#!/bin/bash

LDAPDC=$1
SERVERNAME=$2
PROXY=$3

function usage()
{
        echo "Usage: $0 <Ldap DC> <Server Name> <proxy>"
        echo "Example: $0 test test.librework.com 192.168.100.2"
        exit 1
}

if [ -n "${4}" ] || [ -z "${3}" ]
then
        usage
fi

#install nginx
apt-get install nginx

#remove default site
rm /etc/nginx/sites-available/default
rm /etc/nginx/sites-enabled/default

#config nginx
echo  "server {
   listen 		8000;
   server_name 		${SERVERNAME};
   access_log 		/var/log/nginx/${LDAPDC}_8000.access.log;
   error_log 		/var/log/nginx/${LDAPDC}_8000.error.log;
   return 		301 https://${SERVERNAME}\$request_uri;
}

server {
   listen 		4430;
   server_name 		${SERVERNAME};

   access_log 		/var/log/nginx/${LDAPDC}_443.access.log;
   error_log 		/var/log/nginx/${LDAPDC}_443.error.log;

   # SSL
   ssl on;
   ssl_certificate 	/etc/nginx/ssl/server.crt;
   ssl_certificate_key 	/etc/nginx/ssl/server.key;

   rewrite ^/$ /guacamole;

   location / {
       proxy_pass              http://${PROXY}:8080;
       proxy_next_upstream     error timeout invalid_header http_500 http_502 http_503 http_504;
       proxy_redirect          off;
       proxy_buffering         off;
       proxy_set_header        Host            \$host;
       proxy_set_header        X-Real-IP       \$remote_addr;
       proxy_set_header        X-Forwarded-For \$proxy_add_x_forwarded_for;
       proxy_set_header        X-Forwarded-Proto https;
   }

   location /admin/ {
       proxy_buffering off;
       proxy_pass http://${PROXY}:8080/jsf-blank/;
   }

   location /jsf-blank/ {
       proxy_buffering off;
       proxy_pass http://${PROXY}:8080/jsf-blank/;
   }
}" >>/etc/nginx/sites-available/${LDAPDC}

cd /etc/nginx/sites-enabled/
ln -s /etc/nginx/sites-available/${LDAPDC} ${LDAPDC}

/etc/init.d/nginx restart
