#!/bin/bash

DISK1=$1
DISK2=$2

function usage()
{ 
	echo "Create a new partition to store data. After create a RAID with the swap partition (sda2) and the new data partitions"
	echo "Usage: $0 <disk1> <disk2> "
      	echo "Example: $0 sda sdb"
	exit 1
}

if [ -n "${2}" ] || [ -z "${1}" ]
then
        usage
fi




#*** Set-up software RAID

#- Create partitions with fdisk on both disks (usually /dev/sda and /dev/sdb):

# fdisk /dev/sda
# fdisk /dev/sdb

#Options:
#	p: list partitions
#	n: new partition ('p' for primary). Begin partition on next sector of the last partition
#	t: change type of partition (type 'fd' for RAID)
#	w: save changes and exit

#- (If fdisk says 'no free blocks' use gdisk command instead of fdisk.)

partprobe /dev/sda
partprobe /dev/sdb
 

# Remove /dev/mapper captured devices:

for i in `dmsetup table | awk -F ':' '{print $1}'`
do
	dmsetup remove /dev/mapper/$i
done

mdadm --create /dev/md2 --level=1 --raid-devices=2 /dev/sda2 /dev/sdb2
mdadm --create /dev/md3 --level=1 --raid-devices=2 /dev/sda3 /dev/sdb3


# - Check new md paritions with:
cat /proc/mdstat

# - Change /etc/fstab swap lines.

# For example, add a line like this:

# /dev/md2        swap    swap    defaults        0       0

# and delete old swap partitions such as:

# /dev/sda2	swap	swap	defaults	0	0
# /dev/sdb2	swap	swap	defaults	0	0


mkswap /dev/md2
swapon -a

# - Fix RAID configuration:

/usr/share/mdadm/mkconf > /etc/mdadm/mdadm.conf

