#!/bin/bash

VNETWORK=$1
VMCOREIP=$2

function usage()
{
        echo "Usage: $0 <vnetwork>> <VM-Core ip>"
        echo "Example: $0 dmz0 192.168.100.2"
        exit 1
}

if [ -n "${3}" ] || [ -z "${2}" ]
then
        usage
fi


# Script Start
# Install shorewall
apt-get install shorewall

# Edit /etc/default/shorewall for shorewall automatic startup
sed -i "s/startup=.*/startup=1/" /etc/default/shorewall


# Edit interfaces
echo "#ZONE   INTERFACE       BROADCAST       OPTIONS

net     xenbr0          -               dhcp,tcpflags,nosmurfs,routefilter,logmartians,sourceroute=0

dmz     xapi0           -               tcpflags,nosmurfs,routefilter,logmartians,routeback
">>/etc/shorewall/interfaces


# Edit /etc/shorewall/masq
echo "#INTERFACE              SOURCE          ADDRESS         PROTO   PORT(S) IPSEC   MARK

xenbr0                  10.0.0.0/8,\\
                        169.254.0.0/16,\\
                        172.16.0.0/12,\\
                        192.168.0.0/16
">>/etc/shorewall/masq



# Edit /etc/shorewall/policy
echo "#SOURCE         DEST            POLICY          LOG LEVEL       LIMIT:BURST

dmz             net             ACCEPT

\$FW             net             ACCEPT

net             all             DROP            info

# THE FOLLOWING POLICY MUST BE LAST

all             all             REJECT          info
">>/etc/shorewall/policy



# Edit /etc/shorewall/routestopped
echo "#INTERFACE      HOST(S)                  OPTIONS

xapi0          -
">>/etc/shorewall/routestopped

# Edit /etc/shorewall/shorewall.conf
sed -i "s/IP_FORWARDING=.*/IP_FORWARDING=Yes/" /etc/shorewall/shorewall.conf

# Edit /etc/shorewall/zones
echo "#ZONE   TYPE    OPTIONS                 IN                      OUT

#                                       OPTIONS                 OPTIONS

fw      firewall

net     ipv4

dmz     ipv4
">>/etc/shorewall/zones


# Edit /etc/shorewall/rules

echo "#ACTION         SOURCE          DEST            PROTO   DEST    SOURCE          ORIGINAL        RATE            USER/   MARK    CONNLIMIT       TIME         HEADERS         SWITCH
#                                                       PORT    PORT(S)         DEST            LIMIT           GROUP
#SECTION ALL
#SECTION ESTABLISHED
#SECTION RELATED
SECTION NEW

#       Don't allow connection pickup from the net #
Invalid(DROP)   net             all
#
#       Accept DNS connections from the firewall to the network
#
DNS(ACCEPT)     \$FW             net
#
SSH(ACCEPT)     net                \$FW


REDIRECT	net		8000			tcp	http
REDIRECT        net             4430                    tcp     https
REDIRECT	net		443			tcp	4444

#__DMZ CONFIGURATION__
ACCEPT          \$FW           dmz                      tcp     http,https,8080
DNAT            net            dmz:${VMCOREIP}          tcp     3389
SSH(ACCEPT)     \$FW           dmz
ACCEPT          \$FW           dmz:192.168.100.3       tcp     3389
DNAT            net            dmz:192.168.100.3:3389  tcp     33890



#
#       Allow Ping from the local network
#
Ping(ACCEPT)    dmz             \$FW

#
# Drop Ping from the "bad" net zone.. and prevent your log from being flooded..
#

Ping(DROP)      net             \$FW

ACCEPT          \$FW             dmz             icmp
ACCEPT          \$FW             net             icmp
">>/etc/shorewall/rules


# Restart Shorewall
/etc/init.d/shorewall restart
