#!/bin/bash

DEVICE=$1
VNETWORK=$2
VLAN=$3
DMZIP=$4
DMZMASK=$5


function usage()
{ 
        echo "Usage: $0 <device> <vnetwork> <vlan> <ip> <mask>"
        echo "Example: $0 eth0 dmz0 1 192.168.100.1 255.255.255.0"
        exit 1
}

if [ -n "${6}" ] || [ -z "${5}" ]
then
        usage
fi


echo "openvswitch" > /etc/xcp/network.conf

dpkg-reconfigure xcp-xapi

/etc/init.d/xcp-xapi restart

# Fix qemu network interface script to use openvswitch:

sed -i "s/brctl addif/#brctl addif/g" /etc/xen/scripts/qemu-ifup
echo "ovs-vsctl add-port \$2 \$1" >> /etc/xen/scripts/qemu-ifup

# Configure network interface:

PIF_UUID=`xe pif-list device=${DEVICE} --minimal`
IP=`/sbin/ifconfig ${DEVICE} | grep "inet addr:" | awk -F ':' '{print $2}' | awk '{print $1}'`
MASK=`/sbin/ifconfig ${DEVICE} | grep "inet addr:" | awk -F 'Mask:' '{print $2}' | awk '{print $1}'`
GATEWAY=`netstat -nr | grep "^0.0.0.0" | awk '{print $2}'`
xe pif-reconfigure-ip mode=static DNS=8.8.8.8 \
        ip=${IP} netmask=${MASK} gateway=${GATEWAY} \
        uuid=${PIF_UUID}

cp -n /etc/network/interfaces /etc/network/interfaces.bak-xcp

cat > /etc/network/interfaces << EOF
auto lo
iface lo inet loopback
EOF

# To create a multi-host dmz, using physical interface $DEVICE:

PIF_UUID=`xe pif-list device=${DEVICE} --minimal`
UUID=`xe network-create name-label=${VNETWORK}`
xe vlan-create network-uuid=${UUID} pif-uuid=${PIF_UUID} vlan=${VLAN}
PIF_UUID=`xe pif-list device=${DEVICE} VLAN=${VLAN} --minimal`
xe pif-reconfigure-ip mode=static ip=${DMZIP} netmask=${DMZMASK} uuid=${PIF_UUID}

# This will create a new interface xapiN (xapi0, xapi1, ...) that can be configured on shorewall (you can see who-is-who with ovs-vsctl show, looking for tag parameter equal to vlan on previous command)

# Create VM templates:

/usr/lib/xcp/lib/create_templates

#Configure ISO repository

mkdir /opt/iso

xe sr-create name-label="ISO library" type=iso content-type=iso device-config:legacy_mode=true device-config:location=/opt/iso

#cd /opt/iso && wget ftp://cdimage.debian.org/cdimage/release/current/amd64/iso-cd/debian-*-amd64-netinst.iso

init 6

