#include <QDir>

#include "appinstance.h"

AppInstance::AppInstance ()
{
    pid = 0;
	this->lwClient = 0;
	init();
}

/* Discarding LDAP
AppInstance::AppInstance ( const QString & appName, LdapInfo * ldapInfo )
*/

AppInstance::AppInstance ( const QString & appName )
{
    pid = 0;
	this->lwClient = 0;
	this->appName = appName;

	/* Discarding LDAP
    bool ldapOk = false;
	LdapInfo * ldap = ldapInfo;

	if ( ldap == 0 )
	{
		ldap = new LdapInfo(Settings::getValue("LdapSuffix"), Settings::getValue("LdapServer"), Settings::ldapServerPort() );
		if ( ldap->authenticate( QString("cn=" + Settings::getUserName() + ",cn=users"), Settings::getPassword() ) )
		{
			Settings::logInfo( "LDAP authentication ok on AppInstance." );
			ldapOk = true;
		} else {
			Settings::logError( "LDAP authentication failed." );
			ldapOk = false;
		}
	} else {
		ldapOk = true;
	}

	if ( ldapOk )
	{
		QString ic = firstString( ldap->getValues( "cn=Applications", "cn=" + appName, "iconPath" ));
		command = firstString( ldap->getValues( "cn=Applications", "cn=" + appName, "scriptPath" ));
		applicationType = firstString( ldap->getValues( "cn=Applications", "cn=" + appName, "operatingSystem" ));
		onlyOne = true;
		name = firstString( ldap->getValues( "cn=Applications", "cn=" + appName, "displayName" ));
		iconName = firstString( ldap->getValues( "cn=Applications", "cn=" + appName, "displayName" ));
       		icon = QIcon(ICON_DIR_APPS + QString(QDir::separator()) + ic);
	}
	*/

	init();
}

AppInstance::AppInstance ( const QString & appName, LWClient * lwClient )
{
	Settings::logInfo( QString ("AppInstance. appName: %1").arg( appName ));
    pid = 0;
	this->lwClient = lwClient;
	this->appName = appName;

	if ( lwClient != 0 )
	{
		if ( lwClient->isConnected() )
		{
			QByteArray ic = lwClient->sendMessageByteArray( "get_application_icon?" + appName );
			command = lwClient->sendMessage( "get_application_exec_path?" + appName );
			arguments = parseArguments( lwClient->sendMessage( "get_application_arguments?" + appName ) );
			workingPath = lwClient->sendMessage( "get_application_working_path?" + appName );
			serverId = lwClient->sendMessage( "get_preferred_server?" + appName + "&0.5" );
			server = lwClient->sendMessage( "get_server_address?" + serverId );
			os = lwClient->sendMessage( "get_server_type?" + serverId );
			onlyOne = lwClient->sendMessage( "get_application_onlyone?" + appName ) == "1";
			name = lwClient->sendMessage( "get_application_name?" + appName );
			iconName = lwClient->sendMessage( "get_application_display_name?" + appName );
			QPixmap pixmap;
			pixmap.loadFromData( ic );
			icon = QIcon( pixmap );
		}
	} else {
		setLocalValues();
	}

	init();
}

QStringList AppInstance::parseArguments( QString args )
{
	QStringList ret;
	bool insideQuote = false;
	bool insideSingleQuote = false;

	QString word;
	int i = 0;
	while ( i < args.size() )
	{
		if ( args.at(i) != '"' && args.at(i) != '\'' && args.at(i) != ' ') // Normal character
		{
			word += args.at(i);
			if (i == args.size() - 1) // last character
			{
				ret += word;
			}
		}
		if ( args.at(i) == '"' )
		{
			if (insideSingleQuote) // Cannot understand nested quotes, malformed quotation
			{
				i = args.size();
			}
			if (insideQuote) // End of quoted string
			{
				ret += word;
				word = QString();
				insideQuote = false;
			} else { // Begin of quoted string
				insideQuote = true;
			}
		}
		if ( args.at(i) == '\'' )
		{
			if (insideQuote) // Cannot understand nested quotes, malformed quotation
			{
				i = args.size();
			}
			if (insideSingleQuote) // End of single quoted string
			{
				ret += word;
				word = QString();
				insideSingleQuote = false;
			} else { // Begin of single quoted string
				insideSingleQuote = true;
			}
		}
		if ( args.at(i) == ' ' )
		{
			if (insideQuote || insideSingleQuote) // Space inside quotes
			{
				word += args.at(i);
			} else { // End of word
				ret += word;
				word = QString();
			}
		}
		i++;
	}
	if ( insideQuote || insideSingleQuote ) // malformed quotation
	{
		return args.split(' ');
	} else {
		return ret;
	}
}

void AppInstance::setLocalValues(QIcon icon)
{
    command = Settings::getApplicationValue( appName, "command" );
    arguments = Settings::getApplicationValue( appName, "arguments" ).split(" ");
	workingPath = Settings::homePath();
    os = Settings::getApplicationValue( appName, "os" );
    onlyOne = ( Settings::getApplicationValue( appName, "only-one-instance" ) == "true" );
    name = Settings::getApplicationValue( appName, "name" );
    iconName = Settings::getApplicationValue( appName, "name" );
    if ( icon.isNull() )
    {
        QPixmap pixmap( ICON_DIR_APPS + QString(QDir::separator()) + Settings::getApplicationValue( appName, "icon" ) );
        qDebug() << ICON_DIR_APPS + QString(QDir::separator()) + Settings::getApplicationValue( appName, "icon" );
        this->icon = QIcon( pixmap );
    } else {
        this->icon = icon;
    }
    // Get icon from .exe
    // this->icon = WinInfo::getAppIcon( command );

}

QString AppInstance::firstString( QStringList stringList )
{
	if ( stringList.count() >= 1 )
		return stringList.at(0);
	else
		return QString::null;
}

AppInstance::AppInstance ( const QIcon & icon, const QString & appName )
{
    this->lwClient = 0;
    this->appName = appName;
    setLocalValues( icon );
    init();
}

AppInstance::~AppInstance()
{
}

void AppInstance::init()
{
	window = 0;
}

QProcess * AppInstance::launch( QSize windowSize, quint16 port )
{
    window = 0;
		
	QString exec;

    QProcess * process = new QProcess();

	QProcessEnvironment env = QProcessEnvironment::systemEnvironment();

    if ( ! server.isEmpty() && ! os.isEmpty() && Application::isRemote(server, os) )
	{
        exec = RDP_CLIENT;
	} else {
		if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
		{
#ifdef Q_OS_LINUX
			exec = CHROOT;
#endif
		} else {
#ifdef Q_OS_WIN
            exec = Application::resolveWindowsEnvironment(command);
#else
            exec = command;
#endif
		}
		env.insert("OOO_FORCE_DESKTOP", "kde4");
		env.insert("LANG", "es_ES.UTF-8");
		//env.insert("LANG", "de_DE.UTF-8");
		//env.insert("LANG", "fr_FR.UTF-8");
	}
	process->setProcessEnvironment(env);

    qDebug("Command: '%s'", command.toUtf8().data());
	arguments = Application::getArguments( command, arguments, file, window, windowSize, port, os, server );

	Settings::logInfo( QString ("Launching: %1 %2").arg(exec).arg( arguments.join(" ")) );

	process->setWorkingDirectory( workingPath );
	process->start(exec, arguments );
	process->waitForStarted();

	if ( lwClient != 0 )
	{
		if (lwClient->isConnected())
		{
			pid = process->pid();
            QString res = lwClient->sendMessage( "start_application?" + appName.trimmed() + "&" + QString::number((unsigned long long)pid).trimmed() );
            Settings::logInfo( "start_application?" + appName.trimmed() + "&" + QString::number((unsigned long long)pid).trimmed() + " with result: " + res );
		}
	}

	connect(process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(finish(int, QProcess::ExitStatus)));

	return process;
}

void AppInstance::finish( int exitCode, QProcess::ExitStatus exitStatus )
{
	if ( lwClient != 0 )
	{
		if (lwClient->isConnected())
		{
            QString res = lwClient->sendMessage( "end_application?" + appName.trimmed() + "&" + QString::number((unsigned long long)pid).trimmed() );
            Settings::logInfo( "Sending end_application?" + appName.trimmed() + "&" + QString::number((unsigned long long)pid).trimmed() + " with result: " + res );
		}
	}

	emit finished( exitCode, exitStatus );
}

void AppInstance::deleteAppInstance()
{
	delete this;
}
