#ifndef APPINSTANCE_H
#define APPINSTANCE_H

#include <QObject>
#include <QString>
#include <QStringList>
#include <QIcon>
#include <QPixmap>
#include <QProcess>
#include <QFileInfo>
#include <QMessageBox>

#include "constants.h"
#include "settings.h"
#include "application.h"
/* #include "ldapinfo.h" */
#include "lwclient.h"

class AppInstance : public QObject
{
	Q_OBJECT

	public:
		AppInstance ();
		/* AppInstance ( const QString & appName, LdapInfo * ldap = 0 ); */
		AppInstance ( const QString & appName );
		AppInstance ( const QString & appName, LWClient * lwClient );
		AppInstance ( const QIcon & icon, const QString & appName );
		~AppInstance();

		QStringList parseArguments( QString args );
		void setFile(QString file) { this->file = file; };
		QString getText() { return appName.trimmed(); };
		QString getName() { return name.trimmed(); };
		QString getIconName() { return iconName.trimmed(); };
		QString getCommand() { return command.trimmed(); };
		QString getServer() { return server.trimmed(); };
		QString getServerId() { return serverId.trimmed(); };
        QString getApplicationType() { return os.trimmed(); };
		QIcon getIcon() { return icon; };
		long getWindowId() { return window; };
		bool getOnlyOne() { return onlyOne; };
		QString getFile() { return file; };

        QProcess * launch(QSize size = QSize(), quint16 port = 0);

	public slots:
		void deleteAppInstance();

	signals:
		void finished(int, QProcess::ExitStatus);

	private slots:
		void finish( int exitCode, QProcess::ExitStatus exitStatus );

	private:
		void init();
        void setLocalValues(QIcon icon = QIcon());
		QString firstString( QStringList stringList );

		LWClient * lwClient;

		QString appName;
		QString name;
		QString iconName;
		QString command;
		QString workingPath;
        QString os;
		QString server;
		QString serverId;
		QStringList arguments;
        Q_PID pid;
		QIcon icon;
		long window;
		bool onlyOne;
		QString file;
};

#endif
