#include <QDir>

#include "application.h"

QStringList Application::validApplications = QStringList();

Application::Application() : QObject()
{
}

Application::Application(const QString & name) : QObject()
{
	this->name = name;
	LWClient * lwClient = LWClient::instance();

	if ( lwClient != 0 )
	{
		if ( lwClient->isConnected() )
		{
			QByteArray ic = lwClient->sendMessageByteArray( "get_application_icon?" + name );
			QPixmap pixmap;
			pixmap.loadFromData( ic );
			icon = QIcon( pixmap );
		}
	} else {
		icon = QIcon(ICON_DIR_APPS + QString(QDir::separator()) + Settings::getValue(APPLICATIONS_FILE, name + "/icon"));
	}
}
		
Application::Application ( const QIcon & icon, const QString & name ) : QObject()
{
	this->name = name;
	this->icon = icon;
}

QString Application::getText(QString name)
{
	QString text;

	LWClient * lwClient = LWClient::instance();

	if ( lwClient != 0 )
	{
		text = lwClient->sendMessage( "get_application_display_name?" + name );
    } else {
        text = Settings::getValue( APPLICATIONS_FILE, name +"/name" );
    }

	return text;
}

QString Application::windowsPath(QString file)
{
	QString res;

	res = file.replace(SHARED_DIR, Settings::windowsSharedPath());
	res = res.replace( Settings::homePath(),  Settings::windowsHomePath());
	res = res.replace('/', '\\');

	return res;
}


QString Application::mimeType(QString file)
{
	QFileInfo fileInfo(file);
	return Application::mimeType( fileInfo );
}

QString Application::mimeType(QFileInfo fileInfo)
{
	int count = 0;
	while (fileInfo.isSymLink() && count < MAXLINKDEPTH)
	{
		fileInfo = QFileInfo(fileInfo.symLinkTarget());
		count++;
	}

	if (fileInfo.isSymLink())
		return QString("recursivelink");

	QString ext = "." + fileInfo.suffix();
	QFile extensions (MIME_DIR + QString(QDir::separator()) + "extensions");
	//if ( ! extensions.open(QIODevice::ReadOnly | QIODevice::Text) )
	if ( ! extensions.open(QIODevice::ReadOnly) )
		return QString("application/octet-stream");

	while ( ! extensions.atEnd() )
	{
		QByteArray line = extensions.readLine();
 
		if ( line.startsWith( ext.toUtf8().data() ) )
		{
			line.replace( QString( ext + ": " ).toUtf8().data(), "" );
			return QString( line ).trimmed();
		}
	}

	return QString("application/octet-stream");

/* With magic mime:

	QMimeDatabase mime;

	int count = 0;
	while (fileInfo.isSymLink() && count < MAXLINKDEPTH)
	{
		fileInfo = QFileInfo(fileInfo.symLinkTarget());
		count++;
	}

	if (fileInfo.isSymLink())
		return QString("recursivelink");

	QString res = mime.mimeTypeForFile(fileInfo).name();

	return res.trimmed();
*/
}

QIcon Application::getIcon(QFileInfo file)
{
	QString extension = "." + file.suffix();
	QString program = Application::preferredProgram( extension );

	if ( program.isEmpty() )
	{
		return QIcon();
	}

	Application app(program);
	return app.getIcon();
}

QIcon Application::getIcon(QString applicationName)
{
	Application app(applicationName);
	return app.getIcon();
}

QString Application::preferredProgram(QString extension)
{
	QString app;
	QStringList apps = getValidApplications();

	QList<Application*> appList = applicationList(extension);
	int count = appList.count();
	for (int i = 0; i < count; i++)
	{
		if (apps.contains(appList[i]->getName()))
		{
			app = appList[i]->getName();
			i = count;
		}
	}

	return app;
}

QStringList Application::applicationGroupsList()
{
	QStringList groupList;
	LWClient * lwClient = LWClient::instance();

	if ( lwClient != 0 )
	{
		QString res = lwClient->sendMessage("get_application_groups" );
		if (! res.startsWith("Error:") )
			groupList = res.split(":");
        return groupList.replaceInStrings(QRegExp("^\\s*"),"");
    } else {
        return Settings::getValues(GROUPS_FILE, "name");
    }
}

QStringList Application::applicationGroupIdsList()
{
	QStringList groupIdList;
	LWClient * lwClient = LWClient::instance();

	if ( lwClient != 0 )
	{
		QString res = lwClient->sendMessage("get_application_group_ids" );
		if (! res.startsWith("Error:") )
			groupIdList = res.split(":");
        return groupIdList;
    } else {
        return Settings::getIds(GROUPS_FILE);
    }
}

QList<Application*> Application::applicationListByGroup(QString groupId)
{
	QList<Application*> res;

	QStringList validApplications = getValidApplications();
	QStringList appListByGroup;

	LWClient * lwClient = LWClient::instance();

	if ( lwClient != 0 )
	{
        QString resString = lwClient->sendMessage("get_applications_by_group?" + groupId );
        if (! resString.startsWith("Error:") )
            appListByGroup = resString.split(":");
        int count = appListByGroup.count();
        for (int i = 0; i < count; i++)
        {
            if ( validApplications.contains( appListByGroup[i] ) )
            {
                res.append(new Application(appListByGroup[i]));
            }
        }
    } else {
        for (int i = 0; i < validApplications.count(); i++)
        {
            QStringList groups = Settings::getValue(APPLICATIONS_FILE, validApplications[i] + "/groups").split(":");
            if (groups.contains( groupId))
            {
                res.append(new Application(validApplications[i]));
            }
        }
    }

	return res;
}

QList<Application*> Application::applicationDeletedList(QString extension)
{
	QList<Application*> list;

    QStringList homeAppList = Settings::getValue( MIME_DIR + QString(QDir::separator()) + "extensions", extension).split(":");
	int count = homeAppList.count();
	for (int i = 0; i < count; i++)
	{
		if (homeAppList[i].startsWith("!"))
		{
			list += new Application(homeAppList[i].right( homeAppList[i].length() - 1));
		}
	}

	return list;
}

QList<Application*> Application::applicationList(QString extension)
{
	QList<Application*> list;
	QStringList apps = getValidApplications();

    QStringList homeAppList = Settings::getValue( EXTENSIONS_FILE, extension).split(":");

	QStringList appList;

	LWClient * lwClient = LWClient::instance();

    if ( lwClient == 0 )
	{
        QString type = Settings::getMimetype( extension );
        QStringList applications = Settings::getApplications( type.simplified() );
        for (int i = 0; i < applications.count(); i++)
        {
            if (apps.contains( applications[i] ))
            {
                list += new Application( applications[i] );
            }
        }
        return list;
    } else {
		QString res = lwClient->sendMessage("get_applications?" + extension );
		if (! res.startsWith("Error:") )
			appList = res.split(":");
	}
	QStringList delList;

	int count = homeAppList.count();
	for (int i = 0; i < count; i++)
	{
		if (homeAppList[i].startsWith("!"))
		{
			QString deletedApp = homeAppList[i].right( homeAppList[i].length() - 1);
			appList.removeAll(deletedApp);
			homeAppList.removeAll(homeAppList[i]);
			count = homeAppList.count();
			i--;
		}
	}

	homeAppList += appList;

	homeAppList.removeDuplicates();

	count = homeAppList.count();
	for (int i = 0; i < count; i++)
	{
		if (apps.contains(homeAppList[i]))
		{
			list.append(new Application(homeAppList[i]));
		}
	}

	return list;
}

void Application::deleteList( QList<Application*> list )
{
	for (int i = 0; i < list.count(); i++)
	{
		delete list[i];
	}

	list.clear();
}

QString Application::resolveWindowsEnvironment( QString command )
{
    QString res = command.replace("\\", "\\\\");
    QStringList variables;
    variables += "%programw6432%";
    variables += "%programfiles(x86)%";
    variables += "%programfiles%";
    variables += "%windir%";
    variables += "%systemroot%";
    variables += "%system32%";
    variables += "%system%";
    variables += "%homepath%";
    variables += "%temp%";
    variables += "%tmp%";

    for ( int i = 0; i < variables.count(); i++ )
    {
        res = res.replace( variables[i], QString( qgetenv( variables[i].replace("%", "").toUtf8().data() )).replace("\\", "\\\\"), Qt::CaseInsensitive);
    }
    res = res.replace("\\\\", "\\");
    return res.replace("%", "");
}

void Application::setValidApplications()
{
    validApplications.clear();

    LWClient * lwClient = LWClient::instance();

    if ( lwClient != 0 )
    {
        QString res = lwClient->sendMessage("get_valid_applications");
        if (! res.startsWith("Error:") && ! res.simplified().isEmpty())
        {
            validApplications = res.split(",");
            Settings::logInfo( "Valid applications: " + validApplications.join(",") );
        }
    } else {
        QStringList applications = Settings::getApplicationsList();
        for (int i = 0; i < applications.count(); i++)
        {
            qDebug("Application: %s", applications[i].toUtf8().data());
            QString command = Settings::getApplicationValue( applications[i], "command" );
            QString server = Settings::getApplicationValue( applications[i], "server" );
            QString os = Settings::getApplicationValue( applications[i], "os" );
            QString protocol = Settings::getApplicationValue( applications[i], "protocol" );

            if ( ! command.contains(QDir::separator()) ) // command without path
            {
#ifdef Q_OS_WIN
                QStringList path = QString(qgetenv("PATH")).split(';');
#else
                QStringList path = QString(qgetenv("PATH")).split(':');
#endif
                for (int p = 0; p < path.count(); p++)
                {
                    //qDebug("Checking: %s", QString( path[p] + QDir::separator() + command).toUtf8().data());
                    if ( isValid( path[p] + QDir::separator() + command, server, os, protocol))
                    {
                        validApplications.append(applications[i]);
                        break;
                    }
                }
            }
#ifdef Q_OS_WIN
            command = resolveWindowsEnvironment( command );
#endif

            //qDebug("Command: %s, server: %s, os: %s, localOS: %s, protocol: %s", command.toUtf8().data(), server.toUtf8().data(), os.toUtf8().data(), Settings::getLocalOs().toUtf8().data(), protocol.toUtf8().data());
            if ( isValid( command, server, os, protocol ) )
            {
                validApplications.append(applications[i]);
            }
        }
    }
}

bool Application::isRemote(QString server, QString os)
{
    bool res = true;
    if ( Settings::isLocalhost( server ) && os.compare(Settings::getLocalOs(), Qt::CaseInsensitive ) == 0 )
    {
        res = false;
    }
    return res;
}

bool Application::isValid(QString command, QString server, QString os, QString protocol)
{
    bool res = false;
    if ( QFile::exists( command ) )
    {
        if ( Settings::isLocalhost( server ) )
        {
            if ( os.compare( Settings::getLocalOs(), Qt::CaseInsensitive ) == 0 )
            {
                res = true;
            }
        } else {
            if ( ! protocol.isEmpty() )
            {
                res = true;
            }
        }
    } else {
        if ( ! Settings::isLocalhost( server ) && ! protocol.isEmpty() )
        {
            res = true;
        }
    }
    return res;
}

void Application::setValidApplications(QStringList apps)
{
    Application::validApplications.clear();
	Application::validApplications << apps;
}

QStringList Application::getValidApplications()
{
	return Application::validApplications;
}

void Application::addMimeType(QString mimeType)
{
    Settings::setValue( MIME_DIR + QString(QDir::separator()) + "types", mimeType, QString::null);
}

void Application::addExtension(QString extension)
{
    Settings::setValue( MIME_DIR + QString(QDir::separator()) + "extensions", extension, QString::null);
}

QStringList Application::getExtensionsList()
{
    QStringList res;

	LWClient * lwClient = LWClient::instance();

	if ( lwClient != 0 )
	{
		QString lwRes = lwClient->sendMessage("get_application_extensions_list");
		if ( ! lwRes.startsWith("Error:") )
			res += lwRes.split(",");
    } else {
        QFile extensionsFile(EXTENSIONS_FILE);
        if (extensionsFile.open(QIODevice::ReadOnly))
        {
           QTextStream in(&extensionsFile);
           while (!in.atEnd())
           {
               res += in.readLine();
           }
           extensionsFile.close();
        }
    }

	return res;
}

QString Application::getValue(QString key)
{
	QString primaryFile = Settings::homePath() + "/.desktux/mime/applications";
	QString secondaryFile = MIME_DIR + QString(QDir::separator()) + "applications";

	return Settings::getValue(primaryFile, secondaryFile, key);
}

QStringList Application::getArguments(QString command, QStringList arguments, QString file, long window, QSize windowSize, quint16 port, QString os, QString serverAddress )
{
    if ( ! serverAddress.isEmpty() && ! os.isEmpty() && isRemote(serverAddress, os) )
	{
        return remoteApplicationArguments( command, arguments, file, window, windowSize, port, serverAddress );
    } else {
		QStringList args;
		if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
		{
			args += "-p"; // Preserve environment on schroot
			args += "-c";
			args += "chroot:" + Settings::getUserName();
			args += "--"; // No more options
			args += command.trimmed().split(' ');
            args += arguments;
		} else {
            args = arguments;
		}
		if ( ! file.isEmpty() )
        {
#ifdef Q_OS_WIN
            args += file.replace("/", "\\");
#endif
#ifdef Q_OS_LINUX
            args += file;
#endif
        }
		return args;
	}
}

QStringList Application::remoteApplicationArguments( QString command, QStringList args, QString file, long window, QSize windowSize, quint16 port, QString serverAddress )
{
    QStringList arguments;

	QString shell = "%programfiles%\\Desktux\\fullscreenshell.exe";
//	QString shell = "c:\\fullscreenshell\\fullscreenshell.exe";
//	QString shell = "c:\\seamlessrdp\\seamlessrdpshell.exe";

	QString server = serverAddress;
	QHostAddress address = Settings::getLocalAddress();

	QString commandString;

    commandString = shell + " \"" + command + args.join(" ") +
			"\" \"" + address.toString() +
			"\" \"" + QString::number(port) + "\"";
	if ( ! file.isEmpty() )
		commandString += " \"" + Application::windowsPath(file) + "\"";

	arguments += "-u";
	arguments += Settings::getUserName();

	arguments += "-d";
	arguments += Settings::getValue("SambaDomain");

	arguments += "-p";
	arguments += Settings::getPassword();

	arguments += "-g";
	arguments += QString::number(windowSize.width()) + "x" + QString::number(windowSize.height());

	arguments += "-X";
	arguments += QString::number((long)window);

	arguments += "-s";
	arguments += commandString;

	if ( QString(RDP_CLIENT) == "xfreerdp" )
	{
		arguments += "--no-nla";

		//arguments += "--plugin";
		//arguments += "rdpsnd";

		arguments += "--plugin";
		arguments += "cliprdr";

		arguments += "--plugin";
		arguments += "rdpdr";
		arguments += "--data";
		arguments += "printer";

		arguments += "--";
	} else {
		arguments += "-r";
		arguments += "sound:local";
#ifdef Q_OS_LINUX
		QStringList printers = CupsManager::getPrinters();
		for (int i = 0; i < printers.count(); i++)
		{
			arguments += "-r";
			arguments += "printer:" + printers[i] + "=MS Publisher Imagesetter";
		}
#endif
	}

	arguments += server;

        return arguments;
}
