#ifndef APPLICATION_H
#define APPLICATION_H

#include <QObject>
#include <QString>
#include <QStringList>
#include <QIcon>
#include <QPixmap>
#include <QFileInfo>
#include <QFile>
#include <QMessageBox>
#include <QMimeDatabase>

#include "constants.h"
#include "settings.h"
#include "cupsmanager.h"
#include "lwclient.h"

class Application : public QObject
{
	Q_OBJECT

	public:
		Application();
		Application( const QString & appName );
		Application ( const QIcon & icon, const QString & appName );

		void setName(QString name) { this->name = name; };
		QString getName() { return name; };
		void setIcon(QIcon icon) { this->icon = icon; };
		QIcon getIcon() { return icon; };

		static QString getText(QString name);
		static QString mimeType(QString file);
		static QString mimeType(QFileInfo fileInfo);
		static QIcon getIcon(QFileInfo file);
		static QIcon getIcon(QString applicationName);
		static QString preferredProgram(QString extension);
		static QList<Application*> applicationList(QString extension);
		static QList<Application*> applicationDeletedList(QString extension);
		static QList<Application*> applicationListByGroup(QString group);
		static QStringList applicationGroupsList();
		static QStringList applicationGroupIdsList();
		static QStringList getExtensionsList();
		static void setValidApplications();
		static void setValidApplications(QStringList apps);
		static QStringList getValidApplications();
		static void addMimeType(QString mimeType);
		static void addExtension(QString extension);
		static QString getValue(QString key);
		static Application * openFile(QString file, QString application = QString::null);
		static QString windowsPath(QString file);
		static void deleteList( QList<Application*> list );
		static QStringList getArguments(QString command, QStringList arguments, QString file, long window, QSize windowSize, quint16 port, QString os, QString serverAddress );
		static QStringList remoteApplicationArguments( QString command, QStringList args, QString file, long window, QSize windowSize, quint16 port, QString serverAddress );
		static QString resolveWindowsEnvironment( QString command );
		static bool isRemote(QString server, QString os);
		static bool isValid(QString command, QString server, QString os, QString protocol);
	private:
		QString name;
		QIcon icon;
		static QStringList validApplications;
};

#endif
