#include "appserver.h"

AppServer::AppServer( ThreadButton * button, QHostAddress remoteAddress ) : QTcpServer( button )
{
	socket = 0;
	this->remoteAddress = remoteAddress;
}

AppServer::~AppServer()
{
	delete socket;
}

bool AppServer::listen()
{
	if ( ! Settings::getLocalAddress().isNull() )
	{
		bool res = QTcpServer::listen();
		Settings::logInfo( QString( "Listening on Address %1, port %2" )
			.arg( Settings::getLocalAddress().toString().toUtf8().data() )
			.arg( serverPort() ) );
		return res;
	} else {
		return false;
	}
}

void AppServer::incomingConnection ( int socketDescriptor )
{
	if (socket == 0) // Only accept the first connection
	{
		socket = new QTcpSocket();
		socket->setSocketDescriptor( socketDescriptor );
		if ( socket->peerAddress() == remoteAddress )
		{
			connect( socket, SIGNAL(readyRead()), this, SLOT(readData()), Qt::DirectConnection);
		} else {
			qDebug("!!!Someone is trying to usurp the connection!!!");
			socket->abort();
			delete socket;
			socket = 0;
		}
	} else {
		QTcpSocket * newSocket = new QTcpSocket();
		newSocket->setSocketDescriptor( socketDescriptor );
		newSocket->abort();
		delete newSocket;
	}
}

void AppServer::readData()
{
	qDebug("Ready to read %d bytes", (int)socket->bytesAvailable());
        QByteArray command = socket->readAll();

	qDebug("Command: '%s'", QString(command).toUtf8().data());

	if (QString(command).trimmed() == "running")
	{
		emit running();
	}

        if (QString(command).trimmed() == "close")
	{
		emit closeTab();
	}

	if ( QString(command).startsWith("filename=") )
	{
		emit renameTab( QString(command).remove("filename=") );
	}
}

void AppServer::closeApp()
{
	if (socket != 0)
	{
		Settings::logInfo( QString( "Closing socket on Address %1, port %2" )
			.arg( Settings::getLocalAddress().toString().toUtf8().data() )
			.arg( serverPort() ) );
		socket->write("close");
	}
}

void AppServer::killApp()
{
	if (socket != 0)
		socket->write("kill");
}

void AppServer::disconnectSession()
{
	if (socket != 0)
		socket->write("disconnect");
}
