#include "decorator.h"

static int counter = 0;

Decorator::Decorator ( QPoint pos, QSize wsize, long window, QWidget * parent, int section ) : QWidget (parent, Qt::X11BypassWindowManagerHint)
{
	counter++;
	this->window = window;
	this->section = section;
	this->wsize = wsize;

//	QGraphicsOpacityEffect effect;
//	effect.setOpacity(0.0);
//	setGraphicsEffect(&effect);

	setStyleSheet("background-color: #" BORDERCOLOR);

	setWindowModality(Qt::NonModal);
//	setAutoFillBackground ( false );
//	setWindowOpacity(0.0);
//	setAttribute( Qt::WA_TranslucentBackground, true );
//	setAttribute( Qt::WA_NoSystemBackground );
//	setAttribute( Qt::WA_OpaquePaintEvent );

	windows.append(window);
	decorators.append(this);

	switch (section)
	{
		case Qt::TopSection:
			setGeometry(pos.x(), pos.y() - DECHEIGHT, wsize.width(), DECHEIGHT);
			new Decorator( pos, wsize, window, parent, Qt::TopLeftSection );
			new Decorator( pos, wsize, window, parent, Qt::LeftSection );
			new Decorator( pos, wsize, window, parent, Qt::BottomLeftSection );
			new Decorator( pos, wsize, window, parent, Qt::TopRightSection );
			new Decorator( pos, wsize, window, parent, Qt::RightSection );
			new Decorator( pos, wsize, window, parent, Qt::BottomRightSection );
			new Decorator( pos, wsize, window, parent, Qt::BottomSection );
			break;
		case Qt::TopLeftSection:
			setGeometry(pos.x() - DECWIDTH, pos.y() - DECHEIGHT, DECWIDTH, DECHEIGHT);
			break;
		case Qt::LeftSection:
			setGeometry(pos.x() - DECWIDTH, pos.y(), DECWIDTH, wsize.height());
			break;
		case Qt::BottomLeftSection:
			setGeometry(pos.x() - DECWIDTH, pos.y() + wsize.height(), DECWIDTH, DECHEIGHT);
			break;
		case Qt::TopRightSection:
			setGeometry(pos.x() + wsize.width(), pos.y() - DECHEIGHT, DECWIDTH, DECHEIGHT);
			break;
		case Qt::RightSection:
			setGeometry(pos.x() + wsize.width(), pos.y(), DECWIDTH, wsize.height());
			break;
		case Qt::BottomRightSection:
			setGeometry(pos.x() + wsize.width(), pos.y() + wsize.height(), DECWIDTH, DECHEIGHT);
			break;
		case Qt::BottomSection:
			setGeometry(pos.x(), pos.y() + wsize.height(), wsize.width(), DECHEIGHT);
			break;
		default:
			break;
	}

	show();
}

void Decorator::paintEvent(QPaintEvent *)
{
	QPainter painter(this);

	QPixmap pixmap;

	switch (section)
	{
		case Qt::TopSection:
			pixmap.load( ICON_DIR_WM "/top.png" );
			pixmap = pixmap.scaled(wsize.width(), DECHEIGHT, Qt::IgnoreAspectRatio);
			break;
		case Qt::TopLeftSection:
			pixmap.load( ICON_DIR_WM "/topleft.png" );
			pixmap = pixmap.scaled(DECWIDTH, DECHEIGHT, Qt::IgnoreAspectRatio);
			break;
		case Qt::LeftSection:
			pixmap.load( ICON_DIR_WM "/left.png" );
			pixmap = pixmap.scaled(DECWIDTH, wsize.height(), Qt::IgnoreAspectRatio);
			break;
		case Qt::BottomLeftSection:
			pixmap.load( ICON_DIR_WM "/bottomleft.png" );
			pixmap = pixmap.scaled(DECWIDTH, DECHEIGHT, Qt::IgnoreAspectRatio);
			break;
		case Qt::TopRightSection:
			pixmap.load( ICON_DIR_WM "/topright.png" );
			pixmap = pixmap.scaled(DECWIDTH, DECHEIGHT, Qt::IgnoreAspectRatio);
			break;
		case Qt::RightSection:
			pixmap.load( ICON_DIR_WM "/right.png" );
			pixmap = pixmap.scaled(DECWIDTH, wsize.height(), Qt::IgnoreAspectRatio);
			break;
		case Qt::BottomRightSection:
			pixmap.load( ICON_DIR_WM "/bottomright.png" );
			pixmap = pixmap.scaled(DECWIDTH, DECHEIGHT, Qt::IgnoreAspectRatio);
			break;
		case Qt::BottomSection:
			pixmap.load( ICON_DIR_WM "/bottom.png" );
			pixmap = pixmap.scaled(wsize.width(), DECHEIGHT, Qt::IgnoreAspectRatio);
			break;
		default:
			break;
	}
//	painter.drawPixmap(0, 0, pixmap);
//	painter.fillRect(0, 0, width(), height(), QBrush(pixmap) );
}

void Decorator::setWindow(long window)
{
	this->window = window;
}

long Decorator::getWindow()
{
	return window;
}

void Decorator::closeDecorator(long window)
{
	int num = windows.indexOf(window);

	while ( num != -1 )
	{
		counter--;
		decorators.at(num)->close();
		delete decorators.at(num);
		windows.removeAt(num);
		decorators.removeAt(num);
		num = windows.indexOf(window);
	}
}

bool Decorator::hasDecorator(long window)
{
	return windows.contains( window );
}
