#include "desktop.h"

Desktop::Desktop()
{
	tabBar = 0;
#ifdef Q_OS_WIN
    QString username = qgetenv("USERNAME");
#else
    QString username = qgetenv("USER");
#endif
    Settings::init();
	Settings::setUserName( username );
    Settings::logInfo( QString( "Starting session with user '%1'" ).arg(username) );

    qDebug("Settings file: %s", QString(CONFIG_FILE).toUtf8().data());

	if ( Settings::getValue( "Server/LibreworkServerPort" ).toInt() > 0 )
	{
        QString token;
        qDebug("Looking for token on file: %s", QString(TOKEN_FILE).toUtf8().data());
		if (QFile::exists( TOKEN_FILE ))
        {
			token = Settings::getValue( TOKEN_FILE, "UserToken" );
        }

        if (connectToServer())
        {
            if ( token.isEmpty() )
            {
                InitScreen * initScreen = new InitScreen();
                bool auth = false;

                while (! auth )
                {
                    initScreen->setFixedSize(QApplication::desktop()->size());
                    initScreen->showFullScreen();
                    initScreen->centerDialog();
                    int res = initScreen->exec();
                    if ( res == QDialog::Rejected )
                    {
                        delete initScreen;
                        exitApp();
                        return;
                    } else {
                        QString initPassword = initScreen->getPassword();
                        Settings::setPassword( initPassword );
                        auth = lwClient->authenticate( username, initPassword );
                        if ( auth )
                        {
                            token = lwClient->sendMessage("activate_user?" + username);
                            if (! token.isEmpty()  && ! token.startsWith("Error:") )
                            {
								Settings::setValue(TOKEN_FILE, "UserToken", token);
                            } else {
                                QMessageBox::information(0, tr("Server connection"), tr("Cannot enable user. Contact your system administrator."));
                                exitApp();
                                return;
                            }
                        } else {
                            QMessageBox::information(0, tr("Server connection"), tr("The password you supplied is incorrect. Please, try again."));
                        }
                    }
                }
            } else {
                bool auth = lwClient->authenticate( username, token );
                if (! auth )
                {
					QMessageBox::information(0, tr("Server connection"), tr("Cannot connect to Librework Server with current user token. Contact your system administrator."));
					exitApp();
					return;
                }
            }
        } else {
			QMessageBox::information(0, tr("Server connection"), tr("Cannot connect to Librework Server. Contact your system administrator."));
			exitApp();
			return;
        }
        QString res = lwClient->sendMessage( "start_session" );
        Settings::logInfo( res );
    } else {
        lwClient = 0;
    }
#ifdef Q_OS_LINUX
	QProcess::execute("amixer set Master 100%");
#endif
	currentProcess = 0;
	currentName = "";

	loadingTab = 0;

	connect( QApplication::desktop(), SIGNAL( resized(int) ), this, SLOT( resize(int) ) );

#ifdef Q_OS_WIN
	sessionManager = new SessionManager();
	connect( sessionManager, SIGNAL(clientConnectEvent()), this, SLOT(clientConnected()));
	sessionManager->start();
	manager = new WinManager( this );
#endif
#ifdef Q_OS_LINUX
	manager = new Manager( this );
#endif
	connect( manager, SIGNAL(mapWindow(long long)), this, SLOT(mapWindow(long long))); //, Qt::DirectConnection );
	connect( manager, SIGNAL(unmapWindow(long long)), this, SLOT(unmapWindow(long long))); //, Qt::DirectConnection );
	connect( manager, SIGNAL(titleChanged(long long,QString)), this, SLOT(titleChanged(long long, QString))); //, Qt::DirectConnection );

	tabBar = new TabBar( 0 );
	tabBar->setGeometry( 0,QApplication::desktop()->size().height() - TAB_BUTTON_HEIGHT, QApplication::desktop()->size().width(), TAB_BUTTON_HEIGHT );
	connect (tabBar, SIGNAL(closeButtonClicked(long long)), this, SLOT(closeButtonClicked(long long)) );
	connect (tabBar, SIGNAL(killButtonClicked(long long )), this, SLOT(killButtonClicked(long long)) );
	connect (tabBar, SIGNAL(currentChanged(int)), this, SLOT(setSelected(int)) );
	connect (tabBar, SIGNAL(launchApplication(AppInstance*)), this, SLOT(launchApplication(AppInstance*)) );
	connect (tabBar, SIGNAL(openFile(QString, QString)), this, SLOT(openFile(QString, QString)) );
	connect (tabBar, SIGNAL(mapWidget(QWidget*,bool)), this, SLOT(mapWidget(QWidget*,bool)) );
	connect( tabBar, SIGNAL(exitApp()), this, SLOT(exitApp()));

#ifdef Q_OS_WIN
	manager->start();
#endif
	resize();
	tabBar->showDesktop();
}

bool Desktop::connectToServer()
{
    LWClient::setInstance( new LWClient() );
    lwClient = LWClient::instance();
    if ( lwClient->connect() )
    {
        lwClient->waitForConnected();


        /* chroot environment now is managed by librework-sesman
        res = lwClient->sendMessage( "mount_chroot?" +
                    Settings::getValue( "ChrootHome" ) + "&" +
                    Settings::getValue( "ChrootMount" ) + "&" +
                    Settings::getUserName() + "&" +
					Settings::homePath() + "&" +
                    Settings::getValue( "SambaServer" ) );
        Settings::logInfo( res );
        */
        return true;
    } else {
        delete lwClient;
        LWClient::setInstance( 0 );
        lwClient = 0;
        Settings::logError( tr("Cannot connect to lwserver") );
        return false;
    }
}

Desktop::~Desktop()
{
}

QSize Desktop::getDesktopSize()
{
	return QSize( QApplication::desktop()->size().width(), QApplication::desktop()->size().height() - TAB_BUTTON_HEIGHT);
}

void Desktop::resize(int screen)
{
    Q_UNUSED(screen);
#ifdef Q_OS_WIN
	WinInfo::hideTaskBar();
	manager->resize(QRect( QPoint(0, 0), getDesktopSize()));
#endif
#ifdef Q_OS_LINUX
	for (int i = tabBar->menuButtonCount() - 1; i < tabBar->count(); i++)
	{
		X11Info::attachWindow(tabBar->getButton(i)->getWinId());
	}
#endif
	tabBar->resize( getDesktopSize() );
	tabBar->show();
#ifdef Q_OS_LINUX
	tabBar->showDesktop();
#endif
}

void Desktop::clientConnected()
{
	resize();
}

void Desktop::exitApp()
{
	QString res;

	bool openApplications = false;

	if ( tabBar != 0)
	{
		if (tabBar->getButtonList()->count() > tabBar->menuButtonCount()) // There are open applications
		{
			openApplications = true;
		}

		if ( openApplications )
		{
			if ( QMessageBox::question(NULL, tr("Suspend session"), tr("There are currently running applications. Do you want to suspend session keeping them running?"), QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
			{
#ifdef Q_OS_WIN
				WTSDisconnectSession(WTS_CURRENT_SERVER_HANDLE, WTS_CURRENT_SESSION, FALSE);
				return;
#endif
#ifdef Q_OS_LINUX
#endif
			}
		}
	}

	if (lwClient != 0)
	{
		if (lwClient->isConnected())
		{
			/* chroot environment now is managed by librework-sesman
			if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
			{
				res = lwClient->sendMessage( "umount_chroot?" + Settings::getUserName() );
				Settings::logInfo( res );
			}
			*/
			res = lwClient->sendMessage( "end_session" );
			Settings::logInfo( res );
		}
		delete lwClient;
	}
	if ( tabBar != 0 )
	{
		tabBar->close();
		delete tabBar;
	}
#ifdef Q_OS_WIN
	WinInfo::showTaskBar();
	ExitWindows(0, 0);
#endif
	QApplication::exit(0);
}

void Desktop::timeout()
{
    if (QApplication::desktop()->cursor().shape() == Qt::WaitCursor)
    {
        QApplication::desktop()->setCursor(Qt::ArrowCursor);
    }
	timer.stop();
	disconnect(&timer, SIGNAL(timeout()), this, SLOT(timeout()));
}

void Desktop::openFile(QString file, QString application)
{
	qDebug("Trying to open file %s with application %s", file.toUtf8().data(), application.toUtf8().data());
	AppInstance * app;
	if (application.isEmpty())
	{
		QString extension = "." + QFileInfo( file ).suffix();
		if (Application::mimeType(file) == "recursivelink")
		{
            QMessageBox::critical ( 0, tr("Link error"), tr("This file is a recursive set of links"));
			return;
		}
		QString myApp = Application::preferredProgram(extension);
		if (myApp.isEmpty())
		{
			return;
		}
		app = new AppInstance(myApp, lwClient);
	} else {
		app = new AppInstance(application, lwClient);
	}
	app->setFile(file);
	launchApplication(app);

	connect( app, SIGNAL(finished(int, QProcess::ExitStatus)), app, SLOT(deleteAppInstance()) );
}

/*
void Desktop::openMime(QString location, QString mime)
{
	qDebug("Called dbus mime %s (location %s)", mime.toUtf8().data(), location.toUtf8().data());
	QDBusMessage msg = QDBusMessage::createSignal("/", "com.tuxum.desktux", "openMime");
	msg << "ok";
	QDBusConnection::sessionBus().send(msg);
}
*/

void Desktop::unmapWindow(long long window)
{
	bool stillActive = false;
    if (window != 0)
	{
#ifdef Q_OS_WIN
        DWORD pid = WinInfo::getWindowPid(WinInfo::fromId(window));
        HANDLE process = OpenProcess(SYNCHRONIZE, FALSE, pid);
		DWORD ret = WaitForSingleObject(process, 0);
		CloseHandle(process);
		if (ret == STILL_ACTIVE)
		{
			stillActive = true;
		}
#endif
#ifdef Q_OS_LINUX
        Q_PID pid = X11Info::getWindowPid(window);
		int ret = 0;
		if ( pid > 0 )
		{
			ret = kill( pid, 0 );
		}

        if ( ret != ESRCH  && X11Info::isValidWindow( window ) )
		{
			stillActive = true;
		}
#endif
		if (! stillActive)
		{
			//qDebug("Removing tab for pid %d and ret %d",(int)pid, (int)ret);
            int index = tabBar->indexOf( window );
			//qDebug("Trying to remove index %d", index);
			if ( index >= tabBar->menuButtonCount() )
			{
				tabBar->removeButton( index );
				qDebug("Removed index %d", index);
				return;
			}
		} else {
			//qDebug("Current pid is running, so not removing tab");
			return;
		}
	}
	//qDebug("Cannot remove tab as it contains no window");
}

void Desktop::mapWidget(QWidget * widget, bool force)
{
	qDebug("Mapping widget...");
	int index = tabBar->indexOf( widget );
	qDebug("Index: %d", index);
	if ( ( index >= 0 && index != tabBar->getCurrentIndex() ) || force )
	{
		qDebug("Setting current index to %d", index);
		tabBar->setCurrentIndex( index );
	} else {
		return;
	}
	widget->setWindowFlags(Qt::CustomizeWindowHint | Qt::FramelessWindowHint);
	widget->setFixedSize( getDesktopSize() );
	widget->setGeometry(QRect(QPoint(0, 0), getDesktopSize()));
	widget->setFocus();
	widget->show();
	widget->activateWindow();
    widget->updateGeometry();
#ifdef Q_OS_LINUX
	X11Info::attachWindow( widget->winId() );
#endif
	qDebug() << QString("Widget size: %1x%2").arg(widget->size().width()).arg(widget->size().height()).toUtf8().data();
}

void Desktop::mapNotify(long long window)
{
    Q_UNUSED( window );
}

void Desktop::mapWindow(long long window)
{
#ifdef Q_OS_LINUX
	Q_PID pid = X11Info::getWindowPid(window);
#endif
#ifdef Q_OS_WIN
        unsigned long pid = WinInfo::getWindowPid(WinInfo::fromId(window));
#endif

	int index = tabBar->indexOf(window);

	//qDebug("Index of window %d is %d", window, index);

    if ( index >= 0)
    {
        if ( tabBar->getButton( index )->getType() == MENUBUTTON )
        {
            return; // mapWidget will take care
        }

        tabBar->setCurrentIndex( index );
#ifdef Q_OS_WIN
        manager->showWindow( window );
#endif
#ifdef Q_OS_LINUX
		X11Info::attachWindow( window );
#endif
        setThreadWidgetTitle( window );
        setThreadWidgetName( index );
		return;
    }

	if (currentIcon.isNull())
	{
		currentIcon = QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "default.png");
    }

	bool attach = true;

#ifdef Q_OS_WIN
	attach = WinManager::isAppWindow( window ) && WinInfo::getWindowTitle( (HWND)window ) != "desktux";
#endif

	if (attach) // If is application and is not desktux...
	{
#ifdef Q_OS_WIN
        qDebug("Attaching window %s (desktop size: %dx%d)", WinInfo::getWindowTitle( (HWND)window ).toUtf8().data(), getDesktopSize().width(), getDesktopSize().height());
		WinManager::showWindow( window );
        ThreadButton * button = new ThreadButton(0, THREADBUTTON, currentIcon.pixmap(QSize(TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT)) /*WinInfo::getWindowIcon( WinInfo::fromId( window ) ).pixmap(QSize(TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT))*/, currentName, currentName, window, pid);
#endif

#ifdef Q_OS_LINUX
        ThreadButton * button = new ThreadButton(0, THREADBUTTON, currentIcon.pixmap(QSize(TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT)), currentName, currentName, window, pid);
#endif
        tabBar->addButton( button );
		connect( button, SIGNAL(threadButtonClicked(long long)), this, SLOT(mapWindow(long long)));
		if (QApplication::desktop()->cursor().shape() == Qt::WaitCursor)
		{
			QApplication::desktop()->setCursor(Qt::ArrowCursor);
		}
        setThreadWidgetTitle( window );
        setThreadWidgetName( index );
    }
}

void Desktop::setThreadWidgetName(int index)
{
    QString label;

    if ( index >= 0 )
    {
        QString label = tabBar->getButton(index)->getLabel();
        tabBar->setCurrentThreadName( label );
        if (tabBar->getButton( index )->getWidget() != 0)
        {
            tabBar->setCurrentThreadText( tabBar->getButton( index )->getText() );
        }
    }
}

void Desktop::setThreadWidgetTitle(long long window)
{
    QString text;

    if (window >= 0)
    {
        int index = tabBar->indexOf( window );
        if ( index >= 0 )
        {
            if ( tabBar->getButton(index)->getWidget() != 0 ) // Principal buttons, no need to set title
            {
                return;
            }
        }
#ifdef Q_OS_WIN
        HWND w = WinInfo::fromId(window);
        text = WinInfo::getWindowTitle( w );
#endif

#ifdef Q_OS_LINUX
        text = X11Info::getWindowName( window );
/*
    QString remove = "- " + label;
    name.remove( remove );

    if ( name.isEmpty() )
    {
        name = text;
    }
*/
#endif
        tabBar->setCurrentThreadText( text );
    }
}

void Desktop::titleChanged(long long window, QString newTitle)
{
	//qDebug("Title changed!!! to %s", newTitle.toUtf8().data());
    int index = tabBar->indexOf(window);
    if (index == tabBar->getCurrentIndex())
    {
        tabBar->setCurrentThreadText( newTitle );
    } else {
		//qDebug("Not current title, as current index is %d and window index is %d", tabBar->getCurrentIndex(), index);
    }
}

void Desktop::closeButtonClicked(long long window)
{
#ifdef Q_OS_WIN
	WinInfo::closeWinWindow( WinInfo::fromId( (long) window) );
    WinInfo::selectWindow( WinInfo::fromId( (long) window) );
#endif

#ifdef Q_OS_LINUX
/*
	if (isRemoteApp())
        {
                QList<Window> children = X11Info::getChildren(window);
                int count = children.count();
                if (count >= 1)
                        X11Info::closeWinWindow( children[count - 1] );
        } else {
*/
                X11Info::closeXorgWindow( window );
//        }
#endif
}

void Desktop::killButtonClicked(long long window)
{
#ifdef Q_OS_WIN
	WinInfo::killWinWindow( WinInfo::fromId( (long) window) );
#endif

#ifdef Q_OS_LINUX
/*	if (isRemoteApp( window ))
	{
		QList<Window> children = X11Info::getChildren(window);
		int count = children.count();
		if (count >= 1)
		{
			X11Info::killWinWindow( children[count - 1] );
		}
	} else {
*/
		X11Info::killXorgWindow( window );
//	}
#endif
}

void Desktop::launchApplication(AppInstance * app)
{
	if (QApplication::desktop()->cursor().shape() == Qt::WaitCursor && timer.isActive() )
	{
		return;
	}

	QApplication::desktop()->setCursor(Qt::WaitCursor);
	QApplication::processEvents();

	timer.start(TIMEOUT * 1000);
	connect(&timer, SIGNAL(timeout()), this, SLOT(timeout()));

	QIcon icon = app->getIcon();
	currentIcon = icon;
	QString appName = app->getName();
	currentName = appName;
	QString type = app->getApplicationType();
	bool onlyOne = app->getOnlyOne();
	bool remote = false;
	/* TO-DO: if app->server is localhost then remote is false, else true */

	if ( onlyOne )
	{
		int count = tabBar->count();
		for (int i=0; i<count; i++)
		{
			Settings::logInfo( QString("Looking for onlyOne existing application. AppName(%1): %2")
				.arg(i)
				.arg(appName));
            /*
            if (appName == tabbBar->getName())
			{
                tabBar->setCurrentIndex(i);
				timeout();
				return;
            }*/
		}
	}

	qDebug("Type: %s", type.toUtf8().data());
	if ( remote )
	{
		QHostAddress server = QHostAddress( app->getServer() );
		if ( Settings::getLocalAddress().isNull() )
		{
			QTcpSocket * socket = new QTcpSocket();
			socket->connectToHost( server, 3389 );
			if (socket->waitForConnected(5000))
			{
				Settings::setLocalAddress( socket->localAddress() );
				socket->close();
			}
			delete socket;
		}
		currentIcon = QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "default.png");
	} else {
		currentProcess = app->launch();
	}
	process[currentProcess->pid()] = currentProcess;
	Settings::logInfo( QString("Launching application %1 with PID=%2")
			.arg(appName.toUtf8().data())
            .arg((long long)currentProcess->pid()));
}

void Desktop::setSelected(int index)
{
	ThreadButton * button = tabBar->getButton( index );
	if ( button != 0 )
	{
		setThreadWidgetName( index );
		long window = button->getWinId();
		qDebug("Setting selected index: %d, window: %ld, button type: %d", index, window, button->getType());
		setThreadWidgetTitle( window );
#ifdef Q_OS_LINUX
		X11Info::attachWindow( window );
#endif
#ifdef Q_OS_WIN
		manager->showWindow( window );
#endif
	}
}

void Desktop::loadedFinished()
{
	loadingTab = 0;
}
