#ifndef FILESYSTEMWIDGET_H
#define FILESYSTEMWIDGET_H

#include <QWidget>
#include <QFileSystemModel>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QDesktopWidget>
#include <QTimer>
#include <QMenu>
#include <QMouseEvent>
#include <QMessageBox>
#include <QInputDialog>
#include <QDir>
#include <QFile>
#include <QMutex>
#include <QProgressDialog>

#ifdef Q_OS_WIN
#include <windows.h>
#include <shellapi.h>
#include "wininfo.h"
#endif

#include "constants.h"
#include "remoteclientmanager.h"
#include "fileiconprovider.h"
#include "application.h"

namespace Ui {
class FileSystemWidget;
}

class FileSystemWidget : public QWidget
{
    Q_OBJECT

public:
    explicit FileSystemWidget(QWidget *parent = 0, bool standalone = false, QString openLocation = QString::null);
    ~FileSystemWidget();

    void init();
    void initUi(QString openLocation);
    static bool copyRecursively(QString sourceFile, QString targetFile);

public slots:
    void close();

private slots:
    void setMenu();
    void setNoSelectionMenu();
    void directoryLoaded(QString dir);
    void menuRequested(const QPoint pos);
    void setIndex( QModelIndex index );
    void upButtonPressed();
    void doubleClicked(QModelIndex index);
    void mousePressed(QModelIndex index);
    void fillPlaces();
    void updatePlaces();
    void insertPlace( QIcon icon, QString name );
    void setPlaceIndex( QModelIndex index );
    void placesDoubleClicked(QModelIndex index);
    void setHome();
    void showMenu( QModelIndex index );
    void showNoSelectionMenu();
    void cutSlot();
    void copySlot();
    void pasteSlot();
    void removeSlot();
    void newFolderSlot();
    void renameSlot();
	void openWithSlot();
    void setEnabledButtons();
    void changeView();
	void openFile();
	void findFileSlot();
	void cancelSearch();

    void copyFile();
    void removeFile();
    void renameFile(QString newName);

signals:
    void openFile(QString path, QString application = QString::null);

private:
    void initModel();
    void updatePath(QString *path);
    void disconnectModelSignals();
    void connectModelSignals();
	QString remotePlaceName(int i);
	QString findFile(QString searchString);

    Ui::FileSystemWidget *ui;
    QFileSystemModel * model;
    QStandardItemModel * placesModel;
    FileIconProvider * iconProvider;
    QModelIndex selectedIndex;
	QString selectedApp;
    QFileInfo copiedFile;
    bool cut;
    QMenu * menu;
    QMenu * noSelectionMenu;
    QAction * renameAction;
    QAction * newFolderAction;
    QAction * cutAction;
    QAction * copyAction;
    QAction * pasteAction;
    QAction * removeAction;
	QAction * openWithAction;
	QAction * findFileAction;
	QMutex mutex;
	QProgressDialog * searchProgress;
	bool searching;

    RemoteClientManager * remoteClientManager;
};

#endif // FILESYSTEMWIDGET_H
