#include "googleauthdialog.h"

#include <QApplication>

GoogleAuthDialog::GoogleAuthDialog(QUrl url, QWidget * parent) : QDialog(parent, Qt::WindowCloseButtonHint | Qt::Dialog)
{
	QGridLayout * layout = new QGridLayout(this);
	setFixedSize( 720, 560 );
	webView = new QWebView(this);
	webView->load( url );

	backButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "back.png"), "", this);

	layout->addWidget( webView, 0, 0, 1, 2);
	layout->addWidget( backButton, 1, 1 );

	layout->setRowStretch(0, 100);
	layout->setRowStretch(1, 1);
	layout->setColumnStretch(0, 100);
	layout->setColumnStretch(0, 1);

	connect(backButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect( webView, SIGNAL( loadFinished ( bool ) ), this, SLOT( webViewLoadFinished() ) );
}

int GoogleAuthDialog::exec()
{
	return QDialog::exec();
}


QString GoogleAuthDialog::getCode()
{
	return code;
}

void GoogleAuthDialog::webViewLoadFinished()
{
	QString title = webView->title();

	if (title.startsWith("Success code=") )
	{
		QWebElement element = webView->page()->currentFrame()->documentElement();
		QWebElementCollection allInputs = element.findAll("input");

		for (int i = 0; i < allInputs.count(); i++)
		{
			if (allInputs.at(i).attribute("id") == "code")
			{
				code = allInputs[i].attribute("value", QString::null);
			}
		}
	}

	if (title.startsWith("Denied error=") )
	{
		code = "Denied";
		reject();
	}

	if ( ! code.isEmpty() )
	{
		accept();
	}
}

void GoogleAuthDialog::focusInEvent ( QFocusEvent * )
{
	webView->setFocus();
}
