#include "initscreen.h"
#include "ui_initscreen.h"

InitScreen::InitScreen(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::InitScreen)
{
    ui->setupUi(this);

    QPalette pal(palette());
    pal.setColor(QPalette::Background, QColor("#" BACKGROUNDCOLOR));
    setAutoFillBackground( true );
    setPalette( pal );

    QPalette formPal( palette() );
    formPal.setColor(QPalette::Background, Qt::white);
    ui->formFrame->setAutoFillBackground( true );
    ui->formFrame->setPalette( formPal );

    ui->languageComboBox->insertItem(0, QIcon(ICON_DIR_OTHER "/us_uk_flag.png"), "English");
    ui->languageComboBox->insertItem(0, QIcon(ICON_DIR_OTHER "/es_flag.png"), "Español");
}

void InitScreen::centerDialog()
{
    int width = QApplication::desktop()->width();
    int height = QApplication::desktop()->height();
    int frameWidth = ui->formFrame->width();
    int frameHeight = ui->formFrame->height();
    int vertical = (height - frameHeight)/2;
    int horizontal = (width - frameWidth)/2;

    ui->formFrame->setGeometry( QRect( horizontal, vertical, frameWidth, frameHeight) );
}

QString InitScreen::getPassword()
{
    return ui->passwordLineEdit->text();
}

QString InitScreen::getLanguage()
{
    if (ui->languageComboBox->currentText() == "Spanish")
    {
        return "es";
    }
    return "en";
}

InitScreen::~InitScreen()
{
    delete ui;
}

void InitScreen::on_buttonBox_accepted()
{
    if ( ! ui->passwordLineEdit->text().isEmpty() )
    {
        accept();
    } else {
        QMessageBox::information( this, tr("Empty password"), tr("The password must not be empty"));
    }
}

void InitScreen::on_buttonBox_rejected()
{
    reject();
}
