#include "lwclient.h"

#include <QTimer>

static LWClient * lwClient = 0;

LWClient::LWClient()
{
	tcpSocket = new QTcpSocket(this);
	_isConnected = false;
}

LWClient::~LWClient()
{
	if ( _isConnected )
	{
		QString message = "quit\n";
		tcpSocket->write(message.toUtf8().data());
		tcpSocket->waitForReadyRead();
		tcpSocket->readAll();
	}
	delete tcpSocket;
}

LWClient * LWClient::instance()
{
	return lwClient;
}

void  LWClient::setInstance( LWClient * instance)
{
	lwClient = instance;
}

bool LWClient::connect()
{
	int port = Settings::getValue( "Server/LibreworkServerPort" ).toInt();
	QString address = Settings::getValue( "Server/LibreworkServerAddress" );

	tcpSocket->connectToHost( address, port, QIODevice::ReadWrite );

	if (tcpSocket->waitForConnected( TIMEOUT ))
	{
		qDebug("Connected");
        _isConnected = true;
    } else {
        qDebug("Cannot connect");
        _isConnected = false;
    }
    return _isConnected;
}

bool LWClient::authenticate(QString username, QString password)
{
	if (_isConnected )
	{
		// Authentication
		QString message = "login?" + username + "&&" + password + "\n";
		long bytesWritten = tcpSocket->write(message.toUtf8().data());
		bool waitForBytesWritten = tcpSocket->waitForBytesWritten();
		Q_UNUSED( waitForBytesWritten );
		bool readyRead = tcpSocket->waitForReadyRead();
		Q_UNUSED( readyRead );
		QString response = tcpSocket->readAll();
		if ( bytesWritten == -1 || response.startsWith( "Error" ) )
		{
			return false;
		} else {
			return true;
		}
	} else {
		return false;
	}
}

QByteArray LWClient::sendMessageByteArray(QString message)
{
	mutex.lock();

	QByteArray response;

	if ( _isConnected )
	{
		long bytesWritten = tcpSocket->write( QString( "_byteArray_" + message).toUtf8().data());
		Q_UNUSED( bytesWritten );
		bool waitForBytesWritten = tcpSocket->waitForBytesWritten();
		Q_UNUSED( waitForBytesWritten );
		bool readyRead = tcpSocket->waitForReadyRead();
		Q_UNUSED( readyRead );
		QByteArray resSize = tcpSocket->read( 4 );
		//Settings::logInfo( QString("resSize size: %1").arg(resSize.size()) );
		int responseSize = 0;

		responseSize |= ( (unsigned char) resSize[0] << 24 );
		responseSize |= ( (unsigned char) resSize[1] << 16 );
		responseSize |= ( (unsigned char) resSize[2] <<  8 );
		responseSize |= ( (unsigned char) resSize[3] );

		//Settings::logInfo( QString("Response size: %1").arg(responseSize) );

		QTimer timer(this);
		timer.start(TIMEOUT * 1000);

		while (response.size() < responseSize && timer.isActive() )
		{
			if ( tcpSocket->bytesAvailable() > 0 )
				response += tcpSocket->readAll();
			else
				readyRead = tcpSocket->waitForReadyRead();
			//Settings::logInfo( QString("Message size: %1").arg(response.size()) );
			bool dataFlushed = tcpSocket->flush();
			Q_UNUSED( dataFlushed );
		}

	}

	mutex.unlock();

	return response;
}

QString LWClient::sendMessage(QString message)
{
	mutex.lock();

	QString response;

	if ( _isConnected )
	{
		long bytesWritten = tcpSocket->write( message.toUtf8().data());
		Q_UNUSED( bytesWritten );
		bool waitForBytesWritten = tcpSocket->waitForBytesWritten();
		Q_UNUSED( waitForBytesWritten );
		bool readyRead = tcpSocket->waitForReadyRead();
		Q_UNUSED( readyRead );
		while (! tcpSocket->atEnd())
		{
			response += tcpSocket->read(256);
		}
		bool dataFlushed = tcpSocket->flush();
		Q_UNUSED( dataFlushed );

		response = response.trimmed();
	}

	mutex.unlock();

	return response;
}
