#include "menu.h"

#include <QDir>

Menu::Menu(QWidget * parent, QString color) : PixmapFrame(parent)
{
    principalWidget = this;
//	setPixmap(QPixmap(WALLPAPERS_DIR "metal.jpg"));
//	setScaledContents(true);
//	setStyleSheet("background-color: #99aabb");
//	setStyleSheet("background-color: #185d7b");
//	setStyleSheet("background-color: #396da5");
	setStyleSheet( QString("background-color: #%1").arg(color) );
	
	setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	layout = new QGridLayout();
	layout->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	principalWidget->setLayout(layout);
    //addWidget(principalWidget);
    //setCurrentWidget(principalWidget);
	qDebug("PrincipalWidget winId: %d", (int)principalWidget->winId() );
/*
	QPalette palette;
	palette.setBrush(QPalette::Background, *(new QBrush(*(new QPixmap(WALLPAPERS_DIR "metal.png")))));
	setPalette(palette);
*/
}

void Menu::resize( QSize newSize )
{
	setFixedSize( newSize );

	principalWidget->resize();

	// First, clean grid and get button list

	QList<AppButton*> list;

	for (int i=0; i<columns; i++)
	{
		for (int j=0; j<rows; j++)
		{
			if ( layout->itemAtPosition(j,i)->widget() == 0 )
			{
				layout->removeItem(layout->itemAtPosition(j, i));
			} else {
				AppButton * button = (AppButton*) layout->itemAtPosition(j,i)->widget();
				layout->removeWidget( button );
				list += button;
			}
		}
	}

	delete layout;

	layout = new QGridLayout();
	layout->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	principalWidget->setLayout(layout);

	prepareCells();

	// Finally, locate buttons
	for (int i = 0; i < list.count(); i++)
	{
		locateApplication( list.at(i) );
	}
	
}

long Menu::getWindowId()
{
    return 0; // Otherwise, if return winId() the tab is closed because a destroyNotify is emitted. See manager.cpp
}

void Menu::prepareCells()
{
	int cell_width = AppButton::buttonSize().width();
	int cell_height = AppButton::buttonSize().height();
	int w = width();
	int h = height();

	columns = (int) ((double)w / (double) cell_width);
	rows = (int) ((double)h / (double) cell_height);

	for (int i=0; i<columns; i++)
	{
		for (int j=0; j<rows; j++)
		{
			QSpacerItem * spacer = new QSpacerItem(cell_width, cell_height, QSizePolicy::Fixed, QSizePolicy::Fixed);
			layout->addItem(spacer, j, i);
		}
	}
}

void Menu::addApplications( QStringList validApps, LWClient * lwClient )
{
	prepareCells();

	QStringList validApplications = validApps;

	if ( lwClient == 0 )
	{
		/* Discarding LDAP
		LdapInfo * ldap = new LdapInfo(Settings::getValue("LdapSuffix"), Settings::getValue("LdapServer"), Settings::ldapServerPort() );

		Settings::logInfo( QString( "Adding applications for user %1 from %2" ).arg(Settings::getUserName()).arg(Settings::getValue("LdapSuffix")) );
		if ( ldap->authenticate( QString("cn=" + Settings::getUserName() + ",cn=users"), Settings::getPassword() ) )
		{
			Settings::logInfo( "LDAP authentication ok." );
			validApplications = ldap->getValues( "cn=Users", "cn=" + Settings::getUserName(), "progid" );
			int count = validApplications.count();
			for (int i=0; i<count; i++)
				addApplication(ldap, validApplications[i]);

			ldap->close();
		} else {
			Settings::logError( "LDAP authentication failed. Setting default pool of applications" );
		*/

			int count = validApplications.count();
			for (int i=0; i<count; i++)
				addApplication(lwClient, validApplications[i]);
		/*
		}

		delete ldap;
		*/
	} else {
		if (lwClient->isConnected())
		{
			int count = validApplications.count();
			for (int i=0; i<count; i++)
				addApplication(lwClient, validApplications[i]);
		}
	}

	Application::setValidApplications( validApplications );
}

void Menu::addApplication(LWClient * lwClient, QString appName)
{
	AppInstance * app = new AppInstance(appName, lwClient);
	connect( this, SIGNAL(destroyed()), app, SLOT(deleteAppInstance()) );
	AppButton * pixmap = new AppButton(app, this);
	pixmap->setFlat(true);
	connect(pixmap, SIGNAL(appButtonClicked(AppInstance *)), this, SIGNAL(launchApplication(AppInstance *)));
	locateApplication(pixmap);
}

/* Discarding LDAP
void Menu::addApplication(LdapInfo * ldap, QString appName)
{
	AppInstance * app = new AppInstance(appName, ldap);
	connect( this, SIGNAL(destroyed()), app, SLOT(deleteAppInstance()) );
	AppButton * pixmap = new AppButton(app, this);
	pixmap->setFlat(true);
	connect(pixmap, SIGNAL(appButtonClicked(AppInstance *)), this, SIGNAL(launchApplication(AppInstance *)));
	locateApplication(pixmap);
}
*/

void Menu::locateApplication(QWidget * widget)
{
	for (int i=0; i<columns; i++)
	{
		for (int j=0; j<rows; j++)
		{
			if ( layout->itemAtPosition(j,i)->widget() == 0 )
			{
				layout->removeItem(layout->itemAtPosition(j, i));
				layout->addWidget(widget, j, i);
				i=columns;
				j=rows;
			}
		}
	}
}

void Menu::setLogo()
{
	QByteArray ic = LWClient::instance()->sendMessageByteArray( "get_user_logo?" + Settings::getUserName() );
	QPixmap logo;
	logo.loadFromData( ic );
	if ( logo.isNull() )
	{
		logo = QPixmap(ICON_DIR_OTHER + QString(QDir::separator()) + "librework.png");
	}
	principalWidget->setOpacity( 0.9 );
	//principalWidget->setPixmap( logo.scaledToWidth(600, Qt::SmoothTransformation), true, false );
	principalWidget->setPixmap( logo, true, false );
}
