#ifndef MIMECONFIG_H
#define MIMECONFIG_H

#include <QDialog>
#include <QWidget>
#include <QComboBox>
#include <QListWidget>
#include <QListWidgetItem>
#include <QMenu>
#include <QAction>
#include <QGridLayout>
#include <QMessageBox>
#include <QDir>

#include "toolbutton.h"
#include "settings.h"
#include "application.h"
#include "configurationdialog.h"

class MimeConfig : public QDialog, public ConfigurationDialog
{
	Q_OBJECT

	public:
		MimeConfig(Menu * menu);
		MimeConfig(QString file = QString::null, QWidget * parent = 0);

		void init(QString file = QString::null, QWidget * parent = 0);
		QIcon icon() { return QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "mimetypes.png"); };
		QString name() { return tr("File\nassociations"); };
		QDialog * widget() { return this; };

	signals:
		void mimeChanged();
		

	public slots:
		void accept();
		void reject();

	private slots:
		void deleteSelected();
		void appGroupActivated(int index);
		void updateSelected(int index);
		void extensionsActivated(int index);
		void applicationsToSelectedClicked();
		void upClicked();
		void downClicked();
		void applicationsChanged( int row );
		void selectedChanged( int row );

	private:
		bool selectedContains(QString application);
		void appendApplication(Application * application, QListWidget * listWidget);
		void deleteApplication(QString application);
		QStringList modifiedItems();
		void saveConfig(QStringList modifiedItems);

		QGridLayout * layout;
		QComboBox * appGroup;
		QListWidget * applications;
		ToolButton * applicationsToSelected;
		QComboBox * extensions;
		QListWidget * selected;
		QMenu * selectedMenu;
		QAction * deleteSelectedItem;
		ToolButton * backButton;
		ToolButton * up;
		ToolButton * down;
		ToolButton * deleteButton;
		ToolButton * acceptButton;
		ToolButton * cancelButton;

		QListWidgetItem * selectedItem;
		QStringList deletedItems;
		QStringList appendedItems;
		bool dontclose;
};

#endif
