#ifdef OLDCODE

#include "container.h"

Container::Container (long window, TabWidget * parent) : QObject (parent)
{
    this->window = window;
	tabWidget = parent;
	port = 0;
	name = QString::null;
	process = 0;
	resizing = false;
	finished = false;
	toBeResized = true;
	newSize = QSize(0, 0);
	loadingWidgetCreated = false;
	Settings::logInfo(QString("Creating container %1").arg((long)this) );
}

Container::~Container()
{
	qDebug("Deleting container");
	if (loadingWidgetCreated)
	{
		delete loadingMovie;
		delete loadingWidget;
	}
}


void Container::resize()
{
	if ( newSize.width() > 0 && newSize.height() > 0 )
	{
		QLabel * messageBox = new QLabel(NULL, Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint);
		QFont font;
		font.setBold( true );
		messageBox->setText( tr("Resizing application. Please, wait...") );
		messageBox->setFont( font );
		messageBox->setAttribute( Qt::WA_DeleteOnClose, true );
		messageBox->setFixedSize( 400, 200 );
		messageBox->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
		messageBox->show();
		QApplication::processEvents();

        //setFixedSize( newSize );
		if ( port != 0)
		{
			resizing = true;
			//X11Info::disconnectRdpWindow( window );
			if (process != 0)
			{
				process->kill();
				process->waitForFinished();
			}
			process = app->launch( this, newSize, port );
		} else {
#ifdef Q_OS_LINUX
			X11Info::attachWindow( this, window );
#endif
		}
		toBeResized = false;
		delete messageBox;
	}
}

bool Container::mustBeResized()
{
	return toBeResized;
}

void Container::setToBeResized(QSize newSize)
{
	this->newSize = newSize;
	toBeResized = true;
}

void Container::setProcess( QProcess * process )
{
	this->process = process;
}

QProcess * Container::getProcess()
{
	return process;
}

void Container::setWindow(long window)
{
	this->window = window;
}

long Container::getWindow()
{
	return window;
}

void Container::setAppInstance(AppInstance * app)
{
	this->app = app;
}

AppInstance * Container::getAppInstance()
{
	return app;
}

void Container::setServerPort(quint16 port)
{
	this->port = port;
}

quint16 Container::getServerPort()
{
	return port;
}

void Container::closeTab(int exitCode, QProcess::ExitStatus exitStatus)
{
	Settings::logInfo(QString("Container closing tab. Resizing: %1, Finished: %2, exitCode: %3, existStatus: %4").arg( resizing ).arg( finished ).arg( exitCode ).arg( exitStatus ) );
	if ( ! resizing && ! finished )
	{
		finished = true;
	}

	if ( ! resizing )
	{
		if (exitCode != 0 || exitStatus == QProcess::CrashExit)
		{
			Settings::logInfo(QString("Container closing tab. Something failed. Exit code: %1").arg( exitCode ) );
		}

		if (tabWidget != 0)
		{
            int ix = tabWidget->indexOf(window);
			Settings::logInfo(QString("Container %1 closing tab. Removing tab index: %2").arg((long)this).arg( ix ) );
			tabWidget->removeTab( ix );
		}
	} else {
		resizing = false;
	}
}

void Container::loading()
{
	loadedContainer = false;
	loadingWidget = new QLabel();
	loadingWidgetCreated = true;
	loadingWidget->setStyleSheet("QLabel { background-color : white; }");
	loadingWidget->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    loadingWidget->setFixedSize( QApplication::desktop()->size() );
	loadingWidget->show();
	loadingMovie = new QMovie( ICON_DIR_ANIM + QString(QDir::separator()) + "loading.gif" );
	loadingWidget->setMovie(loadingMovie);
	loadingMovie->start();
	QTimer::singleShot ( 1000 * LOADINGTIMEOUT, this, SLOT(loaded()));
	connect( this, SIGNAL(running()), this, SLOT(loaded()));
}

void Container::loaded()
{
	emit loadedFinished();
	if ( ! loadedContainer )
	{
		loadingWidgetCreated = false;
		loadingMovie->stop();
		delete loadingMovie;
		delete loadingWidget;
		loadedContainer = true;
	}
}

void Container::showLoading()
{
	if (loadingWidgetCreated)
	{
		loadingMovie->start();
		loadingWidget->show();
	}
}

void Container::hideLoading()
{
	if (loadingWidgetCreated)
	{
		loadingMovie->stop();
		loadingWidget->hide();
	}
}

void Container::renameTab(QString name)
{
	this->name = name;
	emit nameChanged(window);
}

void Container::closeTab()
{
	Settings::logInfo(QString("Container closeTab(). Resizing: %1, Finished: %2").arg( resizing ).arg( finished ));
	if ( ! resizing && ! finished )
	{
		finished = true;
		if (tabWidget != 0)
            tabWidget->removeTab(tabWidget->indexOf(window));
	} else {
		resizing = false;
	}
}

void Container::closing()
{
	qDebug("\nClosing container...\n");
}

QString Container::getName()
{
	return name;
}
#endif
