#ifndef DIRWIDGET_H
#define DIRWIDGET_H

#include <QApplication>
#include <QProcess>
#include <QFile>
#include <QListView>
#include <QLabel>
#include <QVBoxLayout>
#include <QCursor>
#include <QModelIndex>
#include <QMessageBox>
#include <QInputDialog>

#include "constants.h"
#include "listview.h"
#include "settings.h"
#include "dirwidgetmenu.h"
#include "search.h"
#include "appaction.h"
#include "application.h"
#include "mimeconfig.h"
#include "fileproperties.h"
#ifdef BASICFSMODE
#include "basicfilesystemmodel.h"
#else
#include "filesystemmodel.h"
#endif

class DirWidget : public QWidget
{
	Q_OBJECT

	public:
		DirWidget(QWidget * parent = 0);
		~DirWidget();

		static void correctPath(QString * newPath);

	public slots:
		void setIconView();
		void setListView();
		void setHome();
		void setShared();
		void setSearchs();
		void setDropbox();
		void setGoogleDrive();
		void setDir(QString dir);
		void goUp();
		void goPrevious();
		void goNext();
		void refresh();

		void listViewClicked(const QModelIndex & index);
		void listViewRightClicked();
		void setLabel(const QModelIndex & index);
		void searchDone();

		void searchFiles(QString exp);
		void removeTriggered();
		void renameTriggered();
		void openWithTriggered();
		void openMimeConfig();
		void cutTriggered();
		void copyTriggered();
		void pasteTriggered();
		void newFolderTriggered();
		void propertiesTriggered();
		void locationTriggered();

	signals:
		void enablePrevious(bool enable);
		void enableNext(bool enable);
		void enableUp(bool enable);
		void atHome();
		void atShared();
		void atSearchs();
		void atDropbox();
		void atGoogleDrive();
		void searchFinished();
		void openFile(QString file, QString application);
		void enablePaste(bool enable);

	private:
		void fillMenu();
		void setRootIndex(const QModelIndex & index);
		bool removeDir(const QString dirName);

		Search * search;
#ifdef BASICFSMODE
        BasicFileSystemModel * model;
#else
        FileSystemModel * model;
#endif
		QLabel * currentDirLabel;
		ListView * currentDirView;
		QList<QModelIndex> browserCache;
		QModelIndex currentIndex;
		int currentBrowserPage;
		DirWidgetMenu * rightClickMenu;
		QMenu * openWithMenu;
		QFileInfo fileInfo;
		QList<QFileInfo> clipboard;
		bool cut;
};

#endif
