#include "dropboxauthdialog.h"

#include <QApplication>

DropboxAuthDialog::DropboxAuthDialog(QUrl url, QWidget * parent) : QDialog(parent, Qt::WindowCloseButtonHint | Qt::Dialog)
{
	QGridLayout * layout = new QGridLayout(this);
	setFixedSize( QApplication::desktop()->screenGeometry().size().width(),
			QApplication::desktop()->screenGeometry().size().height() - 1.5 * TAB_BUTTON_HEIGHT );
	webView = new QWebView(this);
	webView->load( url );

	backButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "back.png"), "", this);

	layout->addWidget( webView, 0, 0, 1, 2);
	layout->addWidget( backButton, 1, 1 );

	layout->setRowStretch(0, 100);
	layout->setRowStretch(1, 1);
	layout->setColumnStretch(0, 100);
	layout->setColumnStretch(0, 1);

	connect(backButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect( webView, SIGNAL( loadFinished ( bool ) ), this, SLOT( webViewLoadFinished() ) );
}

int DropboxAuthDialog::exec()
{
	return QDialog::exec();
}


QString DropboxAuthDialog::getCode()
{
	return code;
}

void DropboxAuthDialog::webViewLoadFinished()
{
	QString title = webView->title();

	QWebElement element = webView->page()->currentFrame()->documentElement();
	QWebElementCollection allInputs = element.findAll("span");

	for (int i = 0; i < allInputs.count(); i++)
	{
		if (allInputs.at(i).attribute("class") == "auth-code")
		{
			code = allInputs[i].toPlainText();
		}
	}

	if (title.startsWith("Denied error=") )
	{
		code = "Denied";
		reject();
	}

	if ( ! code.isEmpty() )
	{
		accept();
	}
}

void DropboxAuthDialog::focusInEvent ( QFocusEvent * )
{
	webView->setFocus();
}
