#include "dropboxmanager.h"

#include <QDir>
#include <QUrl>

DropboxManager::DropboxManager( DropboxService * dropboxService, QObject * parent ) : QStandardItemModel( parent )
{
	this->dropboxService = dropboxService;
	process = 0;
	connect( this, SIGNAL(isAvailable(bool)), dropboxService, SIGNAL(isAvailable(bool)) );
}

DropboxManager::~DropboxManager()
{
}

void DropboxManager::update()
{
	this->clear();

	QStringList accounts = dropboxService->getAccounts();

	int numAccounts = accounts.count();

	if ( numAccounts > 0 )
		emit isAvailable( true );
	else
		emit isAvailable( false );

	for (int i = 0; i < numAccounts; i++)
	{
		QStandardItem * account = new QStandardItem( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "dropbox.png"), accounts[i] );
		account->setEditable( false );
		appendRow( account );
	}
}

QStringList DropboxManager::getAccounts()
{
	return dropboxService->getAccounts();
}

QUrl DropboxManager::addAccount(QString account)
{
	QUrl url;

	process = new QProcess;
	QString program = "dropbox.sh";
	QStringList arguments;
	arguments += dropboxService->getDropboxDir();
	arguments += "configure";
	arguments += account;

        process->start( program, arguments );
	qDebug("Dropbox add account: process start");

	if (process->waitForStarted())
	{
		qDebug("Dropbox add account: process started ok");
		if (process->waitForReadyRead())
		{
			qDebug("Dropbox add account: wait for ready read");
			while ( process->bytesAvailable() > 0)
			{
				QByteArray output = process->readLine();
				qDebug( output.data() );
				if ( output.startsWith("https://www.dropbox.com/") )
				{
					QString urlString = output;
					urlString = urlString.replace( "%3A", ":" );
					urlString = urlString.replace( "%2F", "/" );
					url = QUrl( urlString.trimmed() );
				}
			}
		} else {
			qDebug("Dropbox add account: not ready for read");
		}
	} else {
		qDebug("Dropbox add account: not started");
	}

	return url;
}

void DropboxManager::finishAddAccount(QString key)
{
	if ( process != 0 )
	{
		if ( process->state() == QProcess::Running )
		{
			process->write( QString( key + "\n" ).toUtf8().data() );
			process->waitForFinished();
			Settings::logInfo( QString( "Dropbox account creation output: '%1'" ).arg(process->readAllStandardOutput().data() + QString() + process->readAllStandardError().data() ) );
		}

		delete process;
		process = 0;
	}
}

void DropboxManager::removeAccount(QString account)
{
	dropboxService->unconfigure( account );
}
