#include "dropboxservice.h"

#include <QDir>
#include "settings.h"

DropboxService::DropboxService()
{
}

DropboxService::~DropboxService()
{
}

void DropboxService::start()
{
	emit isUpdating( true );

	Settings::logInfo( "Starting Dropbox..." );
	qDebug("Starting Dropbox..");
	QStringList arguments;
	arguments += "start";
	if ( launchCommand( arguments ) == 0)
	{
		emit isAvailable( true );
		Settings::logInfo( "Done." );
	} else {
		Settings::logInfo( "Not done." );
	}
	emit isUpdating( false );

}

void DropboxService::stop()
{
	QStringList arguments;
	arguments += "stop";

	launchCommand( arguments );

	emit isUpdating( false );
	emit isAvailable( false );
}

bool DropboxService::status()
{
	QStringList arguments;
	arguments += "status";

	if ( launchCommand( arguments ) == 0)
	{
		return true;
	} else {
		return false;
	}
}

void DropboxService::update()
{
	this->stop();
	this->start();
}

void DropboxService::unconfigure()
{
	stop();

	QStringList arguments;
	arguments += "unconfigure";
	arguments += "--all";

	launchCommand( arguments );
}

void DropboxService::unconfigure(QString account)
{
	QStringList arguments;
	arguments += "unconfigure";
	arguments += account;

	launchCommand( arguments );
}

QStringList DropboxService::getAccounts()
{
	QString program = "/usr/bin/dropbox.sh";

	QStringList arguments;
	arguments += getDropboxDir();
	arguments += "list";

	return getCommandOutput( program, arguments );
}

int DropboxService::launchCommand(QStringList subcommands)
{
	QString program = "/usr/bin/dropbox.sh";

	QStringList arguments;
	arguments += getDropboxDir();
	arguments += subcommands;

	Settings::logInfo( QString("Launching command: %1").arg(program + " " + arguments.join(" ") ) );

	Settings::logInfo( QString("Command output: %1").arg( getCommandOutput( program, arguments ).join("\n") ) );
	return true;
}

QString DropboxService::getDropboxDir()
{
	QString dropboxDir;

	//if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
	//	dropboxDir = CHROOTMOUNT + Settings::homePath() + QString(QDir::separator()) + DROPBOX_HOME;
	//else
	dropboxDir = Settings::homePath() + QString(QDir::separator()) + DROPBOX_HOME;

	return dropboxDir;
}
