#include <QDir>

#include "filedialog.h"

FileDialog::FileDialog(QWidget * parent) : QDialog (parent)
{
	setWindowState(Qt::WindowFullScreen);
	setWindowModality(Qt::WindowModal);
	setFixedSize( 600, 400 );
	this->parent = parent;
}

void FileDialog::doCopy(QString source, QString target)
{
	QMovie * movie = new QMovie(ICON_DIR_ANIM + QString(QDir::separator()) + "copy.gif", QByteArray(), this);
	doCopy(movie, source, target, false);
}

void FileDialog::doMove(QString source, QString target)
{
	QMovie * movie = new QMovie(ICON_DIR_ANIM + QString(QDir::separator()) + "copy.gif", QByteArray(), this);
	doCopy(movie, source, target, true);
}

void FileDialog::doRemove(QString filePath)
{
	QMovie * movie = new QMovie(ICON_DIR_ANIM + QString(QDir::separator()) + "trash.gif", QByteArray(), this);
	doCopy(movie, filePath, QString::null, true);
}

void FileDialog::doMoveToTrash(QString filePath)
{
	QMovie * movie = new QMovie(ICON_DIR_ANIM + QString(QDir::separator()) + "trash.gif", QByteArray(), this);
	doCopy(movie, filePath, QDir::homePath() + ".Trash", true);
}

void FileDialog::doCopy(QMovie * movie, QString source, QString target, bool move)
{
	layout = new QVBoxLayout(this);
	animationLabel = new QLabel(this);

	animationLabel->setMovie(movie);
	movie->start();
	layout->addWidget(animationLabel, 0, Qt::AlignHCenter);

	QString labelSource = source;
	DirWidget::correctPath(&labelSource);
	QString labelTarget = target;
	DirWidget::correctPath(&labelTarget);
	
	QHBoxLayout * labelLayout = new QHBoxLayout(this);
	QLabel * sourceLabel = new QLabel(labelSource, this);
	ToolButton * toLabel = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "forward.png"), "", this);
	QLabel * targetLabel = new QLabel(labelTarget);
	labelLayout->addWidget(sourceLabel, 0, Qt::AlignHCenter);
	labelLayout->addWidget(toLabel, 0, Qt::AlignHCenter);
	labelLayout->addWidget(targetLabel, 0, Qt::AlignHCenter);
	QFont font;
	font.setBold(true);
	sourceLabel->setFont(font);
	targetLabel->setFont(font);

	layout->addLayout(labelLayout);

	setVisible( true );

	process = new QProcess(parent);
	connect(process, SIGNAL(finished ( int )), this, SLOT(actionFinished ( int )) );

	if (target.isNull()) // Command = remove
	{
		program = "rm";
		arguments += "-rf";
		arguments += source;
	} else {
		if (move)
		{
			program = "mv";
		} else {
			program = "cp";
			arguments += "-a";
		}
		arguments += source;
		arguments += target;
	}
	process->start(program, arguments);
}

FileDialog::~FileDialog()
{
	delete process;
}

void FileDialog::actionFinished(int)
{
	emit finished();
	close();
}
