#include "filemanager.h"

FileManager::FileManager(QWidget * parent, bool standalone, QString openLocation) : QFrame(parent)
{
	QPalette pal(palette());
	pal.setColor(QPalette::Background, QColor( HEXBACKGROUNDCOLOR ));
	setAutoFillBackground(true);
	setPalette(pal);

	QDir dir( QDir::homePath() );
        dir.mkpath( SEARCH_DIR );
	setMinimumSize(640, 480);
	this->standalone = standalone;
	dbus = false;

	if (standalone)
	{
		QDBusConnection dbusConnection( QDBusConnection::connectToBus(QDBusConnection::SessionBus, "com.tuxum.desktux") );
		if ( dbusConnection.isConnected() )
		{
			QDBusMessage message = QDBusMessage::createSignal("/", "com.tuxum.desktux", "test");
			message << "test";
			dbusConnection.send(message);
			QDBusMessage res = dbusConnection.call( message, QDBus::Block, DBUS_TIMEOUT * 1000 );
			QList<QVariant> response = res.arguments();
			if (response.count() > 0)
			{
				if ( response.at(0) == QVariant("ok") )
				{
					qDebug("Eureka!!!!");
					dbus = true;
				} else {
					qDebug("Not connected to DBUS");
				}
			} else {
				qDebug("Not connected to DBUS");
			}
		}
	}

	layout = new QVBoxLayout(this);
	layout->setContentsMargins( 0, 0, 0, 0 );
	splitter = new QSplitter(this);

	fillButtons();
	fillPlaces();
	fillCurrentDir();

	splitter->setStretchFactor ( 0, 30 );
	splitter->setStretchFactor ( 1, 70 );
	layout->addWidget(splitter);

	if (! openLocation.isEmpty())
	{
		dirWidget->setDir(openLocation);
	}
}

FileManager::~FileManager()
{
}

long FileManager::getWindowId()
{
	return splitter->winId();
}

void FileManager::enablePrevious(bool enable)
{
	previousButton->setEnabled(enable);
}

void FileManager::enableNext(bool enable)
{
	nextButton->setEnabled(enable);
}

void FileManager::enableUp(bool enable)
{
	upButton->setEnabled(enable);
}

void FileManager::enableSearch()
{
	searchField->setEnabled(true);
	searchButton->setEnabled(true);
}
	
void FileManager::disableSearch()
{
	searchField->setEnabled(false);
	searchButton->setEnabled(false);
}

void FileManager::fillButtons()
{
	QWidget * buttonsGroupBox = new QWidget;
	buttonsGroupBox->setFixedHeight( ToolButton::buttonSize().height() );
	QPalette pal(palette());
	pal.setColor(QPalette::Background, Qt::white);
	buttonsGroupBox->setAutoFillBackground(true);
	buttonsGroupBox->setPalette(pal);

	buttonsLayout = new QGridLayout();
	buttonsLayout->setHorizontalSpacing( MARGIN );
	buttonsLayout->setContentsMargins( 0, 0, 0, 0 );

	previousButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "back.png"), tr("Previous") );
	previousButton->addPixmap( QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "back-hover.png") );
	previousButton->setEnabled(false);
	nextButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "forward.png"), tr("Next") );
	nextButton->addPixmap( QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "forward-hover.png") );
	nextButton->setEnabled(false);
	upButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "up.png"), tr("Up") );
	upButton->addPixmap( QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "up-hover.png") );

/*
	iconViewButton = new ToolButton(QIcon(ICON_DIR_48 + QString(QDir::separator()) + "icon-view.png"), tr("Icon view") );
	listViewButton = new ToolButton(QIcon(ICON_DIR_48 + QString(QDir::separator()) + "list-view.png"), tr("List view") );
*/

	searchField = new QLineEdit();
	QFont font;
	font.setPixelSize(TOOLBUTTON_SIZE / 2);
	searchField->setFont(font);
	searchField->setMaximumWidth( TOOLBUTTON_SIZE * 10 );
	searchField->setMinimumWidth( TOOLBUTTON_SIZE * 10 );
	searchButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "search.png"), tr("Find") );

	buttonsLayout->addWidget(previousButton, 0, 0);
	buttonsLayout->addWidget(nextButton, 0, 1);
	buttonsLayout->addWidget(upButton, 0, 2);
	buttonsLayout->addItem(
		new QSpacerItem(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE, QSizePolicy::Maximum, QSizePolicy::Maximum),
		0, 3);
/*
	buttonsLayout->addWidget(iconViewButton, 0, 4);
	buttonsLayout->addWidget(listViewButton, 0, 5);
*/
	buttonsLayout->addItem(
		new QSpacerItem(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE, QSizePolicy::Ignored, QSizePolicy::Ignored),
		0, 6);
	buttonsLayout->setColumnStretch(6, 10);
	buttonsLayout->addWidget(searchField, 0, 7, Qt::AlignVCenter);
	buttonsLayout->addWidget(searchButton, 0, 8);

	buttonsGroupBox->setLayout( buttonsLayout );

	layout->addWidget(buttonsGroupBox);
}

void FileManager::fillPlaces()
{
	QVBoxLayout * placesLayout = new QVBoxLayout;
	placesLayout->setContentsMargins(0, 0, 0, 0);
	QWidget * placesWidget = new QWidget( splitter );
	QHBoxLayout * placesLabelLayout = new QHBoxLayout;

	QLabel * placesLabel = new QLabel(tr("Places"));
	placesLabelLayout->addSpacing( MARGIN );
	placesLabelLayout->addWidget( placesLabel );

	QFont font;
	font.setBold( true );
	placesLabel->setFont( font );

	places = new QListWidget();
	places->setIconSize( QSize(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE) );
	places->setSelectionMode( QAbstractItemView::SingleSelection );

	placesLayout->addLayout( placesLabelLayout );
	placesLayout->addWidget( places );

	placesWidget->setLayout( placesLayout );

	home = new QListWidgetItem ( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "folder-home.png"), tr("Home") );
	shared = new QListWidgetItem ( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "folder-shared.png"), tr("Shared") );
	searchs = new QListWidgetItem ( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "search.png"), tr("Searchs") );
	dropboxItem = new QListWidgetItem ( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "dropbox.png"), tr("Dropbox") );
	googleDriveItem = new QListWidgetItem ( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "google-drive.png"), tr("Google Drive") );

	places->addItem( home );
	places->addItem( shared );
	places->addItem( searchs );
#ifdef Q_OS_LINUX
	places->addItem( dropboxItem );
	places->addItem( googleDriveItem );
#endif

	dropboxItem->setHidden( true );
	googleDriveItem->setHidden( true );

	connect(places, SIGNAL(itemClicked(QListWidgetItem *)), this, SLOT(placesChanged(QListWidgetItem *)) );
}

void FileManager::showDropbox(bool show)
{
	dropboxItem->setHidden( ! show );
}

void FileManager::showGoogleDrive(bool show)
{
	googleDriveItem->setHidden( ! show );
}

void FileManager::showGoogleDriveUpdating(bool show)
{
	if (show)
		googleDriveItem->setText( tr("Google Drive (Updating...)") );
	else
		googleDriveItem->setText( tr("Google Drive") );
}

void FileManager::placesChanged(QListWidgetItem * item )
{
	if (item == home)
		dirWidget->setHome();
	if (item == shared)
		dirWidget->setShared();
	if (item == searchs)
		dirWidget->setSearchs();
	if (item == dropboxItem)
		dirWidget->setDropbox();
	if (item == googleDriveItem)
		dirWidget->setGoogleDrive();
}

void FileManager::refresh()
{
	dirWidget->refresh();
}

void FileManager::fillCurrentDir()
{
	dirWidget = new DirWidget(splitter);
/*
	connect(iconViewButton, SIGNAL(clicked()), dirWidget, SLOT(setIconView()));
	connect(listViewButton, SIGNAL(clicked()), dirWidget, SLOT(setListView()));
*/
	connect(upButton, SIGNAL(clicked()), dirWidget, SLOT(goUp()));
	connect(previousButton, SIGNAL(clicked()), dirWidget, SLOT(goPrevious()));
	connect(nextButton, SIGNAL(clicked()), dirWidget, SLOT(goNext()));
	connect(searchField, SIGNAL(returnPressed()), this, SLOT(searchClicked()));
	connect(searchButton, SIGNAL(clicked()), this, SLOT(searchClicked()));
	connect(dirWidget, SIGNAL(enablePrevious(bool)), this, SLOT(enablePrevious(bool)));
	connect(dirWidget, SIGNAL(enableNext(bool)), this, SLOT(enableNext(bool)));
	connect(dirWidget, SIGNAL(enableUp(bool)), this, SLOT(enableUp(bool)));
	connect(dirWidget, SIGNAL(atHome()), this, SLOT(setHome()));
	connect(dirWidget, SIGNAL(atShared()), this, SLOT(setShared()));
	connect(dirWidget, SIGNAL(atSearchs()), this, SLOT(setSearchs()));
	connect(dirWidget, SIGNAL(atDropbox()), this, SLOT(setDropbox()));
	connect(dirWidget, SIGNAL(atGoogleDrive()), this, SLOT(setGoogleDrive()));
	connect(dirWidget, SIGNAL(searchFinished()), this, SLOT(enableSearch()));
	connect(dirWidget, SIGNAL(openFile(QString, QString)), this, SLOT(openFileSlot(QString, QString)));
	dirWidget->setHome();
}

void FileManager::setHome()
{
	home->setSelected(true);
}

void FileManager::setShared()
{
	shared->setSelected(true);
}

void FileManager::setSearchs()
{
	searchs->setSelected(true);
}

void FileManager::setDropbox()
{
	dropboxItem->setSelected(true);
}

void FileManager::setGoogleDrive()
{
	googleDriveItem->setSelected(true);
}

void FileManager::searchClicked()
{
	QString searchText = searchField->text();
	if (!searchText.isEmpty())
	{
		dirWidget->searchFiles(searchText);
		disableSearch();
	}
}

void FileManager::openFileSlot(QString cfile, QString application)
{
	QString file = cfile;

	if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
                file.replace( CHROOTMOUNT, "" );

	setCursor(Qt::WaitCursor);
	dirWidget->setEnabled(false);
	QApplication::processEvents();

	QTimer::singleShot ( TIMEOUT * 1000, this, SLOT(fileOpened()) );

	if (standalone)
	{
		qDebug("Launching app in standalone mode");
		if ( dbus )
		{
			qDebug("Trying with DBUS...");
			QDBusMessage msg = QDBusMessage::createSignal("/", "com.tuxum.desktux", "openFile");
			msg << file << application;
			QDBusConnection::sessionBus().send(msg);
		} else {
			qDebug("Trying with mime app");
			QString mimeType = Application::mimeType(file);
			if (mimeType == "recursivelink")
			{
				QMessageBox::critical ( this, tr("Link error"), tr("This file is a recursive set of links"));
				return;
			}
			QString extension = "." + QFileInfo( file ).suffix();
			QString myApp = Application::preferredProgram(extension);
			if (myApp.isEmpty())
				return;

			QProcess * process = new QProcess(this);
			process->start( myApp, QStringList(file) );
			process->waitForStarted();
			fileOpened();
		}
	} else {
		qDebug("Launching app in managed mode");
		emit openFile(file, application);
	}
}

void FileManager::fileOpened()
{
	dirWidget->setEnabled(true);
	setCursor(Qt::ArrowCursor);
}
