#ifndef FILEMANAGER_H
#define FILEMANAGER_H

#include <QSplitter>
#include <QFileSystemModel>
#include <QListWidget>
#include <QListWidgetItem>
#include <QListView>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QToolBar>
#include <QLineEdit>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusConnection>
#include <QProcess>

#include "constants.h"
#include "toolbutton.h"
#include "dirwidget.h"

class FileManager : public QFrame
{
	Q_OBJECT

	public:
		FileManager(QWidget * parent = 0, bool standalone = false, QString openLocation = QString::null);
		~FileManager();

        long getWindowId();

	public slots:
		void fileOpened();
		void showDropbox(bool show);
		void showGoogleDrive(bool show);
		void showGoogleDriveUpdating(bool show);
		void refresh();

	private slots:
		void placesChanged(QListWidgetItem * item);
		void enablePrevious(bool enable);
		void enableNext(bool enable);
		void enableUp(bool enable);
		void setHome();
		void setSearchs();
		void setShared();
		void setDropbox();
		void setGoogleDrive();
		void searchClicked();
		void enableSearch();
		void disableSearch();
		void openFileSlot(QString file, QString application);

	signals:
		void openFile(QString file, QString application);

	private:
		void fillButtons();
		void fillPlaces();
		void fillCurrentDir();

		bool standalone;
		bool dbus;

		QVBoxLayout * layout;
		QToolBar * toolBar;
		QSplitter * splitter;

		QListWidget * places;
		QListWidgetItem * home;
		QListWidgetItem * shared;
		QListWidgetItem * searchs;
		QListWidgetItem * dropboxItem;
		QListWidgetItem * googleDriveItem;

		DirWidget * dirWidget;

		QGridLayout * buttonsLayout;
		ToolButton * previousButton;
		ToolButton * nextButton;
		ToolButton * upButton;
		ToolButton * iconViewButton;
		ToolButton * listViewButton;
		ToolButton * searchButton;
		QLineEdit * searchField;
};

#endif
