#include "fileproperties.h"

FileProperties::FileProperties(QFileInfo file, QIcon mimeIcon, QWidget * parent) : QDialog(parent, Qt::WindowCloseButtonHint | Qt::Dialog)
{
	layout = new QGridLayout( this );
	layout->setSpacing(10);
	layout->setMargin(20);

	icon = new QLabel(this);
	icon->setPixmap(mimeIcon.pixmap(QSize(ICON_SIZE, ICON_SIZE)));

	fileName = new QLabel(file.fileName(), this);

	QLabel * mimeTypeLabel = new QLabel(tr("File type"), this);
	QString mime = Application::mimeType(file.absoluteFilePath());
	mimeType = new QLabel( mime, this);

	QLabel * fullPathLabel = new QLabel(tr("File path"), this);
	fullPath = new QLabel( file.absolutePath(), this);

	QLabel * sizeLabel = new QLabel(tr("File size"), this);
	double fileSize = (double) file.size();
	QString sizeString;
	if (fileSize > 1024)
	{
		if (fileSize > 1048576)
		{
			if (fileSize > 1073741824)
			{
				sizeString = QString::number( fileSize / 1073741824, 'f', 2 ) + " GB";
			} else {
				sizeString = QString::number( fileSize / 1048576, 'f', 1 ) + " MB";
			}
		} else {
			sizeString = QString::number( fileSize / 1024, 'f', 0 ) + " KB";
		}
	} else {
		sizeString = QString::number( fileSize, 'f', 0 ) + " B";
	}
	size = new QLabel( sizeString, this );

	QLabel * modifiedLabel = new QLabel(tr("Last modification time"), this);
	modified = new QLabel( file.lastModified().toString("dd-MM-yyyy hh:mm"), this );

	QLabel * accessedLabel = new QLabel(tr("Last accessed time"), this);
	accessed = new QLabel( file.lastRead().toString("dd-MM-yyyy hh:mm"), this );

	closeButton = new ToolButton( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "exit.png"), "", this );
	connect(closeButton, SIGNAL(clicked()), this, SLOT(accept()));

	layout->addWidget(icon, 0, 0);
	layout->addWidget(fileName, 0, 1);
	layout->addWidget(mimeTypeLabel, 1, 0);
	layout->addWidget(mimeType, 1, 1);
	layout->addWidget(fullPathLabel, 2, 0);
	layout->addWidget(fullPath, 2, 1);
	layout->addWidget(sizeLabel, 3, 0);
	layout->addWidget(size, 3, 1);
	layout->addWidget(modifiedLabel, 4, 0);
	layout->addWidget(modified, 4, 1);
	layout->addWidget(accessedLabel, 5, 0);
	layout->addWidget(accessed, 5, 1);
	layout->addWidget(closeButton, 6, 2);
}
