#ifndef FILESYSTEMMODEL_H
#define FILESYSTEMMODEL_H

#include <QStandardItemModel>
#include <QStandardItem>
#include <QPersistentModelIndex>
#include <QFileIconProvider>
#include <QFile>
#include <QIcon>
#include <QSet>
#include <QDir>

#include "settings.h"
#include "fileiconprovider.h"

#define FileIconRole Qt::DecorationRole
#define FilePathRole Qt::UserRole + 1
#define FileNameRole Qt::UserRole + 2
#define FilePermissions Qt::UserRole + 3

class FileSystemModel : public QStandardItemModel
{
	Q_OBJECT

	public:
		FileSystemModel( QString chroot = QString::null, bool showHiddenFiles = false );
		~FileSystemModel();

		QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;
		Qt::ItemFlags flags(const QModelIndex& index) const;
		bool setData(const QModelIndex& index, const QVariant& value, int role);
		bool isChecked(const QModelIndex& index);
		QSet<QPersistentModelIndex> checkedIndexes();
		void clearChecklist();
		QModelIndex setRootPath( const QString & newPath ) const;
		void update( const QModelIndex & index ) const { update(itemFromIndex(index) ); };

		QString filePath( const QModelIndex & index ) const;
		QModelIndex index ( const QString & path ) const;
		QFileInfo fileInfo( const QModelIndex & index ) const;
		QModelIndex parent( const QModelIndex & child ) const;
		void setRootIndex(  const QModelIndex & index ) const;

		void setIconProvider( FileIconProvider * fileIconProvider ) { this->fileIconProvider = fileIconProvider; };
		QIcon fileIcon( const QModelIndex& index ) { return fileIconProvider->icon( fileInfo( index ) ); };

	private:
		void update(QStandardItem * item) const;
		void remove(QStandardItem * item) const;
		QStandardItem * findNameInItemChildren(QString name, QStandardItem * item) const;
		QString filePath( QStandardItem * item ) const;

		QSet<QPersistentModelIndex> checklist;

		QString chroot;
		bool showHiddenFiles;
		QStandardItem * rootItem;
		FileIconProvider * fileIconProvider;
};

#endif
