#include "googledrivemanager.h"

#include <QDir>
#include <QUrl>

GoogleDriveManager::GoogleDriveManager( GoogleDriveService * googleDriveService, QObject * parent ) : QStandardItemModel( parent )
{
	this->googleDriveService = googleDriveService;
	process = 0;
	connect( this, SIGNAL(isAvailable(bool)), googleDriveService, SIGNAL(isAvailable(bool)) );
}

GoogleDriveManager::~GoogleDriveManager()
{
}

void GoogleDriveManager::update()
{
	this->clear();

	QStringList accounts = googleDriveService->getAccounts();

	int numAccounts = accounts.count();

	if ( numAccounts > 0 )
		emit isAvailable( true );
	else
		emit isAvailable( false );

	for (int i = 0; i < numAccounts; i++)
	{
		QStandardItem * account = new QStandardItem( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "google-drive.png"), accounts[i] );
		account->setEditable( false );
		appendRow( account );
	}
}

QStringList GoogleDriveManager::getAccounts()
{
	return googleDriveService->getAccounts();
}

QUrl GoogleDriveManager::addAccount(QString account)
{
	QUrl url;

	process = new QProcess;
	QString program = "googledrive.sh";
	QStringList arguments;
	arguments += googleDriveService->getGoogleDriveDir();
	arguments += "configure";
	arguments += account;

        process->start( program, arguments );
	qDebug("Google add account: process start");

	if (process->waitForStarted())
	{
		qDebug("Google add account: process started ok");
		if (process->waitForReadyRead())
		{
			qDebug("Google add account: wait for ready read");
			while ( process->bytesAvailable() > 0)
			{
				QByteArray output = process->readLine();
				qDebug( output.data() );
				if ( output.contains("https://accounts.google.com") )
				{
					int pos = output.indexOf("https://accounts.google.com");
					int length = output.length() - pos;
					QString urlString = output.right( length );
					urlString = urlString.replace( "%3A", ":" );
					urlString = urlString.replace( "%2F", "/" );
					url = QUrl( urlString.trimmed() );
				}
			}
		} else {
			qDebug("Google add account: not ready for read");
		}
	} else {
		qDebug("Google add account: not started");
	}

	return url;
}

void GoogleDriveManager::finishAddAccount(QString key)
{
	if ( process != 0 )
	{
		if ( process->state() == QProcess::Running )
		{
			process->write( QString( key + "\n" ).toUtf8().data() );
			process->waitForFinished();
			Settings::logInfo( QString( "Google Drive account creation output: '%1'" ).arg(process->readAllStandardOutput().data() + QString() + process->readAllStandardError().data() ) );
		}

		delete process;
		process = 0;
	}
}

void GoogleDriveManager::removeAccount(QString account)
{
	googleDriveService->unconfigure( account );
}
