#include "googledriveservice.h"

#include <QDir>
#include "settings.h"

GoogleDriveService::GoogleDriveService()
{
}

GoogleDriveService::~GoogleDriveService()
{
}

void GoogleDriveService::start()
{
	emit isUpdating( true );

	Settings::logInfo( "Starting Google Drive..." );
	qDebug("Starting Google Drive..");
	QStringList arguments;
	arguments += "start";
	if ( launchCommand( arguments ) == 0)
	{
		emit isAvailable( true );
		Settings::logInfo( "Done." );
	} else {
		Settings::logInfo( "Not done." );
	}
	emit isUpdating( false );

}

void GoogleDriveService::stop()
{
	QStringList arguments;
	arguments += "stop";

	launchCommand( arguments );

	emit isUpdating( false );
	emit isAvailable( false );
}

bool GoogleDriveService::status()
{
	QStringList arguments;
	arguments += "status";

	if ( launchCommand( arguments ) == 0)
	{
		return true;
	} else {
		return false;
	}
}

void GoogleDriveService::update()
{
	this->stop();
	this->start();
}

void GoogleDriveService::unconfigure()
{
	stop();

	QStringList arguments;
	arguments += "unconfigure";
	arguments += "--all";

	launchCommand( arguments );
}

void GoogleDriveService::unconfigure(QString account)
{
	QStringList arguments;
	arguments += "unconfigure";
	arguments += account;

	launchCommand( arguments );
}

QStringList GoogleDriveService::getAccounts()
{
	QString program = "/usr/bin/googledrive.sh";

	QStringList arguments;
	arguments += getGoogleDriveDir();
	arguments += "list";

	return getCommandOutput( program, arguments );
}

int GoogleDriveService::launchCommand(QStringList subcommands)
{
	QString program = "/usr/bin/googledrive.sh";

	QStringList arguments;
	arguments += getGoogleDriveDir();
	arguments += subcommands;

	Settings::logInfo( QString("Launching command: %1").arg(program + " " + arguments.join(" ") ) );

	Settings::logInfo( QString("Command output: %1").arg( getCommandOutput( program, arguments ).join("\n") ) );
	return true;
}

QString GoogleDriveService::getGoogleDriveDir()
{
	QString googleDriveDir;

	//if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
	//	googleDriveDir = CHROOTMOUNT + Settings::homePath() + QString(QDir::separator()) + GOOGLEDRIVE_HOME;
	//else
	googleDriveDir = Settings::homePath() + QString(QDir::separator()) + GOOGLEDRIVE_HOME;

	return googleDriveDir;
}
