#include "listview.h"

ListView::ListView(QStringList validParents, QWidget * parent) : QTreeView(parent)
{
	this->setUniformRowHeights( true );
	this->setItemsExpandable( false );

	this->validParents = validParents;
	connect(&timer, SIGNAL(timeout()), this, SLOT(listViewRightClicked()));
	//connect(this, SIGNAL(indexesMoved(const QModelIndexList & )), this, SLOT( indexesMoved(const QModelIndexList & ) ) );
}

void ListView::resizeEvent ( QResizeEvent * event )
{
	QTreeView::resizeEvent( event );
	this->setColumnWidth(0, width() - 3 * 120);
}

void ListView::indexesMoved(const QModelIndexList & )
{
	//this->resizeColumnToContents( 0 );
}

void ListView::setValidParents(QStringList validParents)
{
	this->validParents = validParents;
}

QStringList ListView::getValidParents()
{
	return validParents;
}

QModelIndexList ListView::selectedIndexes()
{
	return QTreeView::selectedIndexes();
}

void ListView::mousePressEvent ( QMouseEvent * event )
{
	qDebug("Mouse pressed!");
	rightClickPos = event->pos();
	if (event->button() == Qt::RightButton)
	{
		listViewRightClicked();
	}

	if (event->button() == Qt::LeftButton)
	{
		timer.setSingleShot(true);
		timer.start(TIMER_RIGHT_CLICK);
	}
}

void ListView::mouseReleaseEvent ( QMouseEvent * event )
{
	qDebug("mouseRelease at %d, %d", event->pos().x(), event->pos().y());
	if (timer.isActive())
	{
		timer.stop();

		if (event->button() == Qt::LeftButton)
		{
			QModelIndex index = indexAt(event->pos());
			if (index.isValid())
			{
				QStyleOptionButton opt;
				opt.QStyleOption::operator=(viewOptions());
				opt.rect = visualRect(index);
				QRect rect = style()->subElementRect(QStyle::SE_ViewItemCheckIndicator, &opt);
				if (! rect.contains(event->pos()))
				{
					// Released outside the checkbox
					emit open(index);
				}
			}
		}
	}
	QTreeView::mouseReleaseEvent(event);
}

void ListView::listViewRightClicked()
{
	if (timer.isActive())
	{
		timer.stop();
	}

	qDebug("Right clicked pressed! On %d, %d", rightClickPos.x(), rightClickPos.y());

	emit rightClicked();
}

void ListView::setRootIndex(const QModelIndex & index )
{
	if (index.isValid())
	{
#ifndef BASICFSMODE
		FileSystemModel * fsModel = (FileSystemModel *)index.model();
		fsModel->setRootIndex( index );
		QTreeView::setRootIndex(index);
		emit rootIndexChanged(index);
#endif
	}
}

bool ListView::isValidPath(QString path)
{
	Settings::logInfo( QString("Checking path %1...").arg(path.toUtf8().data()) );
	QFileInfo pathInfo(path);
	bool isValid = false;
	int count = validParents.count();
	for (int i=0; i<count; i++)
	{
		if (pathInfo.absoluteFilePath().startsWith(validParents[i]))
		{
			isValid = true;
		}
	}

	if ( ! isValid )
		Settings::logInfo( "Invalid path; no valid parent" );

	return isValid;
}
	

QPoint ListView::getRightClickPos()
{
	return rightClickPos;
}
