#ifndef LISTVIEW_H
#define LISTVIEW_H

#include <QListView>
#include <QTreeView>
#include <QWidget>
#include <QMouseEvent>
#include <QTimer>
#include <QStringList>

#include "constants.h"
#include "settings.h"
#ifdef BASICFSMODE
#include "basicfilesystemmodel.h"
#else
#include "filesystemmodel.h"
#endif

class ListView : public QTreeView
{
	Q_OBJECT

	public:
		ListView(QStringList validParents, QWidget * parent);

		void setValidParents(QStringList validParents);
		QStringList getValidParents();
		QModelIndexList selectedIndexes();
		QPoint getRightClickPos();
		bool isValidPath(QString path);

	public slots:
		void mousePressEvent ( QMouseEvent * event );
		void mouseReleaseEvent ( QMouseEvent * event );
		void listViewRightClicked();
		void setRootIndex(const QModelIndex & index );
		void indexesMoved(const QModelIndexList & indexes);

	signals:
		void rightClicked();
		void open(const QModelIndex & index );
		void rootIndexChanged(const QModelIndex & index);

	private:
		void resizeEvent ( QResizeEvent * event );

		QTimer timer;
		QStringList validParents;
		QPoint rightClickPos;
};

#endif
