#ifdef OLDCODE
#include "tabwidget.h"
#include "container.h"

TabWidget::TabWidget(QObject * parent) : QObject(parent)
{
	qDebug("***Creating TabWidget");
	dropbox = 0;
	googleDrive = 0;
	addStorageServices();

	lastIndex = 0;
	currentIndex = 0;
	closeClicked = false;

	tabBar = new TabBar( NULL ) ;

	addMenu();
	addPlaces();
	addConfiguration();

	connectStorageServices();

	exitButton = new MenuButton( 0, EXITBUTTON,  QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "logoff.png") );
	tabBar->addButton( exitButton );

	connect(exitButton, SIGNAL(clicked()), this, SLOT(exitApp()));
	connect(tabBar, SIGNAL(currentChanged(int)), this, SLOT(currentChanged(int)));
	connect(parent, SIGNAL(fileOpened()), fileManager, SLOT(fileOpened()));
	connect(this, SIGNAL(exiting()), parent, SLOT(exitApp()));
	connect(tabBar, SIGNAL(closeButtonClicked(MenuButton *)), this, SLOT(closeButtonClicked(MenuButton *)) );

	threadManager = 0;

	tabBar->setCurrentIndex( 0 );
	tabBar->resize();
}

TabWidget::~TabWidget()
{
}

void TabWidget::init(LWClient * lwClient)
{
    deskMenu->setCursor(Qt::WaitCursor);
    Application::setValidApplications();
    deskMenu->addApplications( Application::getValidApplications(), lwClient );
    deskMenu->setLogo();
    configure->setFixedSize(deskMenu->size());
    configure->addApplications();
    QApplication::processEvents();
    deskMenu->setCursor(Qt::ArrowCursor);
}

void TabWidget::setName( long window )
{
	tabBar->refresh();

	return;

	MenuButton * button = getMenuButton( window );

	int index = tabBar->getCurrentIndex();

	if ( index != -1)
	{
		if ( button == tabBar->getButton(tabBar->getCurrentIndex()) )
		{
			tabBar->setSelected( button );
		}
	}
}

void TabWidget::resize(int)
{
	emit resized();
	QSize newSize = QApplication::desktop()->screenGeometry().size();

	QSize widgetSize = QSize( newSize.width(), newSize.height() - tabBar->height() );

	deskMenu->resize( widgetSize );
	fileManager->setFixedSize( widgetSize );
	configure->resize( widgetSize );

	tabBar->resize();

	tabBar->setCurrentIndex( tabBar->getCurrentIndex() );
}

void TabWidget::currentChanged(int index)
{
	emit setSelected( index );
	qDebug("CurrentChanged. lastIndex: %d, count(): %d", lastIndex, count());

	int m = tabBar->menuButtonCount();
	if ( count() > m )
	{
		exitButton->setPixmap( QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "task-manager.png") );
	} else {
		exitButton->setPixmap( QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "logoff.png") );
	}

    if ( tabBar->getWindows().at( index ) == fileManager->winId() )
		fileManager->refresh();

	if (closeClicked)
	{
		closeClicked = false;
		if ( lastIndex >= count() )
			lastIndex = 0;
		setCurrentIndex(lastIndex);
		return;
	}

	if ( index != currentIndex )
	{
		lastIndex = currentIndex;
		currentIndex = index;
	}

    long window = tabBar->getWindows().at( index );
#ifdef Q_OS_LINUX
	QList<unsigned long> windows = X11Info::getChildren(window);
	if (windows.count() > 0)
		window = windows[0];
	X11Info::setInputFocus( window );
#endif
}

QList<long> TabWidget::getWindows()
{
	return windows;
}

void TabWidget::exitApp()
{
	int m = tabBar->menuButtonCount();
	if ( count() > m )
	{
		goToStartPage();
		threadManager = new ThreadManager( tabBar, m, NULL );
		threadManager->show();
        connect( tabBar, SIGNAL(widgetRemoved ( int )), threadManager, SLOT(update(int)) );
		connect( threadManager, SIGNAL( finished() ), this, SLOT( goToStartPage() ) );
		return;
	}

	disconnectStorageServices();

	while(count()>0)
	{
		removeTab(count() - 1);
	}

	emit exiting();
}

void TabWidget::setSelected(MenuButton * button)
{
	tabBar->setSelected( button );
    emit setSelected( button->getWinId() );
}

void TabWidget::addStorageServices()
{
#ifdef Q_OS_LINUX
	Settings::logInfo("Creating storage services..");
	dropbox = new DropboxService();
	googleDrive = new GoogleDriveService();
	Settings::logInfo("Done storage services creation.");
#endif
}

void TabWidget::connectStorageServices()
{
#ifdef Q_OS_LINUX
	Settings::logInfo("Connecting storage services...");
	connect( dropbox, SIGNAL(isAvailable(bool)), fileManager, SLOT(showDropbox(bool)));
	dropbox->start();

	connect( googleDrive, SIGNAL(isAvailable(bool)), fileManager, SLOT(showGoogleDrive(bool)));
	connect( googleDrive, SIGNAL(isUpdating(bool)), fileManager, SLOT(showGoogleDriveUpdating(bool)));
	googleDrive->start();
	Settings::logInfo("Done storage services connection.");
#endif
}

void TabWidget::disconnectStorageServices()
{
	if ( dropbox != 0 )
	{
		dropbox->stop();
		delete dropbox;
	}

	if ( googleDrive != 0 )
	{
		googleDrive->stop();
		delete googleDrive;
	}
}

void TabWidget::addMenu()
{
	deskMenu = new DeskMenu(0, BACKGROUNDCOLOR);

    MenuButton * button = new MenuButton( deskMenu, MENUBUTTON, QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "logo.png") , tr("LibreWork"), tr("Desktop"), deskMenu->getWindowId());
    addTab( button );
	tabBar->setSelected( button );
	connect(deskMenu, SIGNAL(launchApplication(AppInstance*)), this, SIGNAL(launchApplication(AppInstance*)));
}

void TabWidget::addPlaces()
{
	fileManager = new FileManager(0, false);
	MenuButton * button = new MenuButton( fileManager, MENUBUTTON, QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "folder-docs.png") , tr("Librework"), tr("Places"), fileManager->getWindowId());
    qDebug("Filemanager window id: %d", (int)fileManager->getWindowId());
    addTab( button );
	connect(fileManager, SIGNAL(openFile(QString, QString)), this, SIGNAL(openFile(QString, QString)));
}

void TabWidget::addConfiguration()
{
	//configure = new Configure(this, "99aabb");
	configure = new Configure(this, BACKGROUNDCOLOR);
	MenuButton * button = new MenuButton( configure, MENUBUTTON, QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "config.png") , tr("Librework"), tr("Configuration"), configure->getWindowId());
	addTab( button );
	connect(configure, SIGNAL(launchApplication(AppInstance*)), this, SIGNAL(launchApplication(AppInstance*)));
}

void TabWidget::removeTab ( int index )
{
	closeClicked = true;
	windows.removeAt( index );
	pids.removeAt( index );
	appNames.removeAt( index );
	tabBar->removeButton( index );
}

int TabWidget::addTab ( MenuButton * button )
{
    if ( button->isRemoteApp() )
	{
		Container * container = (Container*)(button->getContainer());
		windows.append( container->getWindow() );
	} else {
		windows.append(button->getWinId());
	}
	pids.append(button->getPid());
	appNames.append(button->getLabel());

	tabBar->addButton( button );

	connect( button, SIGNAL(menuButtonClicked(MenuButton*)), this, SLOT(setSelected(MenuButton*)) );
	connect( button, SIGNAL(closeMenuButton(int)), this, SLOT(removeTab(int)) );

	return button->getIndex();
}

void TabWidget::closeButtonClicked(MenuButton *)
{
	closeClicked = true;
}

QWidget * TabWidget::getWidgetFromPid( Q_PID pid )
{
    if (pid == 0)
		return 0;

	int index = pids.indexOf(pid);

	if (index != -1)
		return widget( index );
	else
		return 0;
}

MenuButton * TabWidget::getMenuButton( long w )
{
	int index = windows.indexOf( w );

	return tabBar->getButton( index );
}

QString TabWidget::getAppName(int index)
{
	return appNames[index];
}

void TabWidget::goToStartPage()
{
	setCurrentIndex(0);
}

void TabWidget::setCurrentWidget(QWidget * widget)
{
	tabBar->setCurrentWidget(widget);
}

int TabWidget::indexOf(long window)
{
    return tabBar->indexOf(window);
}

int TabWidget::count()
{
	return tabBar->count();
}

void TabWidget::setCurrentIndex(int index)
{
	tabBar->setCurrentIndex(index);
}

#ifdef Q_OS_LINUX
DropboxService * TabWidget::getDropboxService()
{
	return dropbox;
}

GoogleDriveService * TabWidget::getGoogleDriveService()
{
	return googleDrive;
}
#endif
#endif
