#ifndef TABWIDGET_H
#define TABWIDGET_H

#ifdef OLDCODE

#include <QList>
#include <QPushButton>
#include <QVBoxLayout>
#include <QIcon>
#include <QProcess>
#include <QAbstractEventDispatcher>
#include <QVBoxLayout>
#include <QMessageBox>
#include <QProgressDialog>

#include "filemanager.h"
#include "constants.h"
#include "deskmenu.h"
#include "configure.h"
#include "closebutton.h"
#include "appinstance.h"
#include "menubutton.h"
#include "tabbar.h"
#include "threadwidget.h"
#include "threadmanager.h"
#include "dropboxservice.h"
#include "googledriveservice.h"
#include "lwclient.h"

#ifdef Q_OS_LINUX
#include "x11info.h"
#endif

class TabWidget : public QObject
{
	Q_OBJECT

	public:
        TabWidget(QObject *parent = 0);
		~TabWidget();

        void init(LWClient * lwClient);
		QWidget * getWidgetFromPid( Q_PID pid );
		int addTab ( MenuButton * menuButton);
		QList<long> getWindows();
		DeskMenu * getDeskMenu() { return deskMenu; };
		Configure * getConfigure() { return configure; };
		FileManager * getFileManager() { return fileManager; };
		QString getAppName(int index);
		QWidget * widget(int index);
        int indexOf(long window);
		int count();
		TabBar * getTabBar() { return tabBar; };
		MenuButton * getMenuButton( long w );
#ifdef Q_OS_LINUX
		DropboxService * getDropboxService();
		GoogleDriveService * getGoogleDriveService();
#endif

	public slots:
		void removeTab ( int index );
		void setCurrentWidget(QWidget * widget);
		void setSelected(MenuButton * button);
		void setCurrentIndex(int index);
		void resize(int screen = -1);
		void setName( long w );

	signals:
		void launchApplication(AppInstance * app);
		void openFile(QString file, QString application);
		void exiting();
		void resized();
        void setSelected(int);

	private slots:
		void exitApp();
		void closeButtonClicked(MenuButton* button);
		void currentChanged(int index);
		void goToStartPage();

	private:
		void addMenu();
		void addPlaces();
		void addConfiguration();
		void addStorageServices();
		void connectStorageServices();
		void disconnectStorageServices();

		QVBoxLayout * layout;

		TabBar * tabBar;
		ThreadWidget * currentThread;
		ThreadManager * threadManager;
		MenuButton * exitButton;
		QList<long> windows;
		QList<QString> appNames;
		QList<Q_PID> pids;
		DeskMenu * deskMenu;
		Configure * configure;
		FileManager * fileManager;
		int lastIndex;
		int currentIndex;
		bool closeClicked;
		DropboxService * dropbox;
		GoogleDriveService * googleDrive;
};

#endif

#endif
