#include "remoteclientmanager.h"

RemoteClientManager::RemoteClientManager()
{
}

void RemoteClientManager::start()
{
#ifdef Q_OS_WIN
	LWClient * lwClient = LWClient::instance();
	QHash<QString,QString> assignedDrives;
	QHash<QString,QString> sharedFoldersAssignedDrives;
	QStringList redirectedDrives = lwClient->sendMessage(QString("get_redirect_drives?%1").arg(Settings::getFullUserName())).split(",");
	Settings::logInfo(QString("get_redirect_drives output: %1").arg(redirectedDrives.join(",")));
	for (int i = 0; i < redirectedDrives.count(); i++)
	{
		QString drive = redirectedDrives[i].trimmed().section('=', 0, 0).remove("\\").toUpper();
		QString name = redirectedDrives[i].trimmed().section('=', 1, 1);
		if ( name.startsWith("\\\\") ) // Shared folder
		{
			sharedFoldersAssignedDrives[name] = drive;
			Settings::logInfo(QString("Shared folder name: %1 drive: %2").arg(name).arg(drive));
		} else { // Client drive redirect
			name.remove(":").remove("/").remove("\\");
			assignedDrives[name.toLower()] = drive;
			Settings::logInfo(QString("Client redirect name: %1 drive: %2").arg(name).arg(drive));
		}
	}
	names.clear();
	drives.clear();
	remove.clear();
	WinInfo::setRemoteDrives(&names, &drives, &remove, assignedDrives);
	WinInfo::setSharedFoldersDrives(&names, &drives, &remove, sharedFoldersAssignedDrives);
	if ( Settings::homePath().startsWith("\\\\") ) // Home path in shared folder
	{
		WinInfo::setHomeDrive(&names, &drives, &remove);
	}
#endif
    emit update();
}

void RemoteClientManager::stop()
{
    for (int i = 0; i < drives.count(); i++)
    {
        if ( remove[i] )
        {
#ifdef Q_OS_WIN
            WinInfo::removeRemoteDrive( drives[i] );
#endif
        }
    }
}
