#ifndef SETTINGS_H
#define SETTINGS_H

#include <QSettings>
#include <QSet>
#include <QList>
#include <QString>
#include <QFile>
#include <QTcpSocket>
#include <QHostAddress>
#include <QDateTime>
#include <QNetworkInterface>
#include <QHostInfo>
#include <QMap>

#include "constants.h"

class Settings
{
	public:
		Settings();

        static void init();

		static QHostAddress getLocalAddress();
		static void setLocalAddress(QHostAddress address);
		static QString getLocalOs();
		static bool isLocalhost(QString address);

		static void setValue(QString file, QString key, QString value);
		static QString getValue(QString primaryFile, QString secondaryFile, QString key);
		static QString getValue(QString file, QString key);
		static QString getValue(QString key);
		static QStringList allKeys(QString file);
		static bool getBooleanValue(QString key);
		static QStringList getIds(QString file);
		static QStringList getValues(QString file, QString field);

		static QString getApplicationValue( QString appName, QString key );
		static QStringList getApplicationsList();
		static QStringList getMimeList();


		static QString sharedPath();
		static QString searchPath();
		static QString windowsSharedPath();
		static QString windowsHomePath();
		static int ldapServerPort();

		static QString getUserName();
		static QString getPassword();
		static QString getFullUserName();
		static QString getMimetype(QString extension);
		static QStringList getApplications( QString mimetype );
		static void setUserName(QString userName);
		static void setPassword(QString password);
		static void setExtensions();
		static int getWindowManagerBehaviour();
		static QString homePath();
		static void setHomePath( QString homePath );

		static void log(QString type, QString message);
		static void logInfo(QString message);
		static void logError(QString message);
};

#endif
