#include "tabbar.h"

#include <QDir>

/*ApplicationWindow WindowHistory::getLastWindow()
{
	ApplicationWindow appWindow((long long)0);
	return appWindow;
}

void WindowHistory::setLastWindow(ApplicationWindow appWindow)
{

}

void WindowHistory::removeWindow(ApplicationWindow appWindow)
{

}
*/
TabBar::TabBar ( QWidget * parent ) : QWidget( parent, Qt::CustomizeWindowHint | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint )
{
	QPalette pal(palette());
	pal.setColor(QPalette::Background, Qt::white);
	setAutoFillBackground(true);
	setPalette(pal);

	setAttribute(Qt::WA_AlwaysShowToolTips);

	layout = new QHBoxLayout;
	layout->setSizeConstraint( QLayout::SetMinimumSize );
	layout->setSpacing(0);
	layout->setContentsMargins(0, 0, 0, 0);

	menusLayout = new QHBoxLayout;
	menusLayout->setSpacing(0);
	menusLayout->setContentsMargins(0, 0, 0, 0);
	menusLayout->setAlignment( Qt::AlignLeft );
	menusLayout->setSizeConstraint( QLayout::SetMinimumSize );

	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentThread = new ThreadWidget( NULL, false );
		currentThread->setFixedHeight( TAB_BUTTON_HEIGHT );
	}

	threadsBar = new QWidget( NULL );

	threadsLayout = new QHBoxLayout;
	threadsLayout->setSpacing(0);
	threadsLayout->setContentsMargins(0, 0, 0, 0);
	threadsLayout->setAlignment( Qt::AlignLeft );

	threadsBar->setLayout( threadsLayout );

	leftButton = new QPushButton( QIcon( ICON_DIR_64 + QString(QDir::separator()) + "back.png" ), QString::null, this );
	leftButton->setFixedSize( QSize(TAB_BUTTON_WIDTH / 2, TAB_BUTTON_HEIGHT) );
	leftButton->hide();
	leftButton->setAccessibleName( "leftButton" );
	connect( leftButton, SIGNAL(clicked()), this, SLOT(leftButtonClicked()));
	rightButton = new QPushButton( QIcon( ICON_DIR_64 + QString(QDir::separator()) + "forward.png" ), QString::null, this );
	rightButton->setFixedSize( QSize(TAB_BUTTON_WIDTH / 2, TAB_BUTTON_HEIGHT) );
	rightButton->hide();
	rightButton->setAccessibleName( "rightButton" );
	connect( rightButton, SIGNAL(clicked()), this, SLOT(rightButtonClicked()));
	threadsLayout->addWidget(leftButton);
	threadsLayout->addWidget(rightButton);

	exitLayout = new QHBoxLayout;
	exitLayout->setContentsMargins(0, 0, 0, 0);
	exitLayout->setSpacing(0);

	layout->addLayout( menusLayout, 0 );
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		layout->addWidget( currentThread, 0 );
	}
	layout->addWidget( threadsBar, 1 );
	layout->addLayout( exitLayout, 0 );

	setLayout( layout );

	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		connect (currentThread, SIGNAL(closeButtonClicked(ThreadButton *)), this, SLOT(closeButtonClicked(ThreadButton *)) );
		connect (currentThread, SIGNAL(killButtonClicked(ThreadButton *)), this, SLOT(killButtonClicked(ThreadButton *)) );
	}

	lastIndex = -1;
	buttonList = new QList<ThreadButton*>;
	hiddenThreadButtons = 0;
	firstShownThreadButton = 0;

	addMenuButtons();
}

void TabBar::showDesktop()
{
	setCurrentIndex( 0 );
	emit mapWidget( (QWidget*) deskMenu, true );
}

void TabBar::addMenuButtons()
{
	deskMenu = new Menu( 0, BACKGROUNDCOLOR );
	ThreadButton * button = new ThreadButton(deskMenu, MENUBUTTON, QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "logo.png") , tr("Librework"), tr("Desktop"), deskMenu->getWindowId());
	addButton( button );
	connect( deskMenu, SIGNAL(launchApplication(AppInstance*)), this, SIGNAL(launchApplication(AppInstance*)));
	lastMenuButton = (QWidget*) deskMenu;

	fileManager = new FileSystemWidget( 0 );
	ThreadButton * fileButton = new ThreadButton(fileManager, MENUBUTTON, QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "folder-docs.png") , tr("Librework"), tr("Places"), fileManager->winId());
	addButton( fileButton );
	connect( fileManager, SIGNAL(openFile(QString, QString)), this, SIGNAL(openFile(QString, QString)));

	configure = new Configure( 0, BACKGROUNDCOLOR );
	ThreadButton * configureButton = new ThreadButton(configure, MENUBUTTON, QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "config.png") , tr("Librework"), tr("Configuration"), configure->getWindowId());
	addButton( configureButton );
	connect( configure, SIGNAL(selectWidget(QWidget*, bool)), this, SIGNAL(mapWidget(QWidget*, bool)));

	exitButton = new ThreadButton( 0, EXITBUTTON,  QPixmap(ICON_DIR_32 + QString(QDir::separator()) + "logoff.png") );
    addButton( exitButton );
	connect(exitButton, SIGNAL(clicked()), this, SIGNAL(exitApp()));

	Application::setValidApplications();
    deskMenu->addApplications( Application::getValidApplications(), LWClient::instance() );
    deskMenu->setLogo();

    fileManager->init();

    configure->setFixedSize( deskMenu->size() );
    configure->addApplications();
}

TabBar::~TabBar()
{
	deskMenu->close();
	fileManager->close();
	configure->close();

	delete deskMenu;
	delete fileManager;
	delete configure;
}

int TabBar::numThreadButtons()
{
	return threadsLayout->count() - 2;
}

int TabBar::getCurrentThreadIndex()
{
	int res = getCurrentIndex() - menusLayout->count();
	if (res < 0)
		return 0;
	else
		return res;
}

void TabBar::resize( QSize desktopSize )
{
	qDebug("Setting new geometry - x: %d, y: %d, width: %d, height: %d", 0, desktopSize.height(), desktopSize.width(), TAB_BUTTON_HEIGHT);
	setGeometry( QRect(0, desktopSize.height(), desktopSize.width(), TAB_BUTTON_HEIGHT) );
    configure->resize( desktopSize );
    deskMenu->resize( desktopSize );
	updateContents();
}

void TabBar::updateContents()
{
	// First, calculate currentThread (ThreadWidget) width
	int occupiedSize = menusLayout->count() * TAB_BUTTON_WIDTH /* static menus */
					+ TAB_BUTTON_WIDTH /* exitButton*/
					+ TAB_BUTTON_WIDTH /* left & right */ 
					+ TAB_BUTTON_WIDTH /* at least one thread button */;

	int currentThreadMaximumWidth = QApplication::desktop()->screenGeometry().size().width() - occupiedSize;
	if ( currentThreadMaximumWidth > THREAD_WIDGET_WIDTH )
		currentThreadMaximumWidth = THREAD_WIDGET_WIDTH;

	// Hide all thread buttons
	for (int i = 0; i < numThreadButtons(); i++)
		threadsLayout->itemAt( 1 + i )->widget()->hide();

	// Get left space for thread buttons
	int spaceLeft = qApp->desktop()->screenGeometry().size().width()
			- menusLayout->count() * TAB_BUTTON_WIDTH /* static menus */
			- TAB_BUTTON_WIDTH /* exitButton*/
			- currentThreadMaximumWidth /* currentThread width */;

	// Calculate maximum number of buttons on the given space left (c++ integer division always round down)
	int maxButtons = spaceLeft / TAB_BUTTON_WIDTH;

	// Add to currentThread (ThreadWidget) the remaining space
	int remainingSpace = spaceLeft - ( maxButtons * TAB_BUTTON_WIDTH );
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentThread->setFixedSize( currentThreadMaximumWidth + remainingSpace, TAB_BUTTON_HEIGHT );
	}

	// Calculate number of shown and hidden buttons, and the position of the first shown button
	int buttonsShown = 0;
	if ( numThreadButtons() > maxButtons )
	{
		hiddenThreadButtons = numThreadButtons() - maxButtons + 1;
		buttonsShown = maxButtons - 1; // Right and left (scroll) buttons occupy as one button, so substract it
		firstShownThreadButton = hiddenThreadButtons;
	} else {
		hiddenThreadButtons = 0;
		firstShownThreadButton = 0;
		buttonsShown = numThreadButtons();
	}

	// Finally show calculated thread buttons
	if ( hiddenThreadButtons > 0 )
	{
		for (int i = 0; i < numThreadButtons(); i++)
		{
			if ( i >= firstShownThreadButton && i < firstShownThreadButton + buttonsShown )
			{
				qDebug("shown");
				threadsLayout->itemAt( 1 + i )->widget()->show();
			} else {
				qDebug("hidden");
				threadsLayout->itemAt( 1 + i )->widget()->hide();
			}
		}
		showScrollButtons();
	} else { // as ther are no hidden buttons, hide scroll buttons but show everyone else
		hideScrollButtons();
		for (int i = 0; i < numThreadButtons(); i++)
		{
			threadsLayout->itemAt( 1 + i )->widget()->show();
		}
	}
}

void TabBar::removeButton( int index )
{
    qDebug("Removing button. lastIndex is %d", lastIndex);
    ThreadButton * button;

    int ix;

    if (index == -1 ) // Remove current selected button
    {
        ix = getCurrentIndex();
    } else {
        ix = index;
    }

    if (ix < buttonList->count())
    {
        button = buttonList->at( ix );
    } else {
        return;
    }
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		if ( button == currentThread->getThreadButton() )
		{
			currentThread->setThreadButton( 0 );
		}
    }

    buttonList->removeAt( ix );
    threadsLayout->removeWidget( button );
    delete button;

#ifdef Q_OS_WIN
	HWND window = GetForegroundWindow();
	if (window != NULL)
	{
		int newIndex = indexOf((long long) window );
		if ( newIndex != -1 )
		{
			setCurrentIndex( newIndex );
			updateContents();
			return;
		}
	}
#endif

    int count = buttonList->count();
    if (  count > 0 )
    {
        if ( lastIndex >= count || lastIndex < 0)
		{
			doMapWidget( lastMenuButton, true);
        } else {
			setCurrentIndex( lastIndex );
		}
    }

    updateContents();
}

ThreadButton * TabBar::getButton( int index )
{
	if (index >= 0)
		return buttonList->at( index );
	else
		return 0;
}

void TabBar::addToThreadsLayout( ThreadButton * button )
{
	threadsLayout->removeWidget( rightButton );
	threadsLayout->addWidget( button );
	threadsLayout->addWidget( rightButton );
}

void TabBar::addButton(ThreadButton * button)
{
	ThreadButton * currentButton = 0;
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentButton = currentThread->getThreadButton();
	}

	if ( currentButton != 0 )
	{
		int newIndex = buttonList->indexOf(button);
		int ix = buttonList->indexOf(currentButton);
		if ( ix != newIndex )
			lastIndex = ix;
	}

	if (button->getType() != EXITBUTTON)
	{
		int index = indexOf( button->getWinId() );
		buttonList->append( button );

		button->setIndex( index );

		connect( button, SIGNAL(threadButtonClicked(int)), this, SLOT(setCurrentIndex(int)) );
	}

	switch (button->getType())
	{
		case MENUBUTTON:
			menusLayout->addWidget( button );
			connect( button, SIGNAL(threadButtonClicked(QWidget*,bool)), this, SLOT(doMapWidget(QWidget*,bool)) );
			break;

		case THREADBUTTON:
			addToThreadsLayout( button );
			setSelected( button );
			updateContents();
			break;

		case EXITBUTTON:
			exitLayout->addWidget( button );
			break;
	}
}

void TabBar::doMapWidget(QWidget * widget, bool force)
{
	lastMenuButton = widget;
	emit mapWidget(widget, force);
}

ThreadButton * TabBar::getFirstShownThreadButton()
{
	if (numThreadButtons() > 0)
        	return (ThreadButton*) threadsLayout->itemAt( 1 + firstShownThreadButton )->widget();
	else
		return 0;
}

void TabBar::showScrollButtons()
{
	leftButton->show();
	rightButton->show();
	if (firstShownThreadButton == 0)
	{
		leftButton->setEnabled( false );
		rightButton->setEnabled( true );
	} else {
		leftButton->setEnabled( true );
		if (hiddenThreadButtons > firstShownThreadButton)
			rightButton->setEnabled( true );
		else
			rightButton->setEnabled( false );
	}
}

void TabBar::hideScrollButtons()
{
	leftButton->hide();
	rightButton->hide();
}

void TabBar::setSelected( ThreadButton * button )
{
	int newIndex = buttonList->indexOf(button);

	ThreadButton * currentButton = 0;
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentButton = currentThread->getThreadButton();
	}
	if (currentButton != 0)
	{
		currentButton->setSelected(true);
		unsigned long w = button->getWinId();
		if ( w != 0 )
		{
			QPixmap pixmap = QGuiApplication::screens().at(0)->grabWindow( w );
			currentButton->setPreview( pixmap );
			int ix = buttonList->indexOf(currentButton);
			if ( ix != newIndex )
				lastIndex = ix;
		}
	}

	qDebug("Setting selected button. NewIndex: %d, LastIndex: %d", newIndex, lastIndex);

	for (int i = 0; i < buttonList->count(); i++)
	{
		 buttonList->at(i)->setSelected( false );
	}

	button->setSelected( true );
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentThread->setThreadButton(button);
	}
	lastIndex = newIndex;

	if ( button->getWidget() != 0 )
	{
		button->getWidget()->show();
		button->getWidget()->setFocus(Qt::ActiveWindowFocusReason);
	}

	emit currentChanged( newIndex );
}

int TabBar::getCurrentIndex()
{
    return lastIndex;
}

int TabBar::indexOf(long window)
{
    int index = -1;

    for (int i = 0; i < buttonList->count(); i++)
    {
        if (buttonList->at(i)->getWinId() == window)
        {
            index = i;
        }
    }
    return index;
}

int TabBar::indexOf(QWidget *widget)
{
    int index = -1;

    for (int i = 0; i < buttonList->count(); i++)
    {
        if (buttonList->at(i)->getWidget() == widget)
        {
            index = i;
        }
    }
    return index;
}

int TabBar::count()
{
    return buttonList->count();
}

int TabBar::threadButtonCount()
{
	return menusLayout->count();
}

int TabBar::menuButtonCount()
{
    int count = 0;
    for (int i = 0; i < buttonList->count(); i++)
    {
        if ( buttonList->at(i)->getType() == MENUBUTTON )
        {
            count++;
        }
    }
    return count;
}

void TabBar::setCurrentIndex(int index)
{
	if (index >= 0)
	{
		lastIndex = index;
		setSelected( getButton ( index ) );
	}
}

void TabBar::refresh()
{
	setCurrentIndex( getCurrentIndex() );
}

void TabBar::leftButtonClicked()
{
	if (firstShownThreadButton > 0)
	{
		int visibleButtons = numThreadButtons() - hiddenThreadButtons;
		threadsLayout->itemAt( firstShownThreadButton + visibleButtons)->widget()->hide();
		firstShownThreadButton--;
		getFirstShownThreadButton()->show();
	}

	showScrollButtons();
}

void TabBar::rightButtonClicked()
{
	int visibleButtons = numThreadButtons() - hiddenThreadButtons;
	getFirstShownThreadButton()->hide();
	threadsLayout->itemAt( firstShownThreadButton + visibleButtons + 1)->widget()->show();
	firstShownThreadButton++;
	showScrollButtons();
}

void TabBar::closeButtonClicked(ThreadButton * button)
{
	emit closeButtonClicked( button->getWinId() );
}

void TabBar::killButtonClicked(ThreadButton * button)
{
	emit killButtonClicked( button->getWinId() );
}

void TabBar::setCurrentThreadName( QString name )
{
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentThread->setName( name );
	}
}

void TabBar::setCurrentThreadText( QString text )
{
	if ( Settings::getWindowManagerBehaviour() == FULLSCREEN )
	{
		currentThread->setText( text );
	}
}
