#ifndef TABBAR_H
#define TABBAR_H

#include <QPushButton>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QString>
#include <QList>
#include <QIcon>
#include <QPixmap>
#include <QPainter>
#include <QFont>
#include <QWidget>
#include <QSize>
#include <QStackedWidget>
#include <QScrollArea>
#include <QScreen>
#include <QApplication>
#include <QGuiApplication>
#include <QDesktopWidget>

#include "constants.h"
#include "threadbutton.h"
#include "threadwidget.h"
#include "menu.h"
#include "configure.h"
#include "filesystemwidget.h"
#include "appinstance.h"
#include "lwclient.h"

/*class ApplicationWindow
{
public:
	ApplicationWindow(long long window) { _window = window; _widget = 0; };
	ApplicationWindow(QWidget * widget) { _widget = widget; _window = 0; };

	long long window() { return _window; };
	QWidget * widget() { return _widget; };

private:
	long long _window;
	QWidget * _widget;
};

class WindowHistory : public QObject
{
	Q_OBJECT

public:
	ApplicationWindow getLastWindow();
	void setLastWindow(ApplicationWindow appWindow);

public slots:
	void removeWindow(ApplicationWindow appWindow);

private:
	QList<long long> windowHistory;
	QList<QWidget*> widgetHistory;
};
*/

class TabBar : public QWidget
{
	Q_OBJECT

	public:
		TabBar ( QWidget * parent );
		~TabBar();

		void addButton(ThreadButton * button);

		void setSelected( ThreadButton * button );
		ThreadButton * getButton( int index );
		QList<ThreadButton *> * getButtonList() { return buttonList; };

		int indexOf(long window);
		int indexOf(QWidget * widget);
		int count();
		int getCurrentIndex();
		int getCurrentThreadIndex();
		int threadButtonCount();
		int menuButtonCount();

	public slots:
		void showDesktop();
		void removeButton( int index = -1 );
		void setCurrentIndex(int index);
		void refresh();
		void resize( QSize desktopSize );
		void updateContents();
		void setCurrentThreadName( QString name );
		void setCurrentThreadText( QString text );

		void closeButtonClicked(ThreadButton * button);
		void killButtonClicked(ThreadButton * button);

	private slots:
		void doMapWidget(QWidget * widget, bool force);
		void leftButtonClicked();
		void rightButtonClicked();

	signals:
		void buttonClicked(ThreadButton *);
        void closeButtonClicked(long long);
        void killButtonClicked(long long);
		void currentChanged( int );
		void launchApplication(AppInstance*);
		void openFile(QString, QString);
		void mapWidget(QWidget*,bool);
		void exitApp();

	private:
		void addMenuButtons();
		void showScrollButtons();
		void hideScrollButtons();
		ThreadButton * getFirstShownThreadButton();
		int numThreadButtons();
		void addToThreadsLayout( ThreadButton * button );

		QHBoxLayout * layout;
		QHBoxLayout * menusLayout;
		QHBoxLayout * threadsLayout;
		QHBoxLayout * exitLayout;

		ThreadWidget * currentThread;
		QWidget * threadsBar;
		QScrollArea * threadsScroll;
		QList<ThreadButton *> * buttonList;
		QPushButton * leftButton;
		QPushButton * rightButton;

		Menu * deskMenu;
		FileSystemWidget * fileManager;
		Configure * configure;
		ThreadButton * exitButton;

/*		WindowHistory windowHistory;*/
		int lastIndex;
		QWidget * lastMenuButton;
		int hiddenThreadButtons;
		int firstShownThreadButton;
};

#endif
