#include "threadbutton.h"
#include <QDir>

#ifdef Q_OS_LINUX
ThreadButton::ThreadButton (QWidget * widget, int type, QPixmap pixmap, QString label, QString text, long window, Q_PID pid, bool isRemote ) : QPushButton()
#endif
#ifdef Q_OS_WIN
ThreadButton::ThreadButton (QWidget * widget, int type, QPixmap pixmap, QString label, QString text, long long window, unsigned long pid, bool isRemote ) : QPushButton()
#endif
{
	connect(this, SIGNAL(clicked()), this, SLOT(buttonClicked()));
	setFixedSize( QSize(TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT) );
	setIconSize( QSize(TAB_BUTTON_WIDTH - MARGIN, TAB_BUTTON_WIDTH - MARGIN) );
    this->widget = widget;
	this->type = type;
	this->pixmap = pixmap;
	this->label = label;
	this->text = text;
	this->window = window;
	this->pid = pid;
    this->remote = isRemote;

	setAccessibleName( label );

	selected = false;

	index = -1;
}

void ThreadButton::setSelected( bool /*selected*/ )
{
    //this->selected = selected;
#ifdef Q_OS_WIN
    //forceRepaint();
#else
	repaint();
#endif
}

void ThreadButton::forceRepaint()
{
    QSize _size = size();
    setFixedSize(size().width() - 1, size().height() - 1);
    setFixedSize(_size);
}

QSize ThreadButton::buttonSize()
{
	return QSize( TAB_BUTTON_WIDTH, TAB_BUTTON_HEIGHT );
}

void ThreadButton::buttonClicked()
{
    if ( widget != 0 )
    {
        emit threadButtonClicked(widget);
    } else {
        emit threadButtonClicked(window);
    }
}

void ThreadButton::closeButton()
{
    emit closeThreadButton( index );
}

void ThreadButton::setPixmap(QPixmap pixmap)
{
	this->pixmap = pixmap;
	this->update();
}

void ThreadButton::paintEvent ( QPaintEvent * e)
{
	Q_UNUSED( e );

	QPainter p(this);

	QPixmap pPixmap = pixmap;
	//QBrush brush(Qt::SolidPattern);
	QRadialGradient gradient(50, 50, 50, 50, 50);
	gradient.setColorAt(0, Qt::white);
	gradient.setColorAt(1, Qt::lightGray);
	QBrush brush( gradient );
	brush.setColor( QColor( Qt::lightGray ) );

	if (selected && type != EXITBUTTON)
	{
		//QPushButton::paintEvent( e );
		//p.setBackgroundMode(Qt::OpaqueMode);
		//p.setBackground( brush );
	}

	if ( type == EXITBUTTON )
	{
		if ( isDown() )
		{
			pPixmap =  QPixmap(ICON_DIR_64 + QString(QDir::separator()) + "power-pressed.png");
		}
	}

	if ( ! pPixmap.isNull() )
	{
		int pressedMargin = 0;

		if ( isDown() && type != EXITBUTTON )
		{
			pressedMargin = BUTTON_PRESSED_MARGIN;
		}

		int wMargin = ( TAB_BUTTON_WIDTH - MENU_ICON_SIZE ) / 2;
		int hMargin = ( TAB_BUTTON_HEIGHT - MENU_ICON_SIZE ) / 2;

		QPoint upLeft( wMargin + pressedMargin, hMargin + pressedMargin);
		QPoint downRight( TAB_BUTTON_WIDTH - pressedMargin - wMargin, TAB_BUTTON_HEIGHT - pressedMargin - hMargin );
		QRect rectangle (upLeft, downRight);
		if (selected && type != EXITBUTTON)
		{
			p.fillRect( QRect( QPoint(0,0), size() ), brush );
		}
		p.drawPixmap( rectangle, pPixmap );
	}
}
