#ifndef THREADBUTTON_H
#define THREADBUTTON_H

#include <QPushButton>
#include <QString>
#include <QStringList>
#include <QIcon>
#include <QPixmap>
#include <QPainter>
#include <QFont>
#include <QWidget>
#include <QSize>
#include <QProcess>
#include "constants.h"
#include "application.h"

/*
#ifdef Q_OS_LINUX
#include "x11info.h"
#endif
*/

#ifdef Q_OS_WIN
#include "wininfo.h"
#endif

class ThreadButton : public QPushButton
{
	Q_OBJECT

	public:
#ifdef Q_OS_LINUX
		ThreadButton (QWidget *widget, int type, QPixmap pixmap = QPixmap(), QString label = QString::null, QString text = QString::null, long window = 0, Q_PID pid = 0, bool isRemote = false );
#endif
#ifdef Q_OS_WIN
        ThreadButton (QWidget *widget, int type, QPixmap pixmap = QPixmap(), QString label = QString::null, QString text = QString::null, long long window = 0, unsigned long pid = 0, bool isRemote = false );
#endif
		static QSize buttonSize();
		int getType() { return type; };
		QPixmap getPixmap() { return pixmap; };
		void setPixmap(QPixmap pixmap);
		QString getLabel() { return label; };
		QString getText() { return text; };
		long getWinId() { return window; };
		QWidget * getWidget() { return widget; };
#ifdef Q_OS_LINUX
		Q_PID getPid() { return pid; };
#endif
#ifdef Q_OS_WIN
        unsigned long getPid() { return pid; };
#endif
		int getIndex() { return index; };
		void setIndex(int index) { this->index = index; };
		int isRemoteApp() { return remote; };
		QPixmap getPreview() { return preview; };
		void setPreview(QPixmap preview) { this->preview = preview; };
        void setSelected(bool);

	public slots:
		void closeButton();

	signals:
		void threadButtonClicked( int index );
        void threadButtonClicked( long long winId );
		void threadButtonClicked( QWidget * widget, bool = false );
		void closeThreadButton( int index );

	private slots:
		void buttonClicked();

	private:
		void paintEvent ( QPaintEvent * e);
		void forceRepaint();

		bool selected;
		QWidget * widget;
		QString label;
		QString text;
		QPixmap pixmap;
		QPixmap preview;
		int index;
		int type;
        long long window;
#ifdef Q_OS_LINUX
		Q_PID pid;
#endif
#ifdef Q_OS_WIN
        unsigned long pid;
#endif
		bool remote;
};

#endif
