#include "threadmanager.h"

#include <QDir>

ThreadManager::ThreadManager( TabBar * tabBar, int numberOfMenus, QWidget * parent ) : QWidget( parent )
{
	this->threads = tabBar->getButtonList();
	this->tabBar = tabBar;
	this->numberOfMenus = numberOfMenus;

	QVBoxLayout * layout = new QVBoxLayout;

	grid = new QGridLayout;

	QHBoxLayout * buttons = new QHBoxLayout;
	buttons->setAlignment( Qt::AlignRight );
	buttons->setSpacing(SEPARATION);

	sleepButton = new QPushButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "task-manager.png"), tr("Suspend session"), this);
	sleepButton->setIconSize( QSize(ICON_SIZE, ICON_SIZE) );

	returnButton = new QPushButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "back.png"), tr("Return"), this);
	returnButton->setIconSize( QSize(ICON_SIZE, ICON_SIZE) );

	buttons->addWidget(sleepButton);
	buttons->addWidget(returnButton);

	layout->addLayout( grid );

	QFrame* line = new QFrame();
	line->setFrameShape(QFrame::HLine);
	line->setFrameShadow(QFrame::Sunken);
	layout->addWidget(line);

	layout->addLayout( buttons );

	layout->setStretchFactor( grid, 30 );
	layout->setStretchFactor( buttons, 1 );

	setLayout( layout );

	connect( sleepButton, SIGNAL(clicked()), this, SLOT(disconnect()));
	connect( returnButton, SIGNAL(clicked()), this, SLOT(exit()));

	init();
}

void ThreadManager::init()
{
	if (threads->count() == numberOfMenus)
		close();

	qreal n = (qreal) (threads->count() - numberOfMenus);
	qreal s = qSqrt(n);
	rows = qCeil(s);
	if (rows < 3)
		rows = 3;

	setFixedSize(QApplication::desktop()->screenGeometry().size());

	int counter = 0;

	for (int i = 0; i < rows; i++)
	{	
		grid->setColumnStretch(i, 1);
		grid->setRowStretch(i, 1);
		for ( int j = 0; j < rows; j++)
		{
			if (counter < threads->count() -numberOfMenus )
			{
				ThreadWidget * tw = new ThreadWidget( this, true );
				tw->setThreadButton( threads->at(counter + numberOfMenus) );
				grid->addWidget( tw, i, j );
				connect(tw, SIGNAL(clicked(ThreadButton*)), this, SLOT(setSelected(ThreadButton*)));
			} else {
				grid->addWidget( new QWidget(this), i, j );
			}
			counter++; 
		}
	}
}

void ThreadManager::update(int)
{
	int counter = 0;

	for (int i = 0; i < rows; i++)
	{	
		for ( int j = 0; j < rows; j++)
		{
			if (counter < threads->count() -numberOfMenus )
			{
				ThreadWidget * tw = (ThreadWidget*)grid->itemAtPosition(i, j)->widget();
				grid->removeItem( grid->itemAtPosition(i, j) );
				delete tw;
			} else {
				QWidget * w = grid->itemAtPosition(i, j)->widget();
				grid->removeItem( grid->itemAtPosition(i, j) );
				delete w;
			}
			counter++; 
		}
	}

	init();
}

QWidget * ThreadManager::emptyThread()
{
	QWidget * widget = new QWidget(this);
	widget->setStyleSheet( QString("background-color: #%1").arg(BACKGROUNDCOLOR) );
	widget->setAutoFillBackground ( true );
	return widget;
}

void ThreadManager::closeAllThreads()
{
	for (int i = numberOfMenus; i < threads->count() ; i++)
	{
#ifdef Q_OS_LINUX
		emit 
        if ( threads->at(i)->isRemoteApp() )
	{
		tabBar->closeButtonClicked( threads->at(i) );
	} else {
		tabBar->closeButtonClicked( threads->at(i) );
	}
#endif
	}
}

void ThreadManager::killAllThreads()
{
	for (int i = numberOfMenus; i < threads->count() ; i++)
	{
#ifdef Q_OS_LINUX
        if ( threads->at(i)->isRemoteApp() )
	{
		tabBar->killButtonClicked( threads->at(i) );
	} else {
		tabBar->killButtonClicked( threads->at(i) );
	}
#endif
	}
}

void ThreadManager::disconnect()
{
	QLabel * messageBox = new QLabel(NULL, Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint);
	QFont font;
	font.setBold( true );
	messageBox->setText( tr("The session is being suspended. Please, wait.") );
	messageBox->setFont( font );
	messageBox->setAttribute( Qt::WA_DeleteOnClose, true );
	messageBox->setFixedSize( 400, 200 );
	messageBox->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
	messageBox->show();
	QApplication::processEvents();

	QThread::sleep(1);

	QTimer::singleShot(6000, messageBox, SLOT(close()) );
	QTimer::singleShot(6000, this, SLOT(close()) );
#ifdef Q_OS_LINUX
	QProcess::execute ( XRDP_DIS );
#endif
}

void ThreadManager::exit()
{
	emit finished();
	close();
}

void ThreadManager::setSelected( ThreadButton * button )
{
	close();
	tabBar->setSelected( button );
}
