#include "toolbutton.h"

ToolButton::ToolButton ( const QIcon & icon, const QString & text, QWidget * parent ) : QPushButton( parent )
{
	this->icon = icon;
	setToolTip( text );
	init();
	widget = 0;
	connect(this, SIGNAL(clicked()), this, SLOT(emitClicked()));
}

void ToolButton::addPixmap( const QPixmap & pixmap, QIcon::Mode mode, QIcon::State state )
{
	icon.addPixmap( pixmap, mode, state );
}

void ToolButton::emitClicked()
{
	emit clicked(widget);
}

void ToolButton::init()
{
	setFixedSize( buttonSize() );
}

QSize ToolButton::buttonSize()
{
	return QSize( TOOLBUTTON_SIZE + 2 * TOOLBUTTON_MARGIN, TOOLBUTTON_SIZE + 2 * TOOLBUTTON_MARGIN );
}

void ToolButton::enterEvent ( QEvent * event )
{
	Q_UNUSED( event );
	update();
}

void ToolButton::leaveEvent ( QEvent * event )
{
	Q_UNUSED( event );
	update();
}

void ToolButton::paintEvent ( QPaintEvent * e)
{
	Q_UNUSED( e );
	//if (isDown())
	//	QPushButton::paintEvent(e);

	QBrush brush(Qt::SolidPattern);
	brush.setColor( QColor( Qt::lightGray ) );

	QPainter p(this);

	if (isEnabled() == false)
		p.setOpacity(0.25);
	else
		p.setOpacity(1.0);

	if (!icon.isNull())
	{
		QPoint upLeft(TOOLBUTTON_MARGIN, TOOLBUTTON_MARGIN);
		QPoint downRight(TOOLBUTTON_MARGIN + TOOLBUTTON_SIZE, TOOLBUTTON_MARGIN + TOOLBUTTON_SIZE);
		QRect rectangle (upLeft, downRight);
		if ( isDown() )
			p.fillRect( QRect( QPoint(0,0), size() ), brush );
		if ( underMouse() && isEnabled() )
			p.drawPixmap( rectangle, icon.pixmap(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE, QIcon::Active, QIcon::On) );
		else
        		p.drawPixmap( rectangle, icon.pixmap(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE));

	}
}
