#include "wininfo.h"

void WinInfo::selectWindow(HWND hWnd, bool fullScreen)
{
	if ( IsIconic( hWnd ) )
	{
		ShowWindow( hWnd, SW_RESTORE );
	} else {
		ShowWindow( hWnd, SW_SHOW );
	}
    BringWindowToTop( hWnd );
    if ( Settings::getWindowManagerBehaviour() == FULLSCREEN || fullScreen )
    {
        ShowWindow(hWnd, SW_SHOWMAXIMIZED);
    }
}

void WinInfo::followProc(DWORD processId)
{
    qDebug("Following process %d", (int)processId);
    //SetWinEventHook(EVENT_SYSTEM_END, EVENT_SYSTEM_END, NULL, FollowWindowsProc, NULL /*processId*/, NULL, WINEVENT_OUTOFCONTEXT | WINEVENT_SKIPOWNTHREAD);
}

/*
void CALLBACK FollowWindowsProc(HWINEVENTHOOK hook, DWORD event, HWND hWnd, long idObject, long idChild, DWORD dwEventThread, DWORD dwmsEventTime)
{
    qDebug("Followed event %d on window %d", (int) event, (int) hWnd);
}
*/

QList<HWND> WinInfo::getWindows(QProcess * process)
{
    handleData data;
    data.process = process;
    data.windows.clear();
    data.first = (HWND) -1;
    EnumWindows(EnumWindowsProc, (LPARAM)&data);
    return data.windows;
}

QList<HWND> WinInfo::getVisibleWindows()
{
    handleData data;
    data.process = NULL; // enum all visible windows
    data.windows.clear();
    //data.first = (HWND) -1;
    EnumWindows(EnumWindowsProc, (LPARAM)&data);
    EnumChildWindows(WinInfo::fromId(QApplication::desktop()->winId()), EnumWindowsProc, (LPARAM)&data );
    return data.windows;
}

void WinInfo::debugWindowInfo(HWND hWnd)
{
    qDebug("Detected window: %d '%s' with parent: %d isToolWindow: %d isAppWindow: %d isDlgFrame: %d isChild: %d isPopup: %d isPopupWindow: %d  hasSysmenu: %d isOverlapped: %d, isOverlappedWindow: %d, isVisible: %d",
        (long long)hWnd,
        WinInfo::getWindowTitle(hWnd).toUtf8().data(),
        (long long)WinInfo::getParent(hWnd),
        (int)WinInfo::isToolWindow(hWnd),
        (int)WinInfo::isAppWindow(hWnd),
        (int)WinInfo::isDlgFrame(hWnd),
        (int)WinInfo::isChild(hWnd),
        (int)WinInfo::isPopup(hWnd),
        (int)WinInfo::isPopupWindow(hWnd),
        (int)WinInfo::hasSysmenu(hWnd),
        (int)WinInfo::isOverlapped(hWnd),
        (int)WinInfo::isOverlappedWindow(hWnd),
        (int)WinInfo::isVisible(hWnd)
    );
}

void WinInfo::setWindowsInfo( QList<HWND> * windowsList, QList<long> * styles, QList<long> * exStyles, QList<QString> * titles)
{
    windowsList->clear();
    styles->clear();
    exStyles->clear();
    titles->clear();
    QList<HWND> list = WinInfo::getVisibleWindows();
    for (int i = 0; i < list.count(); i++)
    {
        if ( ! windowsList->contains(list[i]) ) // Don't add duplicates!
        {
            tagWINDOWINFO pwi;
            GetWindowInfo(list[i], &pwi);
            QString title = WinInfo::getWindowTitle( list[i] );
            windowsList->append( list[i] );
            styles->append(  pwi.dwStyle );
            exStyles->append( pwi.dwExStyle );
            titles->append(title);
        }
    }
}

unsigned long WinInfo::getWindowPid(HWND hWnd)
{
    unsigned long processId = 0;
    GetWindowThreadProcessId(hWnd, &processId);
    return processId;
}

BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lparam)
{
    handleData * data = (handleData*)lparam;
    //qDebug("data->first: %d, hWnd: %d, Title: %s", (int)data->first, (int)hWnd, WinInfo::getWindowTitle(hWnd).toUtf8().data());
    /*if (data->first == (HWND) -1)
    {
        data->first = hWnd;
    } else {
        if (data->first == hWnd)
        {
            qDebug("Returning false!!! Count: %d", data->windows.count());
            return false;
        }
    }*/

    if ( data->process != NULL ) // enum windows for that process
    {
        unsigned long processId = 0;
        GetWindowThreadProcessId(hWnd, &processId);
        if (data->process->processId() == processId)
        {
            qDebug("Process found!!! %d", (int)processId);
            data->windows += hWnd;
        }
        return true;
    } else { // enum all  windows
        if ( WinInfo::isVisible(hWnd) )
        {
            data->windows += hWnd;
        }
        return true;
    }
}

QString WinInfo::getWindowTitle(HWND hWnd)
{
    int maxString = 256;
    LPWSTR title = new WCHAR[maxString + 1];
    int result = GetWindowText(hWnd, title, maxString);
    QString titleString = QString::fromWCharArray(title);
    delete title;
    if (result > 0)
    {
        return QString(titleString);
    } else {
        return QString::null;
    }
}

QString WinInfo::getWindowExecName(HWND hWnd)
{
    unsigned long pid = getWindowPid(hWnd);
    LPWSTR execName = new WCHAR[256];
    HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS, TRUE,(DWORD) pid);
    if ( hProcess == NULL)
    {
        delete execName;
        return QString::null;
    }
    int result = GetModuleFileNameEx(hProcess, NULL, execName, 256);
    QString execNameString = QString::fromWCharArray(execName);
    delete execName;
    if (result > 0)
    {
        return QString(execNameString).section("\\", -1, -1);
    } else {
        return QString::null;
    }
}

HWND WinInfo::getParent(HWND hWnd)
{
	return GetParent( hWnd );
}

bool WinInfo::hasNoParent(HWND hWnd)
{
	HWND hParent = GetAncestor(hWnd, GA_PARENT);
	return  ! hParent || hParent == GetDesktopWindow();
}

int WinInfo::isWindowStyle(HWND hWnd, long style)
{
    /*tagWINDOWINFO pwi;
    GetWindowInfo(hWnd, &pwi);
    return pwi.dwStyle & style;*/
    long gwlStyle = GetWindowLong(hWnd, GWL_STYLE);
    return gwlStyle & style;
}

int WinInfo::isWindowExStyle(HWND hWnd, long exStyle)
{
    long gwlExStyle = GetWindowLong(hWnd, GWL_EXSTYLE);
    //qDebug("gwlExStyle: %#06x exStyle: %#06x isExStyle: %d", gwlExStyle, exStyle, gwlExStyle & exStyle );
    return gwlExStyle & exStyle;
}

bool WinInfo::isChild(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_CHILD );
}

bool WinInfo::isDlgFrame(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_DLGFRAME );
}

bool WinInfo::isPopupWindow(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_POPUPWINDOW);
}

bool WinInfo::isPopup(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_POPUP);
}

bool WinInfo::hasSysmenu(HWND hWnd)
{
    return isWindowStyle(hWnd, WS_SYSMENU);
}

bool WinInfo::isOverlapped(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_OVERLAPPED );
}

bool WinInfo::isOverlappedWindow(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_OVERLAPPEDWINDOW);
}
bool WinInfo::isVisible(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_VISIBLE);
}

bool WinInfo::isAppWindow(HWND hWnd)
{
    return isWindowExStyle( hWnd, WS_EX_APPWINDOW);
}

bool WinInfo::isToolWindow(HWND hWnd)
{
    return isWindowExStyle( hWnd, WS_EX_TOOLWINDOW);
}

void WinInfo::closeWinWindow(HWND hWnd)
{
    PostMessage(hWnd, WM_SYSCOMMAND, SC_CLOSE, 0);
}

void WinInfo::killWinWindow(HWND hWnd)
{
    PostMessage(hWnd, WM_QUIT, 0, 0);
}

HWND WinInfo::fromId(long long id)
{
    return (HWND) id;
    /*
    QWindow * window = QWindow::fromWinId((WId)id);
    HWND ww = (HWND) QGuiApplication::platformNativeInterface()->nativeResourceForWindow(QByteArrayLiteral("handle"), window);
    return ww;
    */
}

void WinInfo::attachWindow(QWidget * container, HWND hWnd)
{
    DWORD dwStyle = GetWindowLong( hWnd, GWL_STYLE );
    dwStyle &= ~WS_BORDER;
    dwStyle &= ~WS_THICKFRAME;
    DWORD dwExStyle = GetWindowLong( hWnd, GWL_EXSTYLE );
    dwExStyle |= WS_EX_APPWINDOW;
    QWindow * window = QWindow::fromWinId((long long)hWnd);
    Q_UNUSED( window );
    //SetParent(hWnd, fromId(container->windowHandle()->winId()));
    SetWindowLong( hWnd, GWL_STYLE, dwStyle);
    SetWindowLong( hWnd, GWL_EXSTYLE, dwExStyle);
    MoveWindow( hWnd, 0, 0, container->width(), container->height(), TRUE);
    WinInfo::selectWindow( hWnd );
}

void WinInfo::hideTaskBar()
{
    APPBARDATA barData;
    barData.cbSize = sizeof( barData );
    barData.hWnd = FindWindow( L"System_TrayWnd", NULL);
    barData.lParam = ABS_AUTOHIDE;
    SHAppBarMessage( ABM_SETSTATE, &barData );
}

void WinInfo::showTaskBar()
{
    APPBARDATA barData;
    barData.cbSize = sizeof( barData );
    barData.hWnd = FindWindow( L"System_TrayWnd", NULL);
    barData.lParam = ABS_ALWAYSONTOP;
    SHAppBarMessage( ABM_SETSTATE, &barData );
}

void WinInfo::setWallPaper(LPWSTR image)
{
    int res = SystemParametersInfo( SPI_SETDESKWALLPAPER, 0, image, SPIF_UPDATEINIFILE);
    Q_UNUSED(res);
}

QIcon WinInfo::getWindowIcon(HWND hwnd)
{
    HICON iconHandle = (HICON)SendMessage(hwnd,WM_GETICON,ICON_SMALL2,0);
    if(iconHandle == 0)
        iconHandle = (HICON)SendMessage(hwnd,WM_GETICON,ICON_SMALL,0);
    if(iconHandle == 0)
        iconHandle = (HICON)SendMessage(hwnd,WM_GETICON,ICON_BIG,0);
/*    if (iconHandle == 0)
        iconHandle = (HICON)GetClassLongPtr(hwnd, GCL_HICON);
    if (iconHandle == 0)
        iconHandle = (HICON)GetClassLongPtr(hwnd, GCL_HICONSM);
*/
    if(iconHandle == 0)
        return QIcon();
    //Icon icn = Icon.FromHandle(iconHandle);

    QIcon icon = QtWin::fromHICON(iconHandle);

    return icon;
}

QIcon WinInfo::getAppIcon(QString command)
{
    QIcon icon;
    LPCWSTR wCommand = (LPCWSTR)command.utf16();
    HICON hIcon = ExtractIcon(NULL, (LPCWSTR)wCommand, 0);
    if (hIcon != NULL)
    {
        icon = QtWin::fromHICON( hIcon );
    }
    return icon;
}

QStringList WinInfo::getUsedLogicalDrives()
{
    QStringList used;
    DWORD dwSize = MAX_PATH;
    LPWSTR szLogicalDrives = new WCHAR[MAX_PATH]; // = {0};
    DWORD dwResult = GetLogicalDriveStrings(dwSize, szLogicalDrives);

    if (dwResult > 0 && dwResult <= MAX_PATH)
    {
        LPWSTR szSingleDrive = szLogicalDrives;
        while(*szSingleDrive)
        {
            QString drive = QString::fromWCharArray( szSingleDrive );
            used += drive;
            szSingleDrive += 4;
        }
    }
    delete szLogicalDrives;
	return used;
}

QString WinInfo::getAvailableLogicalDrive()
{
	QStringList used = getUsedLogicalDrives();
    for ( char letter = 'E'; letter <= 'Z'; letter++ )
    {
        QString drive = QChar(letter) + QString(":\\");
        if (! used.contains(drive))
        {
            return QChar(letter) + QString(":");
        }
    }
    return QString::null;
}

void WinInfo::removeRemoteDrive(QString drive)
{
    qDebug("Removing drive %s", drive.toUtf8().data());
    WNetCancelConnection2((const wchar_t*)drive.utf16(), 0, TRUE);
}

QString WinInfo::appendRemoteDrive(QString name, QString _drive)
{
    NETRESOURCE NetResource ;
    memset(&NetResource, 0, sizeof(NetResource)) ;
    NetResource.lpProvider		= NULL ;
    NetResource.dwType			= RESOURCETYPE_ANY;

	QString drive;
	if ( _drive.isEmpty() )
	{
		drive = getAvailableLogicalDrive();
	} else {
		if ( getUsedLogicalDrives().contains( _drive ) )
		{
			drive = getAvailableLogicalDrive();
		} else {
			drive = _drive;
		}
	}
    NetResource.lpLocalName = (LPTSTR) drive.utf16();

	QString fullName;
	if ( name.startsWith("\\\\") ) // Shared folder
	{
		fullName = name;
	} else { // Client drive redirect
		fullName = "\\\\tsclient\\" + name;
	}
    NetResource.lpRemoteName = (LPTSTR) fullName.utf16();

    qDebug("name: %ls, drive: %ls\n", NetResource.lpRemoteName, NetResource.lpLocalName );
	Settings::logInfo( QString("Name: %1 Drive: %2").arg(name).arg(drive) );
    if ( WNetAddConnection2(&NetResource, NULL, NULL, FALSE) != NO_ERROR )
    {
        qDebug("Something failed connecting %s to local drive %s", name.toUtf8().data(), drive.toUtf8().data());
		Settings::logError( QString("Something failed connecting Name: %1 Drive: %2").arg(name).arg(drive) );
		return QString::null;
    }

    return drive;
}

void WinInfo::setHomeDrive(QStringList * names, QStringList * drives, QList<bool> *remove)
{
	QString drive = appendRemoteDrive( Settings::homePath() );

	if ( ! drive.isEmpty() )
	{
		names->append( Settings::homePath() );
		drives->append( drive );
		remove->append( true );
		Settings::setHomePath( drive );
	}
}

void WinInfo::setSharedFoldersDrives(QStringList * names, QStringList * drives, QList<bool> *remove, QHash<QString, QString> assignedDrives)
{
	QHashIterator<QString,QString> it(assignedDrives);
	while (it.hasNext())
	{
		it.next();
		QString drive = appendRemoteDrive( it.key(), it.value() );

		if ( ! drive.isEmpty() )
		{
			names->append(it.key());
			drives->append(it.value());
			remove->append( true );
		}
	}
}

void WinInfo::setRemoteDrives(QStringList * names, QStringList * drives, QList<bool> *remove, QHash<QString, QString> assignedDrives)
{
    HANDLE hEnum;
    WNetOpenEnum(RESOURCE_CONNECTED, RESOURCETYPE_DISK, 0, NULL, &hEnum);

	DWORD cbBuffer = 1048576;
    void *buffer = LocalAlloc(LMEM_FIXED, cbBuffer);
    LPNETRESOURCE pnr = (LPNETRESOURCE)buffer;

    DWORD err;
    do
    {
        DWORD cEntries = INFINITE;
        DWORD cb = cbBuffer;
        err = WNetEnumResource(hEnum, &cEntries, buffer, &cb);
        if (err == NO_ERROR || err == ERROR_MORE_DATA)
        {
            for (DWORD i = 0; i < cEntries; i++)
            {
                if (pnr[i].lpLocalName != NULL)
                {
                    drives->append( QString::fromWCharArray( pnr[i].lpLocalName ) );
                    names->append( QString::fromWCharArray( pnr[i].lpRemoteName ).remove("\\\\TSCLIENT\\", Qt::CaseInsensitive) );
                    remove->append( false );
                }
            }
			Settings::logInfo(QString("Appended existent drives; names: %1 drives: %2").arg(names->join(",")).arg(drives->join(",")));
            for (DWORD i = 0; i < cEntries; i++)
            {
                QString name = QString::fromWCharArray( pnr[i].lpRemoteName ).remove("\\\\TSCLIENT\\", Qt::CaseInsensitive);
				Settings::logInfo(QString("Checking name: %1").arg(name));
				if ( ! names->contains( name, Qt::CaseInsensitive ) )
                {
					QString drive = assignedDrives.value( name.toLower(), QString::null );

					drive = appendRemoteDrive( name, drive );

                    if ( ! drive.isEmpty() )
                    {
                        names->append( name );
                        drives->append( drive );
                        remove->append( true );
						Settings::logInfo(QString("Appending new drive: %1 for resource: %2").arg(drive).arg(name));
					} else {
						Settings::logInfo(QString("Not appending new drive for resource: %2").arg(name));
					}
				} else {
					Settings::logInfo(QString("Not appending drive as it has been appended previously. Name: %1").arg(name));
				}
            }
		} else {
			if ( err == ERROR_NO_MORE_ITEMS )
			{
				Settings::logInfo(QString("No items found enumerating net resource"));
			} else {
				Settings::logInfo(QString("Error enumerating net resource: %1").arg(err));
			}
		}
    } while (err == ERROR_MORE_DATA);

    LocalFree(buffer);
    WNetCloseEnum(hEnum);
}
