#include "winmanager.h"

WinManager::WinManager(Desktop *parent) : QThread( parent )
{
    desktop = parent;
    /*qApp->installNativeEventFilter( this );
    hook = SetWindowsHookExA( WH_CALLWNDPROC, HookCallback, NULL, 0 );
    if ( ! hook )
    {
        qDebug("Cannot set windows hook!");
    }*/
}

WinManager::~WinManager()
{
}

void WinManager::run()
{
    mutex.lock();
    stopThread = false;
    mutex.unlock();

    bool first = true;

    while ( true )
    {
        QList<HWND> newWindowsList;
        QList<long> newStyles;
        QList<long> newExStyles;
        QList<QString> newTitles;

        WinInfo::setWindowsInfo(&newWindowsList, &newStyles, &newExStyles, &newTitles);

        QList<HWND> _notInList = notInList(newWindowsList);
        QList<HWND> _newInList = newInList(newWindowsList);
        QList<HWND> _changedTitleInList = changedTitleInList(newWindowsList, newTitles);
        QList<HWND> _changedAppWindowList = changedStyleList(windowsList, newWindowsList, exStyles, newExStyles, WS_EX_APPWINDOW);
        QList<HWND> _changedVisibleWindowList = changedStyleList(windowsList, newWindowsList, styles, newStyles, WS_VISIBLE);

        if (! first)
        {
            for (int i = 0; i < _notInList.count(); i++)
            {
                emit unmapWindow( (long long) _notInList[i] );
                //qDebug("Window %d removed from windows list", _notInList[i]);
            }
            for (int i = 0; i < _newInList.count(); i++)
            {
                emit mapWindow( (long long) _newInList[i] );
                //WinInfo::debugWindowInfo( _newInList[i] );
                //qDebug("Window %d added to windows list", _newInList[i]);
            }
            for (int i = 0; i < _changedAppWindowList.count(); i++)
            {
                int index = newWindowsList.contains( _changedAppWindowList[i] );
                if (index >= 0)
                {
                    if ( newExStyles[index] & WS_EX_APPWINDOW )
                    {
                        emit mapWindow( (long long) _changedAppWindowList[i] );
                        //qDebug("Window %d added to windows list (added WS_EX_APPWINDOW flag)", _changedAppWindowList[i]);
                    } else {
                        emit unmapWindow( (long long) _changedAppWindowList[i] );
                        //qDebug("Window %d removed from windows list (removed WS_EX_APPWINDOW flag)", _changedAppWindowList[i]);
                    }
                }
            }
            for (int i = 0; i < _changedVisibleWindowList.count(); i++)
            {
                int index = newWindowsList.contains( _changedVisibleWindowList[i] );
                if (index >= 0)
                {
                    if ( newStyles[index] & WS_VISIBLE )
                    {
                        emit mapWindow( (long long) _changedVisibleWindowList[i] );
                        //qDebug("Window %d added to windows list (added WS_VISIBLE flag)", _changedVisibleWindowList[i]);
                    } else {
                        emit unmapWindow( (long long) _changedVisibleWindowList[i] );
                        //qDebug("Window %d removed from windows list (removed WS_VISIBLE flag)", _changedVisibleWindowList[i]);
                    }
                }
            }
            for (int i = 0; i < _changedTitleInList.count(); i++)
            {
                QString newTitle = WinInfo::getWindowTitle(_changedTitleInList[i]);
                emit titleChanged( (long long) _changedTitleInList[i], newTitle );
            }
        } else {
            first = false;
        }

        windowsList = newWindowsList;
        styles = newStyles;
        exStyles = newExStyles;
        titles = newTitles;
        QThread::msleep(WINMANAGER_TIMER);
        mutex.lock();
        if (stopThread)
        {
            mutex.unlock();
            break;
        } else {
            mutex.unlock();
        }
    }
}

bool WinManager::isAppWindow(long window)
{
    HWND hWnd = WinInfo::fromId(window);
	if ( WinInfo::hasNoParent(hWnd) && ( WinInfo::isAppWindow(hWnd) || ( WinInfo::isPopupWindow(hWnd) && ! WinInfo::isToolWindow(hWnd) ) ) )
    {
        return true;
    } else {
        return false;
    }
}

void WinManager::showWindow(ThreadButton * button)
{
    long window = button->getWinId();
    HWND hWnd = WinInfo::fromId( window );
    WinInfo::selectWindow( hWnd );
}

void WinManager::showWindow(long window, bool fullScreen)
{
    HWND hWnd = WinInfo::fromId( window );
	WinInfo::selectWindow( hWnd, fullScreen );
}

QList<HWND> WinManager::notInList(QList<HWND> newList)
{
    QList<HWND> res;
    for (int i = 0; i < windowsList.count(); i++)
    {
        if ( ! newList.contains(windowsList[i]) )
        {
            res += windowsList[i];
        }
    }
    return res;
}

QList<HWND> WinManager::newInList(QList<HWND> newList)
{
    QList<HWND> res;
    for (int i = 0; i < newList.count(); i++)
    {
        if ( ! windowsList.contains(newList[i]) )
        {
            res += newList[i];
        }
    }
    return res;
}

QList<HWND> WinManager::changedTitleInList(QList<HWND> newList, QList<QString> newTitles)
{
    QList<HWND> res;
    for (int i = 0; i < newList.count(); i++)
    {
        if ( windowsList.contains(newList[i]) )
        {
            int index = windowsList.indexOf(newList[i]);
            if (newTitles[i] != titles[index])
            {
                res += newList[i];
            }
        }
    }
    return res;
}

QList<HWND> WinManager::changedStyleList(QList<HWND> oldList, QList<HWND> newList, QList<long> oldStyles, QList<long> newStyles, long style)
{
    QList<HWND> res;
    for (int i = 0; i < newList.count(); i++)
    {
        if ( oldList.contains(newList[i]) )
        {
            int index = oldList.indexOf(newList[i]);
            bool isNewStyle = newStyles[i] & style;
            bool isOldStyle = oldStyles[index] & style;
            if ( isNewStyle != isOldStyle )
            {
                    res += newList[i];
            }
        }
    }
    return res;
}

void WinManager::stop()
{
    mutex.lock();
    stopThread = true;
    mutex.unlock();
}
/*
LRESULT __stdcall HookCallback(int nCode, WPARAM wParam, LPARAM lParam )
{
    qDebug("HookCallback wParam: %d", (int)wParam);
    return CallNextHookEx( hook, nCode, wParam, lParam );
}
*/

bool WinManager::nativeEventFilter(const QByteArray & eventType, void * message, long * result)
{
    Q_UNUSED( result );
    Q_UNUSED( eventType );
    Q_UNUSED( message );
    /*
    //return QAbstractNativeEventFilter::nativeEventFilter(eventType, message, result);
    Q_UNUSED(result);
    if ( eventType == "windows_generic_MSG" )
    {
        MSG * eventMessage = (MSG*)message;
        //qDebug("Windows message: %04x", eventMessage->message);

        //return SendNotifyMessage(eventMessage->hwnd, eventMessage->message, eventMessage->wParam, eventMessage->lParam);
        //return PostMessage(eventMessage->hwnd, eventMessage->message, eventMessage->wParam, eventMessage->lParam);
        if ((long)eventMessage->hwnd == currentWindow)
        {
            qDebug("Found event on current window! Event: %d, Window: %d", (int)eventMessage->message, (int)eventMessage->hwnd);
        }

        switch (eventMessage->message)
        {
            case WM_DESTROY:
                //qDebug("Destroy window %d", (int)eventMessage->hwnd);
                //PostQuitMessage(eventMessage->message);
                break;
            case WM_NCDESTROY:
                //qDebug("NCDestroy window %d", (int)eventMessage->hwnd);
                //PostQuitMessage(eventMessage->message);
                break;
            case WM_CLOSE:
                //qDebug("Close window %d", (int)eventMessage->hwnd);
                break;
            case WM_QUIT:
                //qDebug("Quit window %d", (int)eventMessage->hwnd);
                break;
            default:
                //qDebug("Unknown event on window %d", (int)eventMessage->hwnd);
                //TranslateMessage(eventMessage);
                //return DispatchMessage(eventMessage);
                //return DefWindowProc(eventMessage->hwnd, eventMessage->message, eventMessage->wParam, eventMessage->lParam);
                //qDebug() << "Redirecting windows message";
                //PostMessageW(eventMessage->hwnd, eventMessage->message, eventMessage->wParam, eventMessage->lParam);
                //DispatchMessageA(eventMessage);
                break;
        }
    } else if ( eventType == "windows_dispatcher_MSG" ) {
        MSG * eventMessage = (MSG*)message;
        if ((long)eventMessage->hwnd == currentWindow)
        {
            qDebug("Found dispatch event on current window! Event: %d, Window: %d", (int)eventMessage->message, (int)eventMessage->hwnd);
        }
    } else {
        qDebug("Event type: %s", eventType.data());
    }
    */
    return false;
}

long WinManager::getWindow(QProcess *process)
{
    WinInfo::followProc(process->processId());
    QTimer timer;
    timer.setInterval(TIMEOUT * 1000);
    timer.start();
    QList<HWND> list;
    int selected = -1;
    while ( timer.isActive() && selected == -1 )
    {
        list = WinInfo::getWindows(process);

        if ( list.count() > 0 )
        {
            for ( int i = 0; i < list.count(); i++ )
            {
                //WinInfo::debugWindowInfo(list[i]);
                //qDebug("Window title: %s", WinInfo::getWindowTitle(list[i]).toUtf8().data());

                if ( WinInfo::isVisible( list[i]) || ( WinInfo::hasSysmenu( list[i] ) && ! WinInfo::getWindowTitle( list[i] ).isEmpty() ) )
                {
                    if ( selected == -1 || WinInfo::isOverlappedWindow( list[i] ) || WinInfo::hasSysmenu( list[i] ) )
                    {
                        selected = i;
                    }
                }
            }
        }
    }

    timer.stop();

    if ( selected != -1 )
    {
        //qDebug("Returning selected window %d", selected);
        currentWindow = (long long)list[selected];
        return (long long)list[selected];
    } else {
        //qDebug("Returning no window");
        return 0;
    }
}

void WinManager::resize(QRect qRect)
{
    qDebug("Resizing to %dx%d", qRect.width(), qRect.height());
    // Create work area
    RECT rect;
    rect.top = qRect.top();
    rect.bottom = qRect.bottom();
    rect.left = qRect.left();
    rect.right = qRect.right();
    bool result = SystemParametersInfo(SPI_SETWORKAREA, 0, (PVOID*)&rect, SPIF_SENDCHANGE);
    if (!result)
    {
        qDebug() <<  "Cannot change work area";
    } else {
        LPCWSTR className = (LPCWSTR)"Shell_TrayWnd";
        HWND lHwnd = FindWindow( className, NULL);
        LPARAM Zero = 0;
        SendMessage(lHwnd, WM_COMMAND, MIN_ALL, Zero);
        QThread::msleep(10);
        SendMessage(lHwnd, WM_COMMAND, MIN_ALL_UNDO, Zero);
        QThread::msleep(10);
    }
}
