#ifndef WINMANAGER_H
#define WINMANAGER_H

#include <windows.h>

#include <QObject>
#include <QRect>
#include <QThread>
#include <QMutex>
#include <QDebug>
#include <QApplication>
#include <QAbstractNativeEventFilter>
#include <QByteArray>
#include <QList>

#include "desktop.h"
#include "constants.h"

class Desktop;

#define MIN_ALL 419
#define MIN_ALL_UNDO 416

class WinManager : public QThread, public QAbstractNativeEventFilter
{
    Q_OBJECT

    public:
        WinManager(Desktop * parent = 0);
        ~WinManager();

        bool nativeEventFilter(const QByteArray & eventType, void * message, long * result);
        long getWindow(QProcess * process);

        static bool isAppWindow(long window);
        static void showWindow(ThreadButton * button);
		static void showWindow(long window, bool fullScreen = false);

    signals:
        void titleChanged( long long window, QString newTitle );
        void unmapWindow( long long window );
        void mapWindow( long long window );

    public slots:
        void resize(QRect rect);
        void stop();

    private:
        void run();
        QList<HWND> notInList(QList<HWND> newList);
        QList<HWND> newInList(QList<HWND> newList);
        QList<HWND> changedTitleInList(QList<HWND> newList, QList<QString> newTitles);
        QList<HWND> changedStyleList(QList<HWND> oldList, QList<HWND> newList, QList<long> oldStyles, QList<long> newStyles, long style);

        Desktop * desktop;

        QMutex mutex;
        bool stopThread;

        QList<HWND> windowsList;
        QList<long> styles;
        QList<long> exStyles;
        QList<QString> titles;
};

static long currentWindow = -1;
//static HHOOK hook;
//LRESULT __stdcall HookCallback(int nCode, WPARAM wParam, LPARAM lParam );

#endif // WINMANAGER_H
