#!/bin/bash

DROPBOXDIR=$1
SUFFIX=-dropbox.ini

getName()
{
	ACCOUNT=$1

	NAME=`basename "${ACCOUNT}" ${SUFFIX}`x
	NAME=${NAME:1:-1}

	echo ${NAME}
}

start()
{
	set +m
	shopt -s lastpipe

	if ! [ -d "${DROPBOXDIR}" ]
	then
		mkdir -p "${DROPBOXDIR}"
	fi

	cd "${DROPBOXDIR}"

	ACCOUNTS=`ls -a "${DROPBOXDIR}" | grep "^\..*\${SUFFIX}$"`

	FAILED=1

	if [ -n "${ACCOUNTS}" ]
	then
		FAILED=1
		echo "${ACCOUNTS}" | while read ACCOUNT
		do
			NAME=`getName "${ACCOUNT}"`
			if ! [ -d "${NAME}" ]
			then
				mkdir "${DROPBOXDIR}/${NAME}"
			fi

			lwcloudfs -s -o allow_other --fstype=dropbox --credsfile="${ACCOUNT}" "${DROPBOXDIR}/${NAME}"

			if [ $? == 0 ]
			then
				FAILED=0
			fi
		done
	fi

	exit $FAILED
}

stop()
{
	if [ -d "${DROPBOXDIR}" ]
	then
		cd "${DROPBOXDIR}"

		ACCOUNTS=`ls -a "${DROPBOXDIR}" | grep "^\..*\${SUFFIX}$"`

		if [ -n "${ACCOUNTS}" ]
		then
			echo "${ACCOUNTS}" | while read ACCOUNT
			do
				NAME=`getName "${ACCOUNT}"`
				if [ -d "${DROPBOXDIR}/${NAME}" ]
				then
					fusermount -u "${DROPBOXDIR}/${NAME}"
				fi
			done
		fi
	fi
}

list()
{
	if [ -d "${DROPBOXDIR}" ]
	then
		cd "${DROPBOXDIR}"

		ACCOUNTS=`ls -a "${DROPBOXDIR}" | grep "^\..*\${SUFFIX}$"`

		if [ -n "${ACCOUNTS}" ]
		then
			echo "${ACCOUNTS}" | while read ACCOUNT
			do
				NAME=`getName "${ACCOUNT}"`
				echo $NAME
			done
		fi
	fi
}

configure()
{
	NAME=$1

	mkdir -p "${DROPBOXDIR}/${NAME}"

	python -u /usr/bin/lwcloudfs --fstype=dropbox --credsfile="${DROPBOXDIR}/.${NAME}${SUFFIX}" --authorize=console

	if [ $? != 0 ]
	then
		rmdir "${DROPBOXDIR}/${NAME}"
		rm -f "${DROPBOXDIR}/.${NAME}${SUFFIX}"
	else
		lwcloudfs -s -o allow_other --fstype=dropbox --credsfile="${DROPBOXDIR}/.${NAME}${SUFFIX}" "${DROPBOXDIR}/${NAME}"
	fi
}

unconfigure()
{
	NAME=$1

	if [ -d "${DROPBOXDIR}/${NAME}" ]
	then
		fusermount -u "${DROPBOXDIR}/${NAME}"
		rm -f "${DROPBOXDIR}/.${NAME}${SUFFIX}"
		rmdir "${DROPBOXDIR}/${NAME}"
	fi
}

usage()
{
	echo "Usage: $0 <Dropbox Dir> {start|stop|list|configure <acount name>|unconfigure <account name>|unconfigure}" >&2
	echo "Example: $0 /home/user/Dropbox configure myAccount" >&2
	exit 1
}

case "$2" in
	start)
		if [ -z "$3" ]
		then
			start
		else
			usage
		fi
		;;
	stop)
		if [ -z "$3" ]
		then
			stop
		else
			usage
		fi
		;;
	list)
		if [ -z "$3" ]
		then
			list
		else
			usage
		fi
		;;
	configure)
		if [ -n "$3" ] && [ -z "$4" ]
		then
			configure "$3"
		else
			usage
		fi
		;;
	unconfigure)
		if [ -n "$3" ] && [ -z "$4" ]
		then
			unconfigure "$3"
		else
			usage
		fi
		;;
	*)
		usage
		;;
esac

exit 0
