#!/bin/bash

GDRIVEDIR=$1
SUFFIX=.json

getName()
{
	ACCOUNT=$1

	NAME=`basename "${ACCOUNT}" ${SUFFIX}`x
	NAME=${NAME:1:-1}

	echo ${NAME}
}

start()
{
	set +m
	shopt -s lastpipe

	if ! [ -d "${GDRIVEDIR}" ]
	then
		mkdir -p "${GDRIVEDIR}"
	fi

	cd "${GDRIVEDIR}"

	ACCOUNTS=`ls -a "${GDRIVEDIR}" | grep "^\..*\${SUFFIX}$"`

	FAILED=1

	if [ -n "${ACCOUNTS}" ]
	then
		FAILED=1
		echo "${ACCOUNTS}" | while read ACCOUNT
		do
			NAME=`getName "${ACCOUNT}"`
			if ! [ -d "${NAME}" ]
			then
				mkdir "${GDRIVEDIR}/${NAME}"
			fi

			lwcloudfs -s -o allow_other --fstype=googledrive --credsfile="${ACCOUNT}" "${GDRIVEDIR}/${NAME}"

			if [ $? == 0 ]
			then
				FAILED=0
			fi
		done
	fi

	exit $FAILED
}

stop()
{
	if [ -d "${GDRIVEDIR}" ]
	then
		cd "${GDRIVEDIR}"

		ACCOUNTS=`ls -a "${GDRIVEDIR}" | grep "^\..*\${SUFFIX}$"`

		if [ -n "${ACCOUNTS}" ]
		then
			echo "${ACCOUNTS}" | while read ACCOUNT
			do
				NAME=`getName "${ACCOUNT}"`
				if [ -d "${GDRIVEDIR}/${NAME}" ]
				then
					fusermount -u "${GDRIVEDIR}/${NAME}"
				fi
			done
		fi
	fi
}

list()
{
	if [ -d "${GDRIVEDIR}" ]
	then
		cd "${GDRIVEDIR}"

		ACCOUNTS=`ls -a "${GDRIVEDIR}" | grep "^\..*\${SUFFIX}$"`

		if [ -n "${ACCOUNTS}" ]
		then
			echo "${ACCOUNTS}" | while read ACCOUNT
			do
				NAME=`getName "${ACCOUNT}"`
				echo $NAME
			done
		fi
	fi
}

configure()
{
	NAME=$1

	mkdir -p "${GDRIVEDIR}/${NAME}"

	python -u /usr/bin/lwcloudfs --fstype=googledrive --credsfile="${GDRIVEDIR}/.${NAME}${SUFFIX}" --authorize=console

	if [ $? != 0 ]
	then
		rmdir "${GDRIVEDIR}/${NAME}"
		rm -f "${GDRIVEDIR}/.${NAME}${SUFFIX}"
	else
		lwcloudfs -s -o allow_other --fstype=googledrive --credsfile="${GDRIVEDIR}/.${NAME}${SUFFIX}" "${GDRIVEDIR}/${NAME}"
	fi
}

unconfigure()
{
	NAME=$1

	if [ -d "${GDRIVEDIR}/${NAME}" ]
	then
		fusermount -u "${GDRIVEDIR}/${NAME}"
		rm -f "${GDRIVEDIR}/.${NAME}${SUFFIX}"
		rmdir "${GDRIVEDIR}/${NAME}"
	fi
}

usage()
{
	echo "Usage: $0 <Google Drive Dir> {start|stop|list|configure <acount name>|unconfigure <account name>|unconfigure}" >&2
	echo "Example: $0 /home/user/GoogleDrive configure myAccount" >&2
	exit 1
}

case "$2" in
	start)
		if [ -z "$3" ]
		then
			start
		else
			usage
		fi
		;;
	stop)
		if [ -z "$3" ]
		then
			stop
		else
			usage
		fi
		;;
	list)
		if [ -z "$3" ]
		then
			list
		else
			usage
		fi
		;;
	configure)
		if [ -n "$3" ] && [ -z "$4" ]
		then
			configure "$3"
		else
			usage
		fi
		;;
	unconfigure)
		if [ -n "$3" ] && [ -z "$4" ]
		then
			unconfigure "$3"
		else
			usage
		fi
		;;
	*)
		usage
		;;
esac

exit 0
