#!/bin/bash

GDRIVEDIR=$1
SUFFIX=.json

ID=1088235002695-u0iutihaupnnlu9spk5vmovp2n1o8ehh.apps.googleusercontent.com
SECRET=NRZlG6AYVEKwub8CsXH9B2HU

CONFDIR=$HOME/.gdfuse

start()
{
	set +m
	shopt -s lastpipe

	if ! [ -d "${GDRIVEDIR}" ]
	then
		mkdir -p "${GDRIVEDIR}"
	fi

	cd "${GDRIVEDIR}"

	ACCOUNTS=`ls "${CONFDIR}"`

	FAILED=1

	if [ -n "${ACCOUNTS}" ]
	then
		FAILED=1
		echo "${ACCOUNTS}" | while read ACCOUNT
		do
			if ! [ -d "${ACCOUNT}" ]
			then
				mkdir "${GDRIVEDIR}/${ACCOUNT}"
			fi

			google-drive-ocamlfuse -headless -id ${ID} -secret ${SECRET} -label "${ACCOUNT}" "${GDRIVEDIR}/${ACCOUNT}"

			if [ $? == 0 ]
			then
				FAILED=0
			fi
		done
	fi

	exit $FAILED
}

stop()
{
	if [ -d "${GDRIVEDIR}" ]
	then
		cd "${GDRIVEDIR}"

		ACCOUNTS=`ls "${CONFDIR}"`

		if [ -n "${ACCOUNTS}" ]
		then
			echo "${ACCOUNTS}" | while read ACCOUNT
			do
				if [ -d "${GDRIVEDIR}/${ACCOUNT}" ]
				then
					fusermount -u "${GDRIVEDIR}/${ACCOUNT}"
				fi
			done
		fi
	fi
}

list()
{
	if [ -d "${GDRIVEDIR}" ]
	then
		cd "${GDRIVEDIR}"

		ACCOUNTS=`ls "${CONFDIR}"`

		if [ -n "${ACCOUNTS}" ]
		then
			echo "${ACCOUNTS}" | while read ACCOUNT
			do
				echo ${ACCOUNT}
			done
		fi
	fi
}

configure()
{
	NAME=$1

	mkdir -p "${GDRIVEDIR}/${NAME}"

	google-drive-ocamlfuse -headless -id ${ID} -secret ${SECRET} -label "${NAME}"

	if [ $? != 0 ]
	then
		rmdir "${GDRIVEDIR}/${NAME}"
		rm -f "${GDRIVEDIR}/.${NAME}${SUFFIX}"
	else
		google-drive-ocamlfuse -headless -id ${ID} -secret ${SECRET} -label "${NAME}" "${GDRIVEDIR}/${NAME}"
	fi
}

unconfigure()
{
	NAME=$1

	if [ -d "${GDRIVEDIR}/${NAME}" ]
	then
		fusermount -u "${GDRIVEDIR}/${NAME}"
		rm -f "${GDRIVEDIR}/${NAME}"
		rm -rf "${CONFDIR}/${NAME}"
	fi
}

usage()
{
	echo "Usage: $0 <Google Drive Dir> {start|stop|list|configure <acount name>|unconfigure <account name>|unconfigure}" >&2
	echo "Example: $0 /home/user/GoogleDrive configure myAccount" >&2
	exit 1
}

case "$2" in
	start)
		if [ -z "$3" ]
		then
			start
		else
			usage
		fi
		;;
	stop)
		if [ -z "$3" ]
		then
			stop
		else
			usage
		fi
		;;
	list)
		if [ -z "$3" ]
		then
			list
		else
			usage
		fi
		;;
	configure)
		if [ -n "$3" ] && [ -z "$4" ]
		then
			configure "$3"
		else
			usage
		fi
		;;
	unconfigure)
		if [ -n "$3" ] && [ -z "$4" ]
		then
			unconfigure "$3"
		else
			usage
		fi
		;;
	*)
		usage
		;;
esac

exit 0
