#!/bin/bash

GRIVEDIR=$1
COMMAND=$2

PID=$$

export LC_ALL=C

function usage()
{
	echo "Usage: grive.sh <basedir> <command>"
	echo "Commands:"
	echo "    update - Updates all configured accounts in <basedir>"
	echo "    list   - Lists all configured accounts in <basedir>"
	exit 1
}

if [ -z "${COMMAND}" ]
then
	usage
fi

if [ -n "$3" ]
then
	usage
fi

if [ -e "${GRIVEDIR}/.grive" ]
then
	mkdir -p "${GRIVEDIR}/default"
	for i in "${GRIVEDIR}"/*
	do
		if [ "$i" != "${GRIVEDIR}/default" ]
		then
			mv "$i" "${GRIVEDIR}/default"
		fi
	done
	mv "${GRIVEDIR}/.grive" "${GRIVEDIR}/default"
	if [ -e "${GRIVEDIR}/.grive_state" ]
	then
		mv "${GRIVEDIR}/.grive_state" "${GRIVEDIR}/default"
	fi
fi

if [ "$COMMAND" == "update" ]
then
	EXISTS=false
	if [ -e "${GRIVEDIR}/.grive.pid" ]
	then
		LASTPID=`cat "${GRIVEDIR}/.grive.pid"`
		ISRUNNING=`ps -ef | awk '{print $2}' | grep -w "$LASTPID"`

		if [ -z "${ISRUNNING}" ]
		then
			rm -f ${GRIVEDIR}/.grive.pid
		else
			echo "Another instance of grive.sh is running. Exiting."
			exit 0
		fi
	fi

	echo $$ > "${GRIVEDIR}/.grive.pid"

	cd "${GRIVEDIR}"

	for i in *
	do
		if [ -e "$i/.grive" ]
		then
			EXISTS=true
			grive -p "${GRIVEDIR}/$i"
		fi
	done

	if [ "${EXISTS}" == "false" ]
	then
		echo "No accounts defined"
	fi

	rm -f "${GRIVEDIR}/.grive.pid"
fi

if [ "$COMMAND" == "list" ]
then
	for i in "${GRIVEDIR}"/*
	do
		if [ -e "$i/.grive" ]
		then
			echo `basename "$i"`
		fi
	done
fi
