#!/usr/bin/env python

import stat
import os
import sys
import errno
import ConfigParser
import webbrowser
import fuse

import filesystem
import fsstat

from dropbox_api import *

import json
from datetime import datetime
from dateutil import parser
from cache import Cache

CACHETTL=120

from logger import create_logger
_logger=create_logger('lwcloudfs.dropbox')

APP_KEY='xtwvbwftzedke70'
APP_SECRET='oyhgesy6c8t4azo'

class Dropbox(filesystem.FileSystem):
	def __init__( self ):
		filesystem.FileSystem.__init__(self)
		self.cacheJsonData = Cache(CACHETTL)
		self.cacheId = Cache(CACHETTL)

	def init( self, credentialsFile ):
		client.DropboxOAuth2FlowNoRedirect(APP_KEY, APP_SECRET)

		try:
			config = ConfigParser.ConfigParser()
			accessToken = ""
			if len( config.read( credentialsFile ) ) > 0:
				accessToken = config.get( 'client', 'access_token' )

			if len( accessToken ) == 0:
				print "Error: invalid credentials file"
				exit(4)
		except ConfigParser.MissingSectionHeaderError:
			print "Error: invalid credentials file. No valid header."
			exit(4)

		self.client = client.DropboxClient(accessToken)

	def umount( self ):
		_logger.debug('Dropbox: umount()')

	def authorize( self, type, credentialsFile ):
		accessToken = None
		flow = client.DropboxOAuth2FlowNoRedirect(APP_KEY, APP_SECRET)

		auth_uri = flow.start()
		if type == "console":
			print("Open this URL accepting access to LibreWork CloudFS: \n\n" + auth_uri + "\n")

		if type == "browser":
			webbrowser.open(auth_uri)

		code = raw_input('Code from Dropbox: ')
		try:
			print("")
			accessToken, userId = flow.finish(code)
		except:
			print("\nError: The code does not seem to be valid.\n")
			exit(5)
		config = ConfigParser.RawConfigParser()
		config.add_section('client')
		config.set('client', 'access_token', accessToken)
		with open(credentialsFile, 'wb') as configfile:
			config.write(configfile)
			configfile.close()

		if accessToken is None:
			print "Error creating credentials"
			exit(6)
		else:
			print "Successfully created credentials file " + credentialsFile
			exit(0)

	def downloadFile( self, fileName, localFile ):
		_logger.debug('Dropbox: downloadFile(%s)', fileName)
		try:
			f = self.client.get_file( fileName )
			out = open( localFile, 'w' )
			out.write(f.read())
			out.close()
		except:
			pass

	def uploadFile( self, path, localFile ):
		_logger.debug('Dropbox: uploadFile(%s)', path)
		f = open( localFile, 'r' )
		try:
			self.client.put_file(path, f, True)
		except:
			pass
		f.close()
		self.cacheId.remove( path )
		_logger.debug('Dropbox: File(%s) uploaded', path)

	def createDir( self, dirName ):
		_logger.debug('Dropbox: createDir(%s)', dirName)
		try:
			self.client.file_create_folder( dirName )
		except rest.ErrorResponse:
			print 'An error occured'

	def rename( self, oldPath, newPath ):
                _logger.debug('Dropbox: rename(%s to %s)', oldPath, newPath)
		try:
			jsonData = self.client.file_move(oldPath, newPath) 
			self.cacheJsonData.remove( oldPath )
			self.cacheJsonData.remove( newPath )
			self.cacheJsonData.set( newPath, jsonData )
			self.cacheId.remove( oldPath )
			self.cacheId.remove( newPath )
                	_logger.debug('Dropbox: File renamed (%s to %s)', oldPath, newPath)
			return True
		except rest.ErrorResponse:
                	_logger.debug('Dropbox: An error ocurred renaming %s to %s', oldPath, newPath)
			print 'An error occured'
			return False

	def remove( self, path ):
		_logger.debug('Dropbox: remove(%s)', path)
		try:
			self.client.file_delete( path )
		except Exception,e:
			_logger.debug('An error occurred: %s'%str(e))
		self.cacheJsonData.remove( path )
		self.cacheId.remove( path )

	def statFs( self ):
		st = fuse.StatVfs()
		st.f_bsize = 1
		st.f_frsize = 1
		info = self.client.account_info()
		total = info['quota_info']['quota']
		used = info['quota_info']['normal'] + info['quota_info']['shared']
		st.f_blocks = total
		st.f_bfree = total - used
		st.f_bavail = total - used
		return st

	def getStat( self, path ):
		_logger.debug('Dropbox: getStat(%s)', path)

		_logger.debug('Dropbox: getStat: Retrieving jsonData for path %s', path)

		jsonData = self.retrieveJsonData( path )

		if jsonData == None:
			_logger.debug( "File %s not exists", path )
			return -errno.ENOENT
		else:
			_logger.debug( "File %s exists", path )
			_logger.debug( "Dropbox: returning stat from json" )
			return self.getStatFromJsonData( jsonData )

	def getStatFromJsonData( self, jsonData ):
		_logger.debug('Dropbox: getStat: jsonData retrieved')
		st = fsstat.FSStat()

		st.st_uid = os.getuid()
		st.st_gid = os.getgid()

		if jsonData == None:
			_logger.debug('Dropbox: getStat: jsonData is null')
			st.st_mode = stat.S_IFREG | 0644
			st.st_nlink = 1
			st.st_size = 0
			return st

		_logger.debug('Dropbox: getStat: jsonData not null')

		if jsonData['is_dir']:
			st.st_mode = stat.S_IFDIR | 0755
			st.st_nlink = 2
			st.st_size = 4096
		else:
			st.st_mode = stat.S_IFREG | 0644
			st.st_nlink = 1
			st.st_size = jsonData['bytes']
		st.st_ctime = self.getSeconds( jsonData['modified'].encode('utf8') )
		st.st_mtime = self.getSeconds( jsonData['modified'].encode('utf8') )
		st.st_atime = self.getSeconds( jsonData['modified'].encode('utf8') )
		return st

	def exists( self, path ):
		if self.retrieveJsonData( path ) != None:
			return True
		else:
			return False

	def getSeconds( self, myDateTime ):
		dateTime = parser.parse( myDateTime )
		dateFrom = parser.parse( "1970-01-01 00:00:00 +0000" )
		delta = dateTime - dateFrom
		return delta.days * 24 * 3600 + delta.seconds
			
	def getDirContents( self, dirName ):
		res = [ '.', '..' ]

		try:
			jsonData = self.retrieveAllFiles( dirName )
			data = json.loads(json.dumps(jsonData))
			contents = data['contents']
			for _file in contents:
				path = _file['path'].encode('utf8')
				_logger.debug('Caching data for path %s', path)
				self.cacheJsonData.set( path, _file )
				res.append( self.fileName( path ) )
		except:
			_logger.debug('Error: %s', sys.exc_info()[0] )

		return res

	def retrieveAllFiles( self, path ):
		""" Retrieve a list of File resources.
		Args:
			path: the parent dir
		Returns:
			List of File resources.
		"""
		_logger.debug('Dropbox: retrieveAllFiles(%s)', path)
		folderMetadata = self.client.metadata( path )
		return folderMetadata

	def retrieveJsonData(self, path):
		_logger.debug('Dropbox: retrieveJsonData(%s)', path)
		if self.cacheJsonData.get( path ) != None:
			_logger.debug('Dropbox: jsonData(%s) retrieved from cache', path)
			jsonData = self.cacheJsonData.get( path )
		else:
			try:
				jsonData = self.client.metadata( path )
				if 'is_deleted' in jsonData:
					if jsonData['is_deleted']:
						jsonData = None
			except:
				_logger.debug( 'Dropbox: retrieveJsonData: No JSON data for path %s', path )
				jsonData = None
		return jsonData

	def retrieveId( self, path ):
		_id = self.cacheId.get( path )
		if _id != None:
			return _id

		if self.exists( path ):
			_id = self.retrieveNameHash( path )
		else:
			_id = None
		self.cacheId.set( path, _id )
		return _id

	def fileName( self, path ):
		_tuple = path.rpartition( "/" )
		return _tuple[2]
