#! /usr/bin/env python
# -*- coding: utf-8 -*-

from lwcloudfs import *
import fuse
import sys

BLUE = '\033[94m'
RED = '\033[91m'
ENDC = '\033[0m'
GREEN = '\033[92m'

def done():
	print GREEN + "Done.\n" + ENDC

def fail():
	print RED + "Test Failed" + ENDC
	exit(1)

def do(myString):
	print BLUE + myString + "..." + ENDC

def main():
        __version__ = "0.1"
        usage = """
LibreWork cloud file system test program

""" + fuse.Fuse.fusage

        server = LWCloudFS(version = 'LibreWork Cloud File System - LWCloudFS version: ' + __version__,
                                usage = usage,
                                dash_s_do='setsingle')

	do("Mounting filesystem")
	print server.fuse_args.mountpoint
	done()

	do ("Getting FS stats")
	st = server.statfs()
	print "Total: " + str(st.f_blocks / 1048576) + " MB, Available: " + str(st.f_bavail / 1048576) + " MB"
	done()

	do("Accessing file system")
	if server.access( "/", 0 ) < 0:
		fail()
	else:
		done()

	do("Listing file system root contents")
	print "\n".join(server.fs.getDirContents( "/" ))
	done()

	TESTDIR="Lw cloudfsá.testdir"
	do("Creating directory /" + TESTDIR)
	server.mkdir("/" + TESTDIR, 0600)
	if TESTDIR in server.fs.getDirContents( "/" ):
		done()
	else:
		fail()

	do ("Removing directory /" + TESTDIR)
	server.rmdir( "/" + TESTDIR )
	if TESTDIR in server.fs.getDirContents( "/" ):
		fail()
	else:
		done()

	do ("Getting attributes")
	for i in server.fs.getDirContents( "/" ):
		st = server.getattr( i )
		print "File: " + i + \
				" Mode: " + str( st.st_mode ) + \
				" Uid: " + str( st.st_uid ) + \
				" Gid: " + str( st.st_gid ) + \
				" Size: " + str( st.st_size )
	done()

	TESTFILE="lw cloudfsá.testfile"
	writeString = "Hello, world!"
	do ('Creating file "' + TESTFILE + '" and writing "' + writeString + '" to it')
	server.mknod( "/" + TESTFILE, 0, 0 )
	server.open( "/" + TESTFILE, 0 )
	server.write( "/" + TESTFILE, writeString, 0 )
	server.release( "/" + TESTFILE, 0 )
	done()

	do ('Showing file "' + TESTFILE + '" contents')
	server.open( "/" + TESTFILE, 0 )
	read = server.read( "/" + TESTFILE, 1024, 0)
	print read
	server.release( "/" + TESTFILE, 0 )
	done()

	newContent = "\nNew line"
	do ('Modifying file "' + TESTFILE + '" contents')
	server.open( "/" + TESTFILE, 0 )
	server.write( "/" + TESTFILE, newContent, len(read) )
	server.release( "/" + TESTFILE, 0 )
	done()

	do ('Showing again file "' + TESTFILE + '" contents')
	server.open( "/" + TESTFILE, 0 )
	read = server.read( "/" + TESTFILE, 1024, 0)
	print read
	server.release( "/" + TESTFILE, 0 )
	done()

	do ('Getting attr from ' +TESTFILE )
	server.getattr( TESTFILE )
	server.fs.getStat( TESTFILE )
	done()

	TESTFILE2="lwcloudfs.testfile"

	do ('Renaming file "' + TESTFILE +'" to "' + TESTFILE2 + '"')
	server.rename( "/" + TESTFILE, "/" + TESTFILE2 )
	do ('Getting attributes of new file')
	st = server.getattr( "/" + TESTFILE2 )
	print "File: " + TESTFILE2 + \
			" Mode: " + str( st.st_mode ) + \
			" Uid: " + str( st.st_uid ) + \
			" Gid: " + str( st.st_gid ) + \
			" Size: " + str( st.st_size )
	done()

	do ('Removing file "' + TESTFILE2 + '"')
	server.unlink( "/" + TESTFILE2 )
	done()

	NONEXISTENTFILE="/Sin título 1.jhfu554negkl33243fd23"
	do ('Getting attributes of non existent file')
	server.getattr( NONEXISTENTFILE )
	done()

	do ("Umounting file system")
	server.fsdestroy()
	done()

       	#server.main()

if __name__ == '__main__':
        main()
