#!/bin/sh

TMPDIR=/tmp/test-smb
USERNAME=${SUDO_USER:dmoreno}
GROUPNAME=adm

if [ "$( whoami )" != "root" ]; then
	echo "Run as root"
	exit 1
fi

function off(){
	#umount -f /tmp/samba/home
	umount -f /tmp/samba/google
	umount -f $TMPDIR
	service sshd stop
	service firewalld start
	service smb stop
	setenforce 1
}

function on(){
	setenforce 0
	service sshd start
	service firewalld stop
	service smb start

	mkdir -p /tmp/samba/home/
	mkdir -p /tmp/samba/google/
	chown $USERNAME:$GROUPNAME /tmp/samba -R
	
	#echo ssh
	#sshfs -o allow_other $USERNAME@localhost:Documents /tmp/samba/home || exit 1

	echo google
	pushd .. >/dev/null
	#./lwcloudfs -o allow_other,uid=1000,gid=1000 /tmp/samba/google || exit 1
	./lwcloudfs -o allow_other,uid=1000,gid=1000 --fstype=dropbox --credsfile=credentials.json.dropbox /tmp/samba/google || exit 1
	popd > /dev/null
	
	mkdir -p $TMPDIR
	chown $USERNAME:$GROUPNAME $TMPDIR
	echo samba
	mount -t cifs -o username=$USERNAME,noperm //localhost/tmp $TMPDIR || exit 1
	
	echo $TMPDIR
}

function fail(){
	RED='\033[0;31m'
	NORMAL='\033[m'
	echo -e "${RED}Failed $*$NORMAL"
}

function echob(){
	BLUE='\033[0;34m'
	NORMAL='\033[m'
	echo -e "$BLUE$*$NORMAL"
}

function testopen(){
	for d in ${1:-/tmp/samba/ /tmp/samba/google /tmp/test-smb /tmp/test-smb/google}; do
		echo
		echob Test $d
		
		rm -f $d/touch
		[ "$( ls $d | grep touch )" ] && fail $d/touch exists ls
		[ -e $d/touch ] && fail $d/touch exists 
		date > $d/touch
		[ "$( ls $d | grep touch )" ] || fail $d/touch do not exist ls
		[ -e $d/touch ] || fail $d/touch does not exist
		echo "Wait 2 seconds"
		sleep 2
		[ "$( ls $d | grep touch )" ] || fail $d/touch do not exist ls		
		continue
		
		cp -v todo.ods $d || fail
		md5sum $d/todo.ods || fail
		md5sum $d/todo.ods || fail
		[ "$( md5sum $d/todo.ods | awk '{ print $1 }' )" = 'a546e51f44731610b59afc33ded711a2' ] || fail
	done
}

case "$1" in
-a|all)
	off
	on
	testopen $2
	off
;;
-u|off)
	off
	echo "All umounted"
;;
on)
	on
	echo "Ready"
;;
""|help)
	echo "mount-all.sh < all | off | on | help >"
;;
esac

