package com.librework.console.dao.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import com.librework.console.dao.IApplicationsDao;
import com.librework.console.model.AppGroup;
import com.librework.console.model.AppGroupPK;
import com.librework.console.model.AppServer;
import com.librework.console.model.AppServerPK;
import com.librework.console.model.Applications;
import com.librework.console.model.GroupApp;
import com.librework.console.model.GroupUser;
import com.librework.console.model.GroupUserApp;
import com.librework.console.model.GroupUserAppPK;
import com.librework.console.model.Server;
import com.librework.console.model.User;
import com.librework.console.model.UserApp;
import com.librework.console.model.UserAppPK;
import com.librework.console.model.UserGroup;
import com.librework.console.model.UserGroupPK;
import com.librework.console.utils.HibernateUtil;

public class ApplicationsDaoImpl implements IApplicationsDao, Serializable {

	private static final long serialVersionUID = 1L;
	
	private SessionFactory sessionFactory;
	
	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	public boolean createApplications(Applications applications) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.save(applications);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		return blnRespuesta;
	}

	@Override
	public Applications getApplicationsId(long applicationsId) {
		// TODO Auto-generated method stub
		Applications app = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applicationsId != 0) {
				criteria.add(Restrictions.eq("app_id", applicationsId));
			
				List<Applications> lista = criteria.list();
				if(!lista.isEmpty()){
					app = lista.get(0);
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return app;
	}

	@Override
	public List<Applications> getApplicationsName(String applicationsName) {
		// TODO Auto-generated method stub
		List<Applications> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applicationsName != null) {
				if (! applicationsName.equals("")) {
					criteria.add(Restrictions.like("applicationsName", applicationsName + "%"));
				
					List<Applications> lista = criteria.list();
					if(!lista.isEmpty()){
						list = lista;
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return list;
	}

	@Override
	public List<Applications> getAllApplications() {
		// TODO Auto-generated method stub
		List<Applications> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session.createCriteria(Applications.class);
			List<Applications> lista = criteria.list();
			if (!lista.isEmpty()) {
				list = lista;
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean updateApplications(Applications applications) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.update(applications);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public boolean deleteApplications(Applications applications) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.delete(applications);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public List<AppGroup> getAllGroupApp(Applications applications) {
		// TODO Auto-generated method stub
		List<AppGroup> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applications.getApp_id() != 0) {
				
					criteria.add(Restrictions.eq("app_id", applications.getApp_id()));
				
					List<Applications> listaappGroup = criteria.list();
					if(!listaappGroup.isEmpty()){
						List<AppGroup> lista = listaappGroup.get(0).getAppGroups();
						list = lista;
					}
				
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addGroupApp(Applications applications, GroupApp groupApp, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppGroupPK appGroupPK = new AppGroupPK(groupApp, applications);
		    AppGroup appGroup = new AppGroup();
		    appGroup.setAppGroupPK(appGroupPK);
		    appGroup.setFk_agr_gap_id(groupApp.getGap_id());
		    appGroup.setFk_agr_app_id(applications.getApp_id());
		    appGroup.setAgr_datmod(new Date());
		    appGroup.setAgr_usemod(userId);
		    applications.getAppGroups().add(appGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(appGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeGroupApp(Applications applications, GroupApp groupApp) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppGroupPK appGroupPK = new AppGroupPK(groupApp, applications);
		    AppGroup appGroup = new AppGroup();
		    appGroup.setAppGroupPK(appGroupPK);
		    appGroup.setFk_agr_gap_id(groupApp.getGap_id());
		    appGroup.setFk_agr_app_id(applications.getApp_id());
		    appGroup.setAgr_datmod(new Date());
		    applications.getAppGroups().remove(appGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(appGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		} 
		return blnRespuesta;
	}

	@Override
	public GroupApp getGroupApp(Applications applications, GroupApp gropuApp) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<UserApp> getAllAppUser(Applications applications) {
		// TODO Auto-generated method stub
		List<UserApp> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applications.getApp_id() != 0) {
				
					criteria.add(Restrictions.eq("app_id", applications.getApp_id()));
				
					List<Applications> listaAppUser = criteria.list();
					if(!listaAppUser.isEmpty()){
						List<UserApp> lista = listaAppUser.get(0).getUserApps();
						list = lista;
					}
				
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return list;
	}

	@Override
	public List<UserApp> getActiveAppUser(Applications applications) {
		List<UserApp> lstSalida = new ArrayList<UserApp>();
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applications.getApp_id() != 0) {
				criteria.add(Restrictions.eq("app_id", applications.getApp_id()));
				List<Applications> listaApp = criteria.list();
				if (! listaApp.isEmpty()) {
					List<UserApp> lista = listaApp.get(0).getUserApps();
					if (! lista.isEmpty()) {
						for (int i = 0; i < lista.size(); i++) {
							if (lista.get(i).getUap_end_date_process() == null) {
								lstSalida.add(lista.get(i));
							}
						}
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		
		return lstSalida;
	}
	@Override
	public boolean addUserApp(Applications applications, User user, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			UserAppPK userAppPK = new UserAppPK(user, applications);
		    UserApp userApp = new UserApp();
		    userApp.setUserAppPK(userAppPK);
		    userApp.setFk_uap_app_id(applications.getApp_id());
		    userApp.setFk_uap_use_id(user.getUse_id());
		    userApp.setUap_start_date_process(new Date());
		    userApp.setUap_end_date_process(new Date());
		    userApp.setUap_datmod(new Date());
		    userApp.setUap_usemod(userId);
		    applications.getUserApps().add(userApp);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(userApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeUserApp(Applications applications, User user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			UserAppPK userAppPK = new UserAppPK(user, applications);
		    UserApp userApp = new UserApp();
		    userApp.setUserAppPK(userAppPK);
		    userApp.setFk_uap_app_id(applications.getApp_id());
		    userApp.setFk_uap_use_id(user.getUse_id());
		    userApp.setUap_datmod(new Date());
		    applications.getUserApps().remove(userApp);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(userApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public User getUser(Applications applications, User user) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<AppServer> getAllAppServer(Applications applications) {
		// TODO Auto-generated method stub
		List<AppServer> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applications.getApp_id() != 0) {
				
					criteria.add(Restrictions.eq("app_id", applications.getApp_id()));
				
					List<Applications> listaAppServer = criteria.list();
					if(!listaAppServer.isEmpty()){
						List<AppServer> lista = listaAppServer.get(0).getAppServers();
						list = lista;
					}
				
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addAppServer(Applications applications, Server server, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppServerPK appServerPK = new AppServerPK(applications, server);
			AppServer appServer = new AppServer();
			appServer.setAppServerPK(appServerPK);
			appServer.setFk_ase_app_id(applications.getApp_id());
			appServer.setFk_ase_ser_id(server.getSer_id());
			appServer.setAse_datmod(new Date());
			appServer.setAse_usemod(userId);
		    applications.getAppServers().add(appServer);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(appServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeAppServer(Applications applications,
			Server server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppServerPK appServerPK = new AppServerPK(applications, server);
			AppServer appServer = new AppServer();
			appServer.setAppServerPK(appServerPK);
			appServer.setFk_ase_app_id(applications.getApp_id());
			appServer.setFk_ase_ser_id(server.getSer_id());
			appServer.setAse_datmod(new Date());
		    applications.getAppServers().remove(appServer);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(appServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public Server getServer(Applications applications, Server server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<GroupUserApp> getAllGroupUser(Applications applications) {
		// TODO Auto-generated method stub
		List<GroupUserApp> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Applications.class);
			if (applications.getApp_id() != 0) {
				
					criteria.add(Restrictions.eq("app_id", applications.getApp_id()));
				
					List<Applications> listaGroupUser = criteria.list();
					if(!listaGroupUser.isEmpty()){
						List<GroupUserApp> lista = listaGroupUser.get(0).getGroupUserApps();
						list = lista;
					}
				
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GETUSERBYID:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addGroupUser(Applications applications, GroupUser groupUser, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			GroupUserAppPK groupUserAppPK = new GroupUserAppPK(groupUser, applications);
			GroupUserApp groupUserApp = new GroupUserApp();
			groupUserApp.setGroupUserAppPK(groupUserAppPK);
			groupUserApp.setFk_gua_app_id(applications.getApp_id());
			groupUserApp.setFk_gua_gus_id(groupUser.getGus_id());
			groupUserApp.setGua_datmod(new Date());
			groupUserApp.setGua_usemod(userId);
		    applications.getGroupUserApps().add(groupUserApp);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(groupUserApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeGroupUser(Applications applications,
			GroupUser groupUser) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			GroupUserAppPK groupUserAppPK = new GroupUserAppPK(groupUser, applications);
			GroupUserApp groupUserApp = new GroupUserApp();
			groupUserApp.setGroupUserAppPK(groupUserAppPK);
			groupUserApp.setFk_gua_app_id(applications.getApp_id());
			groupUserApp.setFk_gua_gus_id(groupUser.getGus_id());
			groupUserApp.setGua_datmod(new Date());
		    applications.getGroupUserApps().remove(groupUserApp);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(groupUserApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public GroupUser getGroupUser(Applications applications, GroupUser groupUser) {
		// TODO Auto-generated method stub
		return null;
	}
	

}
