package com.librework.console.dao.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import com.librework.console.dao.IGroupAppDao;
import com.librework.console.model.AppGroup;
import com.librework.console.model.AppGroupPK;
import com.librework.console.model.Applications;
import com.librework.console.model.Events;
import com.librework.console.model.GroupApp;
import com.librework.console.model.GroupUser;
import com.librework.console.utils.HibernateUtil;

public class GroupAppDaoImpl implements IGroupAppDao, Serializable {

	private static final long serialVersionUID = 1L;
	
	private SessionFactory sessionFactory;

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	@Override
	public boolean createGroupApp(GroupApp groupApp) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.save(groupApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		return blnRespuesta;
	}

	@Override
	public GroupApp getGroupAppId(long groupAppId) {
		// TODO Auto-generated method stub
		GroupApp groupApp = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupApp.class);
			if (groupAppId != 0) {
				
				criteria.add(Restrictions.eq("gap_id", groupAppId));
			
				List<GroupApp> lista = criteria.list();
				if(!lista.isEmpty()){
					
					groupApp = lista.get(0);
					
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GROUPAPP:" + e.toString());
		} 
		return groupApp;
	}

	@Override
	public List<GroupApp> getGroupAppName(String groupAppName) {
		// TODO Auto-generated method stub
		List<GroupApp> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupApp.class);
			if (groupAppName != null) {
				if (!groupAppName.equals("")) {
					
					criteria.add(Restrictions.like("gap_name", groupAppName + "%-"));
				
					List<GroupApp> lista = criteria.list();
					if(!lista.isEmpty()){
						
						list =  lista;
						
					}
				}
			}
			
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("groupapp:" + e.toString());
		} 
		return list;
	}

	@Override
	public List<GroupApp> getAllGroupApp() {
		// TODO Auto-generated method stub
		List<GroupApp> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session.createCriteria(GroupApp.class);
			List<GroupApp> lista = criteria.list();
			if (!lista.isEmpty()) {
				list = lista;
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("groupapp:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean updateGroupApp(GroupApp groupApp) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.update(groupApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public boolean deleteGroupApp(GroupApp groupApp) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.delete(groupApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public boolean addApplications(GroupApp groupApp, Applications applications, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppGroupPK appGroupPK = new AppGroupPK(groupApp, applications);
		    AppGroup appGroup = new AppGroup();
		    appGroup.setAppGroupPK(appGroupPK);
		    appGroup.setFk_agr_gap_id(groupApp.getGap_id());
		    appGroup.setFk_agr_app_id(applications.getApp_id());
		    appGroup.setAgr_datmod(new Date());
		    appGroup.setAgr_usemod(userId);
		    applications.getAppGroups().add(appGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(appGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<AppGroup> getAllApplications(GroupApp groupApp) {
		// TODO Auto-generated method stub
		List<AppGroup> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
						.createCriteria(GroupApp.class);
				if (groupApp.getGap_id() != 0) {
					
						criteria.add(Restrictions.eq("gap_id", groupApp.getGap_id()));
					
						List<GroupApp> listaGroupApp = criteria.list();
						if(!listaGroupApp.isEmpty()){
							List<AppGroup> lista = listaGroupApp.get(0).getAppGroups();
							list = lista;
						}
					
				}
				tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("groupapp:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean removeApplications(GroupApp groupApp,
			Applications applications) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppGroupPK appGroupPK = new AppGroupPK(groupApp, applications);
		    AppGroup appGroup = new AppGroup();
		    appGroup.setAppGroupPK(appGroupPK);
		    appGroup.setFk_agr_gap_id(groupApp.getGap_id());
		    appGroup.setFk_agr_app_id(applications.getApp_id());
		    appGroup.setAgr_datmod(new Date());
		    applications.getAppGroups().remove(appGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(appGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public Applications getApplications(GroupApp groupApp,
			Applications applications) {
		// TODO Auto-generated method stub
		return null;
	}
}
