package com.librework.console.dao.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import com.librework.console.dao.IGroupServerDao;
import com.librework.console.model.AppGroup;
import com.librework.console.model.AppGroupPK;
import com.librework.console.model.GroupApp;
import com.librework.console.model.GroupServer;
import com.librework.console.model.Server;
import com.librework.console.model.ServerGroup;
import com.librework.console.model.ServerGroupPK;
import com.librework.console.utils.HibernateUtil;

public class GroupServerDaoImpl implements IGroupServerDao, Serializable {

	private static final long serialVersionUID = 1L;
	
	private SessionFactory sessionFactory;

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}
	
	@Override
	public boolean createGroupServer(GroupServer groupServer) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.save(groupServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		return blnRespuesta;
	}

	@Override
	public GroupServer getGroupServerId(long groupServerId) {
		// TODO Auto-generated method stub
		GroupServer groupServer = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupServer.class);
			if (groupServerId != 0) {
				
				criteria.add(Restrictions.eq("grs_id", groupServerId));
			
				List<GroupServer> lista = criteria.list();
				if(!lista.isEmpty()){
					
					groupServer = lista.get(0);
					
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GROUPSERVER:" + e.toString());
		} 
		return groupServer;
	}

	@Override
	public List<GroupServer> getAllGroupServer() {
		// TODO Auto-generated method stub
		List<GroupServer> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session.createCriteria(GroupServer.class);
			List<GroupServer> lista = criteria.list();
			if (!lista.isEmpty()) {
				list = lista;
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GROUPSERVER:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean updateGroupServer(GroupServer groupServer) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.update(groupServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public boolean deleteGroupServer(GroupServer groupServer) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.delete(groupServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public List<ServerGroup> getAllServer(GroupServer groupServer) {
		// TODO Auto-generated method stub
		List<ServerGroup> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupServer.class);
			if (groupServer.getGrs_id() != 0) {
				
					criteria.add(Restrictions.eq("grs_id", groupServer.getGrs_id()));
				
					List<GroupServer> listaGroupServer = criteria.list();
					if(!listaGroupServer.isEmpty()){
						List<ServerGroup> lista = listaGroupServer.get(0).getServerGroups();
						list =  lista;
					}
				
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GROUPSERVER:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addServer(GroupServer groupServer, Server server,
			String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			ServerGroupPK serverGroupPK = new ServerGroupPK(server, groupServer);
		    ServerGroup serverGroup = new ServerGroup();
		    serverGroup.setServerGroupPK(serverGroupPK);
		    serverGroup.setFk_seg_grs_id(groupServer.getGrs_id());
		    serverGroup.setFk_seg_ser_id(server.getSer_id());
		    serverGroup.setSeg_datmod(new Date());
		    serverGroup.setSeg_usemod(userId);
		    server.getServerGroups().add(serverGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(serverGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeServer(GroupServer groupServer, Server server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			ServerGroupPK serverGroupPK = new ServerGroupPK(server, groupServer);
		    ServerGroup serverGroup = new ServerGroup();
		    serverGroup.setServerGroupPK(serverGroupPK);
		    serverGroup.setFk_seg_grs_id(groupServer.getGrs_id());
		    serverGroup.setFk_seg_ser_id(server.getSer_id());
		    serverGroup.setSeg_datmod(new Date());
		    server.getServerGroups().remove(serverGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(serverGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public Server getServer(GroupServer groupServer, Server server) {
		// TODO Auto-generated method stub
		return null;
	}

}
