package com.librework.console.dao.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import com.librework.console.dao.IGroupUserDao;
import com.librework.console.model.Applications;
import com.librework.console.model.GroupServer;
import com.librework.console.model.GroupUser;
import com.librework.console.model.GroupUserApp;
import com.librework.console.model.GroupUserAppPK;
import com.librework.console.model.User;
import com.librework.console.model.UserGroup;
import com.librework.console.model.UserGroupPK;
import com.librework.console.utils.HibernateUtil;

public class GroupUserDaoImpl implements IGroupUserDao, Serializable {

	private static final long serialVersionUID = 1L;
	
	private SessionFactory sessionFactory;

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}
	
	@Override
	public boolean createGroupUser(GroupUser groupUser) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.save(groupUser);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		return blnRespuesta;
	}

	

	@Override
	public GroupUser getGroupUserId(long groupUserId) {
		// TODO Auto-generated method stub
		GroupUser groupUser = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupUser.class);
			if (groupUserId != 0) {
				
				criteria.add(Restrictions.eq("gus_id", groupUserId));
			
				List<GroupUser> lista = criteria.list();
				if(!lista.isEmpty()){
					
					groupUser = lista.get(0);
					
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("groupUser:" + e.toString());
		} 
		return groupUser;
	}

	@Override
	public GroupUser getGroupUserCn(String groupUserCn) {
		// TODO Auto-generated method stub
		GroupUser groupUser = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupUser.class);
			if (groupUserCn != null) {
				if (! groupUserCn.equals("")) {
					criteria.add(Restrictions.eq("gus_cn", groupUserCn));
				
					List<GroupUser> lista = criteria.list();
					if(!lista.isEmpty()){
						groupUser = lista.get(0);
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("groupUser:" + e.toString());
		} 
		return groupUser;
	}
	
	@Override
	public List<GroupUser> getAllGroupUser() {
		// TODO Auto-generated method stub
		List<GroupUser> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session.createCriteria(GroupUser.class);
			List<GroupUser> lista = criteria.list();
			if (!lista.isEmpty()) {
				list = lista;
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GroupUser:" + e.toString());
		} 
		return list;
	}
	
	@Override
	public boolean updateGroupUser(GroupUser groupUser) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.update(groupUser);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public boolean deleteGroupUser(GroupUser groupUser) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			session.delete(groupUser);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			tx.rollback();
			blnRespuesta = false;
		}
		
		return blnRespuesta;
	}

	@Override
	public List<UserGroup> getAllUser(GroupUser groupUser) {
		// TODO Auto-generated method stub
		List<UserGroup> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupUser.class);
			if (groupUser.getGus_id() != 0) {
				criteria.add(Restrictions.eq("gus_id", groupUser.getGus_id()));
			
				List<GroupUser> listaGroupUser = criteria.list();
				if(!listaGroupUser.isEmpty()){
					List<UserGroup> lista = listaGroupUser.get(0).getUserGroups();
					list = lista;
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GroupUser:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addUser(GroupUser groupUser, User user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			UserGroupPK userGroupPK = new UserGroupPK(user, groupUser);
		    UserGroup userGroup = new UserGroup();
		    userGroup.setUserGroupPK(userGroupPK);
		    userGroup.setFk_ugr_gus_id(groupUser.getGus_id());
		    userGroup.setFk_ugr_use_id(user.getUse_id());
		    userGroup.setUgr_datmod(new Date());
		    userGroup.setUgr_usemod(user.getUse_id());
		    user.getUserGroups().add(userGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(userGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		} 
		return blnRespuesta;
	}

	@Override
	public boolean removeUser(GroupUser groupUser, User user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			UserGroup userGroup = new UserGroup();
			UserGroupPK userGroupPK = new UserGroupPK(user, groupUser);
			userGroup.setUserGroupPK(userGroupPK);
			userGroup.setFk_ugr_gus_id(groupUser.getGus_id());
			userGroup.setFk_ugr_use_id(user.getUse_id());
			userGroup.setUgr_datmod(new Date());
			userGroup.setUgr_usemod(user.getUse_id());
			
			user.getUserGroups().remove(userGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(userGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public User getUser(GroupUser groupUser, User user) {
		// TODO Auto-generated method stub
		
		return null;
	}

	@Override
	public List<GroupUserApp> getAllApplications(GroupUser groupUser) {
		// TODO Auto-generated method stub
		List<GroupUserApp> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(GroupUser.class);
			if (groupUser.getGus_id() != 0) {
				criteria.add(Restrictions.eq("gus_id", groupUser.getGus_id()));
			
				List<GroupUser> listaGroupUser = criteria.list();
				if(!listaGroupUser.isEmpty()){
					List<GroupUserApp> lista = listaGroupUser.get(0).getGroupUserApps();
					list = lista;
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("GroupUser:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addApplications(GroupUser groupUser,
			Applications applications, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			GroupUserAppPK groupUserAppPK = new GroupUserAppPK(groupUser, applications);
		    GroupUserApp groupUserApp = new GroupUserApp();
		    groupUserApp.setGroupUserAppPK(groupUserAppPK);
		    groupUserApp.setFk_gua_gus_id(groupUser.getGus_id());
		    groupUserApp.setFk_gua_app_id(applications.getApp_id());
		    groupUserApp.setGua_datmod(new Date());
		    groupUserApp.setGua_usemod(userId);
		    applications.getGroupUserApps().add(groupUserApp);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(groupUserApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeApplications(GroupUser groupUser,
			Applications applications) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			GroupUserAppPK groupUserAppPK = new GroupUserAppPK(groupUser, applications);
		    GroupUserApp groupUserApp = new GroupUserApp();
		    groupUserApp.setGroupUserAppPK(groupUserAppPK);
		    groupUserApp.setFk_gua_gus_id(groupUser.getGus_id());
		    groupUserApp.setFk_gua_app_id(applications.getApp_id());
		    groupUserApp.setGua_datmod(new Date());
		    applications.getGroupUserApps().remove(groupUserApp);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(groupUserApp);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public Applications getApplications(GroupUser groupUser,
			Applications applications) {
		// TODO Auto-generated method stub
		return null;
	}

	

}
