package com.librework.console.dao.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import com.librework.console.dao.IServerDao;
import com.librework.console.model.AppServer;
import com.librework.console.model.AppServerPK;
import com.librework.console.model.Applications;
import com.librework.console.model.GroupServer;
import com.librework.console.model.Server;
import com.librework.console.model.ServerGroup;
import com.librework.console.model.ServerGroupPK;
import com.librework.console.model.ServerHDD;
import com.librework.console.model.UserApp;
import com.librework.console.utils.HibernateUtil;



public class ServerDaoImpl implements IServerDao, Serializable {

	private static final long serialVersionUID = 1L;
	
	private SessionFactory sessionFactory;
	
	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	public boolean createServer(Server server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(server);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println("CREATE SERVER - " + e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public Server getServerId(String serverId) {
		// TODO Auto-generated method stub
		Server server = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (serverId != null) {
				if (!serverId.equals("")) {
					criteria.add(Restrictions.eq("ser_id", serverId));
					
					List<Server> lista = criteria.list();
					if (! lista.isEmpty()) {
						server = lista.get(0);
					}
				}
			}
		tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return server;
	}

	@Override
	public List<Server> getServerHost(String serverHost) {
		// TODO Auto-generated method stub
		List<Server> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (serverHost != null) {
				if (!serverHost.equals("")) {
					criteria.add(Restrictions.like("ser_host", serverHost + "%"));
					List<Server> lista = criteria.list();
					if (! lista.isEmpty()) {
						list = lista;
					}
				}
			}
		tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public List<Server> getServerDescription(String serverDescription) {
		// TODO Auto-generated method stub
		List<Server> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (serverDescription != null) {
				if (!serverDescription.equals("")) {
					criteria.add(Restrictions.eq("ser_description", serverDescription));
					List<Server> lista = criteria.list();
					if (! lista.isEmpty()) {
						list = lista;
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public List<Server> getAllServer() {
		// TODO Auto-generated method stub
		List<Server> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
					.createCriteria(Server.class);
			list = criteria.list();
			if (list.isEmpty()) {
				list = null;
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean updateServer(Server server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.update(server);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println("UPDATE SERVER - " + e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean deleteServer(Server server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(server);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println("UPDATE SERVER - " + e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<ServerHDD> getAllServerHDD(Server server) {
		// TODO Auto-generated method stub
		List<ServerHDD> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (server.getSer_id() != null) {
				if (! server.getSer_id().equals("")) {
					criteria.add(Restrictions.eq("ser_id", server.getSer_id()));
				
					List<Server> listaServer = criteria.list();
					if(!listaServer.isEmpty()){
						List<ServerHDD> lista = listaServer.get(0).getServerHDDs();
						list = lista;
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addServerHDD(Server server, ServerHDD serverHDD) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
		    server.getServerHDDs().add(serverHDD);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(serverHDD);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeServerHDD(Server server, ServerHDD serverHDD) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
		    server.getServerHDDs().remove(serverHDD);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(serverHDD);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public ServerHDD getServerHDD(Server server, ServerHDD serverHDD) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<ServerGroup> getAllGroupServer(Server server) {
		// TODO Auto-generated method stub
		List<ServerGroup> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (server.getSer_id() != null) {
				if (! server.getSer_id().equals("")) {
					criteria.add(Restrictions.eq("ser_id", server.getSer_id()));
				
					List<Server> listaServer = criteria.list();
					if(!listaServer.isEmpty()){
						List<ServerGroup> lista = listaServer.get(0).getServerGroups();
						list = lista;
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addGroupServer(Server server, GroupServer groupServer, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			ServerGroupPK serverGroupPK = new ServerGroupPK(server, groupServer);
		    ServerGroup serverGroup = new ServerGroup();
		    serverGroup.setServerGroupPK(serverGroupPK);
		    serverGroup.setFk_seg_grs_id(groupServer.getGrs_id());
		    serverGroup.setFk_seg_ser_id(server.getSer_id());
		    serverGroup.setSeg_datmod(new Date());
		    serverGroup.setSeg_usemod(userId);
		    server.getServerGroups().add(serverGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(serverGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeGroupServer(Server server, GroupServer groupServer) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			ServerGroupPK serverGroupPK = new ServerGroupPK(server, groupServer);
		    ServerGroup serverGroup = new ServerGroup();
		    serverGroup.setServerGroupPK(serverGroupPK);
		    serverGroup.setFk_seg_grs_id(groupServer.getGrs_id());
		    serverGroup.setFk_seg_ser_id(server.getSer_id());
		    serverGroup.setSeg_datmod(new Date());
		    server.getServerGroups().remove(serverGroup);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(serverGroup);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public GroupServer getGroupServer(Server server, GroupServer groupServer) {
		// TODO Auto-generated method stub
		
		return null;
	}

	@Override
	public List<AppServer> getAllApplications(Server server) {
		// TODO Auto-generated method stub
		List<AppServer> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (server.getSer_id() != null) {
				if (! server.getSer_id().equals("")) {
					criteria.add(Restrictions.eq("ser_id", server.getSer_id()));
				
					List<Server> listaServer = criteria.list();
					if(!listaServer.isEmpty()){
						List<AppServer> lista = listaServer.get(0).getAppServers();
						list = lista;
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public boolean addApplications(Server server, Applications applications, String userId) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppServerPK appServerPK = new AppServerPK(applications, server);
		    AppServer appServer = new AppServer();
		    appServer.setAppServerPK(appServerPK);
		    appServer.setFk_ase_app_id(applications.getApp_id());
		    appServer.setFk_ase_ser_id(server.getSer_id());
		    appServer.setAse_datmod(new Date());
		    appServer.setAse_usemod(userId);
		    server.getAppServers().add(appServer);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(appServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeApplications(Server server, Applications applications) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			AppServerPK appServerPK = new AppServerPK(applications, server);
		    AppServer appServer = new AppServer();
		    appServer.setAppServerPK(appServerPK);
		    appServer.setFk_ase_app_id(applications.getApp_id());
		    appServer.setFk_ase_ser_id(server.getSer_id());
		    appServer.setAse_datmod(new Date());
		    server.getAppServers().remove(appServer);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(appServer);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}
	

	@Override
	public Applications getApplications(Server server, Applications applications) {
		// TODO Auto-generated method stub
		
		return null;
	}

	@Override
	public List<UserApp> getAllUserApp(Server server) {
		// TODO Auto-generated method stub
		List<UserApp> list = null;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (server.getSer_id() != null) {
				if (! server.getSer_id().equals("")) {
					criteria.add(Restrictions.eq("ser_id", server.getSer_id()));
				
					List<Server> listaServer = criteria.list();
					if(!listaServer.isEmpty()){
						List<UserApp> lista = listaServer.get(0).getUserApps();
						list = lista;
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

	@Override
	public List<UserApp> getActiveUserApp(Server server) {
		// TODO Auto-generated method stub
		List<UserApp> list = new ArrayList<UserApp>();;
		Session session = null;
		Transaction tx = null;
		
		try {
			session = HibernateUtil.getSessionFactory().getCurrentSession();
			tx = session.beginTransaction();
			Criteria criteria = session
				.createCriteria(Server.class);
			if (server.getSer_id() != null) {
				if (! server.getSer_id().equals("")) {
					criteria.add(Restrictions.eq("ser_id", server.getSer_id()));
				
					List<Server> listaServer = criteria.list();
					if(!listaServer.isEmpty()){
						List<UserApp> lista = listaServer.get(0).getUserApps();
						for (int i=0; i<lista.size(); i++) {
							if (lista.get(i).getUap_end_date_process() == null) {
								list.add(lista.get(i));
							}
						}
					}
				}
			}
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			System.out.println("Server:" + e.toString());
		} 
		return list;
	}

}
