package com.librework.console.dao.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import com.librework.console.dao.ISiteDao;
import com.librework.console.model.Server;
import com.librework.console.model.ServerHDD;
import com.librework.console.model.Site;
import com.librework.console.model.User;
import com.librework.console.model.UserApp;
import com.librework.console.model.UserAppPK;
import com.librework.console.model.UserSite;
import com.librework.console.model.UserSitePK;
import com.librework.console.utils.HibernateUtil;

public class SiteDaoImpl implements ISiteDao, Serializable {

	private static final long serialVersionUID = 1L;
	
	private SessionFactory sessionFactory;
	
	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	public boolean createSite(Site site) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(site);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println("CREATE SERVER - " + e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public Site getSiteId(long siteId) {
		// TODO Auto-generated method stub
		Criteria criteria = HibernateUtil.getSessionFactory().getCurrentSession()
				.createCriteria(Site.class);
		if (siteId != 0) {
			
				criteria.add(Restrictions.eq("sit_id", siteId));
				
				List<Site> lista = criteria.list();
				if (! lista.isEmpty()) {
					return lista.get(0);
				
			}
		}
		return null;
	}

	@Override
	public List<Site> getSiteDescription(String siteDescription) {
		// TODO Auto-generated method stub
		Criteria criteria = HibernateUtil.getSessionFactory().getCurrentSession()
				.createCriteria(Site.class);
		if (siteDescription != null) {
			if (!siteDescription.equals("")) {
				criteria.add(Restrictions.like("sit_description", siteDescription + "%"));
				List<Site> lista = criteria.list();
				if (! lista.isEmpty()) {
					return lista;
				}
			}
		}
		return null;
	}

	@Override
	public List<Site> getAllSite() {
		// TODO Auto-generated method stub
		Criteria criteria = HibernateUtil.getSessionFactory().getCurrentSession()
				.createCriteria(Site.class);
		List<Site> lista = criteria.list();
		if (! lista.isEmpty()) {
			return lista;
		}
		return null;
	}

	@Override
	public boolean updateSite(Site site) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.update(site);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println("UPDATE SERVER - " + e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean deleteSite(Site site) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session = null;
		Transaction tx = null;
		try {
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(site);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println("UPDATE SERVER - " + e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<UserSite> getAllUser(Site site) {
		// TODO Auto-generated method stub
		Criteria criteria = HibernateUtil.getSessionFactory().getCurrentSession()
				.createCriteria(Site.class);
		if (site.getSit_id() != 0) {
			
				criteria.add(Restrictions.eq("sit_id", site.getSit_id()));
			
				List<Site> listaSite = criteria.list();
				if(!listaSite.isEmpty()){
					List<UserSite> lista = listaSite.get(0).getUserSites();
					return lista;
				}
			
		}
		return null;
	}

	@Override
	public boolean addUser(Site site, User user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			UserSitePK userSitePK = new UserSitePK(user, site);
		    UserSite userSite = new UserSite();
		    userSite.setUserSitePK(userSitePK);
		    userSite.setFk_usi_sit_id(site.getSit_id());
		    userSite.setFk_usi_use_id(user.getUse_id());
		    userSite.setUsi_datmod(new Date());
		    userSite.setUsi_usemod(user.getUse_id());
		    site.getUserSites().add(userSite);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.save(userSite);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeUser(Site site, User user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		Session session=null;  
        Transaction tx=null;
		try {
			UserSitePK userSitePK = new UserSitePK(user, site);
		    UserSite userSite = new UserSite();
		    userSite.setUserSitePK(userSitePK);
		    userSite.setFk_usi_sit_id(site.getSit_id());
		    userSite.setFk_usi_use_id(user.getUse_id());
		    userSite.setUsi_datmod(new Date());
		    userSite.setUsi_usemod(user.getUse_id());
		    site.getUserSites().remove(userSite);
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			session.delete(userSite);
			tx.commit();
			blnRespuesta = true;
		} catch (Exception e) {
			if (session != null) {
				tx.rollback();
			}
			blnRespuesta = false;
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public User getUser(Site site, User user) {
		// TODO Auto-generated method stub
		return null;
	}
}
