package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IAppGroupLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.AppGroupLDAP;

public class AppGroupLDAPDaoImpl implements IAppGroupLDAPDao {

	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	
	protected ContextMapper getAppGroupContextMapper() {
		return new AppGroupContextMapper();
	}
	
	private static class AppGroupContextMapper extends AbstractContextMapper {
		
		public Object doMapFromContext(DirContextOperations context) {
			AppGroupLDAP appGroup = new AppGroupLDAP();
			appGroup.setGroupsApp(context.getStringAttributes(Constant.KEY_STR_ATTR_PROGGROUP));
			return appGroup;
		}
	}

	@Override
	public boolean createAppGroup(AppGroupLDAP appGroup) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, appGroup.getCnApplication());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.ADD_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_PROGGROUP, appGroup.getCnGroup()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<AppGroupLDAP> getGroupsApp(String cnApp) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_APPLICATIONS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_APPLICATION)).and
			(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cnApp));
		 List groupsApp = ldapTemplate.search(dn, filter.encode(), getAppGroupContextMapper());
		
		if (! groupsApp.isEmpty()) {
			return groupsApp;
		}
		
		return null;
	}

	@Override
	public boolean removeAppGroup(AppGroupLDAP appGroup) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, appGroup.getCnApplication());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_PROGGROUP, appGroup.getCnGroup()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}
}
