package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IAppServerLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.AppServerLDAP;

public class AppServerLDAPDaoImpl implements IAppServerLDAPDao {

	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	
	protected ContextMapper getAppServerContextMapper() {
		return new AppServerContextMapper();
	}
	
	private static class AppServerContextMapper extends AbstractContextMapper {
		
		public Object doMapFromContext(DirContextOperations context) {
			AppServerLDAP appServer = new AppServerLDAP();
			appServer.setServersApp(context.getStringAttributes(Constant.KEY_STR_ATTR_HOST));
			return appServer;
		}
		
	}
	@Override
	public boolean createAppServer(AppServerLDAP appServer) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, appServer.getCnApplication());
			//Creamos servidor a la aplicacin
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_HOST, appServer.getCnServer()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<AppServerLDAP> getAppServer(String cnApplication) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_APPLICATIONS);
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_APPLICATION))
			.and (new EqualsFilter(Constant.KEY_STR_DNAME_CN, cnApplication));
		List appServer = ldapTemplate.search(dn, filter.encode(), getAppServerContextMapper());
		
		if (! appServer.isEmpty()) {
			return appServer;
		}
		
		return null;
	}

	@Override
	public boolean removeAppServer(AppServerLDAP appServer) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, appServer.getCnApplication());
			//Creamos servidor a la aplicacin
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_HOST, appServer.getCnServer()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

}
