package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IApplicationLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.ApplicationLDAP;
import com.librework.console.modelLDAP.GroupUserLDAP;

public class ApplicationLDAPDaoImpl implements IApplicationLDAPDao {

	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	
	protected ContextMapper getContextMapper() {
		return new ApplicationContextMapper();
	}
	
	private static class ApplicationContextMapper extends AbstractContextMapper {
		
		public Object doMapFromContext(DirContextOperations context) {
			ApplicationLDAP applicationLDAP = new ApplicationLDAP();
			applicationLDAP.setCn(context.getStringAttribute(Constant.KEY_STR_ATTR_CN));
			applicationLDAP.setDescription(context.getStringAttribute(Constant.KEY_STR_ATTR_DESCRIPTION));
			applicationLDAP.setName(context.getStringAttribute(Constant.KEY_STR_ATTR_NAME));
			applicationLDAP.setDisplayName(context.getStringAttribute(Constant.KEY_STR_ATTR_DISPLAYNAME));
			applicationLDAP.setDistinguishedName(context.getStringAttribute(Constant.KEY_STR_ATTR_DISTINGUISHEDNAME));
			applicationLDAP.setIcon(context.getStringAttribute(Constant.KEY_STR_ATTR_ICON));
			applicationLDAP.setScriptPath(context.getStringAttribute(Constant.KEY_STR_ATTR_SCRIPTPATH));
			applicationLDAP.setOperatingSystem(context.getStringAttribute(Constant.KEY_STR_ATTR_OPERATINGSYSTEM));
			applicationLDAP.setProgGroup(context.getStringAttribute(Constant.KEY_STR_ATTR_PROGGROUP));
			return applicationLDAP;
		}
	}
	
	private Attributes builApplicationAttributes(final ApplicationLDAP applicationLDAP) {
		
		Attributes attrs = new BasicAttributes();
		BasicAttribute ocattr = new BasicAttribute(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS);
		ocattr.add(Constant.KEY_STR_FILTERLDAP_TYPE_APPLICATION);
		attrs.put(ocattr);
		attrs.put(Constant.KEY_STR_ATTR_CN, applicationLDAP.getCn());
		attrs.put(Constant.KEY_STR_ATTR_NAME, applicationLDAP.getName());
		attrs.put(Constant.KEY_STR_ATTR_DESCRIPTION, applicationLDAP.getDescription());
		attrs.put(Constant.KEY_STR_ATTR_DISPLAYNAME, applicationLDAP.getDisplayName());
		attrs.put(Constant.KEY_STR_ATTR_ICON, applicationLDAP.getIcon());
		attrs.put(Constant.KEY_STR_ATTR_SCRIPTPATH, applicationLDAP.getScriptPath());
		attrs.put(Constant.KEY_STR_ATTR_OPERATINGSYSTEM, applicationLDAP.getOperatingSystem());
		attrs.put(Constant.KEY_STR_ATTR_PROGGROUP, applicationLDAP.getProgGroup());
		return attrs;
	}

	@Override
	public boolean createApplication(ApplicationLDAP application) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, application.getCn());
			ldapTemplate.bind(dn, null, builApplicationAttributes(application));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public ApplicationLDAP getApplicationCn(String cn) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_APPLICATIONS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_APPLICATION))
			.and(new EqualsFilter(Constant.KEY_STR_ATTR_CN, cn));
		List<ApplicationLDAP> applications = ldapTemplate.search(dn, filter.encode(), getContextMapper());
		if (! applications.isEmpty()) {
			return applications.get(0);
		}
		return null;
	}

	@Override
	public List<ApplicationLDAP> getAllApplications() {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_APPLICATIONS);
		System.out.println(ldapTemplate.getContextSource().toString());
		EqualsFilter filter = new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_APPLICATION);
		return ldapTemplate.search(dn, filter.encode(), getContextMapper());
	}

	@Override
	public boolean updateApplication(ApplicationLDAP application) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, application.getCn());
			ldapTemplate.rebind(dn, null, builApplicationAttributes(application));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean deleteApplciation(ApplicationLDAP application) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_APPLICATIONS);
			dn.add(Constant.KEY_STR_DNAME_CN, application.getCn());
			ldapTemplate.unbind(dn);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}
	

}
