package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IGroupLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.GroupUserLDAP;

public class GroupLDAPDaoImpl implements IGroupLDAPDao {

	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}

	//Recuperar el contextMapper de la clase Builtin
	protected ContextMapper getContextMapper() {
		return new GroupContextMapper();
	}
	
	//Mapeo del contexto de GroupLDAP con sus atributos
	private static class GroupContextMapper extends AbstractContextMapper {
		
		public Object doMapFromContext(DirContextOperations context) {
			GroupUserLDAP groupUserLDAP = new GroupUserLDAP();
			groupUserLDAP.setCn(context.getStringAttribute(Constant.KEY_STR_ATTR_CN));
			groupUserLDAP.setName(context.getStringAttribute(Constant.KEY_STR_ATTR_NAME));
			groupUserLDAP.setDescription(context.getStringAttribute(Constant.KEY_STR_ATTR_DESCRIPTION));
			groupUserLDAP.setDistinguishedName(context.getStringAttribute(Constant.KEY_STR_ATTR_DISTINGUISHEDNAME));
			return groupUserLDAP;
		}
	}
	
	private Attributes builGroupUserAttributes(final GroupUserLDAP groupUser) {
		
		Attributes attrs = new BasicAttributes();
		BasicAttribute ocattr = new BasicAttribute(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS);
		ocattr.add(Constant.KEY_STR_FILTERLDAP_TYPE_GROUP);
		attrs.put(ocattr);
		attrs.put(Constant.KEY_STR_ATTR_CN, groupUser.getCn());
		attrs.put(Constant.KEY_STR_ATTR_NAME, groupUser.getName());
		attrs.put(Constant.KEY_STR_ATTR_DESCRIPTION, groupUser.getDescription());
		return attrs;
	}
	@Override
	public boolean createGroup(GroupUserLDAP group) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_BUILTIN);
			dn.add(Constant.KEY_STR_DNAME_CN, group.getCn());
			ldapTemplate.bind(dn, null, builGroupUserAttributes(group));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public GroupUserLDAP getGroupCn(String cn) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_BUILTIN);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_GROUP))
			.and(new EqualsFilter(Constant.KEY_STR_ATTR_CN, cn));
		List<GroupUserLDAP> groups = ldapTemplate.search(dn, filter.encode(), getContextMapper());
		if (! groups.isEmpty()) {
			return groups.get(0);
		}
		return null;
	}

	@Override
	public GroupUserLDAP getGroupDistinguishedName(String distinguishedName) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_BUILTIN);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_GROUP))
			.and(new EqualsFilter(Constant.KEY_STR_ATTR_DISTINGUISHEDNAME, distinguishedName));
		List<GroupUserLDAP> groups = ldapTemplate.search(dn, filter.encode(), getContextMapper());
		if (! groups.isEmpty()) {
			return groups.get(0);
		}
		return null;
	}

	@Override
	public List<GroupUserLDAP> getAllGroups() {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_BUILTIN);
		System.out.println(ldapTemplate.getContextSource().toString());
		EqualsFilter filter = new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_GROUP);
		return ldapTemplate.search(dn, filter.encode(), getContextMapper());
	}

	@Override
	public boolean updateGroup(GroupUserLDAP group) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_BUILTIN);
			dn.add(Constant.KEY_STR_DNAME_CN, group.getCn());
			ldapTemplate.rebind(dn, null, builGroupUserAttributes(group));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean deleteGroup(GroupUserLDAP group) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_BUILTIN);
			dn.add(Constant.KEY_STR_DNAME_CN, group.getCn());
			ldapTemplate.unbind(dn);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}
	
	
}
