package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IServerGroupLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.ServerGroupLDAP;

public class ServerGroupLDAPDaoImpl implements IServerGroupLDAPDao {

private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	
	protected ContextMapper getServerGroupContextMapper() {
		return new ServerGroupContextMapper();
	}
	
	private static class ServerGroupContextMapper extends AbstractContextMapper {
			
			public Object doMapFromContext(DirContextOperations context) {
				ServerGroupLDAP serverGroup = new ServerGroupLDAP();
				serverGroup.setGroupsServer(context.getStringAttributes(Constant.KEY_STR_ATTR_SERVERGROUP));
				return serverGroup;
			}
		}

	@Override
	public boolean createServerGroup(ServerGroupLDAP serverGroup) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_SERVERS);
			dn.add(Constant.KEY_STR_DNAME_CN, serverGroup.getCnServer());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.ADD_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_SERVERGROUP, serverGroup.getCnGroup()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<ServerGroupLDAP> getGroupsServer(String cnServer) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_SERVERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_SERVER)).and
			(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cnServer));
		 List serverGroup = ldapTemplate.search(dn, filter.encode(), getServerGroupContextMapper());
		
		if (! serverGroup.isEmpty()) {
			return serverGroup;
		}
		
		return null;
	}

	@Override
	public boolean deleteServerGroup(ServerGroupLDAP serverGroup) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_SERVERS);
			dn.add(Constant.KEY_STR_DNAME_CN, serverGroup.getCnServer());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_SERVERGROUP, serverGroup.getCnGroup()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}
}
