package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IServerLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.ApplicationLDAP;
import com.librework.console.modelLDAP.ServerLDAP;

public class ServerLDAPDaoImpl implements IServerLDAPDao {
	
	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}

	protected ContextMapper getContextMapper() {
		return new ServerContextMapper();
	}
	
	private static class ServerContextMapper extends AbstractContextMapper {
		
		public Object doMapFromContext(DirContextOperations context) {
			ServerLDAP server = new ServerLDAP();
			server.setCn(context.getStringAttribute(Constant.KEY_STR_ATTR_CN));
			server.setDescription(context.getStringAttribute(Constant.KEY_STR_ATTR_DESCRIPTION));
			server.setDistinguishedName(context.getStringAttribute(Constant.KEY_STR_ATTR_DISTINGUISHEDNAME));
			server.setHost(context.getStringAttribute(Constant.KEY_STR_ATTR_HOST));
			server.setName(context.getStringAttribute(Constant.KEY_STR_ATTR_NAME));
			server.setOperatingSystem(context.getStringAttribute(Constant.KEY_STR_ATTR_OPERATINGSYSTEM));
			server.setServerGroup(context.getStringAttribute(Constant.KEY_STR_ATTR_SERVERGROUP));
			return server;
		}
	}
	
	private Attributes builServerAttributes(final ServerLDAP server) {
		
		Attributes attrs = new BasicAttributes();
		BasicAttribute ocattr = new BasicAttribute(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS);
		ocattr.add(Constant.KEY_STR_FILTERLDAP_TYPE_SERVER);
		attrs.put(ocattr);
		attrs.put(Constant.KEY_STR_ATTR_CN, server.getCn());
		attrs.put(Constant.KEY_STR_ATTR_DESCRIPTION, server.getDescription());
		attrs.put(Constant.KEY_STR_ATTR_HOST, server.getHost());
		attrs.put(Constant.KEY_STR_ATTR_NAME, server.getName());
		attrs.put(Constant.KEY_STR_ATTR_OPERATINGSYSTEM, server.getOperatingSystem());
		attrs.put(Constant.KEY_STR_ATTR_SERVERGROUP, server.getServerGroup());
		return attrs;
	}

	@Override
	public boolean createServer(ServerLDAP server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_SERVERS);
			dn.add(Constant.KEY_STR_DNAME_CN, server.getCn());
			ldapTemplate.bind(dn, null, builServerAttributes(server));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public ServerLDAP getServerCn(String cn) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_SERVERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_SERVER))
			.and(new EqualsFilter(Constant.KEY_STR_ATTR_CN, cn));
		List<ServerLDAP> servers = ldapTemplate.search(dn, filter.encode(), getContextMapper());
		if (! servers.isEmpty()) {
			return servers.get(0);
		}
		return null;
	}

	@Override
	public List<ServerLDAP> getAllServers() {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_SERVERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		EqualsFilter filter = new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_SERVER);
		return ldapTemplate.search(dn, filter.encode(), getContextMapper());
	}

	@Override
	public boolean updateServer(ServerLDAP server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_SERVERS);
			dn.add(Constant.KEY_STR_DNAME_CN, server.getCn());
			ldapTemplate.rebind(dn, null, builServerAttributes(server));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean deleteServer(ServerLDAP server) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_SERVERS);
			dn.add(Constant.KEY_STR_DNAME_CN, server.getCn());
			ldapTemplate.unbind(dn);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}
	
	
}
