package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IUserAppLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.UserAppLDAP;

public class UserAppLDAPDaoImpl implements IUserAppLDAPDao {
	
	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	
	protected ContextMapper getApplicationsUserContextMapper() {
		return new ApplicationsUserContextMapper();
	}
	
	private static class ApplicationsUserContextMapper extends AbstractContextMapper {
		
		public Object doMapFromContext(DirContextOperations context) {
			UserAppLDAP userApp = new UserAppLDAP();
			userApp.setApplicationsUser(context.getStringAttributes(Constant.KEY_STR_ATTR_PROGID));
			return userApp;
		}
	}

	@Override
	public boolean createUserApp(UserAppLDAP userApp) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_USERS);
			dn.add(Constant.KEY_STR_DNAME_CN, userApp.getCnUser());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.ADD_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_PROGID, userApp.getCnApplication()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public List<UserAppLDAP> getApplicationsUser(String cnUser) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_PERSON)).and
			(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cnUser));
		 List usersApp = ldapTemplate.search(dn, filter.encode(), getApplicationsUserContextMapper());
		
		if (! usersApp.isEmpty()) {
			return usersApp;
		}
		
		return null;
	}

	@Override
	public boolean deleteUserApp(UserAppLDAP userApp) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_USERS);
			dn.add(Constant.KEY_STR_DNAME_CN, userApp.getCnUser());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_PROGID, userApp.getCnApplication()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

}
