package com.librework.console.daoLDAP.impl;

import java.util.Arrays;
import java.util.List;

import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;

import com.librework.console.daoLDAP.IUserGroupLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.GroupUserLDAP;
import com.librework.console.modelLDAP.UserGroupLDAP;
import com.librework.console.modelLDAP.UserLDAP;

public class UserGroupLDAPDaoImpl implements IUserGroupLDAPDao {
	
	private LdapTemplate ldapTemplate;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	
	//Recuperar el contextMapper de la clase Usuarios
	protected ContextMapper getGroupsContextMapper(){
		return new GroupsUserGroupContextMapper();
	}
	//Mapeo del contexto de UserLDAP con sus atributos
	private static class GroupsUserGroupContextMapper extends AbstractContextMapper {

		public Object doMapFromContext(DirContextOperations context) {
			UserGroupLDAP usergroupLDAP = new UserGroupLDAP();
			usergroupLDAP.setGroupsUser(context.getStringAttributes(Constant.KEY_STR_ATTR_MEMBER));
			return usergroupLDAP;			
		}
	}
	
	//Recuperar el contextMapper de la clase Usuarios
		protected ContextMapper getUsersContextMapper(){
			return new UsersUserGroupContextMapper();
		}
		//Mapeo del contexto de UserLDAP con sus atributos
		private static class UsersUserGroupContextMapper extends AbstractContextMapper {

			public Object doMapFromContext(DirContextOperations context) {
				UserGroupLDAP usergroupLDAP = new UserGroupLDAP();
				usergroupLDAP.setUsersGroup(context.getStringAttributes(Constant.KEY_STR_ATTR_MEMBEROF));
				return usergroupLDAP;			
			}
		}
	
	@Override
	public boolean createUserGroup(UserGroupLDAP usergroup) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_BUILTIN);
			dn.add(Constant.KEY_STR_DNAME_CN, usergroup.getCnGroup());
			//Creamos un grupo a un usario
			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.ADD_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_MEMBER, usergroup.getDnUser()));
			ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
		}
		return blnRespuesta;
	}

	@Override
	public boolean removeUserGroup(UserGroupLDAP usergroup) {
		// TODO Auto-generated method stub
				boolean blnRespuesta = false;
				try {
					DistinguishedName dn = new DistinguishedName();
					dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_BUILTIN);
					dn.add(Constant.KEY_STR_DNAME_CN, usergroup.getCnGroup());
					//Creamos un grupo a un usario
					ModificationItem[] mods = new ModificationItem[1];
					mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_MEMBER, usergroup.getDnUser()));
					ldapTemplate.modifyAttributes(dn, mods);
					blnRespuesta = true;
				} catch (Exception e) {
					System.out.println(e.toString());
				}
				return blnRespuesta;
	}

	@Override
	public List getUserGroupByUser(String cnUser) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_PERSON)).and
			(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cnUser));
		 List usersGroup = ldapTemplate.search(dn, filter.encode(), getUsersContextMapper());
		
		if (! usersGroup.isEmpty()) {
			return usersGroup;
		}
		
		return null;
	}

	@Override
	public List<UserGroupLDAP> getUserGroupByGroup(String cnGroup) {
		// TODO Auto-generated method stub
		
				DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_BUILTIN);
				System.out.println(ldapTemplate.getContextSource().toString());
				AndFilter filter = new AndFilter();
				filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_GROUP)).and
					(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cnGroup));
				List usersGroup = ldapTemplate.search(dn, filter.encode(), getGroupsContextMapper());
				
				if (! usersGroup.isEmpty()) {
					return usersGroup;
				}
				
				return null;
	}


}
