package com.librework.console.daoLDAP.impl;

import java.util.List;

import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;

import org.springframework.ldap.core.AttributesMapper;
import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextAdapter;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DirContextProcessor;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.simple.ParameterizedContextMapper;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;
import org.springframework.ldap.filter.WhitespaceWildcardsFilter;



import com.librework.console.daoLDAP.IUserLDAPDao;
import com.librework.console.general.Constant;
import com.librework.console.modelLDAP.UserLDAP;

public class UserLDAPDaoImpl implements IUserLDAPDao {

	private LdapTemplate ldapTemplate;
	
	//Declaracin de variables para el tipo de cuento igual que lmaccess.h
	private static int UF_ACCOUNTDISABLE = 0x0002;
	private static int UF_PASSWD_NOTREQD = 0x0020;
	private static int UF_PASSWD_CANT_CHANGE = 0x0040;
	private static int UF_NORMAL_ACCOUNT = 0x0200;
	private static int UF_DONT_EXPIRE_PASSWD = 0x10000;
	private static int UF_PASSWORD_EXPIRED = 0x800000;
	
	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}
	//Recuperar el contextMapper de la clase Usuarios
	protected ContextMapper getContextMapper(){
		return new UserContextMapper();
	}
	//Mapeo del contexto de UserLDAP con sus atributos
	private static class UserContextMapper extends AbstractContextMapper {

		public Object doMapFromContext(DirContextOperations context) {
			UserLDAP userLDAP = new UserLDAP();
			userLDAP.setCn(context.getStringAttribute(Constant.KEY_STR_ATTR_CN));
			userLDAP.setSAMAccountName(context.getStringAttribute(Constant.KEY_STR_ATTR_SAMACCOUNTNAME));
			userLDAP.setUnicodePwd (context.getStringAttribute(Constant.KEY_STR_ATTR_UNICODEPWD));
			userLDAP.setUserAccountControl(context.getStringAttribute(Constant.KEY_STR_ATTR_USERACCOUNTCONTROL));
			userLDAP.setName(context.getStringAttribute(Constant.KEY_STR_ATTR_NAME));
			userLDAP.setDescription(context.getStringAttribute(Constant.KEY_STR_ATTR_DESCRIPTION));
			userLDAP.setDisplayName(context.getStringAttribute(Constant.KEY_STR_ATTR_DISPLAYNAME));
			userLDAP.setRegisteredAddress(context.getStringAttribute(Constant.KEY_STR_ATTR_REGISTEREDADDRESS));
			userLDAP.setTelephoneNumber(context.getStringAttribute(Constant.KEY_STR_ATTR_TELEPHONENUMBER));
			userLDAP.setMail(context.getStringAttribute(Constant.KEY_STR_ATTR_MAIL));
			userLDAP.setLastLogoff(context.getStringAttribute(Constant.KEY_STR_ATTR_LASTLOGOFF));
			userLDAP.setLastLogon(context.getStringAttribute(Constant.KEY_STR_ATTR_LASTLOGON));
			userLDAP.setDistinguishedName(context.getStringAttribute(Constant.KEY_STR_ATTR_DISTINGUISHEDNAME));
			return userLDAP;			
		}
	}
	
	private Attributes buildUserAttributes(final UserLDAP user) {
		
		Attributes attrs = new BasicAttributes();
		BasicAttribute ocattr = new BasicAttribute(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS);
		ocattr.add(Constant.KEY_STR_FILTERLDAP_TYPE_USER);
		attrs.put(ocattr);
		attrs.put(Constant.KEY_STR_ATTR_CN, user.getCn());
		attrs.put(Constant.KEY_STR_ATTR_SAMACCOUNTNAME, user.getSAMAccountName());
		attrs.put(Constant.KEY_STR_ATTR_NAME, user.getName());
		attrs.put(Constant.KEY_STR_ATTR_DESCRIPTION, user.getDescription());
		attrs.put(Constant.KEY_STR_ATTR_DISPLAYNAME, user.getDisplayName());
		attrs.put(Constant.KEY_STR_ATTR_REGISTEREDADDRESS, user.getRegisteredAddress());
		if (user.getTelephoneNumber() != null) {
			attrs.put(Constant.KEY_STR_ATTR_TELEPHONENUMBER, user.getTelephoneNumber());
		}
		attrs.put(Constant.KEY_STR_ATTR_MAIL, user.getMail());
		//Tener en cuenta que se necesita crear el objeto de usuario antes de poder
        // establecer la contrasea. Por lo tanto como el usuario se crea sin
        // contrasea, AccountControl debe de establecerse de la siguiente forma
		attrs.put(Constant.KEY_STR_ATTR_USERACCOUNTCONTROL,Integer.toString(UF_NORMAL_ACCOUNT + UF_PASSWD_NOTREQD + UF_PASSWORD_EXPIRED+ UF_ACCOUNTDISABLE));
		
		return attrs;
	}
	
	@Override
	public boolean createUser(UserLDAP user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_USERS);
			dn.add(Constant.KEY_STR_DNAME_CN, user.getCn());
			ldapTemplate.bind(dn, null, buildUserAttributes(user));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
			
		}
		return blnRespuesta;
	}
	
	@Override
	public List getAllUsersNames() {
		// TODO Auto-generated method stub
		
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		EqualsFilter filter = new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS,Constant.KEY_STR_FILTERLDAP_TYPE_PERSON);
		
		//filter.and(new EqualsFilter()).and(new WhitespaceWildcardsFilter("cn",name));
		return ldapTemplate.search(dn, filter.encode(),
				new AttributesMapper() {
					public Object mapFromAttributes(Attributes attrs)
							throws NamingException {
						return attrs.get(Constant.KEY_STR_ATTR_NAME).get();
					}
				});
	}

	@Override
	public List getAllUsers() {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		EqualsFilter filter = new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS,Constant.KEY_STR_FILTERLDAP_TYPE_PERSON);
		return ldapTemplate.search(dn, filter.encode(), getContextMapper());
	}
	
	@Override
	public UserLDAP getUserCn(String cn) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_PERSON)).and
			(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cn));
		List<UserLDAP> users = ldapTemplate.search(dn, filter.encode(), getContextMapper());
		
		if (! users.isEmpty()) {
			return users.get(0);
		}
		
		return null;
	}
	
	@Override
	public UserLDAP getUserDistinguishedName(String distinguishedName) {
		// TODO Auto-generated method stub
		DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
		System.out.println(ldapTemplate.getContextSource().toString());
		AndFilter filter = new AndFilter();
		filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_PERSON)).and
			(new EqualsFilter(Constant.KEY_STR_ATTR_DISTINGUISHEDNAME, distinguishedName));
		
		List<UserLDAP> users = ldapTemplate.search(dn, filter.encode(), getContextMapper());
		
		if (! users.isEmpty()) {
			return users.get(0);
		}
		
		return null;
	}
	
	@Override
	public boolean authenticate(String cn, String password) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName(Constant.KEY_STR_DNAME_CN_USERS);
			AndFilter filter = new AndFilter();
			filter.and(new EqualsFilter(Constant.KEY_STR_FILTERLDAP_OBJECTCLASS, Constant.KEY_STR_FILTERLDAP_TYPE_PERSON)).and
				(new EqualsFilter(Constant.KEY_STR_DNAME_CN, cn));
			blnRespuesta = ldapTemplate.authenticate(dn, filter.toString(), password);
		} catch (Exception e) {
			blnRespuesta = false;
		}
		return blnRespuesta;
		
	}
	
	@Override
	public boolean updateUser(UserLDAP user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN, Constant.KEY_STR_DNAME_USERS);
			dn.add(Constant.KEY_STR_DNAME_CN, user.getCn());
			ldapTemplate.rebind(dn, null, buildUserAttributes(user));
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
			
		}
		return blnRespuesta;
	}
	@Override
	public boolean deleteUser(UserLDAP user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_USERS);
			dn.add(Constant.KEY_STR_DNAME_CN, user.getCn());
			ldapTemplate.unbind(dn);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
			
		}
		return blnRespuesta;
	}
	
	@Override
	public boolean setPasswordUser(UserLDAP user) {
		// TODO Auto-generated method stub
		boolean blnRespuesta = false;
		
		try {
			DistinguishedName dn = new DistinguishedName();
			dn.add(Constant.KEY_STR_DNAME_CN,Constant.KEY_STR_DNAME_USERS);
			dn.add(Constant.KEY_STR_DNAME_CN, user.getCn());
			//Establecer la contrasea en una modificacin en LDAP
            //Tener en cuenta que tambien hay que cambiar el AccountControl
            //podemos forzar que el usuario tenga que cambiar la contrasea despues del primer login
            //o que la contrasea nunca expire
            ModificationItem[] mods = new ModificationItem[2];
       
            //Reemplezar el attributo "unicdodePwd" con un nuevo valor
            //La contrasea debe tener Unicode y una cadena entre comillas
            String newQuotedPassword = "\"" + user.getUnicodePwd() + "\"";
            byte[] newUnicodePassword = newQuotedPassword.getBytes("UTF-16LE");

            mods[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_UNICODEPWD, newUnicodePassword));
            mods[1] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, new BasicAttribute(Constant.KEY_STR_ATTR_USERACCOUNTCONTROL,Integer.toString(UF_NORMAL_ACCOUNT + UF_DONT_EXPIRE_PASSWD)));
       
            // Ejecutanto la actualizacin
            ldapTemplate.modifyAttributes(dn, mods);
			blnRespuesta = true;
		} catch (Exception e) {
			System.out.println(e.toString());
			
		}
		return blnRespuesta;
	}
	
	
}
