package com.librework.console.managed.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.primefaces.context.RequestContext;

import com.librework.console.general.Constant;
import com.librework.console.managed.IGroupServerManaged;
import com.librework.console.model.GroupServer;
import com.librework.console.model.Server;
import com.librework.console.model.ServerGroup;
import com.librework.console.service.IGroupServerService;
import com.librework.console.service.IServerService;
import com.librework.console.utils.LanguageUtils;
import com.librework.console.utils.PropertiesUtils;

@ManagedBean(name="groupServerBean")
@ViewScoped
public class GroupServerManagedImpl implements IGroupServerManaged, Serializable {

	private static final long serialVersionUID = 1L;
	
	@ManagedProperty(value="#{GroupServerService}")
	private IGroupServerService groupServerService;
	
	@ManagedProperty(value="#{ServerService}")
	private IServerService serverService;
	
	@ManagedProperty(value="#{logon}")
	private LogonManagedImpl logon;
	
	private List<GroupServer> tableGroupServer = new ArrayList<GroupServer>();
	private List<GroupServer> filteredtableGroupServer = new ArrayList<GroupServer>();
	private GroupServer selectedGroupServer;
	//Datos grupo
	private long grs_id;
	private String grs_description;
	private boolean groupCreated;
	private List<Server> tableServerGroup;
	private List<Server> filteredtableServerGroup;
	
	public GroupServerManagedImpl() {
		tableGroupServer = new ArrayList<GroupServer>();
		filteredtableGroupServer = new ArrayList<GroupServer>();
		selectedGroupServer = new GroupServer();
		grs_id = 0;
		grs_description = "";
		groupCreated = false;
		tableServerGroup = new ArrayList<Server>();
		filteredtableServerGroup = new ArrayList<Server>();
	}
	
	@PostConstruct
	public void init() {
		try {
			getGroupServer();
		} catch (Exception e) {
			System.out.println("ERROR GROUPSERVER:" + e.toString());
		}
	}
	

	@Override
	public void getGroupServer() {
		// TODO Auto-generated method stub
		//GenericApplicationContext appContext = AppUtils.getContext();
		//appContext.refresh();
		List<GroupServer> listGroupServer = getGroupServerService().getAllGroupServer();
		if (listGroupServer != null) {
			if (!listGroupServer.isEmpty()) {
				setTableGroupServer(listGroupServer);
			}
		}
	}

	@Override
	public void createGroupServer() {
		FacesMessage msg = null;
		LanguageUtils lang = new LanguageUtils();
		try {
			if (lfCheckData()) {
				PropertiesUtils props = new PropertiesUtils(Constant.CONFIG_FILE_LIBREWORK);
				GroupServer groupServer = new GroupServer();
				groupServer.setGrs_description(grs_description);
				groupServer.setGrs_datmod(new Date());
				groupServer.setGrs_usemod(logon.getUserId());
				getGroupServerService().createGroupServer(groupServer);
				groupCreated = true;
				getGroupServer();
				msg = new FacesMessage(FacesMessage.SEVERITY_INFO, lang.getMessage("gen_correct"),
						lang.getMessage("gen_save_ok"));
				FacesContext.getCurrentInstance().addMessage(null, msg);
				RequestContext.getCurrentInstance().execute(Constant.WEB_CLOSE_MSG_NEWGROUPSERVER);
			} 
		} catch (Exception e) {
			msg = new FacesMessage(FacesMessage.SEVERITY_FATAL, lang.getMessage("gen_error"),
					lang.getMessage("gen_save_ko"));
			FacesContext.getCurrentInstance().addMessage(null, msg);
		}
	}

	@Override
	public void dialogNewGroupServer() {
		grs_id = 0;
		grs_description = "";
		groupCreated = false;
		RequestContext.getCurrentInstance().execute(Constant.WEB_MSG_NEWGROUPSERVER);
	}
	
	@Override
	public void getServerGroup() {
		List<Server> lstTemp = new ArrayList<Server>();
		if (selectedGroupServer.getGrs_id() != 0) {
			List<ServerGroup> serverGroups = getGroupServerService().getAllServer(selectedGroupServer);
			if (serverGroups != null) {
				for (int i = 0; i < serverGroups.size(); i++) {
					lstTemp.add(serverGroups.get(i).getServerGroupPK().getServer());
				}
				setTableServerGroup(lstTemp);
			}
			setTableServerGroup(lstTemp);
		}
		
	}
	
	private boolean lfCheckData() {
		boolean blnCorrect = true;
		FacesMessage msg = null;
		LanguageUtils lang = new LanguageUtils();
		if (getGrs_description().equals("")) {
			blnCorrect = false;
			msg = new FacesMessage(FacesMessage.SEVERITY_INFO, lang.getMessage("gen_description"),
					lang.getMessage("gen_requiered"));
			FacesContext.getCurrentInstance().addMessage(null, msg);
		}
		return blnCorrect;
	}
	
	
	
	
	
	
	/**
	 * @return the groupServerService
	 */
	public IGroupServerService getGroupServerService() {
		return groupServerService;
	}


	/**
	 * @param groupServerService the groupServerService to set
	 */
	public void setGroupServerService(IGroupServerService groupServerService) {
		this.groupServerService = groupServerService;
	}


	/**
	 * @return the tableGroupServer
	 */
	public List<GroupServer> getTableGroupServer() {
		return tableGroupServer;
	}


	/**
	 * @param tableGroupServer the tableGroupServer to set
	 */
	public void setTableGroupServer(List<GroupServer> tableGroupServer) {
		this.tableGroupServer = tableGroupServer;
	}


	/**
	 * @return the filteredtableGroupServer
	 */
	public List<GroupServer> getFilteredtableGroupServer() {
		return filteredtableGroupServer;
	}


	/**
	 * @param filteredtableGroupServer the filteredtableGroupServer to set
	 */
	public void setFilteredtableGroupServer(
			List<GroupServer> filteredtableGroupServer) {
		this.filteredtableGroupServer = filteredtableGroupServer;
	}


	/**
	 * @return the selectedGroupServer
	 */
	public GroupServer getSelectedGroupServer() {
		return selectedGroupServer;
	}


	/**
	 * @param selectedGroupServer the selectedGroupServer to set
	 */
	public void setSelectedGroupServer(GroupServer selectedGroupServer) {
		this.selectedGroupServer = selectedGroupServer;
		getServerGroup();
	}

	/**
	 * @return the serverService
	 */
	public IServerService getServerService() {
		return serverService;
	}

	/**
	 * @param serverService the serverService to set
	 */
	public void setServerService(IServerService serverService) {
		this.serverService = serverService;
	}

	/**
	 * @return the logon
	 */
	public LogonManagedImpl getLogon() {
		return logon;
	}

	/**
	 * @param logon the logon to set
	 */
	public void setLogon(LogonManagedImpl logon) {
		this.logon = logon;
	}

	/**
	 * @return the grs_id
	 */
	public long getGrs_id() {
		return grs_id;
	}

	/**
	 * @param grs_id the grs_id to set
	 */
	public void setGrs_id(long grs_id) {
		this.grs_id = grs_id;
	}

	/**
	 * @return the grs_description
	 */
	public String getGrs_description() {
		return grs_description;
	}

	/**
	 * @param grs_description the grs_description to set
	 */
	public void setGrs_description(String grs_description) {
		this.grs_description = grs_description;
	}

	/**
	 * @return the groupCreated
	 */
	public boolean isGroupCreated() {
		return groupCreated;
	}

	/**
	 * @param groupCreated the groupCreated to set
	 */
	public void setGroupCreated(boolean groupCreated) {
		this.groupCreated = groupCreated;
	}

	/**
	 * @return the tableServerGroup
	 */
	public List<Server> getTableServerGroup() {
		return tableServerGroup;
	}

	/**
	 * @param tableServerGroup the tableServerGroup to set
	 */
	public void setTableServerGroup(List<Server> tableServerGroup) {
		this.tableServerGroup = tableServerGroup;
	}

	/**
	 * @return the filteredtableServerGroup
	 */
	public List<Server> getFilteredtableServerGroup() {
		return filteredtableServerGroup;
	}

	/**
	 * @param filteredtableServerGroup the filteredtableServerGroup to set
	 */
	public void setFilteredtableServerGroup(List<Server> filteredtableServerGroup) {
		this.filteredtableServerGroup = filteredtableServerGroup;
	}
	

}
