package com.librework.console.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.AssociationOverride;
import javax.persistence.AssociationOverrides;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Table(name="lw_app_server")
@AssociationOverrides({
	@AssociationOverride(name="fk_ase_app_id",
				joinColumns = @JoinColumn(name="ASE_APP_ID", referencedColumnName="APP_ID", insertable=false, updatable=false)),
	@AssociationOverride(name="fk_ase_ser_id",
				joinColumns = @JoinColumn(name="ASE_SER_ID", referencedColumnName="SER_ID", insertable=false, updatable=false))
})
public class AppServer implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	private AppServerPK appServerPK = new AppServerPK();
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="ASE_DATMOD")
	private Date ase_datmod;

	@Column(name="ASE_USEMOD")
	private String ase_usemod;
	
	//Bidirectional relations with lw_server, lw_applications
	@SuppressWarnings("unused")
	@Column(name="ASE_APP_ID", nullable=false, updatable=false, insertable=false)
	private Long fk_ase_app_id;
	
	@SuppressWarnings("unused")
	@Column(name="ASE_SER_ID", nullable=false, updatable=false, insertable=false)
	private String fk_ase_ser_id;

	public AppServer(AppServerPK appServerPK, Date ase_datmod,
			String ase_usemod, Long fk_ase_app_id, String fk_ase_ser_id) {
		super();
		this.appServerPK = appServerPK;
		this.ase_datmod = ase_datmod;
		this.ase_usemod = ase_usemod;
		this.fk_ase_app_id = fk_ase_app_id;
		this.fk_ase_ser_id = fk_ase_ser_id;
	}

	public AppServer() {
		
	}

	/**
	 * @return the appServerPK
	 */
	public AppServerPK getAppServerPK() {
		return appServerPK;
	}

	/**
	 * @param appServerPK the appServerPK to set
	 */
	public void setAppServerPK(AppServerPK appServerPK) {
		this.appServerPK = appServerPK;
	}

	/**
	 * @return the ase_datmod
	 */
	public Date getAse_datmod() {
		return ase_datmod;
	}

	/**
	 * @param ase_datmod the ase_datmod to set
	 */
	public void setAse_datmod(Date ase_datmod) {
		this.ase_datmod = ase_datmod;
	}

	/**
	 * @return the ase_usemod
	 */
	public String getAse_usemod() {
		return ase_usemod;
	}

	/**
	 * @param ase_usemod the ase_usemod to set
	 */
	public void setAse_usemod(String ase_usemod) {
		this.ase_usemod = ase_usemod;
	}

	/**
	 * @return the fk_ase_app_id
	 */
	public Long getFk_ase_app_id() {
		return fk_ase_app_id;
	}

	/**
	 * @param fk_ase_app_id the fk_ase_app_id to set
	 */
	public void setFk_ase_app_id(Long fk_ase_app_id) {
		this.fk_ase_app_id = fk_ase_app_id;
	}

	/**
	 * @return the fk_ase_ser_id
	 */
	public String getFk_ase_ser_id() {
		return fk_ase_ser_id;
	}

	/**
	 * @param fk_ase_ser_id the fk_ase_ser_id to set
	 */
	public void setFk_ase_ser_id(String fk_ase_ser_id) {
		this.fk_ase_ser_id = fk_ase_ser_id;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((appServerPK == null) ? 0 : appServerPK.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AppServer other = (AppServer) obj;
		if (appServerPK == null) {
			if (other.appServerPK != null)
				return false;
		} else if (!appServerPK.equals(other.appServerPK))
			return false;
		return true;
	}

	
}
