package com.librework.console.model;

import com.librework.console.model.SystemOperating;


import static javax.persistence.GenerationType.IDENTITY;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinColumns;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

@Entity
@Table(name="lw_applications")
public class Applications implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy=IDENTITY)
	@Column(name="APP_ID", unique=true, nullable= false)
	private long app_id;

	@Column(name="APP_NAME", nullable=false)
	private String app_name;

	@Column(name="APP_DISPLAY_NAME", nullable=false)
	private String app_display_name;

	@Column(name="APP_DESCRIPTION")
	private String app_description;

	@Column(name="APP_PATH_EXE")
	private String app_path_exe;

	@Lob
	@Basic(fetch=FetchType.LAZY)
	@Column(name="APP_ICON")
	private byte[] app_icon;

	@ManyToOne(fetch=FetchType.EAGER)
	@JoinColumns( { @JoinColumn(name = "APP_SYO_ID", referencedColumnName = "SYO_ID") }) 
	private SystemOperating fk_app_syo_id;

	@Temporal(TemporalType.DATE)
	@Column(name="APP_DATMOD")
	private Date app_datmod;

	@Column(name="APP_USEMOD")
	private String app_usemod;
	
	//Bidierectional asociation with AppGroup
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_agr_app_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<AppGroup> appGroups = new ArrayList<AppGroup>();

	//Bidirectional asociation with AppServer
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_ase_app_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<AppServer> appServers = new ArrayList<AppServer>();
	
	//Biderectional asociation with UserApp
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_uap_app_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<UserApp> userApps = new ArrayList<UserApp>();
	
	//Biderectional asociation with GroupUserApp
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_gua_app_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<GroupUserApp> groupUserApps = new ArrayList<GroupUserApp>();

	public Applications(long app_id, String app_name, String app_display_name,
			String app_description, String app_path_exe, byte[] app_icon,
			SystemOperating fk_app_syo_id, Date app_datmod, String app_usemod,
			List<AppGroup> appGroups, List<AppServer> appServers,
			List<UserApp> userApps, List<GroupUserApp> groupUserApps) {
		super();
		this.app_id = app_id;
		this.app_name = app_name;
		this.app_display_name = app_display_name;
		this.app_description = app_description;
		this.app_path_exe = app_path_exe;
		this.app_icon = app_icon;
		this.fk_app_syo_id = fk_app_syo_id;
		this.app_datmod = app_datmod;
		this.app_usemod = app_usemod;
		this.appGroups = appGroups;
		this.appServers = appServers;
		this.userApps = userApps;
		this.groupUserApps = groupUserApps;
	}

	public Applications() {

	}

	/**
	 * @return the app_id
	 */
	public long getApp_id() {
		return app_id;
	}

	/**
	 * @param app_id the app_id to set
	 */
	public void setApp_id(long app_id) {
		this.app_id = app_id;
	}

	/**
	 * @return the app_name
	 */
	public String getApp_name() {
		return app_name;
	}

	/**
	 * @param app_name the app_name to set
	 */
	public void setApp_name(String app_name) {
		this.app_name = app_name;
	}

	/**
	 * @return the app_display_name
	 */
	public String getApp_display_name() {
		return app_display_name;
	}

	/**
	 * @param app_display_name the app_display_name to set
	 */
	public void setApp_display_name(String app_display_name) {
		this.app_display_name = app_display_name;
	}

	/**
	 * @return the app_description
	 */
	public String getApp_description() {
		return app_description;
	}

	/**
	 * @param app_description the app_description to set
	 */
	public void setApp_description(String app_description) {
		this.app_description = app_description;
	}

	/**
	 * @return the app_path_exe
	 */
	public String getApp_path_exe() {
		return app_path_exe;
	}

	/**
	 * @param app_path_exe the app_path_exe to set
	 */
	public void setApp_path_exe(String app_path_exe) {
		this.app_path_exe = app_path_exe;
	}

	/**
	 * @return the app_icon
	 */
	public byte[] getApp_icon() {
		return app_icon;
	}

	/**
	 * @param app_icon the app_icon to set
	 */
	public void setApp_icon(byte[] app_icon) {
		this.app_icon = app_icon;
	}

	/**
	 * @return the fk_app_syo_id
	 */
	public SystemOperating getFk_app_syo_id() {
		return fk_app_syo_id;
	}

	/**
	 * @param fk_app_syo_id the fk_app_syo_id to set
	 */
	public void setFk_app_syo_id(SystemOperating fk_app_syo_id) {
		this.fk_app_syo_id = fk_app_syo_id;
	}

	/**
	 * @return the app_datmod
	 */
	public Date getApp_datmod() {
		return app_datmod;
	}

	/**
	 * @param app_datmod the app_datmod to set
	 */
	public void setApp_datmod(Date app_datmod) {
		this.app_datmod = app_datmod;
	}

	/**
	 * @return the app_usemod
	 */
	public String getApp_usemod() {
		return app_usemod;
	}

	/**
	 * @param app_usemod the app_usemod to set
	 */
	public void setApp_usemod(String app_usemod) {
		this.app_usemod = app_usemod;
	}

	/**
	 * @return the appGroups
	 */
	public List<AppGroup> getAppGroups() {
		return appGroups;
	}

	/**
	 * @param appGroups the appGroups to set
	 */
	public void setAppGroups(List<AppGroup> appGroups) {
		this.appGroups = appGroups;
	}

	/**
	 * @return the appServers
	 */
	public List<AppServer> getAppServers() {
		return appServers;
	}

	/**
	 * @param appServers the appServers to set
	 */
	public void setAppServers(List<AppServer> appServers) {
		this.appServers = appServers;
	}

	/**
	 * @return the userApps
	 */
	public List<UserApp> getUserApps() {
		return userApps;
	}

	/**
	 * @param userApps the userApps to set
	 */
	public void setUserApps(List<UserApp> userApps) {
		this.userApps = userApps;
	}

	/**
	 * @return the groupUserApps
	 */
	public List<GroupUserApp> getGroupUserApps() {
		return groupUserApps;
	}

	/**
	 * @param groupUserApps the groupUserApps to set
	 */
	public void setGroupUserApps(List<GroupUserApp> groupUserApps) {
		this.groupUserApps = groupUserApps;
	}

	
}
