package com.librework.console.model;

import static javax.persistence.GenerationType.IDENTITY;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinColumns;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

@Entity
@Table(name="lw_server")
public class Server implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@Column(name="SER_ID", unique=true, nullable= false)
	private String ser_id;

	@Column(name="SER_DESCRIPTION")
	private String ser_description;

	@ManyToOne(fetch=FetchType.EAGER)
	@JoinColumns( { @JoinColumn(name = "SER_SYO_ID", referencedColumnName = "SYO_ID") }) 
	private SystemOperating fk_ser_syo_id;

	@Column(name="SER_HOST")
	private String ser_host;

	@Column(name="SER_MEMTOT")
	private float ser_mentot;

	@Column(name="SER_MEMUSE")
	private float ser_menuse;

	@Column(name="SER_CPUUSE")
	private float ser_cpuuse;

	@Column(name="SER_REDUSE")
	private float ser_reduse;

	@Temporal(TemporalType.DATE)
	@Column(name="SER_DATMOD")
	private Date ser_datmod;

	@Column(name="SER_USEMOD")
	private String ser_usemod;
	
	//Bidierectional asociation with ServerHDD
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_shd_ser_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<ServerHDD> serverHDDs = new ArrayList<ServerHDD>();

	//Bidirectional asociation with AppServer
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_ase_ser_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<AppServer> appServers = new ArrayList<AppServer>();
	
	//Biderectional asociation with UserApp
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_uap_ser_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<UserApp> userApps = new ArrayList<UserApp>();
	
	//Biderectional asociation with ServerGroup
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_seg_ser_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<ServerGroup> serverGroups = new ArrayList<ServerGroup>();

	public Server(String ser_id, String ser_description,
			SystemOperating fk_ser_syo_id, String ser_host, float ser_mentot,
			float ser_menuse, float ser_cpuuse, float ser_reduse,
			Date ser_datmod, String ser_usemod, List<ServerHDD> serverHDDs,
			List<AppServer> appServers, List<UserApp> userApps,
			List<ServerGroup> serverGroups) {
		super();
		this.ser_id = ser_id;
		this.ser_description = ser_description;
		this.fk_ser_syo_id = fk_ser_syo_id;
		this.ser_host = ser_host;
		this.ser_mentot = ser_mentot;
		this.ser_menuse = ser_menuse;
		this.ser_cpuuse = ser_cpuuse;
		this.ser_reduse = ser_reduse;
		this.ser_datmod = ser_datmod;
		this.ser_usemod = ser_usemod;
		this.serverHDDs = serverHDDs;
		this.appServers = appServers;
		this.userApps = userApps;
		this.serverGroups = serverGroups;
	}

	public Server() {
		
	}

	/**
	 * @return the ser_id
	 */
	public String getSer_id() {
		return ser_id;
	}

	/**
	 * @param ser_id the ser_id to set
	 */
	public void setSer_id(String ser_id) {
		this.ser_id = ser_id;
	}

	/**
	 * @return the ser_description
	 */
	public String getSer_description() {
		return ser_description;
	}

	/**
	 * @param ser_description the ser_description to set
	 */
	public void setSer_description(String ser_description) {
		this.ser_description = ser_description;
	}

	/**
	 * @return the fk_ser_syo_id
	 */
	public SystemOperating getFk_ser_syo_id() {
		return fk_ser_syo_id;
	}

	/**
	 * @param fk_ser_syo_id the fk_ser_syo_id to set
	 */
	public void setFk_ser_syo_id(SystemOperating fk_ser_syo_id) {
		this.fk_ser_syo_id = fk_ser_syo_id;
	}

	/**
	 * @return the ser_host
	 */
	public String getSer_host() {
		return ser_host;
	}

	/**
	 * @param ser_host the ser_host to set
	 */
	public void setSer_host(String ser_host) {
		this.ser_host = ser_host;
	}

	/**
	 * @return the ser_mentot
	 */
	public float getSer_mentot() {
		return ser_mentot;
	}

	/**
	 * @param ser_mentot the ser_mentot to set
	 */
	public void setSer_mentot(float ser_mentot) {
		this.ser_mentot = ser_mentot;
	}

	/**
	 * @return the ser_menuse
	 */
	public float getSer_menuse() {
		return ser_menuse;
	}

	/**
	 * @param ser_menuse the ser_menuse to set
	 */
	public void setSer_menuse(float ser_menuse) {
		this.ser_menuse = ser_menuse;
	}

	/**
	 * @return the ser_cpuuse
	 */
	public float getSer_cpuuse() {
		return ser_cpuuse;
	}

	/**
	 * @param ser_cpuuse the ser_cpuuse to set
	 */
	public void setSer_cpuuse(float ser_cpuuse) {
		this.ser_cpuuse = ser_cpuuse;
	}

	/**
	 * @return the ser_reduse
	 */
	public float getSer_reduse() {
		return ser_reduse;
	}

	/**
	 * @param ser_reduse the ser_reduse to set
	 */
	public void setSer_reduse(float ser_reduse) {
		this.ser_reduse = ser_reduse;
	}

	/**
	 * @return the ser_datmod
	 */
	public Date getSer_datmod() {
		return ser_datmod;
	}

	/**
	 * @param ser_datmod the ser_datmod to set
	 */
	public void setSer_datmod(Date ser_datmod) {
		this.ser_datmod = ser_datmod;
	}

	/**
	 * @return the ser_usemod
	 */
	public String getSer_usemod() {
		return ser_usemod;
	}

	/**
	 * @param ser_usemod the ser_usemod to set
	 */
	public void setSer_usemod(String ser_usemod) {
		this.ser_usemod = ser_usemod;
	}

	/**
	 * @return the serverHDDs
	 */
	public List<ServerHDD> getServerHDDs() {
		return serverHDDs;
	}

	/**
	 * @param serverHDDs the serverHDDs to set
	 */
	public void setServerHDDs(List<ServerHDD> serverHDDs) {
		this.serverHDDs = serverHDDs;
	}

	/**
	 * @return the appServers
	 */
	public List<AppServer> getAppServers() {
		return appServers;
	}

	/**
	 * @param appServers the appServers to set
	 */
	public void setAppServers(List<AppServer> appServers) {
		this.appServers = appServers;
	}

	/**
	 * @return the userApps
	 */
	public List<UserApp> getUserApps() {
		return userApps;
	}

	/**
	 * @param userApps the userApps to set
	 */
	public void setUserApps(List<UserApp> userApps) {
		this.userApps = userApps;
	}

	/**
	 * @return the serverGroups
	 */
	public List<ServerGroup> getServerGroups() {
		return serverGroups;
	}

	/**
	 * @param serverGroups the serverGroups to set
	 */
	public void setServerGroups(List<ServerGroup> serverGroups) {
		this.serverGroups = serverGroups;
	}

		
}
