package com.librework.console.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static javax.persistence.GenerationType.IDENTITY;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

@Entity
@Table(name="lw_operatingsystem")
public class SystemOperating implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy=IDENTITY)
	@Column(name="SYO_ID", unique=true, nullable= false)
	private long syo_id;

	@Column(name="SYO_DESCRIPTION")
	private String syo_description;

	@Temporal(TemporalType.DATE)
	@Column(name="SYO_DATMOD")
	private Date syo_datmod;

	@Column(name="SYO_USEMOD")
	private String syo_usemod;
	
	//Bidirectional asociation with server
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_ser_syo_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<Server> servers = new ArrayList<Server>();
	
	//Biderectional asociation with applications
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_app_syo_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<Applications> applications = new ArrayList<Applications>();

	public SystemOperating(long syo_id, String syo_description,
			Date syo_datmod, String syo_usemod, List<Server> servers,
			List<Applications> applications) {
		super();
		this.syo_id = syo_id;
		this.syo_description = syo_description;
		this.syo_datmod = syo_datmod;
		this.syo_usemod = syo_usemod;
		this.servers = servers;
		this.applications = applications;
	}

	public SystemOperating() {
	
	}

	/**
	 * @return the syo_id
	 */
	public long getSyo_id() {
		return syo_id;
	}

	/**
	 * @param syo_id the syo_id to set
	 */
	public void setSyo_id(long syo_id) {
		this.syo_id = syo_id;
	}

	/**
	 * @return the syo_description
	 */
	public String getSyo_description() {
		return syo_description;
	}

	/**
	 * @param syo_description the syo_description to set
	 */
	public void setSyo_description(String syo_description) {
		this.syo_description = syo_description;
	}

	/**
	 * @return the syo_datmod
	 */
	public Date getSyo_datmod() {
		return syo_datmod;
	}

	/**
	 * @param syo_datmod the syo_datmod to set
	 */
	public void setSyo_datmod(Date syo_datmod) {
		this.syo_datmod = syo_datmod;
	}

	/**
	 * @return the syo_usemod
	 */
	public String getSyo_usemod() {
		return syo_usemod;
	}

	/**
	 * @param syo_usemod the syo_usemod to set
	 */
	public void setSyo_usemod(String syo_usemod) {
		this.syo_usemod = syo_usemod;
	}

	/**
	 * @return the servers
	 */
	public List<Server> getServers() {
		return servers;
	}

	/**
	 * @param servers the servers to set
	 */
	public void setServers(List<Server> servers) {
		this.servers = servers;
	}

	/**
	 * @return the applications
	 */
	public List<Applications> getApplications() {
		return applications;
	}

	/**
	 * @param applications the applications to set
	 */
	public void setApplications(List<Applications> applications) {
		this.applications = applications;
	}

	
}
