package com.librework.console.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.bean.ManagedBean;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

@Entity
@Table(name="lw_user")
public class User implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@Column(name="USE_ID", unique=true, nullable=false)
	private String use_id;

	@Column(name="USE_NAME", nullable=false)
	private String use_name;
	
	@Column(name="USE_SURNAME1")
	private String use_surname1;
	
	@Column(name="USE_SURNAME2")
	private String use_surname2;
	
	@Column(name="USE_DIRECTION")
	private String use_direction;

	@Column(name="USE_EMAIL")
	private String use_email;
	
	@Column(name="USE_PHONE")
	private String use_phone;

	@Column(name="USE_PASSWORD")
	private String use_password;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="USE_START_LAST_CONNECTION")
	private Date use_start_last_connection;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="USE_END_LAST_CONNECTION")
	private Date use_end_last_connection;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="USE_DATMOD")
	private Date use_datmod;
	
	@Column(name="USE_USEMOD")
	private String use_usemod;
	
	//Bidirectional asociation with UserGroup
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_ugr_use_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<UserGroup> userGroups = new ArrayList<UserGroup>();

	//Bidirectional asociation with UserApp
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_uap_use_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<UserApp> userApps = new ArrayList<UserApp>();
	
	//Bidirectional asociation with UserSite
	@OneToMany(cascade=CascadeType.ALL, mappedBy="fk_usi_use_id", orphanRemoval=true)
	@LazyCollection(LazyCollectionOption.TRUE)
	private List<UserSite> userSites = new ArrayList<UserSite>();

	

	

	public User(String use_id, String use_name, String use_surname1,
			String use_surname2, String use_direction, String use_email,
			String use_phone, String use_password,
			Date use_start_last_connection, Date use_end_last_connection,
			Date use_datmod, String use_usemod, List<UserGroup> userGroups,
			List<UserApp> userApps, List<UserSite> userSites) {
		super();
		this.use_id = use_id;
		this.use_name = use_name;
		this.use_surname1 = use_surname1;
		this.use_surname2 = use_surname2;
		this.use_direction = use_direction;
		this.use_email = use_email;
		this.use_phone = use_phone;
		this.use_password = use_password;
		this.use_start_last_connection = use_start_last_connection;
		this.use_end_last_connection = use_end_last_connection;
		this.use_datmod = use_datmod;
		this.use_usemod = use_usemod;
		this.userGroups = userGroups;
		this.userApps = userApps;
		this.userSites = userSites;
	}

	public User() {
		
	}

	/**
	 * @return the use_id
	 */
	public String getUse_id() {
		return use_id;
	}

	/**
	 * @param use_id the use_id to set
	 */
	public void setUse_id(String use_id) {
		this.use_id = use_id;
	}

	/**
	 * @return the use_name
	 */
	public String getUse_name() {
		return use_name;
	}

	/**
	 * @param use_name the use_name to set
	 */
	public void setUse_name(String use_name) {
		this.use_name = use_name;
	}

	/**
	 * @return the use_surname1
	 */
	public String getUse_surname1() {
		return use_surname1;
	}

	/**
	 * @param use_surname1 the use_surname1 to set
	 */
	public void setUse_surname1(String use_surname1) {
		this.use_surname1 = use_surname1;
	}

	/**
	 * @return the use_surname2
	 */
	public String getUse_surname2() {
		return use_surname2;
	}

	/**
	 * @param use_surname2 the use_surname2 to set
	 */
	public void setUse_surname2(String use_surname2) {
		this.use_surname2 = use_surname2;
	}

	/**
	 * @return the use_direction
	 */
	public String getUse_direction() {
		return use_direction;
	}

	/**
	 * @param use_direction the use_direction to set
	 */
	public void setUse_direction(String use_direction) {
		this.use_direction = use_direction;
	}

	/**
	 * @return the use_email
	 */
	public String getUse_email() {
		return use_email;
	}

	/**
	 * @param use_email the use_email to set
	 */
	public void setUse_email(String use_email) {
		this.use_email = use_email;
	}

	/**
	 * @return the use_phone
	 */
	public String getUse_phone() {
		return use_phone;
	}

	/**
	 * @param use_phone the use_phone to set
	 */
	public void setUse_phone(String use_phone) {
		this.use_phone = use_phone;
	}

	/**
	 * @return the use_password
	 */
	public String getUse_password() {
		return use_password;
	}

	/**
	 * @param use_password the use_password to set
	 */
	public void setUse_password(String use_password) {
		this.use_password = use_password;
	}

	/**
	 * @return the use_start_last_connection
	 */
	public Date getUse_start_last_connection() {
		return use_start_last_connection;
	}

	/**
	 * @param use_start_last_connection the use_start_last_connection to set
	 */
	public void setUse_start_last_connection(Date use_start_last_connection) {
		this.use_start_last_connection = use_start_last_connection;
	}

	/**
	 * @return the use_end_last_connection
	 */
	public Date getUse_end_last_connection() {
		return use_end_last_connection;
	}

	/**
	 * @param use_end_last_connection the use_end_last_connection to set
	 */
	public void setUse_end_last_connection(Date use_end_last_connection) {
		this.use_end_last_connection = use_end_last_connection;
	}
	
	/**
	 * @return the use_datmod
	 */
	public Date getUse_datmod() {
		return use_datmod;
	}

	/**
	 * @param use_datmod the use_datmod to set
	 */
	public void setUse_datmod(Date use_datmod) {
		this.use_datmod = use_datmod;
	}

	/**
	 * @return the use_usemod
	 */
	public String getUse_usemod() {
		return use_usemod;
	}

	/**
	 * @param use_usemod the use_usemod to set
	 */
	public void setUse_usemod(String use_usemod) {
		this.use_usemod = use_usemod;
	}

	/**
	 * @return the userGroups
	 */
	public List<UserGroup> getUserGroups() {
		return userGroups;
	}

	/**
	 * @param userGroups the userGroups to set
	 */
	public void setUserGroups(List<UserGroup> userGroups) {
		this.userGroups = userGroups;
	}

	/**
	 * @return the userApps
	 */
	public List<UserApp> getUserApps() {
		return userApps;
	}

	/**
	 * @param userApps the userApps to set
	 */
	public void setUserApps(List<UserApp> userApps) {
		this.userApps = userApps;
	}

	/**
	 * @return the userSites
	 */
	public List<UserSite> getUserSites() {
		return userSites;
	}

	/**
	 * @param userSites the userSites to set
	 */
	public void setUserSites(List<UserSite> userSites) {
		this.userSites = userSites;
	}
	
		
}
