#!/bin/bash

function usage()
{
	echo "Usage: $0"
	exit 1
}

rojo()
{
        echo -e "\033[31m${1}\033[0m"
}

azul()
{
        echo -e "\033[36m${1}\033[0m"
}

verde()
{
        echo -e "\033[32m${1}\033[0m"
}

configure_ipaddress()
{


	IFACE=`ifconfig -a | grep "Ethernet" | awk '{print $1}' | grep -m 1 ""`

	if [ -n "${IFACE}" ]
	then
		cat > /etc/network/interfaces << EOF
auto lo
iface lo inet loopback

auto ${IFACE}
iface ${IFACE} inet static
	address ${ADDRESS}
	netmask 255.255.255.0
	gateway ${GATEWAY}
EOF
	fi
/etc/init.d/networking restart

}

configure_samba()
{
	rm -f /etc/samba/smb.conf
	samba-tool domain provision --domain=${DOMAIN} --realm=${DOMAIN}.librework --server-role=dc --dns-backend=SAMBA_INTERNAL --adminpass=${ADMINPASS}
	cp /var/lib/samba/private/krb5.conf /etc/krb5.conf
	cat >> /etc/krb5.conf << EOF
[realms]
        ${DOMAIN}.librework = {
                kdc = localhost:88
	}
EOF
	sed -i /etc/samba/smb.conf -e "s#\[global\]#\[global\]\n\ttemplate shell = /bin/bash\n\tdsdb:schema update allowed = yes\n#g" -e "s/dns forwarder = 127.0.0.1/dns forwarder = ${FORWARDER}/g"
	cat >> /etc/samba/smb.conf << EOF
[homes]
	comment = Home Directories
	browseable = no
	read only = no
	create mask = 0700
	directory mask = 0700
	valid users = %S

[printers]
	path = /var/spool/samba
	printable = yes
	printing = CUPS
EOF
	/etc/init.d/sernet-samba-ad restart
	echo "nameserver 127.0.0.1" > /etc/resolv.conf
	sleep 5
	kinit administrator < /home/.librework/adminpassword
	samba-tool domain passwordsettings set --min-pwd-age=0
	samba-tool domain passwordsettings set --max-pwd-age=999
	samba-tool domain passwordsettings set --complexity=off
	cd /provision/samba
	/provision/samba/updatesamba.sh ${ADDRESS} dc=${DOMAIN},dc=librework Administrator
}

configure_guacamole()
{
	cat > /etc/guacamole/guacamole.properties <<EOF
#    Guacamole - Clientless Remote Desktop
#    Copyright (C) 2010  Michael Jumper
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Hostname and port of guacamole proxy
guacd-hostname: localhost
guacd-port:     4822

# Auth provider class (authenticates user/pass combination, needed if using the provided login screen)
#auth-provider: net.sourceforge.guacamole.net.basic.BasicFileAuthenticationProvider
#basic-user-mapping: /etc/guacamole/user-mapping.xml

auth-provider: net.sourceforge.guacamole.net.auth.ldap.LDAPAuthenticationProvider
ldap-hostname: localhost
ldap-port: 389
ldap-suffix: cn=Users,dc=${DOMAIN},dc=librework
ldap-guac-protocol: rdp
ldap-guac-protocol-hostname: ${ADDRESS}
ldap-guac-protocol-port: 3389
EOF
}

configure_chroot()
{
	apt-get --force-yes -y install schroot debootstrap
	mkdir /home/linux
	debootstrap wheezy /home/linux
	cp /etc/apt/sources.list /home/linux/etc/apt/
	chroot /home/linux apt-get update
	chroot /home/linux mount -t proc proc /proc
	chroot /home/linux mount -t devpts devpts /dev/pts
	chroot /home/linux env DEBIAN_FRONTEND=noninteractive
	for i in libreoffice libreoffice-kde libreoffice-l10n-es libreoffice-help-es myspell-dictionary-es ark kwrite kcalc okular gwenview krita kde-l10n-es
	do
		chroot /home/linux apt-get --force-yes -y install $i
	done
	sed -i "s/# es_ES.UTF-8 UTF-8/es_ES.UTF-8 UTF-8/g" /home/linux/etc/locale.gen
	chroot /home/linux locale-gen
	chroot /home/linux umount /proc
	chroot /home/linux umount /dev/pts
	sed -i -e "s/passwd:\s\+compat/passwd:\t\tcompat winbind/g" -e "s/group:\s\+compat/group:\t\tcompat winbind/g" /home/linux/etc/nsswitch.conf
}

configure_desktux()
{
	cat > /etc/desktux/desktux.ini << EOF
WindowsRDPServer=${NETWORK}.3
WindowsRDPPort=3389
SambaDomain=${DOMAIN}
SambaServer=lxserver
SambaSharedPath=\shared
SharedPath=/home/shared
ChrootEnvironment=true
ChrootHome=/home/linux
ChrootMount=/var/tmp
LibreworkServerPort=4445
LdapServer=${ADDRESS}
LdapServerPort=389
LdapSuffix="dc=${DOMAIN},dc=librework"
EOF
}

if [ -n "$1" ]
then
	usage
fi

if [ -e /home/.librework ]
then
	if [ -e /home/.librework/network ]
	then
		NETWORK=`cat /home/.librework/network`
		ADDRESS=${NETWORK}.2
		GATEWAY=${NETWORK}.1
		configure_ipaddress
	fi

	if [ -e /home/.librework/adminpassword ] && [ -e /home/.librework/ldapdc ] && [ -e /home/.librework/dnsforwarder ]
	then
		DOMAIN=`cat /home/.librework/ldapdc`
		FORWARDER=`cat /home/.librework/dnsforwarder`
		ADMINPASS=`cat /home/.librework/adminpassword`
		configure_samba
		configure_guacamole
		configure_chroot
		configure_desktux
	fi
	/etc/init.d/tomcat7 restart
	rm -rf /home/.librework
fi
