#!/bin/bash

NETWORK=$1
ADMINPASSWORD=$2
LDAPDC=$3
DNS=$4
SIZE=$5
TEMPLATE=$6
LVGROUP=$7
VMNAME=$8
MEMORY=$9
LVDEVICE=${10}
VNETWORK=${11}
MNT=mnt.tmp
DATADISKTYPE=sparse

function usage()
{
	echo "Usage: $0 <network> <administrator password> <ldap dc> <dns forwarder> <data image size in GB><template vm file> <target vgroup> <vmname> <memory> <lvdevice> <vnetwork>"$
        echo "Example:  $0 192.168.100 Dinsa.2013 dinsa 8.8.8.8 200 /var/lib/xen/templates/linux.tar.gz vgxen lxserver 8192 /dev/md3 dmz0"
        exit 1
}

rojo()
{
        echo -e "\033[31m${1}\033[0m"
}

azul()
{
        echo -e "\033[36m${1}\033[0m"
}

verde()
{
        echo -e "\033[32m${1}\033[0m"
}

if [ -n "${12}" ] || [ -z "${11}" ]
then
	usage
fi

apt-get install kpartx



azul "Checking OS template size..."
OSSIZE=`tar ztvf ${TEMPLATE} | awk '{print $3}'`
let DATASIZE=SIZE*1024*1024*1024
azul "Checking total size (including data)..."
let TOTALSIZE=OSSIZE+DATASIZE+1024*1024*1024
azul "Initializing physical volume for use with logical volumes..."
pvcreate -ff  ${LVDEVICE}
azul "Creating  volume group..."
vgcreate ${LVGROUP} ${LVDEVICE}
azul "Creating logical volume..."
lvcreate ${LVGROUP} -L ${TOTALSIZE}B -n ${VMNAME}


#cd /usr/lib/xcp/sm
ln -s /usr/lib/xcp/sm/LVHDSR.py /usr/lib/xcp/sm/LVHDSR
/etc/init.d/xcp-xapi restart





#pv -r -b -e -p ${TEMPLATE} | tar zxvO > /dev/${LVGROUP}/${TARGET}

azul "Creating storage repository..."
SRUUID=`xe sr-create type=lvhd device-config:device=/dev/${LVGROUP}/${VMNAME} name-label=${VMNAME}-storage`
verde "sr created!.\n"

# Create VM
azul "Creating virtual machine..."
VMUUID=`xe vm-install sr-uuid=${SRUUID} new-name-label=${VMNAME} template="Debian Wheezy 7.0 (64-bit)"`
verde "vm created!.\n"

azul "Resizing auto-generated OS virtual disk..."
OSUUID=`xe vbd-list vm-uuid=${VMUUID} | grep -w "vdi-uuid" | awk -F ': ' '{print $2}'`
xe vdi-resize uuid=${OSUUID} disk-size=${OSSIZE}
verde "Done."

#OSUUID=`xe vdi-create sr-uuid=${SRUUID} name-label=${VMNAME}-os type=user virtual-size=${OSSIZE}`
azul "Creating data virtual disk..."
DATAUUID=`xe vdi-create sr-uuid=${SRUUID} name-label=${VMNAME}-data type=user virtual-size=${DATASIZE}`
verde "vdi created!.\n"

azul "Decompresing OS template...\n"
if [ -e template.img ]
then
	echo "Using existing template"
else
	tar zxOf ${TEMPLATE} > template.img
fi

azul "Importing template.\n"
xe vdi-import filename=template.img uuid=${OSUUID}
verde "Done.\n"

#Mount data VDI

DOM0UUID=`xe vm-list | grep -B2 "Control domain on host:" | grep "uuid" | awk '{print $5}'`
VBDUUID=`xe vbd-create device=0 vm-uuid=${DOM0UUID} vdi-uuid=${DATAUUID} bootable=false mode=RW type=Disk`
xe vbd-plug uuid=${VBDUUID}

DEVICE=/dev/`xe vbd-param-list uuid=${VBDUUID} | grep -w "device" | awk -F ': ' '{print $2}'`

parted -s ${DEVICE} mklabel msdos


parted -s ${DEVICE} mkpart primary ext3 0 100%

partprobe ${DEVICE}

PART=`kpartx -l ${DEVICE} | awk '{print $1}'`

PART=/dev/mapper/${PART}

kpartx -a ${DEVICE}
 
mkfs.ext3 ${PART}



# Mount filesystem
azul "Mounting filesystem..."

MNT=/tmp/lwcreateVM

mkdir -p ${MNT}
umount ${MNT}
mount -t ext3 ${PART} ${MNT}
verde "Done.\n"

# Create configuration files on filesystem
azul "Creating configuration files..."
mkdir -p ${MNT}/.librework
echo ${NETWORK} > ${MNT}/.librework/network
echo ${ADMINPASSWORD} > ${MNT}/.librework/adminpassword
echo ${LDAPDC} > ${MNT}/.librework/ldapdc
echo ${DNS} > ${MNT}/.librework/dnsforwarder
verde "Done.\n"

# Unmount filesystem
azul "Unmounting filesystem..."
umount ${MNT}
kpartx -d ${DEVICE}

xe vbd-unplug uuid=${VBDUUID}
xe vbd-destroy uuid=${VBDUUID}

verde "Done.\n"

#Create VM

#VMUUID=`xe vm-install sr-uuid=${SRUUID} new-name-label=${VMNAME} template="Debian Wheezy 7.0 (64-bit)"`

#verde "VM Create.\n"


# Assign memory

xe vm-memory-limits-set uuid=${VMUUID} static-min=${MEMORY}MiB dynamic-min=${MEMORY}MiB dynamic-max=${MEMORY}MiB static-max=${MEMORY}MiB


#Attach VDI OS

#VBDOSUUID=`xe vbd-create device=0 vm-uuid=${VMUUID} vdi-uuid=${OSUUID} bootable=true mode=RW type=Disk`

#xe vbd-plug uuid=${VBDOSUUID}

#Attach VDI DATA

VBDDATAUUID=`xe vbd-create device=1 vm-uuid=${VMUUID} vdi-uuid=${DATAUUID} bootable=false mode=RW type=Disk`

# this is only needed for hotplug conection if the VM is running
# xe vbd-plug uuid=${VBDDATAUUID}

# Configuring virtual network interfaces for VMs

VNETWORKUUID=`xe network-list name-label=${VNETWORK} --minimal`

VIFUUID=`xe vif-create vm-uuid=${VMUUID} network-uuid=${VNETWORKUUID} device=0`

# this is only needed for hotplug conection if the VM is running
# If the VM is halted, VIF will be attached in VM start 
# xe vif-plug uuid=${VIFUUID}



xe vm-param-set uuid=${VMUUID} other-config:install-repository="http://ftp.de.debian.org/debian/"

#Start VM

xe vm-start uuid=${VMUUID}

sleep 5

xe vm-shutdown uuid=${VMUUID}

sleep 5

xe vm-start uuid=${VMUUID}




# Configure shorewall firewall entry

# Configure nginx entry
