#!/bin/bash

TEMPLATE=$1
LVGROUP=$2
VMNAME=$3
MEMORY=$4
VNETWORK=$5
VHDSIZE=$6



function usage()
{
        echo "Usage: $0 <template> <lvgroup> <vmname> <memory> <vnetwork> <VHD size(GB)>"
        echo "Example:  $0 winserver-template.img vgxen winserver 8192 dmz0 100"
        exit 1
}

rojo()
{
        echo -e "\033[31m${1}\033[0m"
}

azul()
{
        echo -e "\033[36m${1}\033[0m"
}

verde()
{
        echo -e "\033[32m${1}\033[0m"
}


if [ -n "${7}" ] || [ -z "${6}" ]
then
        usage
fi

apt-get install pv

azul "Checking OS template size..."
OSSIZE=`tar ztvf ${TEMPLATE} | awk '{print $3}'`
let USERSIZE=${VHDSIZE}
let TOTALSIZE=USERSIZE*1024*1024*1024+OSSIZE+1024*1024*1024
let DISKSIZE=TOTALSIZE-1024*1024*1024

azul "Creating logical volume..."
lvcreate ${LVGROUP} -L ${TOTALSIZE}B -n ${VMNAME}


azul "Creating storage repository..."
SRUUID=`xe sr-create type=lvhd device-config:device=/dev/${LVGROUP}/${VMNAME} name-label=${VMNAME}-storage`
verde "sr created!.\n"

azul "Creating virtual machine..."
VMUUID=`xe vm-install sr-uuid=${SRUUID} new-name-label=${VMNAME} template="Other install media"`
verde "vm created!.\n"

azul "Creating OS disk..."
OSUUID=`xe vdi-create sr-uuid=${SRUUID} name-label=${VMNAME}-os type=user virtual-size=${DISKSIZE}`
verde "vdi created!.\n"

azul "Decompresing OS template...\n"
if [ -e template.img ]
then
        echo "Using existing template"
else
        tar zxOf ${TEMPLATE} > template.img
	#pv -s ${OSSIZE} ${TEMPLATE} | tar zxO > template.img
fi

azul "Importing template.\n"
xe vdi-import filename=template.img uuid=${OSUUID}
verde "Done.\n"

#resize vdi
#xe vdi-resize uuid=${OSUUID} disk-size=${TOTALSIZE}


# Assign memory
xe vm-memory-limits-set uuid=${VMUUID} static-min=${MEMORY}MiB dynamic-min=${MEMORY}MiB dynamic-max=${MEMORY}MiB static-max=${MEMORY}MiB


#Attach VDI OS
VBDOSUUID=`xe vbd-create device=0 vm-uuid=${VMUUID} vdi-uuid=${OSUUID} bootable=true mode=RW type=Disk`
xe vbd-plug uuid=${VBDOSUUID}

VNETWORKUUID=`xe network-list name-label=${VNETWORK} --minimal`
VIFUUID=`xe vif-create vm-uuid=${VMUUID} network-uuid=${VNETWORKUUID} device=0`


