#!/bin/bash

TEMPLATE_VM="android-template"
USERS_FILE="/etc/librework/users"
LOCK_FILE="/var/lock/lwandroid.run"
VNCBASEPORT=5900
LWBASEPORT=30314
MAXCOUNTER=40

ACTION=$1

wait_temp()
{
	lockfile -1 -r ${MAXCOUNTER} ${LOCK_FILE}
}

usage()
{
	echo "Usage: $0 <ACTION> [ <PARAMETERS> ]"
	echo "Valid actions: create | delete | start | stop | vncport | lwport"
	if [ -z "$1" ]
	then
		echo "Valid parameters: type $0 <action> without parameters to show valid ones for each action"
	else
		echo $1
	fi
	bye 1
}

user_id()
{
	cat ${USERS_FILE} | grep -w "${1}" | awk -F '=' '{print $1}'
}

min_user()
{
	i=0
	sort -n ${USERS_FILE} |
	{
		while read line
		do
			NUMBER=`echo $line | awk -F '=' '{print $1}'`
			if [ "${NUMBER}" == "${i}" ]
			then
				let i=i+1
			else
				break
			fi
		done

		echo ${i}
	}
}

remove_user()
{
	sed -i -e "/^[0-9]*=${1}$/d" ${USERS_FILE}
}

exists_user()
{
	grep -w "${1}" ${USERS_FILE}
}

exists_vm()
{
	VBoxManage list vms | grep -w "android-${1}"
}

bye()
{
	rm -f ${LOCK_FILE}
	exit ${1}
}

if [ -z "${ACTION}" ]
then
	usage
fi

wait_temp

if [ "${ACTION}" == "create" ]
then
	USERNAME=$2

	if [ -z "${USERNAME}" ] || [ -n "$3" ]
	then
		usage "Valid parameters: $0 create <USERNAME>"
	fi

	EXISTS_VM=`exists_vm ${USERNAME}`
	if [ -z "${EXISTS_VM}" ]
	then
		EXISTS_USER=`exists_user ${USERNAME}`
		if [ -z "${EXISTS_USER}" ]
		then
			USER_ID=`min_user`
			echo "${USER_ID}=${USERNAME}" >> ${USERS_FILE}
		else
			USER_ID=`user_id ${USERNAME}`
		fi
		let NATPORT=LWBASEPORT+USER_ID
		VBoxManage clonevm "${TEMPLATE_VM}" --name "android-${USERNAME}" --register
		VBoxManage modifyvm "android-${USERNAME}" --natpf1 "guestlw,tcp,,${NATPORT},,${LWBASEPORT}"

		bye $?
	else
		echo "This virtual machine currently exists"
		bye 2
	fi
fi

if [ "${ACTION}" == "delete" ]
then
	USERNAME=$2

	if [ -z "${USERNAME}" ] || [ -n "$3" ]
	then
		usage "Valid parameters: $0 delete <USERNAME>"
	fi

	EXISTS_VM=`exists_vm ${USERNAME}`
	if [ -n "${EXISTS_VM}" ]
	then
		VBoxManage modifyvm "android-${USERNAME}" --natpf1 delete "guestlw"
		VBoxManage unregistervm "android-${USERNAME}" --delete
	else
		echo "The virtual machine doesn't exist"
		bye 2
	fi

	EXISTS_USER=`exists_user ${USERNAME}`
	if [ -n "${EXISTS_USER}" ]
	then
		remove_user ${USERNAME}
	else
		echo "The user doesn't exist"
		bye 2
	fi

	bye 0
fi

if [ "${ACTION}" == "start" ]
then
	USERNAME=$2
	PASSWORD=$3

	if [ -z "${PASSWORD}" ] || [ -n "$4" ]
	then
		usage "Valid parameters: $0 start <USERNAME> <PASSWORD>"
	fi

	USER_ID=`user_id ${USERNAME}`
	let PORT=VNCBASEPORT+USER_ID

	VBoxHeadless -v "on" -e "TCP/Ports=${PORT}" -e "TCP/Address=0.0.0.0" -e "VNCPassword=${PASSWORD}" -s "android-${USERNAME}" &

	bye $?
fi

if [ "${ACTION}" == "stop" ]
then
	USERNAME=$2

	if [ -z "${USERNAME}" ] || [ -n "$3" ]
	then
		usage "Valid parameters: $0 stop <USERNAME>"
	fi

	VBoxManage controlvm "android-${USERNAME}" poweroff

	bye $?
fi

if [ "${ACTION}" == "vncport" ]
then
	USERNAME=$2

	if [ -z "${USERNAME}" ] || [ -n "$3" ]
	then
		usage "Valid parameters: $0 vncport <USERNAME>"
	fi

	EXISTS_USER=`exists_user ${USERNAME}`
	if [ -n "${EXISTS_USER}" ]
	then
		USER_ID=`user_id ${USERNAME}`
		let PORT=VNCBASEPORT+USER_ID

		echo ${PORT}

		bye 0
	else
		bye 2
	fi
fi

if [ "${ACTION}" == "lwport" ]
then
	USERNAME=$2

	if [ -z "${USERNAME}" ] || [ -n "$3" ]
	then
		usage "Valid parameters: $0 lwport <USERNAME>"
	fi

	EXISTS_USER=`exists_user ${USERNAME}`
	if [ -n "${EXISTS_USER}" ]
	then
		USER_ID=`user_id ${USERNAME}`
		let NATPORT=LWBASEPORT+USER_ID

		echo ${NATPORT}

		bye 0
	else
		bye 2
	fi
fi

usage
