#include <stdlib.h>
#include <signal.h>

#include "proxyserver.h"

#include <QCoreApplication>
#include <QObject>
#include <QString>
#include <QStringList>
#include <QHostAddress>

#define DEFAULT_PORT 3389
#define DEFAULT_REMOTE_PORT 3389

ProxyServer * server;

QString error = QString::null;

void usage(char * command)
{
	printf("usage: %s <options>\n", command);
	printf("Options:\n");
	printf("  -a, --address ADDRESS           Serve on IP address ADDRESS\n");
	printf("  -p, --port PORT                 Serve on port number PORT\n");
	printf("  -r, --remoteaddress ADDRESS     Serve on IP address ADDRESS\n");
	printf("  -q, --remoteport PORT           Serve on port number PORT\n");
}

QString checkArguments(int argc, char ** argv, QString parameter1, QString parameter2)
{
	QStringList validArguments;
	validArguments << "-a" << "--address" << "-p" << "--port" << "-r" << "--remoteaddress" << "-q" << "--remoteport";
	QStringList neededArguments;
	neededArguments << "-r";

	for (int i=1; i<argc; i++)
	{
		if (validArguments.indexOf(argv[i]) == -1)
			error = QObject::tr("Invalid option. Check parameters.");

		if (argv[i] == parameter1 || argv[i] == parameter2)
		{
			if (i + 1 < argc)
			{
				return QString(argv[i+1]);
			} else {
				error = QObject::tr("The parameter needs a value.");
				return QString::null;
			}		
		} else {
			i++;
		}
	}
	return QString::null;
}

void finish(int signal)
{
	if (signal == SIGINT) // CTRL-C pressed
	{
		printf("%s\n", QObject::tr("Stopping LibreWork RDP proxy server...").toUtf8().data());
		delete server;
		printf("%s\n", QObject::tr("done.").toUtf8().data());
		exit(0);
	}
}

int main(int argc, char ** argv)
{
	signal(SIGINT, finish);

	QCoreApplication app(argc, argv);
        app.setOrganizationName("Librework");
        app.setApplicationName("Librework RDP Proxy Server");

	int port = DEFAULT_PORT;
	QHostAddress address = QHostAddress::Any;

	QString strport = checkArguments(argc, argv, "--port", "-p");
	QString straddress = checkArguments(argc, argv, "--address", "-a");
	QString remoteport = checkArguments(argc, argv, "--remoteport", "-q");
	QString remoteaddress = checkArguments(argc, argv, "--remoteaddress", "-r");

	if ( remoteaddress.isEmpty() )
		error = QObject::tr("The remote address must be given.");

	if (error != QString::null)
	{
		printf("%s\n", error.toUtf8().data());
		usage(argv[0]);
		exit(1);
	}

	if (!strport.isNull())
		port = strport.toInt();

	if (!straddress.isNull())
		address = QHostAddress(straddress);

	int rPort = remoteport.toInt();
	if (rPort == 0)
		rPort = DEFAULT_REMOTE_PORT;
	server = new ProxyServer(remoteaddress, rPort);

	if (!server->listen(address, port))
	{
		printf("%s\n", QObject::tr("Unable to start LibreWork RDP proxy server").toUtf8().data());
		exit(2);
	}

	return app.exec();
}
