#include "proxyserverthread.h"

#include <QtNetwork>

#define TIMEOUT 30000

struct pam_response *reply;

/**
        @short ProxyServerThread constructor

        @param int socketDescriptor is the socket descriptor
        @param QObject* parent is the parent object of the new thread
	@param QString address
	@param int port

	LWServerThread is the class that creates the thread that will
	allow multiple connections to the server (LWServer)
*/
ProxyServerThread::ProxyServerThread(int socketDescriptor, QObject *parent, QString address, int port)
	: QThread(parent), socketDescriptor(socketDescriptor)
{
	this->remoteAddress = address;
	this->remotePort = port;
}

ProxyServerThread::~ProxyServerThread()
{
}

void ProxyServerThread::run()
{
	qDebug("Running thread...");

	// Set server socket
	QSslSocket socket;
	if (!socket.setSocketDescriptor(socketDescriptor))
	{
		emit error(socket.error());
		qDebug("Bad socket");
		quit();
	}

	tcpSocket = &socket;

	QSslConfiguration sslConfiguration;
	QFile certFile(QString("cert.pem"));
	QFile keyFile(QString("key.pem"));
	certFile.open(QIODevice::ReadOnly);
	keyFile.open(QIODevice::ReadOnly);
	QSslCertificate certificate(&certFile, QSsl::Pem);
	QSslKey sslKey(&keyFile, QSsl::Rsa, QSsl::Pem);
	certFile.close();
	keyFile.close();
	sslConfiguration.setPeerVerifyMode(QSslSocket::VerifyNone);
	sslConfiguration.setLocalCertificate(certificate);
	sslConfiguration.setPrivateKey(sslKey);
	sslConfiguration.setProtocol(QSsl::TlsV1SslV3);
	tcpSocket->setSslConfiguration(sslConfiguration);
	tcpSocket->startServerEncryption();


/*

	tcpSocket->setLocalCertificate("/home/pechanove/tuxum/desktux/src/proxyrdp/cert.pem");
	tcpSocket->setPrivateKey("/home/pechanove/tuxum/desktux/src/proxyrdp/key.pem");
*/

	qRegisterMetaType<QList<QSslError> >( "QSslErrorList" );

	connect(tcpSocket, SIGNAL(encrypted()), this, SLOT(readData()), Qt::DirectConnection);
	connect(tcpSocket, SIGNAL(disconnected()), this, SLOT(quit()));
	connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(readData()), Qt::DirectConnection);
	connect(tcpSocket, SIGNAL(sslErrors(const QList<QSslError>&)), this, SLOT(sslErrors(const QList<QSslError>&)));

/*
	if (tcpSocket->waitForEncrypted())
	{
		qDebug("Connected server and started encryption");
	} else {
		qDebug("Cannot encrypt server communication");
		qDebug() << tcpSocket->errorString();
		quit();
	}
*/

	// Set client socket
	remoteSocket = new QSslSocket();

	connect(remoteSocket, SIGNAL(disconnected()), this, SLOT(quit()));
	connect(remoteSocket, SIGNAL(readyRead()), this, SLOT(readRemoteData()), Qt::DirectConnection);
	connect(remoteSocket, SIGNAL(sslErrors(const QList<QSslError>&)), this, SLOT(remoteSslErrors(const QList<QSslError>&)));

	remoteSocket->setProtocol(QSsl::TlsV1);
/*
	remoteSocket->addCaCertificates("/home/pechanove/tuxum/desktux/src/proxyrdp/cert.pem");
	remoteSocket->setLocalCertificate("/home/pechanove/tuxum/desktux/src/proxyrdp/client-cert.pem", QSsl::Pem);
	remoteSocket->setPrivateKey("/home/pechanove/tuxum/desktux/src/proxyrdp/client-key.pem", QSsl::Rsa, QSsl::Pem);
	//remoteSocket->setPeerVerifyName( "192.168.100.3" );
	if(remoteSocket->localCertificate().isNull())
	{
		qDebug("Error: Local certificate is null");
	}

	if (remoteSocket->privateKey().isNull())
	{
		qDebug("Error: Private key is null");
	}
*/

	QList<QSslCertificate> trustedCas = QSslCertificate::fromPath("cert.pem");
	if (trustedCas.empty())
	{
		qFatal("Error: no trusted Cas");
	}
	remoteSocket->setCaCertificates(trustedCas);
	remoteSocket->setPrivateKey("client-key.pem");
	remoteSocket->setLocalCertificate("client-cert.pem");

	remoteSocket->ignoreSslErrors();

	qDebug("Connecting to host %s on port %d", remoteAddress.toUtf8().data(), remotePort);
	remoteSocket->connectToHostEncrypted( remoteAddress, remotePort, QIODevice::ReadWrite );

	if (remoteSocket->waitForConnected())
	{
		qDebug("Connected to remote host");
	} else {
		qDebug("Cannot connect to remote host");
		qDebug() << remoteSocket->errorString();
		quit();
	}

	qDebug() << "Local Certificate:" << remoteSocket->localCertificate().toPem().data();
	qDebug() << "Peer Certificate:" << remoteSocket->peerCertificate().toPem().data();

	qDebug() << "Local certificate issuer info common name:" << remoteSocket->localCertificate().issuerInfo(QSslCertificate::CommonName);
	qDebug() << "Local certificate subject info common name:" << remoteSocket->localCertificate().subjectInfo(QSslCertificate::CommonName);
	qDebug() << "Peer certificate issuer info common name:" << remoteSocket->peerCertificate().issuerInfo(QSslCertificate::CommonName);
	qDebug() << "Peer certificate subject info common name:" << remoteSocket->peerCertificate().subjectInfo(QSslCertificate::CommonName);

	qDebug() << "Peer verify name" << remoteSocket->peerVerifyName();

	if ( tcpSocket->state() == QAbstractSocket::ConnectedState && remoteSocket->state() == QAbstractSocket::ConnectedState )
	{
		exec();
	} else {
		qDebug("Something failed");
		return;
	}
}

void ProxyServerThread::readData()
{
	qDebug("Ready to read %d bytes", (int)tcpSocket->bytesAvailable());
	qDebug("Getting remote certificate...");
	QSslCertificate remoteCertificate = tcpSocket->peerCertificate();
	qDebug(remoteCertificate.issuerInfo( QSslCertificate::CommonName ).join().toUtf8().data() );
	qDebug(remoteCertificate.toPem().data() );
	qDebug("Protocol: %d", tcpSocket->protocol() );

	QByteArray question = tcpSocket->readAll();

	//qDebug(question.data());

	remoteSocket->write( question );
}

void ProxyServerThread::readRemoteData()
{
	qDebug("Ready to read %d bytes from remote", (int)remoteSocket->bytesAvailable());
	QByteArray question = remoteSocket->readAll();

	//qDebug(question.data());

	tcpSocket->write( question );
}

void ProxyServerThread::sslErrors(const QList<QSslError> & errorList)
{
	for (int i=0; i<errorList.count(); i++)
	{
		qDebug("SSL Error: %s", errorList.at(i).errorString().toUtf8().data());
	}
}

void ProxyServerThread::remoteSslErrors(const QList<QSslError> & errorList)
{
	for (int i=0; i<errorList.count(); i++)
	{
		qDebug("Remote SSL Error: %s", errorList.at(i).errorString().toUtf8().data());
	}
}

int functionConversation(int, const struct pam_message **, struct pam_response ** resp, void *)  
{  
	resp[0] = reply;
	return PAM_SUCCESS;
}

bool ProxyServerThread::validate( QString username, QString password )
{
	if ( username.isEmpty() || password.isEmpty() )
	{
		return false;
	}

	char * user = new char[username.length() + 1];
	char * pass = new char[ password.length() + 1 ];
	char * service = new char[6];

	strcpy( user, username.toUtf8().data() );
	strcpy( pass, password.toUtf8().data() );
	strcpy( service, "login" );

	bool res = true;

	qDebug("Username: %s, password: %s", user, pass);
	//qDebug("Username: %s", user);

	const struct pam_conv local_conversation = { functionConversation, NULL };
	pam_handle_t *local_auth_handle = NULL; // this gets set by pam_start

	int retval;
	retval = pam_start(service, user, &local_conversation, &local_auth_handle);

	if (retval != PAM_SUCCESS)
	{
		res = false;
	}
  
	if ( res )
	{
		reply = (struct pam_response *)malloc(sizeof(struct pam_response));

		reply[0].resp = strdup( pass );
		reply[0].resp_retcode = 0;
		retval = pam_authenticate(local_auth_handle, 0);

		if (retval != PAM_SUCCESS)
		{
			res = false;
		}
	}

	retval = pam_end(local_auth_handle, retval);

	if ( res )
	{
		if (retval != PAM_SUCCESS)
		{
			res = false;
		}
	}

	delete[] user;
	delete[] pass;
	delete[] service;

	return res;
}
