#ifndef PROXYSERVERTHREAD_H
#define PROXYSERVERTHREAD_H

#include <QThread>
#include <QMutex>
#include <QSslSocket>
#include <QSslCertificate>
#include <QString>
#include <QByteArray>
#include <QSettings>

#include <security/pam_appl.h> 

typedef QList<QSslError> QSSLErrorList;

class ProxyServerThread : public QThread
{
	Q_OBJECT

	public:
		ProxyServerThread(int socketDescriptor, QObject *parent, QString address, int port);
		~ProxyServerThread();

		void run();

	signals:
		void error(QTcpSocket::SocketError socketError);
		void configured();

	private slots:
		void readData();
		void readRemoteData();
		void sslErrors(const QList<QSslError> & errorList);
		void remoteSslErrors(const QList<QSslError> & errorList);
		bool validate( QString username, QString password );

	private:
		QSslSocket * tcpSocket;
		QSslSocket * remoteSocket;
		int socketDescriptor;
		QString username;
		QString password;
		QMutex mutex;
		QString remoteAddress;
		int remotePort;
};

#endif
