#include "initdialog.h"
#include "ui_initdialog.h"

InitDialog::InitDialog(QString username, QString password, QWidget *parent) :
	QDialog(parent),
	ui(new Ui::InitDialog)
{
	interfaces = QNetworkInterface::allInterfaces();

	ui->setupUi(this);
	ui->usernameLineEdit->setText(username);
	ui->passwordLineEdit->setText(password);
	ui->progressBar->setRange(0, 100);
#ifndef SET_INTERFACE
	ui->interfaceLabel->hide();
	ui->interfaceComboBox->hide();
#endif
#ifdef SET_INTERFACE
	for (int i = 0; i < interfaces.count(); i++)
	{
		for (int j = 0; j < interfaces.at(i).addressEntries().count(); j++)
		{
			QHostAddress interfaceAddress = interfaces.at(i).addressEntries().at(j).ip();
			if ( interfaceAddress.protocol() == QAbstractSocket::IPv4Protocol)
			{
				if ( interfaceAddress.toString() == address )
				{
					selected = i;
				}
				QString name = interfaces.at(i).humanReadableName() + " (" + interfaceAddress.toString() +")";
				ui->interfaceComboBox->addItem(QIcon(":/network.png"), name, interfaceAddress.toString());
			}
		}
	}

	if ( ui->interfaceComboBox->count() == 0 )
	{
		QMessageBox::critical(this, tr("No network interface"), tr("There are no IPv4 network interfaces on this server"));
		reject();
		QMetaObject::invokeMethod(this, "close", Qt::QueuedConnection);
		return;
	}

	ui->interfaceComboBox->setCurrentIndex( selected );
#endif
}

InitDialog::~InitDialog()
{
	delete ui;
}

void InitDialog::on_buttonBox_accepted()
{
	if ( ui->usernameLineEdit->text().isEmpty() || ui->passwordLineEdit->text().isEmpty() )
	{
		QMessageBox::critical(this, tr("Empty fields"), tr("The admin user name and password fields must not be empty"));
		return;
	}

#ifdef SET_INTERFACE
	QString hostAddress = ui->interfaceComboBox->currentData().toString();
	QSettings settings(CONFIG_FILE, QSettings::IniFormat);
	settings.setValue("Server/ServerPort", hostAddress);
	settings.sync();
#endif
	LWServer * server = new LWServer(ui->usernameLineEdit->text(), ui->passwordLineEdit->text());
	connect(server, SIGNAL(progress(int)), ui->progressBar, SLOT(setValue(int)));
	connect(server, SIGNAL(progressText(QString)), ui->progressLabel, SLOT(setText(QString)));
	server->sendServerInformation(ui->usernameLineEdit->text(), ui->passwordLineEdit->text());
	disconnect(server, SIGNAL(progress(int)), ui->progressBar, SLOT(setValue(int)));
	disconnect(server, SIGNAL(progressText(QString)), ui->progressLabel, SLOT(setText(QString)));
	delete server;
	QMessageBox::information(this, tr("Initialization dialog"), tr("Initialization done"));
}
