#include "lwapplicationcommand.h"

static QHash<long,Application> applicationCache;

QString LWApplicationCommand::getApplicationIds(QString username, MySqlInfo * mySqlInfo)
{
	if ( username.isEmpty() )
	{
		return QString::null;
	}

	QStringList uname = username.split("\\");
	QStringList apps = mySqlInfo->getValues("lw_application_users", "application_id", "user_id='" + uname[uname.count() - 1] + "'");

	QStringList userGroups = mySqlInfo->getValues("lw_usergroup_users", "usergroup_id", "user_id='" + uname[uname.count() - 1] + "'");
	for (int i = 0; i < userGroups.count(); i++)
	{
		apps += mySqlInfo->getValues("lw_application_usergroups", "application_id", "usergroup_id='" + userGroups[i] + "'");

		QStringList groupAppGroups = mySqlInfo->getValues("lw_usergroup_applicationgroups", "applicationgroup_id", "usergroup_id='" + userGroups[i] + "'");
		for (int j = 0; j < groupAppGroups.count(); j++)
		{
			apps += mySqlInfo->getValues("lw_applicationgroup_applications", "application_id", "applicationgroup_id='" + groupAppGroups[j] + "'");
		}
	}

	QStringList userAppGroups = mySqlInfo->getValues("lw_user_applicationgroups", "applicationgroup_id", "user_id='" + uname[uname.count() - 1] + "'");
	for (int i = 0; i < userAppGroups.count(); i++)
	{
		apps += mySqlInfo->getValues("lw_applicationgroup_applications", "application_id", "applicationgroup_id='" + userAppGroups[i] + "'");
	}

	apps.removeDuplicates();
	return apps.join(",");
}

QString LWApplicationCommand::getApplicationExtensionsList(MySqlInfo * mySqlInfo)
{
	return mySqlInfo->getValues("lw_extension", "name").join(",");
}

QString LWApplicationCommand::getApplicationGroupIds(MySqlInfo * mySqlInfo)
{
	return mySqlInfo->getValues("lw_applicationgroup", "id").join(":");
}

QString LWApplicationCommand::getApplicationGroups(MySqlInfo * mySqlInfo)
{
	return mySqlInfo->getValues("lw_applicationgroup", "name").join(":");
}

QString LWApplicationCommand::getApplicationsByGroup( long groupId, MySqlInfo * mySqlInfo )
{
	if ( groupId == -1 )
	{
		QStringList fields;
		fields += "AGR_APP_ID";
		QString queryString = "SELECT id FROM lw_application WHERE lw_application.id NOT IN (SELECT application_id FROM lw_applicationgroup_applications)";
		QList<QStringList> query = mySqlInfo->getQueryValues(queryString, fields);
		if (query.count() > 0)
		{
			return query[0].join(":");
		} else {
			return QString::null;
		}
	} else {
		return mySqlInfo->getValues("lw_applicationgroup_applications", "application_id", "applicationgroup_id=" + QString::number(groupId)).join(":");
	}
}

QByteArray LWApplicationCommand::getApplicationIcon( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).icon;
}

QString LWApplicationCommand::getApplicationName( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).name;
}

QString LWApplicationCommand::getApplicationOnlyOne( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).onlyOne;
}

QString LWApplicationCommand::getApplicationDisplayName( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).name;
}

QString LWApplicationCommand::getApplicationDescription( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).description;
}

QString LWApplicationCommand::getApplicationExecPath( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).execPath;
}

QString LWApplicationCommand::getApplicationArguments( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).arguments;
}

QString LWApplicationCommand::getApplicationWorkingPath( long applicationId, MySqlInfo * mySqlInfo )
{
    return getApplication( applicationId, mySqlInfo ).workingPath;
}

QString LWApplicationCommand::getPreferredServer( long applicationId, double cpuWeight, MySqlInfo * mySqlInfo )
{
	QStringList servers = getApplication( applicationId, mySqlInfo ).serverIds;
	QList<double> cpuUsage;
	QList<double> freeMemory;

	for (int i = 0; i < servers.count(); i++)
	{
		QString serverAddress = getServerAddress( mySqlInfo, servers[i].toLongLong() );
		double cpu = 100.0;
		cpuUsage.append( cpu );
		double freeMem = 0.0;
		freeMemory.append( freeMem );
	}

	return LWSystemCommand::getPreferredServer( cpuUsage, freeMemory, servers, cpuWeight );
}

QString LWApplicationCommand::getServerHosts( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).serverHosts.join(":");
}

QString LWApplicationCommand::getServerIds( long applicationId, MySqlInfo * mySqlInfo )
{
	return getApplication( applicationId, mySqlInfo ).serverIds.join(":");
}

long long LWApplicationCommand::getServerId( QString serverAddress, MySqlInfo * mySqlInfo )
{
	QStringList id = mySqlInfo->getValues("lw_server", "id", "address='" + serverAddress +"'");
	if (id.count() > 0)
	{
		return id[0].toLongLong();
	} else {
		QStringList attributes;
		QList<QVariant> values;
		attributes += "address";
		values += QVariant(serverAddress);
		bool ok = mySqlInfo->insertRecord( "lw_server", attributes, values );
		if (ok)
		{
			id = mySqlInfo->getValues("lw_server", "id", "address='" + serverAddress +"'");
			if (id.count() > 0)
			{
				return id[0].toLongLong();
			} else {
				return 0;
			}
		} else {
			return 0;
		}
	}
}

QString LWApplicationCommand::getServerType( long serverId, MySqlInfo * mySqlInfo )
{
	QStringList type = mySqlInfo->getValues("lw_server", "type", "id=" + QString::number( serverId ) );
	if (type.count() > 0)
	{
		return type[0];
	} else {
		return QString::null;
	}
}

QString LWApplicationCommand::getServerAddress( MySqlInfo * mySqlInfo, long serverId )
{
	QStringList address = mySqlInfo->getValues("lw_server", "address", "id=" + QString::number( serverId ) );
	if (address.count() > 0)
	{
		return address[0];
	} else {
		return QString::null;
	}
}

QString LWApplicationCommand::getApplications( QString extension, MySqlInfo * mySqlInfo )
{
	QString query = "SELECT lw_application_mime_types.application_id "
			" FROM lw_application_mime_types,lw_mimetype_extensions "
			" WHERE lw_mimetype_extensions.extension_id = '" + extension.remove('.') + "' AND lw_application_mime_types.mimetype_id = lw_mimetype_extensions.mimetype_id";

	qDebug() << query;

	QList<QStringList> queryResult = mySqlInfo->getQueryValues(query, QStringList("app_id"));

	QString res;

	if (queryResult.count() > 0)
	{
		if (queryResult[0].count() > 0)
		{
			res = queryResult[0].join(":");
		}
	}

	return res;
}

QByteArray LWApplicationCommand::getUserLogo( QString username, MySqlInfo * mySqlInfo )
{
	if ( username.isEmpty() )
	{
		return QByteArray();
	}

	QStringList uname = username.split("\\");
	QString user = uname[uname.count() - 1];
	QStringList group = mySqlInfo->getValues("lw_usergroup_users", "usergroup_id", "user_id='" + user + "'");

	if ( group.count() >= 1 )
	{
		QStringList logoName = mySqlInfo->getValues("lw_usergroup", "logo", "id=" + group.at(0));

		if ( logoName.count() >= 1 )
		{
			return downloadIcon( logoName.at(0) );
		}
	}
	return QByteArray();
}

QByteArray LWApplicationCommand::downloadIcon( QString iconFile )
{
	QSettings settings(CONFIG_FILE, QSettings::IniFormat);
	QString baseUrl = settings.value( "Server/ConsoleMediaBaseURL", "http://localhost/media/" ).toString();
	QDir rootDir("/");
	rootDir.mkpath( DOWNLOAD_DIR );
	QProcess process;
	QString command = "wget";
	QStringList arguments;
	arguments += "-q";
	arguments += "-O";
	arguments += DOWNLOAD_DIR + QFileInfo(iconFile).fileName();
	arguments += baseUrl + iconFile;
	process.start( command, arguments );
	process.waitForStarted();
	process.waitForFinished();
	if ( process.exitStatus() == QProcess::NormalExit )
	{
		QFile file( DOWNLOAD_DIR + QFileInfo(iconFile).fileName() );
		file.open( QIODevice::ReadOnly );
		QByteArray res = file.readAll();
		file.close();
		return res;
	} else {
		return QByteArray();
	}
}

QString LWApplicationCommand::applicationUpdated( long applicationId )
{
	applicationCache.remove( applicationId );
	return QString::null;
}

Application LWApplicationCommand::getApplication( long applicationId, MySqlInfo * mySqlInfo )
{
	if ( applicationCache.contains( applicationId ) )
	{
		return applicationCache[ applicationId ];
	}

	QStringList fields;

	fields += "name";		// 0
	fields += "description";// 1
	fields += "command";	// 2
	fields += "args";		// 3
	fields += "only_one";	// 4
	fields += "init_path";  // 5
	fields += "icon";		// 6

	QStringList record = mySqlInfo->getRecord("lw_application", "id=" + QString::number( applicationId ), fields );

	Application app;
	QString icon;

	app.applicationId = applicationId;

	if ( record.count() > 0 )
	{
		app.name = record[0];
		app.description = record[1];
		app.execPath = record[2];
		app.arguments = record[3];
		app.onlyOne = record[4];
		app.workingPath = record[5];
		icon = record[6];
	}

	app.icon = downloadIcon( icon );

	app.serverIds = mySqlInfo->getValues("lw_server_applications", "server_id", "application_id=" + QString::number( applicationId ));
	QStringList idsFilter;
	for ( int i = 0; i < app.serverIds.count(); i++ )
	{
		idsFilter += "server_id=" + app.serverIds[i];
	}

	if ( ! idsFilter.isEmpty() )
	{
		app.serverHosts = mySqlInfo->getValues("lw_server", "address", idsFilter.join(" OR "));
	}

	applicationCache.insert( applicationId, app );

	return app;
}
