#ifndef LWAPPLICATIONCOMMAND_H
#define LWAPPLICATIONCOMMAND_H

#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QString>
#include <QStringList>
#include <QProcess>
#include <QHash>
#include <QPixmap>
#include <QSettings>

#include "constants.h"
#include "mysqlinfo.h"
#include "lwsystemcommand.h"

class Application
{
	public:
		long applicationId;
		QString name;
		QString description;
		QString execPath;
        QString workingPath;
		QString arguments;
		QByteArray icon;
		QStringList serverHosts;
		QStringList serverIds;
		QString onlyOne;
};

class LWApplicationCommand : public QObject
{
	Q_OBJECT

	public:
		static QString getApplicationIds(QString username, MySqlInfo * mySqlInfo);
		static QString getApplicationExtensionsList(MySqlInfo * mySqlInfo);
		static QString getApplicationGroupIds(MySqlInfo * mySqlInfo);
		static QString getApplicationGroups(MySqlInfo * mySqlInfo);
		static QString getApplicationsByGroup( long groupId, MySqlInfo * mySqlInfo);
		static QByteArray getApplicationIcon( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationName( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationOnlyOne( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationDisplayName( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationDescription( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationExecPath( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationArguments( long applicationId, MySqlInfo * mySqlInfo );
		static QString getApplicationWorkingPath( long applicationId, MySqlInfo * mySqlInfo );
		static QString getPreferredServer( long applicationId, double cpuWeight, MySqlInfo * mySqlInfo );
		static QString getServerHosts( long applicationId, MySqlInfo * mySqlInfo );
		static QString getServerIds( long applicationId, MySqlInfo * mySqlInfo );
		static long long getServerId( QString serverAddress, MySqlInfo * mySqlInfo );

		static QByteArray getUserLogo( QString username, MySqlInfo * mySqlInfo );

		static QString getServerType( long serverId, MySqlInfo * mySqlInfo );
		static QString getServerAddress( MySqlInfo * mySqlInfo, long serverId );
		static QString getApplications( QString extension, MySqlInfo * mySqlInfo );
		static QString applicationUpdated( long applicationId );

	private:
		static Application getApplication( long applicationId, MySqlInfo * mySqlInfo );
		static QByteArray downloadIcon(QString iconFile);
		static QList<Application*> applications;
};

#endif
