#include "lwserver.h"

static volatile int keepRunning = 1;
static int client_sock = 0;
#ifdef Q_OS_WIN
static HANDLE mutex;
#else
static sem_t mutex;
#endif

LWServer::LWServer(QString adminuser, QString password) : QObject()
{
	this->password = password;
	this->adminuser = adminuser;
}

void LWServer::intHandler(int)
{
	keepRunning = 0;
	qDebug() << "Shutting down socket...\n";
	shutdown( client_sock, 2);
#ifdef Q_OS_WIN
	closesocket( client_sock );
#else
	close( client_sock );
#endif
}
 
int LWServer::listenConnections( char * address, int port )
{
	int c;
#ifdef Q_OS_WIN
	SOCKET socket_desc;
	WORD wVersionRequested;
	WSADATA wsaData;

	wVersionRequested = MAKEWORD(2, 2);

	if ( WSAStartup(wVersionRequested, &wsaData) != 0 )
	{
		_lastError = "Could not find a usable Winsock DLL";
		return -1;
	}
	if (LOBYTE(wsaData.wVersion) != 2 || HIBYTE(wsaData.wVersion) != 2)
	{
		_lastError = "Could not find a usable version of Winsock DLL";
		WSACleanup();
		return -1;
	}
#else
	int socket_desc;
#endif
	struct sockaddr_in server , client;
     
	//Create socket
	socket_desc = socket(AF_INET , SOCK_STREAM , 0);

#ifdef Q_OS_WIN
	if (socket_desc == INVALID_SOCKET)
#else
	if (socket_desc == -1)
#endif
	{
		_lastError = QString("Could not create socket. Error: %1").arg(SHOWERROR);
		qDebug(_lastError.toUtf8().data());
	}
	qDebug() << "Socket created\n";
     
	//Prepare the sockaddr_in structure
	server.sin_family = AF_INET;
	//server.sin_addr.s_addr = INADDR_ANY;
	server.sin_addr.s_addr = inet_addr(address);;
	server.sin_port = htons( port );

	// Set reuse address socket
	int yes = 1;
#ifdef Q_OS_WIN
	if ( setsockopt( socket_desc, SOL_SOCKET, SO_REUSEADDR, (char*)&yes, sizeof(int) ) < 0 )
#else
	if ( setsockopt( socket_desc, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int) ) < 0 )
#endif
	{
		//print the error message
		_lastError = QString("Setsockopt failed: %1").arg(SHOWERROR);
		qDebug(_lastError.toUtf8().data());
		return 1;
	}
		
	//Bind
#ifdef Q_OS_WIN
	if ( ::bind( socket_desc, (struct sockaddr *)&server, sizeof(server) ) < 0 )
#else
	if ( bind( socket_desc, (struct sockaddr *)&server, sizeof(server) ) < 0 )
#endif
	{
		//print the error message
		_lastError = QString("Bind failed: %1").arg(SHOWERROR);
		qDebug(_lastError.toUtf8().data());
		return 1;
	}
	qDebug() << "Bind done\n";

	// Sends server information to Librework principal server
	int ret = sendServerInformation( adminuser, password );
	if ( ! adminuser.isNull() && ! password.isNull() )
	{
#ifdef Q_OS_WIN
		WSACleanup();
#endif
		return ret;
	}
     
	//Listen
	listen(socket_desc , 3);
     
	//Accept and incoming connection
	qDebug() << "Waiting for incoming connections...\n";
	c = sizeof(struct sockaddr_in);

#ifdef Q_OS_WIN
	mutex = CreateSemaphore( NULL, 0, 1, NULL );
#else
	sem_init( &mutex, 0, 1 );
#endif

	while( keepRunning == 1 )
	{
#ifdef Q_OS_WIN
		client_sock = accept(socket_desc, (struct sockaddr *)&client, (int*)&c);
#else
		client_sock = accept(socket_desc, (struct sockaddr *)&client, (socklen_t*)&c);
#endif
		struct sockaddr_in* pV4Addr = (struct sockaddr_in*)&client;
		struct in_addr ipAddr = pV4Addr->sin_addr;
		int * threadSock = (int*)malloc( sizeof( int ) );
		* threadSock = client_sock;
		struct connectionStruct * param = (struct connectionStruct *)malloc(sizeof(struct connectionStruct));
		param->sock = threadSock;
		param->remoteAddress = (char *)malloc(INET_ADDRSTRLEN * sizeof(char));
		inet_ntop( AF_INET, &ipAddr, param->remoteAddress, INET_ADDRSTRLEN );
#ifdef Q_OS_WIN
		DWORD serverId;
		HANDLE serverThread = CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE) connection_handler, (void*)param, NULL, &serverId );
		if ( serverThread == NULL )
#else
		pthread_t serverThread;
		if( pthread_create( &serverThread, NULL, connection_handler, (void*) param) < 0)
#endif
		{
			perror("Could not create server thread. Error:");
			return 1;
		}
#ifdef Q_OS_WIN
		CloseHandle( serverThread );
#else
		pthread_detach( serverThread );
#endif
	}

#ifdef Q_OS_WIN
	CloseHandle( mutex );
	WSACleanup();
#else
	sem_destroy(&mutex);
#endif
     
	if (client_sock < 0)
	{
#ifdef DEBUG
		qDebug() << "Accept failed\n";
#endif
		return 1;
	}
	return 0;
}
 
/*
 * This will handle connection for each client
 * */
void * LWServer::connection_handler(void *connectionParam)
{
	qDebug("New thread...");
	struct connectionStruct * param = (struct connectionStruct *)connectionParam;
	//Get the socket descriptor
	int sock = *(int*)param->sock;
	QString clientAddress( param->remoteAddress );
	qDebug("Cient address: %s", clientAddress.toUtf8().data());

	free( param->sock );
	free( param->remoteAddress );
	free( connectionParam );

	int read_size;
	char client_message[MAX_MESSAGE_SIZE];

	LWServerThread * serverThread = new LWServerThread(QString::number(sock), clientAddress);

	//Receive a message from client
	while( keepRunning == 1 && (read_size = recv(sock , client_message , MAX_MESSAGE_SIZE , 0)) > 0 )
	{
		//end of string marker
		client_message[read_size] = '\0';
		string clientMessageString( client_message );
		
		if ( clientMessageString.compare(0, 4, "quit") == 0)
		{
			qDebug("Shutting down socket...");
			shutdown( sock, 2);
		} else {
			if ( clientMessageString.compare(0, 11, "_byteArray_") == 0)
			{
				QByteArray res = serverThread->dispatchByteArrayMessage( client_message );
				send(sock, res.data(), res.size(), 0);
			} else {
				string res = serverThread->dispatchMessage( client_message );
				send(sock, res.c_str(), (int)res.length(), 0);
			}
		}
		//clear the message buffer
		memset(client_message, 0, MAX_MESSAGE_SIZE);
	}

	if(read_size >= 0)
	{
		fflush(stdout);
	} else if(read_size == -1) {
#ifdef DEBUG
		qDebug() << "recv failed\n";
#endif
	}

#ifdef Q_OS_WIN
	closesocket( sock );
#else
	close( sock );
#endif
	delete serverThread;
#ifndef Q_OS_WIN
	pthread_exit(NULL);
#endif
	return 0;
}

/*
 * Sends server information to Librework principal server
 */
int LWServer::sendServerInformation( QString adminuser, QString password )
{
	emit progress(10);
	emit progressText(tr("Starting server information delivery..."));
	int ret = 0;
	QSettings settings(CONFIG_FILE, QSettings::IniFormat);
	QString address = settings.value( "Server/LibreworkPrincipalServerAddress", "127.0.0.1" ).toString();
	int port = settings.value( "Server/LibreworkPrincipalServerPort", DEFAULT_PORT ).toInt();
	QString serverPort = settings.value( "Server/ServerPort", DEFAULT_PORT).toString();
	bool isPrincipal = settings.value( "Server/LibreworkServer", 0).toInt() == 1;
	QString token = settings.value( "Server/ServerToken", QVariant()).toString();
	bool emptyToken = token.isEmpty();

	if ( token.isEmpty() )
	{
		token = LWUserCommand::generateToken();
		settings.setValue("Server/ServerToken", token);
		settings.sync();
	}

	QStringList dbServerAttributes;
	dbServerAttributes << "librework_server" << "console_server" << "domain_server" << "desktop_server" << "application_server" << "rdp_proxy_server";
	dbServerAttributes << "web_client_server" << "home_storage_server" << "storage_server" << "infrastructure_server" << "pxe_server";
	QStringList attributes;
	attributes << "LibreworkServer" << "ConsoleServer" << "DomainServer" << "DesktopServer" << "ApplicationServer" << "RDPProxyServer";
	attributes << "WebClientServer" << "HomeStorageServer" << "StorageServer" << "VirtualizationServer" << "PXEServer";

	if ( isPrincipal )
	{
		QString mySqlHost = settings.value( "Server/MySqlServerAddress", "localhost" ).toString();
		int mySqlPort = settings.value( "Server/MySqlServerPort", 3306 ).toInt();
		QString mySqlUsername = settings.value( "Server/MySqlUsername", "root" ).toString();
		QString mySqlPassword = settings.value( "Server/MySqlPassword" ).toString();

		qDebug("Connecting to MySql database (server: %s:%d, user: %s, password: %s)...", mySqlHost.toUtf8().data(), mySqlPort, mySqlUsername.toUtf8().data(), mySqlPassword.toUtf8().data());

		MySqlInfo * mySqlInfo = new MySqlInfo( mySqlHost, mySqlPort );
		if (mySqlInfo->authenticate( mySqlUsername, mySqlPassword, "initialize" ) )
		{
			qDebug("MySql server authentication succesful");
			if ( emptyToken )
			{
				LWServerCommand::setServerToken( mySqlInfo, address, serverPort.toInt(), token );
			}
			for (int i = 0; i < attributes.count(); i++)
			{
				QString value = settings.value("Server/" + attributes[i], "0").toString();
				QString response = LWServerCommand::setServerInformation( mySqlInfo, token, dbServerAttributes[i], value, address);
				if ( response != "Ok" )
				{
					qDebug("Failed setting attribute %s with value %s", dbServerAttributes[i].toUtf8().data(), value.toUtf8().data());
					qDebug("Reponse: %s", response.toUtf8().data());
					i = attributes.count();
				}
			}
		} else {
			qDebug("MySql server authentication failed");
		}
		mySqlInfo->close();
		delete mySqlInfo;
	} else {
		QTcpSocket * tcpSocket = new QTcpSocket();
		tcpSocket->connectToHost( address, port, QIODevice::ReadWrite );
		emit progress(30);
		emit progressText(tr("Connected to server"));

		if (tcpSocket->waitForConnected( SOCKET_TIMEOUT ))
		{
			if ( adminuser.isEmpty() || password.isEmpty() )
			{
				for (int i = 0; i < attributes.count(); i++)
				{
					QString value = settings.value("Server/" + attributes[i], "0").toString();
					QString response = sendMessage( tcpSocket, "set_server_information?" + token + "&" + dbServerAttributes[i] + "&" + value);
					if ( response != "Ok" )
					{
						ret = 1;
					}
				}
				QString osId = "1";
#ifdef Q_OS_LINUX
				osId = "1";
#endif
#ifdef Q_OS_WIN
				osId = "2";
#endif
				QString response = sendMessage( tcpSocket, "set_server_information?" + token + "&" "os_id" "&" + osId );
				if ( response != "Ok ")
				{
					ret = 2;
				}
			} else {
				// Authentication
				QString message = "login?" + adminuser + "&&" + password + "\n";
				QString response = sendMessage( tcpSocket, message );
				if ( response.startsWith( "Error" ) )
				{
					tcpSocket->close();
					delete tcpSocket;
					return 255;
				}

				// Send information
				message = "set_server_token?" + QString::number(port) + "&" + token;
				response = sendMessage( tcpSocket, message );
				if ( response.startsWith( "Error" ) )
				{
					ret = 254;
				}
			}
		} else {
			ret = 253;
		}
		emit progress(99);
		emit progressText(tr("Closing socket"));
		tcpSocket->close();
		delete tcpSocket;
		emit progress(100);
		emit progressText(tr("Done"));
	}
	return ret;
}

QString LWServer::sendMessage(QTcpSocket * tcpSocket, QString message)
{
	QString response;

	long bytesWritten = tcpSocket->write( message.toUtf8().data());
	if ( bytesWritten == -1 )
	{
		return QString("Error sending message");
	}
	bool waitForBytesWritten = tcpSocket->waitForBytesWritten();
	Q_UNUSED( waitForBytesWritten );
	bool readyRead = tcpSocket->waitForReadyRead();
	Q_UNUSED( readyRead );
	while (! tcpSocket->atEnd())
	{
		response += tcpSocket->read(256);
	}
	bool dataFlushed = tcpSocket->flush();
	Q_UNUSED( dataFlushed );

	response = response.trimmed();
	return response;
}
