#include "lwservercommand.h"

LWServerCommand::LWServerCommand(MySqlInfo * mySqlInfo, QString username, QString password, QString serverAddress, bool isAdministrator) : QObject()
{
	this->serverAddress = serverAddress;
	this->username = username;
	this->password = password;
	this->mySqlInfo = mySqlInfo;
	this->isAdministrator = isAdministrator;
}

QByteArray LWServerCommand::doResponseByteArray(QString question)
{
	QString command = question.trimmed().section('?', 0, 0);

	QStringList parameters = question.trimmed().section('?', 1, 1).split('&');

	if ( parameters.count() == 1 && parameters[0].isNull() )
		parameters.removeFirst();

	bool badParameters = 0;

	if (command == "_byteArray_get_application_icon")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationIcon( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}
	if (command == "_byteArray_get_user_logo")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getUserLogo( parameters[0], mySqlInfo );
		else
			badParameters = 1;
	}

	if (badParameters == 1)
		return writeError(QString("Invalid number of parameters for ByteArray command %1").arg(command)).toUtf8();
	else
		return writeError(QString("Unknown ByteArray command %1").arg(command)).toUtf8();
}

QString LWServerCommand::doResponse(QString question)
{
	QString command = question.trimmed().section('?', 0, 0);

	QStringList parameters = question.trimmed().section('?', 1, 1).split('&');

	if ( parameters.count() == 1 && parameters[0].isNull() )
		parameters.removeFirst();

	bool badParameters = 0;

	if (command == "add_google_printer")
	{
		if (parameters.count() == 3)
			return LWPrinterCommand::addGooglePrinter(parameters[0], parameters[1], username, parameters[2]);
		else
			badParameters = 1;
	}

	if (command == "remove_google_printer")
	{
		if (parameters.count() == 3)
			return LWPrinterCommand::removeGooglePrinter(parameters[0], parameters[1], username, parameters[2]);
		else
			badParameters = 1;
	}

	if (command == "prune_google_printers")
	{
		if (parameters.count() == 1)
			return LWPrinterCommand::pruneGooglePrinters(parameters[0], username);
		else
			badParameters = 1;
	}

	if (command == "add_printer")
	{
		if (parameters.count() == 2)
			return LWPrinterCommand::addPrinter(parameters[0], parameters[1]);
		else
			badParameters = 1;
	}

	if (command == "remove_printer")
	{
		if (parameters.count() == 1)
			return LWPrinterCommand::removePrinter(parameters[0]);
		else
			badParameters = 1;
	}

	if (command == "share_printer")
	{
		if (parameters.count() == 2)
			return LWPrinterCommand::sharePrinter(parameters[0], parameters[1]);
		else
			badParameters = 1;
	}

	if (command == "mount_chroot")
	{
		if (parameters.count() == 5) // chrootHome, chrootMount, username, homePath, sambaServer
			return LWSystemCommand::mountChroot(parameters[0], parameters[1], parameters[2], password, parameters[3], parameters[4]);
		else
			badParameters = 1;
	}

	if (command == "umount_chroot")
	{
		if (parameters.count() == 1) // username
			return LWSystemCommand::umountChroot(parameters[0]);
		else
			badParameters = 1;
	}

	if (command == "get_valid_applications")
	{
		if (parameters.count() == 0)
			return LWApplicationCommand::getApplicationIds( username, mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_extensions_list")
	{
		if (parameters.count() == 0)
			return LWApplicationCommand::getApplicationExtensionsList( mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_groups")
	{
		if (parameters.count() == 0)
			return LWApplicationCommand::getApplicationGroups( mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_group_ids")
	{
		if (parameters.count() == 0)
			return LWApplicationCommand::getApplicationGroupIds( mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_name")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationName( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_applications_by_group")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationsByGroup( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_onlyone")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationOnlyOne( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_display_name")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationDisplayName( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_description")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationDescription( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_exec_path")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationExecPath( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_arguments")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationArguments( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_application_working_path")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplicationWorkingPath( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_server_hosts")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getServerHosts( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_server_ids")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getServerIds( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "start_session")
	{
		if (parameters.count() == 0)
			return startSession();
		else
			badParameters = 1;
	}

	if (command == "end_session")
	{
		if (parameters.count() == 0)
			return endSession();
		else
			badParameters = 1;
	}

	if (command == "start_application")
	{
		if (parameters.count() == 2)
			return startApplication( parameters[0].toLongLong(), parameters[1].toLongLong() );
		else
			badParameters = 1;
	}

	if (command == "end_application")
	{
		if (parameters.count() == 2)
			return endApplication( parameters[0].toLongLong(), parameters[1].toLongLong() );
		else
			badParameters = 1;
	}

	if (command == "get_preferred_server")
	{
		if (parameters.count() == 2)
			return LWApplicationCommand::getPreferredServer( parameters[0].toLongLong(), parameters[1].toDouble(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_server_address")
	{
		if (parameters.count() == 0)
			return LWApplicationCommand::getServerAddress( mySqlInfo, LWApplicationCommand::getServerId( serverAddress, mySqlInfo ) );
		else
			badParameters = 1;
	}

	if (command == "get_server_type")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getServerType( parameters[0].toLongLong(), mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_applications")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::getApplications( parameters[0], mySqlInfo );
		else
			badParameters = 1;
	}

	if (command == "get_user_list")
	{
		if (parameters.count() == 1)
			return LWUserCommand::getUserList( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "get_group_list")
	{
		if (parameters.count() == 0)
			return LWUserCommand::getGroups();
		else
			badParameters = 1;
	}

	if (command == "kill_session")
	{
		if (parameters.count() == 1) // username
			return LWSystemCommand::killSession( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "create_user")
	{
		if (parameters.count() == 3 || parameters.count() == 4) // username, password, group, [domain]
		{
			QString create = LWUserCommand::createUser( parameters[0], parameters[1] );
			if (create.isEmpty() )
			{
				QString ret = LWUserCommand::enableUser( parameters[0], parameters[2] );
				if ( parameters.count() == 4 )
				{
					return LWUserCommand::enableUser( parameters[0], parameters[3] );
				} else {
					return ret;
				}
			} else {
				return create;
			}
		} else {
			badParameters = 1;
		}
	}

	if (command == "create_user_folder")
	{
		if (parameters.count() == 2) // username, basepath
			return LWUserCommand::createUserFolder( parameters[0], parameters[1] );
		else
			badParameters = 1;
	}

	if (command == "remove_user_folder")
	{
		if (parameters.count() == 2) // username, basepath
			return LWUserCommand::removeUserFolder( parameters[0], parameters[1] );
		else
			badParameters = 1;
	}

	if (command == "create_group")
	{
		if (parameters.count() == 1) // group
			return LWUserCommand::createGroup( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "modify_user_password")
	{
		if (parameters.count() == 3) // username, oldpassword, newpassword
		{
			if ( parameters[1].isEmpty() && ! isAdministrator ) // Not allow empty old password unless admin
			{
				badParameters = 1;
			} else {
				return LWUserCommand::modifyUserPassword( parameters[0], parameters[1], parameters[2] );
			}
		} else {
			badParameters = 1;
		}
	}

	if (command == "remove_user")
	{
		if (parameters.count() == 1) // username
			return LWUserCommand::removeUser( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "remove_group")
	{
		if (parameters.count() == 1) // group
			return LWUserCommand::removeGroup( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "activate_user")
	{
		if (parameters.count() == 1) // username
			return activateUser( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "enable_user")
	{
		if (parameters.count() == 2) // username, group
			return LWUserCommand::enableUser( parameters[0], parameters[1] );
		else
			badParameters = 1;
	}

	if (command == "disable_user")
	{
		if (parameters.count() == 2) // username, group
			return LWUserCommand::disableUser( parameters[0], parameters[1] );
		else
			badParameters = 1;
	}

	if (command == "application_updated")
	{
		if (parameters.count() == 1)
			return LWApplicationCommand::applicationUpdated( parameters[0].toLongLong() );
		else
			badParameters = 1;
	}

	if (command == "get_system_memory")
	{
		if (parameters.count() == 0)
			return LWSystemCommand::getSystemMemory();
		else
			badParameters = 1;
	}

	if (command == "get_system_disk_space")
	{
		if (parameters.count() == 0)
			return LWSystemCommand::getSystemDiskSpace();
		else
			badParameters = 1;
	}

	if (command == "get_system_load")
	{
		if (parameters.count() == 0)
			return LWSystemCommand::getSystemLoad();
		else
			badParameters = 1;
	}

	if (command == "get_redirect_drives")
	{
		if (parameters.count() == 1)
			return LWStorageCommand::getRedirectDrives(serverAddress, parameters[0], mySqlInfo);
		else
			badParameters = 1;
	}

	if (command == "set_server_token") // port, token
	{
		if ( ! isAdministrator )
			return QString::null;

		if (parameters.count() == 2)
			return setServerToken(mySqlInfo, serverAddress, parameters[0].toInt(), parameters[1]);
		else
			badParameters = 1;
	}

	if (command == "start_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::startServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "start_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::startServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "start_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::startServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "start_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::startServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "pause_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::pauseServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "hard_stop_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::hardStopServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "resume_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::resumeServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "hard_reboot_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::hardRebootServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "soft_stop_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::softStopServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "soft_reboot_server") // serverName
	{
		if (parameters.count() == 1)
			return LWVirtCommand::softRebootServer( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "server_list")
	{
		if (parameters.count() == 0)
			return LWVirtCommand::serverList().join(",");
		else
			badParameters = 1;
	}

	if (command == "get_server_status")
	{
		if (parameters.count() == 1)
			return LWVirtCommand::getServerStatus( parameters[0] );
		else
			badParameters = 1;
	}

	if (command == "quit")
	{
		return "true";
	}

	if (badParameters == 1)
		return writeError(QString("Invalid number of parameters for command %1").arg(command));
	else
		return writeError(QString("Unknown command %1").arg(command));
}

QString LWServerCommand::writeError(QString errorString)
{
        return "Error: " + errorString + "(" + QDateTime::currentDateTime().toString(Qt::SystemLocaleShortDate) + ")";
}

QString LWServerCommand::writeInfo(QString infoString)
{
        return "Info: " + infoString + "(" + QDateTime::currentDateTime().toString(Qt::SystemLocaleShortDate) + ")";
}

#ifdef Q_OS_LINUX
QString LWServerCommand::libreworkGroup()
{
	QStringList group = mySqlInfo->getValues("lw_parameters", "librework_group");
	if ( group.count() > 0 )
	{
		return(group[0]);
	} else {
		return QString::null;
	}
}

QString LWServerCommand::libreworkConsoleGroup()
{
	QStringList group = mySqlInfo->getValues("lw_parameters", "console_group");
	if ( group.count() > 0 )
	{
		return(group[0]);
	} else {
		return QString::null;
	}
}
#endif

QString LWServerCommand::writeToken(QString username, QString token)
{
	if ( mySqlInfo->setValue("lw_user", "username='" + username + "'", "token", token) )
	{
		return token;
	} else {
		qDebug("Something failed writing token on database");
		return QString::null;
	}
}

QString LWServerCommand::setServerToken(MySqlInfo * mySqlInfo, QString serverAddress, int port, QString token)
{
	QStringList res = mySqlInfo->getValues("lw_server", "id", "address='" + serverAddress + "' AND port=" + QString::number(port) );
	if ( res.count() == 0 )
	{
		qDebug("Server not found. Creating new server record...");
		QStringList attributes;
		QList<QVariant> values;
		attributes << "name";
		values << serverAddress;
		attributes << "address";
		values << serverAddress;
#ifdef Q_OS_LINUX
		attributes << "os_id";
		values << 1;
#endif
#ifdef Q_OS_WIN
		attributes << "os_id";
		values << 2;
#endif
		attributes << "domain_server";
		values << 0;
		attributes << "port";
		values << port;
		attributes << "application_server";
		values << 0;
		attributes << "console_server";
		values << 0;
		attributes << "desktop_server";
		values << 0;
		attributes << "home_storage_server";
		values << 0;
		attributes << "librework_server";
		values << 0;
		attributes << "rdp_proxy_server";
		values << 0;
		attributes << "storage_server";
		values << 0;
		attributes << "web_client_server";
		values << 0;
		attributes << "infrastructure_server";
		values << 0;
		attributes << "pxe_server";
		values << 0;
		if ( ! mySqlInfo->insertRecord("lw_server", attributes, values ) )
		{
			qDebug("Something failed creating server record");
		} else {
			qDebug("Record succesfully created");
		}
	}
	if ( mySqlInfo->setValue("lw_server", "address='" + serverAddress + "' AND port=" + QString::number(port), "token", token) )
	{
		return QString("Ok");
	} else {
		qDebug("Something failed writing token on database");
		return QString("Error setting token");
	}
}

QString LWServerCommand::setServerInformation(MySqlInfo * mySqlInfo, QString token, QString serverType, QString status, QString clientAddress)
{
	QStringList serverTypes;
	serverTypes << "librework_server" << "console_server" << "domain_server" << "desktop_server" << "application_server" << "rdp_proxy_server";
	serverTypes << "web_client_server" << "home_storage_server" << "storage_server" << "infrastructure_server" << "pxe_server" << "os_id";
	QStringList dbToken = mySqlInfo->getValues( "lw_server", "token", "address='" + clientAddress + "'" );
	if (dbToken.count() >= 1)
	{
		if ( dbToken[0].trimmed().isEmpty() )
		{
			if ( mySqlInfo->setValue("lw_server", "address='" + clientAddress + "'", "token", token) )
			{
				dbToken[0] = token;
			} else {
				qDebug("Cannot write server token on database");
			}
		}

		if ( dbToken[0] == token )
		{
			if ( serverTypes.contains(serverType) )
			{
				if ( mySqlInfo->setValue("lw_server", "address='" + clientAddress + "'", serverType, status) )
				{
					return "Ok";
				} else {
					return QString("Error setting server type " + serverType + " on server: " + clientAddress);
				}
			} else {
				return QString("Invalid server type " + serverType);
			}
		} else {
			return QString("Invalid token for server " + clientAddress);
		}
	} else {
		return QString("No server " + clientAddress);
	}
}

QString LWServerCommand::activateUser(QString username)
{
	return writeToken( username, LWUserCommand::generateToken() );
}

QString LWServerCommand::fillEvent(long long typeId, long long appId, QString text)
{
/*
	QStringList uname = username.split("\\");
	QString user = uname[uname.count() - 1];
*/
	QStringList attributes;
	QList<QVariant> values;

	attributes += "server_id";
	values += QVariant( LWApplicationCommand::getServerId( serverAddress, mySqlInfo ) );

	attributes += "user_id";
	values += QVariant( username );

	if ( appId > 0 )
	{
		attributes += "app_id";
		values += QVariant( appId );
	}

	attributes += "text";
	values += QString( text );

	attributes += "time_stamp";
	values += QDateTime::currentDateTime().toUTC();

	attributes += "type_id";
	values += QVariant( typeId );

	bool ret = mySqlInfo->insertRecord( "lw_log", attributes, values );
	if ( ret )
		return QString("Ok");
	else
		return QString("Error inserting record on events database");
}

QString LWServerCommand::startSession( )
{
	return fillEvent( EVENT_START_SESSION, 0, tr("User %1 started session on server %2").arg(username).arg(serverAddress) );
}

QString LWServerCommand::endSession()
{
	return fillEvent( EVENT_END_SESSION, 0, tr("User %1 ended session on server %2").arg(username).arg(serverAddress) );
}

QString LWServerCommand::startApplication( long long applicationId, long processId )
{
        QString application = LWApplicationCommand::getApplicationName( applicationId, mySqlInfo );
        return fillEvent( EVENT_START_APPLICATION, applicationId, tr("User %1 started application %2 on server %3 with process Id %4").arg(username).arg(application).arg(serverAddress).arg(processId) );
}

QString LWServerCommand::endApplication( long long applicationId, long processId )
{
        QString application = LWApplicationCommand::getApplicationName( applicationId, mySqlInfo );
        return fillEvent( EVENT_END_APPLICATION, applicationId, tr("User %1 started application %2 on server %3 with process Id %4").arg(username).arg(application).arg(serverAddress).arg(processId) );
}
