#include "lwusercommand.h"

QStringList LWUserCommand::getGroups(QString username)
{
    QStringList groupList;
#ifdef Q_OS_WIN
    LPBYTE buffer;
    DWORD entries, total_entries;
    QString logonServer= qgetenv("LOGONSERVER");
    NetUserGetGroups( (LPCWSTR)logonServer.utf16(), (LPCWSTR)username.utf16(), 0, &buffer, MAX_PREFERRED_LENGTH, &entries, &total_entries);
    GROUP_USERS_INFO_0 *ggroups = (GROUP_USERS_INFO_0*)buffer;
    for (uint i=0; i<entries; i++)
    {
        groupList += QString::fromUtf16((const ushort *)ggroups[i].grui0_name);

    }
    NetApiBufferFree(buffer);
#endif
    return groupList;
}

QString LWUserCommand::getGroups()
{
	QStringList res;
#ifdef Q_OS_LINUX
	QString command = "samba-tool";
	QStringList arguments;

	arguments += "group";
	arguments += "list";

	QProcess process;
	process.start(command, arguments);
	process.waitForFinished();
	res = QString(process.readAllStandardOutput()).trimmed().split('\n');
	if ( process.exitStatus() != QProcess::NormalExit )
	{
		return _ERROR_( QString(process.readAllStandardError()) );
	}
#endif
#ifdef Q_OS_WIN
	QString logonServer= qgetenv("LOGONSERVER");
	bool err = false;
	DWORD ret, dwRec, i = 0;
	PNET_DISPLAY_GROUP pBuff, p;

	do
	{
		ret = NetQueryDisplayInformation((LPCWSTR)logonServer.utf16(), 3, i, 1000, MAX_PREFERRED_LENGTH, &dwRec, (PVOID*) &pBuff);
		if( (ret == ERROR_SUCCESS) || (ret == ERROR_MORE_DATA) )
		{
			p = pBuff;
			for( ; dwRec>=0; dwRec--)
			{
				res += QString::fromUtf16((const ushort *)p->grpi3_name);
				i = p->grpi3_next_index;
				p++;
			}
			NetApiBufferFree(pBuff);
		} else {
			qDebug("Error: %u\n", ret);
			err = true;
		}
	} while (ret == ERROR_MORE_DATA);
	if ( err )
	{
		return _ERROR_( tr("Unknown error listing groups (%1)").arg(ret) );
	}
/*
	HRESULT hr = E_FAIL;
	int groupType = 4; // 4: Specifies a group with domain local scope. 8: Specifies a group with universal scope.

	// Construct the ADsPath to bind to the search root.
	CComBSTR sbstrADsPath = "LDAP://";
	sbstrADsPath += (LPCWSTR)logonServer.utf16();

	// Bind to the container.
	CComPtr<IDirectorySearch> spSearch;
	const IID IID_IDirectorySearch = {0x109BA8EC, 0x92F0, 0x11D0, {0xA7,0x90,0x00,0xC0,0x4F,0xD8,0xD5,0xA8}};
	hr = ADsGetObject(sbstrADsPath, IID_IDirectorySearch, (LPVOID*)&spSearch);
	if(SUCCEEDED(hr))
	{
		ADS_SEARCHPREF_INFO SearchPref[1];

		// Set the scope of the search to be all of the subtree.
		SearchPref[0].dwSearchPref = ADS_SEARCHPREF_SEARCH_SCOPE;
		SearchPref[0].vValue.dwType = ADSTYPE_INTEGER;
		SearchPref[0].vValue.Integer = ADS_SCOPE_SUBTREE;

		hr = spSearch->SetSearchPreference(SearchPref, sizeof(SearchPref)/sizeof(ADS_SEARCHPREF_INFO));
		if(FAILED(hr))
		{
			return hr;
		}

		ADS_SEARCH_HANDLE hSearch = NULL;
		LPWSTR pwszDN = (LPWSTR)L"distinguishedName";
		LPWSTR pwszAttributes[1] = {pwszDN};

		// Convert the group type to search for into a string.
		WCHAR wszGroupType[2];
		swprintf_s(wszGroupType, L"%d", groupType);

		// Construct the search filter.
		CComBSTR sbstrSearchFilter;
		sbstrSearchFilter = "(&(objectClass=group)(groupType:";
		sbstrSearchFilter += LDAP_MATCHING_RULE_BIT_AND;
		sbstrSearchFilter += ":=";
		sbstrSearchFilter += wszGroupType;
		sbstrSearchFilter += "))";

		// Execute the search.
		hr = spSearch->ExecuteSearch(sbstrSearchFilter,
			pwszAttributes,
			sizeof(pwszAttributes)/sizeof(LPWSTR),
			&hSearch);

		if(FAILED(hr))
		{
			return QString::null;
		}

		// Get the first result row. There should never be more than one result.
		hr = spSearch->GetFirstRow(hSearch);
		while(S_OK == hr)
		{
			ADS_SEARCH_COLUMN col;

			// Get the distinguished name for the current result.
			hr = spSearch->GetColumn(hSearch, pwszDN, &col);
			if(SUCCEEDED(hr))
			{
				if(ADSTYPE_DN_STRING  == col.dwADsType)
				{
					res += QString::fromUtf16( (const ushort*)col.pADsValues[0].DNString );
				}

				// Free the column.
				spSearch->FreeColumn(&col);
			}

			hr = spSearch->GetNextRow(hSearch);
		}
		// Close the search handle to cleanup.
		hr = spSearch->CloseSearchHandle(hSearch);
	}
	*/
#endif
	return res.join(",");
}

bool LWUserCommand::isUserInGroup(QString username, QString groupname)
{
	QStringList groupUsers = getUserList( groupname ).split(",");
	qDebug() << groupUsers.join(",");
	return groupUsers.contains( username, Qt::CaseInsensitive );
/*#ifdef Q_OS_WIN
    QStringList groupList = getGroups( username );
    return groupList.contains( groupname );
#endif
*/
}

QString LWUserCommand::createGroup(QString group)
{
#ifdef Q_OS_LINUX
	QStringList arguments;
	arguments += "group";
	arguments += "add";
	arguments += group;

	QString res = PythonRun::runSambaTool( arguments );

	return res;
#endif
#ifdef Q_OS_WIN
	GROUP_INFO_0 gi;
	DWORD dwError = 0;
	NET_API_STATUS nStatus;

	gi.grpi0_name = (LPWSTR)group.utf16();

	QString logonServer= qgetenv("LOGONSERVER");
	nStatus = NetGroupAdd( (LPCWSTR)logonServer.utf16(), 0, (LPBYTE)&gi, &dwError);

	switch (nStatus)
	{
	case NERR_Success:
		return QString::null;
	case NERR_GroupExists:
		return _ERROR_( tr("Group already exists") );
	case NERR_NotPrimary:
		return _ERROR_( tr("Not primary domain controller for domain") );
	default:
		return _ERROR_( tr("Error: Unknown error creating group") );
	}
#endif
}

QString LWUserCommand::removeGroup(QString group)
{
#ifdef Q_OS_LINUX
	QStringList arguments;
	arguments += "group";
	arguments += "delete";
	arguments += group;

	QString res = PythonRun::runSambaTool( arguments );
	if ( res.isEmpty() )
	{
		return _SUCCESS_( tr("Group %1 removed succesfully").arg(group) );
	} else {
		return _ERROR_( tr("Cannot remove group %1").arg(group) );
	}
#endif
#ifdef Q_OS_WIN
	NET_API_STATUS nStatus;

	QString logonServer= qgetenv("LOGONSERVER");
	nStatus = NetGroupDel((LPCWSTR)logonServer.utf16(), (LPCWSTR)group.utf16());

	if (nStatus == NERR_Success)
	{
		return _SUCCESS_( tr("Group %1 removed succesfully").arg(group) );
	} else {
		QString errorMessage;
		switch (nStatus)
		{
		case ERROR_ACCESS_DENIED:
			errorMessage = tr("The user does not have access to the requested information");
			break;
		case NERR_InvalidComputer:
			errorMessage = tr("The computer name is invalid");
			break;
		case NERR_NotPrimary:
			errorMessage = tr("The operation is allowed only on the primary domain controller of the domain");
			break;
		case NERR_GroupNotFound:
			errorMessage = tr("The group name could not be found");
			break;
		default:
			errorMessage = tr("Unknown error removing group");
		}
		return _ERROR_( errorMessage );
	}
#endif
}

QString LWUserCommand::modifyUserPassword(QString username, QString oldPassword, QString newPassword)
{
#ifdef Q_OS_LINUX
	Q_UNUSED( oldPassword );
	QStringList arguments;
	arguments += "user";
	arguments += "setpassword";
	arguments += username;
	arguments += newPassword;

	QString res = PythonRun::runSambaTool( arguments );
	if ( res.isEmpty() )
	{
		return _SUCCESS_( tr("Password for user %1 changed succesfully").arg(username) );
	} else {
		return _ERROR_( tr("Cannot change password") );
	}
#endif
#ifdef Q_OS_WIN
	if ( ! oldPassword.isEmpty() )
	{
		NET_API_STATUS nStatus = NetUserChangePassword( NULL, (LPCWSTR) username.utf16(), (LPCWSTR) oldPassword.utf16(), (LPCWSTR) newPassword.utf16());
		if (nStatus == NERR_Success)
		{
			return _SUCCESS_( tr("Password for user %1 changed succesfully").arg(username) );
		} else {
			return _ERROR_( tr("Cannot change password") );
		}
	} else {
		USER_INFO_1003 ui;
		DWORD dwError = 0;
		NET_API_STATUS nStatus;

		ui.usri1003_password = (LPWSTR)newPassword.utf16();

		QString logonServer= qgetenv("LOGONSERVER");
		nStatus = NetUserSetInfo( (LPCWSTR)logonServer.utf16(), (LPCWSTR)username.utf16(), 1003, (LPBYTE)&ui, &dwError);

		switch (nStatus)
		{
		case NERR_Success:
			return _SUCCESS_( tr("Password for user %1 changed succesfully").arg(username) );
		case NERR_PasswordTooShort:
			return _ERROR_( tr("Invalid password") );
		case NERR_NotPrimary:
			return _ERROR_( tr("The operation is allowed only on the primary domain controller of the domain") );
		case NERR_UserNotFound:
			return _ERROR_( tr("The user name could not be found") );
		default:
			return _ERROR_( tr("Unknown error changing user password") );
		}
	}
#endif
}

QString LWUserCommand::getUserList(QString group)
{
	QStringList res;
#ifdef Q_OS_LINUX
	QString command = "samba-tool";
	QStringList arguments;

	if ( ! group.isEmpty() )
	{
		arguments += "group";
		arguments += "listmembers";
		arguments += group;

		qDebug("Running process: %s", QString(command + " " + arguments.join(" ")).toUtf8().data());
		QProcess process;
		process.start(command, arguments);
		process.waitForFinished();
		res = QString(process.readAllStandardOutput()).split('\n');
		if ( process.exitStatus() != QProcess::NormalExit )
		{
			return _ERROR_( QString(process.readAllStandardError()) );
		} else {
			return res.join(",");
		}
	} else {
		return _ERROR_( tr("Cannot get default librework group from database") );
	}
#endif

#ifdef Q_OS_WIN
	LPBYTE buffer;
	DWORD entries, total_entries;
	QString logonServer= qgetenv("LOGONSERVER");
	NetGroupGetUsers((LPCWSTR)logonServer.utf16(), (LPCWSTR)group.utf16(), 0, &buffer, MAX_PREFERRED_LENGTH, &entries, &total_entries, NULL);
	GROUP_USERS_INFO_0 *ggroups = (GROUP_USERS_INFO_0*)buffer;
	for (uint i=0; i<entries; i++)
	{
		res += QString::fromUtf16((const ushort *)ggroups[i].grui0_name);
	}
	qDebug("Total users: %d", total_entries);
	NetApiBufferFree(buffer);
#endif
/* Another way of doing this
#ifdef Q_OS_WIN
    LPUSER_INFO_0 pBuf = NULL;
    LPUSER_INFO_0 pTmpBuf;
    DWORD dwPrefMaxLen = MAX_PREFERRED_LENGTH;
    DWORD dwEntriesRead = 0;
    DWORD dwTotalEntries = 0;
    DWORD dwResumeHandle = 0;
    QString logonServer= qgetenv("LOGONSERVER");
    NET_API_STATUS nStatus = NetUserEnum((LPCWSTR)logonServer.utf16(), 0, FILTER_NORMAL_ACCOUNT, (LPBYTE*)&pBuf, dwPrefMaxLen, &dwEntriesRead, &dwTotalEntries, &dwResumeHandle);
    if ((nStatus == NERR_Success) || (nStatus == ERROR_MORE_DATA))
    {
        pTmpBuf = pBuf;
        if ( pTmpBuf != NULL)
        {
            for (uint i = 0; (i < dwEntriesRead); i++)
            {
                if (pTmpBuf == NULL)
                {
                    fprintf(stderr, "An access violation has occurred\n");
                    break;
                }
                QString user = QString::fromUtf16((const ushort *)pTmpBuf->usri0_name);
                qDebug() << user;
                if ( isUserInGroup( user, group ) )
                {
                    res += user;
                }

                pTmpBuf++;
            }
        }
    } else {
        fprintf(stderr, "A system error has occurred: %d\n", nStatus);
        if (pBuf != NULL)
        {
            NetApiBufferFree(pBuf);
            pBuf = NULL;
        }
    }
    while (nStatus == ERROR_MORE_DATA); // end do
    if (pBuf != NULL)
    {
        NetApiBufferFree(pBuf);
    }
#endif
*/
	return res.join(',');
}

QString LWUserCommand::createUser(QString username, QString password)
{
#ifdef Q_OS_LINUX
	QStringList arguments;
	arguments += "user";
	arguments += "add";
	arguments += username;
	arguments += password;

	QString res = PythonRun::runSambaTool( arguments );

	return res;
#endif
#ifdef Q_OS_WIN
	DWORD dwError = 0;
	NET_API_STATUS nStatus;
	USER_INFO_4 ui = getUserInfo4( username, password, QString::null, QString::null);

	QString logonServer= qgetenv("LOGONSERVER");
	nStatus = NetUserAdd( (LPCWSTR)logonServer.utf16(), 4, (LPBYTE)&ui, &dwError);

	switch (nStatus)
	{
	case NERR_Success:
		return QString::null;
	case NERR_UserExists:
		return _ERROR_( tr("User already exists") );
	case NERR_PasswordTooShort:
		return _ERROR_( tr("Invalid password") );
	default:
		return _ERROR_( tr("Unknown error creating user (Error: %1)").arg(nStatus) );
	}
#endif
}

QString LWUserCommand::removeUserFolder(QString username, QString basePath)
{
	QString ret;
#ifdef Q_OS_WIN
	QString serverName = qgetenv("COMPUTERNAME");
	NET_API_STATUS res;
	res = NetShareDel((LPWSTR)serverName.utf16(), (LPWSTR)username.utf16(), NULL);
	if(res == 0)
	{
		ret = "Ok";
	} else {
		ret = "Error: cannot unshare user folder";
	}
#endif
	QDir baseDir(basePath + QDir::separator() + username);
	baseDir.removeRecursively();
	return ret;
}

QString LWUserCommand::createUserFolder(QString username, QString basePath)
{
	QStringList messages;
	// Setup roaming profile (only windows, samba does this automatically)
#ifdef Q_OS_WIN
	QString serverName = qgetenv("COMPUTERNAME");
	QString homeDir = "\\\\" + serverName.simplified() + "\\" + username.simplified();
	QString profileDir = homeDir + "\\profile";
	DWORD dwError = 0;
	NET_API_STATUS nStatus;
	USER_INFO_4 ui = getUserInfo4( username, QString::null, homeDir, profileDir );

	QString logonServer= qgetenv("LOGONSERVER");
	nStatus = NetUserSetInfo((LPCWSTR)logonServer.utf16(), (LPCWSTR)username.utf16(), 4, (LPBYTE)&ui, &dwError);

	if ( nStatus != NERR_Success )
	{
		messages += QString("Cannot set profile on user (Error %1)").arg(nStatus);
	}
#endif

	// Create dirs
	QStringList subdirs;
	subdirs << "AppData" << "Contacts" << "Desktop" << "Documents" << "Downloads"
			<< "Favorites" << "Links" << "Music" << "Pictures" << "Saved Games"
			<< "Searches" << "Start Menu" << "Videos";
	QDir baseDir(basePath);
	baseDir.mkdir(username);
	baseDir.refresh();
	QDir userDir(basePath + QDir::separator() + username);
	for ( int i = 0; i < subdirs.count(); i++ )
	{
		userDir.mkdir(subdirs.at(i));
	}
	userDir.refresh();

	// Set owner and permissions
#ifdef Q_OS_LINUX
	// Set owner and permissions recursively on basePath
	QStringList args;
	args += "-R";
	args += username;
	args += basePath + QDir::separator() + username;
	QProcess::execute("chown", args);
	args.clear();
	args += "-R";
	args += "0700";
	args += basePath + QDir::separator() + username;
	QProcess::execute("chmod", args);
#endif
#ifdef Q_OS_WIN
	PSID usernameSID = NULL;
	PACL pACL = NULL;
	EXPLICIT_ACCESS ea[1];

	if ( getSid((LPCWSTR)username.utf16(), &usernameSID) != MQ_OK )
	{
		messages += QString("getSid error %1 (username=%2)").arg(GetLastError()).arg(username);
	} else {
		subdirs += "";
		for ( int i = 0; i < subdirs.count(); i++ )
		{
			ZeroMemory(&ea, 1 * sizeof(EXPLICIT_ACCESS));
			ea[0].grfAccessPermissions = GENERIC_ALL;
			ea[0].grfAccessMode = SET_ACCESS;
			ea[0].grfInheritance = NO_INHERITANCE;
			ea[0].Trustee.TrusteeForm = TRUSTEE_IS_SID;
			ea[0].Trustee.TrusteeType = TRUSTEE_IS_GROUP;
			ea[0].Trustee.ptstrName = (LPTSTR) usernameSID;

			if (SetEntriesInAcl(1, ea, NULL, &pACL) != ERROR_SUCCESS)
			{
				messages += "Failed SetEntriesInAcl";
			} else {
				// Try to modify the object's DACL.
				DWORD dwRes = SetNamedSecurityInfo(
					(LPTSTR)QString( basePath + QDir::separator() + username + QDir::separator() + subdirs.at(i)).utf16(),
					SE_FILE_OBJECT,              // type of object
					DACL_SECURITY_INFORMATION,   // change only object's DACL
					NULL, NULL,                  // do not change owner or group
					pACL,                        // DACL specified
					NULL);                       // do not change SACL

				if (dwRes != ERROR_SUCCESS)
				{
					messages += "Failed SetNamedSecurityInfo";
				}
			}
			// Cleanup
			if (pACL != NULL)
			{
				LocalFree(pACL);
				pACL = NULL;
			}
		}
	}
#endif

	// Share dir
#ifdef Q_OS_LINUX
	QString domain = PythonRun::getSambaParameter("global", "workgroup").simplified();
	QString homeDir = PythonRun::getSambaParameter("global", "template homedir").simplified();
	if ( ! domain.isEmpty() && ! homeDir.isEmpty() )
	{
		homeDir = homeDir.replace("%D", domain, Qt::CaseInsensitive);
		homeDir = homeDir.replace("%U", username, Qt::CaseInsensitive );

		QStringList arguments;
		arguments += "-s";
		arguments += basePath + QDir::separator() + username;
		arguments += homeDir;
		if ( QProcess::execute("ln", arguments) == 0 )
		{
			return "Ok";
		} else {
			return "Error: creating link";
		}
	} else {
		return "Error: cannot find domain or home dir";
	}
#endif
#ifdef Q_OS_WIN
	NET_API_STATUS res;
	SHARE_INFO_2 p;
	DWORD parm_err = 0;

	p.shi2_netname = (LPWSTR)username.utf16();
	p.shi2_type = STYPE_DISKTREE | STYPE_SPECIAL; // hidden disk drive
	p.shi2_remark = (LPWSTR)username.utf16();
	p.shi2_permissions = 0;
	p.shi2_max_uses = 4;
	p.shi2_current_uses = 0;
	p.shi2_path = (LPWSTR) QString( basePath + QDir::separator() + username).utf16();
	p.shi2_passwd = NULL; // no password

	res=NetShareAdd( NULL, 2, (LPBYTE) &p, &parm_err);
	if(res==0)
	{
		return "Ok";
	} else {
		return "Error: cannot share user folder: " + messages.join(",");
	}
#endif
}

#ifdef Q_OS_WIN

USER_INFO_4 LWUserCommand::getUserInfo4(QString username, QString password, QString homeDir, QString profileDir)
{
	USER_INFO_4 ui;

	ui.usri4_acct_expires = TIMEQ_FOREVER;
	ui.usri4_auth_flags = 0;
	ui.usri4_full_name = (LPWSTR)username.utf16();
	ui.usri4_usr_comment = NULL;
	ui.usri4_parms = NULL;
	ui.usri4_workstations = NULL;
	ui.usri4_logon_server = NULL;
	ui.usri4_name = (LPWSTR)username.utf16();
	if ( password.isEmpty() )
		ui.usri4_password = NULL;
	else
		ui.usri4_password = (LPWSTR)password.utf16();
	ui.usri4_priv = USER_PRIV_USER;
	ui.usri4_primary_group_id = DOMAIN_GROUP_RID_USERS;
	if ( homeDir.isEmpty() )
		ui.usri4_home_dir = NULL;
	else
		ui.usri4_home_dir = (LPWSTR)homeDir.utf16();
	if ( profileDir.isEmpty())
		ui.usri4_profile = NULL;
	else
		ui.usri4_profile = (LPWSTR)profileDir.utf16();
	ui.usri4_max_storage = USER_MAXSTORAGE_UNLIMITED;
	ui.usri4_comment = NULL;
	ui.usri4_flags = UF_SCRIPT;
	ui.usri4_script_path = NULL;
	ui.usri4_password_expired = NULL;
	ui.usri4_bad_pw_count = NULL;
	ui.usri4_code_page = NULL;
	ui.usri4_country_code = NULL;
	ui.usri4_home_dir_drive = NULL;
	ui.usri4_last_logoff = NULL;
	ui.usri4_last_logon = NULL;
	ui.usri4_logon_hours = NULL;
	ui.usri4_num_logons = NULL;
	ui.usri4_password_age = NULL;
	ui.usri4_units_per_week = NULL;
	ui.usri4_user_sid = NULL;

	return ui;
}

HRESULT LWUserCommand::getSid(LPCWSTR wszAccName, PSID * ppSid)
{
	// Validate the input parameters.
	if (wszAccName == NULL || ppSid == NULL)
	{
		return MQ_ERROR_INVALID_PARAMETER;
	}

	// Create buffers that may be large enough.
	// If a buffer is too small, the count parameter will be set to the size needed.
	const DWORD INITIAL_SIZE = 32;
	DWORD cbSid = 0;
	DWORD dwSidBufferSize = INITIAL_SIZE;
	DWORD cchDomainName = 0;
	DWORD dwDomainBufferSize = INITIAL_SIZE;
	WCHAR * wszDomainName = NULL;
	SID_NAME_USE eSidType;
	DWORD dwErrorCode = 0;
	HRESULT hr = MQ_OK;

	// Create buffers for the SID and the domain name.
	*ppSid = (PSID) new BYTE[dwSidBufferSize];
	if (*ppSid == NULL)
	{
		return MQ_ERROR_INSUFFICIENT_RESOURCES;
	}

	memset(*ppSid, 0, dwSidBufferSize);
	wszDomainName = new WCHAR[dwDomainBufferSize];
	if (wszDomainName == NULL)
	{
		return MQ_ERROR_INSUFFICIENT_RESOURCES;
	}

	memset(wszDomainName, 0, dwDomainBufferSize*sizeof(WCHAR));

	// Obtain the SID for the account name passed.
	for ( ; ; )
	{
		// Set the count variables to the buffer sizes and retrieve the SID.
		cbSid = dwSidBufferSize;
		cchDomainName = dwDomainBufferSize;

		if (LookupAccountNameW(
						   NULL,            // Computer name. NULL for the local computer
						   wszAccName,
						   *ppSid,          // Pointer to the SID buffer. Use NULL to get the size needed,
						   &cbSid,          // Size of the SID buffer needed.
						   wszDomainName,   // wszDomainName,
						   &cchDomainName,
						   &eSidType
						   ))
		{
			if (IsValidSid(*ppSid) == FALSE)
			{
				qDebug("The SID for %s is invalid.", wszAccName);
				dwErrorCode = MQ_ERROR;
			}
			break;
		}

		dwErrorCode = GetLastError();

		// Check if one of the buffers was too small.
		if (dwErrorCode == ERROR_INSUFFICIENT_BUFFER)
		{
			if (cbSid > dwSidBufferSize)
			{
				// Reallocate memory for the SID buffer.
				qDebug("The SID buffer was too small. It will be reallocated.\n");
				FreeSid(*ppSid);
				*ppSid = (PSID) new BYTE[cbSid];
				if (*ppSid == NULL)
				{
					return MQ_ERROR_INSUFFICIENT_RESOURCES;
				}
				memset(*ppSid, 0, cbSid);
				dwSidBufferSize = cbSid;
			}
			if (cchDomainName > dwDomainBufferSize)
			{
				// Reallocate memory for the domain name buffer.
				qDebug("The domain name buffer was too small. It will be reallocated.\n");
				delete [] wszDomainName;
				wszDomainName = new WCHAR[cchDomainName];
				if (wszDomainName == NULL)
				{
					return MQ_ERROR_INSUFFICIENT_RESOURCES;
				}
				memset(wszDomainName, 0, cchDomainName*sizeof(WCHAR));
				dwDomainBufferSize = cchDomainName;
			}
		} else {
			qDebug("LookupAccountNameW failed. GetLastError returned: %d\n", dwErrorCode);
			hr = HRESULT_FROM_WIN32(dwErrorCode);
			break;
		}
	}
	delete [] wszDomainName;
	return hr;
}
#endif

QString LWUserCommand::removeUser(QString username)
{
#ifdef Q_OS_LINUX
	QStringList arguments;
	arguments += "user";
	arguments += "delete";
	arguments += username;

	QString res = PythonRun::runSambaTool( arguments );
	if ( res.isEmpty() )
	{
		return _SUCCESS_( tr("User %1 removed succesfully").arg(username) );
	} else {
		return _ERROR_( tr("Error removing user %1").arg(username) );
	}
#endif
#ifdef Q_OS_WIN
	NET_API_STATUS nStatus;

	QString logonServer= qgetenv("LOGONSERVER");
	nStatus = NetUserDel((LPCWSTR)logonServer.utf16(), (LPCWSTR)username.utf16());

	if (nStatus == NERR_Success)
	{
		return _SUCCESS_( tr("User %1 removed succesfully").arg(username) );
	} else {
		QString errorMessage;
		switch (nStatus)
		{
		case ERROR_ACCESS_DENIED:
			errorMessage = tr("The user does not have access to the requested information");
			break;
		case NERR_InvalidComputer:
			errorMessage = tr("The computer name is invalid");
			break;
		case NERR_NotPrimary:
			errorMessage = tr("The operation is allowed only on the primary domain controller of the domain");
			break;
		case NERR_UserNotFound:
			errorMessage = tr("The user name could not be found");
			break;
		default:
			errorMessage = tr("Unknown error removing user");
		}
		return _ERROR_( errorMessage );
	}
#endif
}

QString LWUserCommand::enableUser(QString username, QString group)
{
	if ( group.isEmpty() )
	{
		return tr("Error: LibreWork group is not defined");
	}
#ifdef Q_OS_LINUX
	QStringList groupArguments;
	groupArguments += "group";
	groupArguments += "addmembers";
	groupArguments += group;
	groupArguments += username;

	QString res = PythonRun::runSambaTool( groupArguments );
	if ( res.isEmpty() )
	{
        QString token = generateToken();
        if ( token.isEmpty() )
        {
            return _ERROR_( tr("Token generation failed") );
        } else {
            return token;
        }
    } else {
        return res;
    }
#endif
#ifdef Q_OS_WIN
    NET_API_STATUS nStatus;
	/*
	size_t buf_size = 0;
	DWORD dwLevel = 0;
	GROUP_USERS_INFO_0 *gi = NULL;

	uint8_t *buffer = NULL;
    buf_size = sizeof(GROUP_USERS_INFO_0);

	nStatus = NetApiBufferAllocate((DWORD)buf_size, (void **)&gi);
    if (nStatus)
    {
        return tr("NetApiBufferAllocate failed");
    }

    wchar_t * _group = (wchar_t*)malloc(sizeof(wchar_t*) * group.size() + 1);
    group.toWCharArray(_group);
    _group[group.size()] = 0;
    gi->grui0_name = (LPWSTR) _group;
	nStatus = NetUserSetGroups(NULL, (LPCWSTR)username.utf16(), dwLevel, (LPBYTE)gi, 1);
	NetApiBufferFree(buffer);
	*/
    //QString logonServer= qgetenv("LOGONSERVER");
	nStatus = NetGroupAddUser( NULL, (LPCWSTR)group.utf16(), (LPCWSTR)username.utf16());


    if (nStatus == NERR_Success)
    {
        QString token = generateToken();
        if ( token.isEmpty() )
        {
            return _ERROR_( tr("Token generation failed") );
        } else {
            return token;
        }
    } else {
        QString errorMessage;
        switch (nStatus)
        {
            case ERROR_ACCESS_DENIED:
                errorMessage = tr("The user does not have access to the requested information");
                break;
            case ERROR_INVALID_LEVEL:
                errorMessage = tr("The system call level is not correct. This error is returned if the level parameter was specified as a value other than 0 or 1.");
                break;
            case ERROR_INVALID_PARAMETER:
                errorMessage = tr("A parameter passed was not valid. This error is returned if the num_entries parameter was not valid");
                break;
            case ERROR_NOT_ENOUGH_MEMORY:
                errorMessage = tr("Insufficient memory was available to complete the operation");
                break;
            case NERR_InvalidComputer:
                errorMessage = tr("The computer name is invalid");
                break;
            case NERR_NotPrimary:
                errorMessage = tr("The operation is allowed only on the primary domain controller of the domain");
                break;
            case NERR_GroupNotFound:
				errorMessage = tr("The group name specified by the grui*_name in the GROUP_USERS_INFO_* structure pointed to by the buf parameter does not exist");
                break;
            case NERR_InternalError:
                errorMessage = tr("An internal error occurred");
                break;
			case NERR_SpeGroupOp:
				errorMessage = tr("This operation is not allowed on certain special groups");
			break;
            case NERR_UserNotFound:
                errorMessage = tr("The user name could not be found");
                break;
            default:
                errorMessage = tr("Unknown error adding user to Librework default group");
        }
        return _ERROR_( errorMessage );
    }
#endif
}

QString LWUserCommand::disableUser(QString username, QString group)
{
	if ( group.isEmpty() )
	{
		return _ERROR_( tr("Error: LibreWork group is not defined") );
	}
#ifdef Q_OS_LINUX
	QStringList groupArguments;
	groupArguments += "group";
	groupArguments += "removemembers";
	groupArguments += group;
	groupArguments += username;

	QString res = PythonRun::runSambaTool( groupArguments );
	if ( res.isEmpty() )
	{
		return _SUCCESS_( tr("User %1 disabled succesfully").arg(username) );
    } else {
        return _ERROR_( tr("Cannot disable user %1").arg(username) );
    }
#endif
#ifdef Q_OS_WIN
    NET_API_STATUS nStatus;

    QString logonServer= qgetenv("LOGONSERVER");
    nStatus = NetGroupDelUser((LPCWSTR)logonServer.utf16(), (LPCWSTR)group.utf16(), (LPCWSTR)username.utf16());

    if (nStatus == NERR_Success)
    {
        return _SUCCESS_( tr("User %1 disabled succesfully" ).arg(username) );
    } else {
        return _ERROR_( tr("Cannot disable user %1").arg(username) );
    }
#endif
}

QString LWUserCommand::generateToken()
{
	const QString possibleCharacters("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789");
	uint t = (uint)QTime::currentTime().msecsSinceStartOfDay();
	qsrand( t );

	QString token;
	for(int i = 0; i < TOKEN_LENGTH; i++)
	{

		int index = qrand() % possibleCharacters.length();
		QChar nextChar = possibleCharacters.at(index);
		token.append(nextChar);
	}

	return token;
}
