#include "lwvirtcommand.h"

QString LWVirtCommand::startServer(QString serverName)
{
#ifdef Q_OS_LINUX
    virDomainPtr dom;
    virConnectPtr conn;
    QString filePath;
    int res;

    conn = createConnection();

    filePath = getFilePath( serverName );
    res = virDomainRestore(conn, filePath.toLatin1().data());
    dom = virDomainLookupByName( conn, serverName.toLatin1().data() );
    res = virDomainResume(dom);
    if (res > -1){
        return QString("start ok");
    }
    else {
        return QString("error start");
    }

#endif
    return QString::null;
}

QString LWVirtCommand::pauseServer(QString serverName)
{

 #ifdef Q_OS_LINUX
    virDomainPtr dom;
    virConnectPtr conn;
    QString filePath;
    int res;

    conn = createConnection();
    dom = virDomainLookupByName( conn, serverName.toLatin1().data() );
    //filePath = getFilePath( serverName );
    res = virDomainSuspend(dom);
    if (res > -1){
        return QString("pause ok");
    }
    else {
        return QString("error pause");
    }

#endif
	return QString::null;
}

QString LWVirtCommand::hardStopServer(QString serverName)
{
#ifdef Q_OS_LINUX
#endif
    return QString::null;
}

QString LWVirtCommand::resumeServer(QString serverName)
{
#ifdef Q_OS_LINUX
#endif
	return QString::null;
}

QString LWVirtCommand::hardRebootServer(QString serverName)
{
#ifdef Q_OS_LINUX
#endif
	return QString::null;
}

QString LWVirtCommand::softStopServer(QString serverName)
{

#ifdef Q_OS_LINUX
    virDomainPtr dom;
    virConnectPtr conn;
    QString filePath;
    int res;

    conn = createConnection();
    dom = virDomainLookupByName( conn, serverName.toLatin1().data() );
    filePath = getFilePath( serverName );
    res = virDomainSave(dom, filePath.toLatin1().data());
    if (res > -1){
        return QString("stop ok");
    }
    else {
        return QString("error stop");
    }
#endif
    return QString::null;

}

QString LWVirtCommand::softRebootServer(QString serverName)
{
#ifdef Q_OS_LINUX
#endif
	return QString::null;
}

QStringList LWVirtCommand::serverList()
{

#ifdef Q_OS_LINUX
    virConnectPtr conn;
    QStringList servers;
    int numNames, i;
    int flags = VIR_CONNECT_LIST_DOMAINS_ACTIVE |
                VIR_CONNECT_LIST_DOMAINS_INACTIVE;
    virDomainPtr *nameList = NULL;

    conn = createConnection();
    if (!conn){

       servers << "error libvirt connection";
       return servers;
    }
    numNames = virConnectListAllDomains(conn, &nameList, flags);


    if (numNames == -1) {
        servers << "error libvirt list domains";
        return servers;
    }
    for (i = 0; i < numNames; i++) {
        servers << virDomainGetName(nameList[i]);
    }
    free(nameList);
    return servers;
#endif
    return QStringList();

}

QString LWVirtCommand::getServerStatus(QString serverName)
{
#ifdef Q_OS_LINUX
    //return SERVER_PAUSED;
    //return SERVER_RUNNING;
    virDomainPtr dom;
    virConnectPtr conn;
    int res;
    //virDomainInfoPtr info;
    virDomainInfo *info;
    virDomainInfo val;

    conn = createConnection();
    dom = virDomainLookupByName( conn, serverName.toLatin1().data() );

    res = virDomainGetInfo(dom, &val);
    if (res > -1) {
        if (val.state == VIR_DOMAIN_SHUTDOWN || val.state == VIR_DOMAIN_SHUTOFF){
            return SERVER_STOPPED;
        } else if (val.state == VIR_DOMAIN_RUNNING){
            return SERVER_RUNNING;
        } else if (val.state == VIR_DOMAIN_PAUSED){
            return SERVER_PAUSED;
        }
    }
    return SERVER_UNKNOWN;

#endif
	return QString::null;
}


#ifdef Q_OS_LINUX
//private linux funs

virConnectPtr LWVirtCommand::createConnection(){

    virConnectPtr conn;

    conn = virConnectOpenAuth("", virConnectAuthPtrDefault, 0);
    return conn;

}


QString LWVirtCommand::getFilePath(QString serverName){
    return SERVER_VIRT_FILES_SAVED_DIR + serverName + ".img";

}



#endif
