#! /usr/bin/env python2.7

import cups, sys, os, json
from auth import Auth
from printer import Printer

def usage():
	print("Usage: manageprinters.py <command> [ <config_file> <account_name> <cups_name> <google_printer_prefix> ]")
	print("Commands:")
	print("connect <config_file> <account_name> <printer_name> <google_printer_prefix> <username>")
	print("disconnect <config_file> <account_name> <printer_name> <google_printer_prefix> <username>")
	print("prune <config_file> <google_printer_prefix>")
	sys.exit(1)

try:
	command = sys.argv[1]
	configFile = sys.argv[2]
except:
	usage()

Auth.config = configFile
requestors, storage = Auth.SetupAuth(True)

if os.path.exists(Auth.config):
	try:
		content_file = open(Auth.config, 'r')
		content = content_file.read()
		data = json.loads(content)
	except:
		# remove old config file
		os.remove(Auth.config)

if (command != "connect") & (command != "disconnect") & (command != "prune"):
	usage()

if len(sys.argv) > 7:
	usage()

if command == "prune":
	if len(sys.argv) > 4:
		usage()
	prefix = sys.argv[3]
	printeruris = []
	printer = Printer(requestors)
	printers = printer.getPrinters()
	for foundprinter in printers:
		printeruris.append(printer.printerNameToUri(foundprinter['account'], foundprinter['name'].encode('ascii', 'replace')))

	# check for printers to prune

	prunePrinters = []
	connection = cups.Connection()
	cupsprinters = connection.getPrinters()

	for cupsprinter in cupsprinters:
		if cupsprinters[cupsprinter]['device-uri'].startswith( printer.PROTOCOL ):
			if cupsprinters[cupsprinter]['device-uri'] not in printeruris:
				if cupsprinters[cupsprinter]['printer-info'].startswith( prefix ):
					prunePrinters.append(cupsprinter)

	if len( prunePrinters ) > 0 :
		print("Found " + str(len( prunePrinters )) + " printers which no longer exist on cloud print:")
		for printer in prunePrinters:
			print(printer)
		for printer in prunePrinters:
			connection.deletePrinter(printer)
			print("Deleted",printer)

	sys.exit(0)

try:
	accountName = sys.argv[3]
	printerName = sys.argv[4]
	prefix = sys.argv[5]
	username = sys.argv[6]
except:
	usage()

selected = False

for requestor in requestors:
	addedCount = 0
	if requestor.getAccount() == accountName:
		selectedRequestor = requestor
		selected = True
		break
if selected == False:
	print("No account with that name")
	sys.exit(1)

connection = cups.Connection()
cupsprinters = connection.getPrinters()
printer = Printer(requestor)
printers = printer.getPrinters()
if printers == None:
	print("No Printers Found")
	sys.exit(1)

if command == "connect":
	for ccpprinter in printers:
		if ccpprinter['name'] != printerName:
			continue
		uri = printer.printerNameToUri(ccpprinter['account'], ccpprinter['name'].encode('ascii', 'replace'))
		found = False
		for cupsprinter in cupsprinters:
			if cupsprinters[cupsprinter]['device-uri'] == uri:
				found = True
		
		if found == False:
			printername = prefix + ccpprinter['name']
			printername = printername.replace( ' ', '_' ).replace( '\\', '_' ).replace( '-', '_' )
			
			# check if printer name already exists
			foundbyname = False
			for ccpprinter2 in cupsprinters:
				if printer.sanitizePrinterName(cupsprinters[ccpprinter2]['printer-info']) == printer.sanitizePrinterName(printername):
					foundbyname = True
			
			if printername != "":
				printer.addPrinter(printername, uri, connection)
				connection.setPrinterUsersAllowed(printername, [username])
				addedCount+=1
			
	if addedCount > 0:
		print("Added " + str(addedCount) + " new printers to CUPS")
	else:
		print("No new printers to add")

if command == "disconnect":
	printername = printerName.replace( ' ', '_' ).replace( '\\', '_' ).replace( '-', '_' )
	try:
		connection.deletePrinter(prefix + printername)
	except:
		print("Printer not found");
	
