#include "pythonrun.h"

PythonRun::PythonRun( QStringList program )
{
	process = new QProcess();
	this->program = program;
}

PythonRun::~PythonRun()
{
	delete process;
}

QProcess::ExitStatus PythonRun::run()
{
	QString command = "python";
	QStringList arguments;
	arguments += "-i";
	arguments += "-u"; // Unbuffered python
	process->setReadChannel( QProcess::StandardError );
	process->start( command, arguments );
	process->waitForStarted();
	process->waitForReadyRead();
	process->waitForReadyRead();
	QByteArray startText = process->readAllStandardError();
	for (int i = 0; i < program.count(); i++)
	{
		runLine( program[i] + "\n" );
	}
	process->waitForFinished( PYTHONRUNTIMEOUT );
	return process->exitStatus();
}

void PythonRun::runLine( QString line )
{
	process->write( line.toUtf8() );
	process->waitForBytesWritten();
}

QByteArray PythonRun::readAllStandardError()
{
	return process->readAllStandardError();
}

int PythonRun::exitCode()
{
	return process->exitCode();
}

QString PythonRun::runSambaTool(QStringList arguments)
{
	if ( arguments.count() == 0 )
	{
		return QString::null;
	}

	QStringList program;
	program += "import sys";
	program += "from samba.netcmd.main import cmd_sambatool";
	program += "cmd = cmd_sambatool()";
	program += "subcommand = \"" + arguments[0] + "\"";
	QString args;
	for (int i = 1; i < arguments.count(); i++)
	{
		args += "\"" + arguments[i] + "\"";
		if ( i < arguments.count() - 1 )
		{
			args += ", ";
		}
	}
	program += "args = (" + args + ")";
	program += "retval = cmd._run(\"samba-tool\", subcommand, *args)";
	program += "sys.exit( retval )";

	PythonRun prun( program );
	if ( prun.run() == QProcess::NormalExit )
	{
		if ( prun.exitCode() == 0 )
		{
			return QString::null;
		} else {
			return prun.readAllStandardError().replace(">>>", "").simplified();
		}
	} else {
		return "Error: " + prun.readAllStandardError().replace(">>>", "").simplified();
	}
}

QString PythonRun::getSambaParameter(QString section, QString parameter)
{
	QStringList arguments;
	arguments += "-l";
	arguments += "-s";
	arguments += "--parameter-name=" + parameter;
	arguments += "--section-name=" + section;
	QProcess process;
	process.start("testparm", arguments);
	process.waitForFinished();
	return process.readAllStandardOutput();
}
